var Locale = {
    name: "en",
    fields: { },
    ui: { },
    set: function(locale, option) {
        option = option || { };
        option.fields = option.fields || "erpscan_logs";
        Locale.name = locale;
        Locale.load.fields(option.fields);
        Locale.load.ui();
    },
    loadPage: function(page, is_return) {
        var locale = Helpers.loadJSON("/config/locale/ui/pages/{level_1}/{level_2}/{level_3}{locale}.json".format({
            level_1: page[0],
            level_2: page[1],
            level_3: page[2] ? page[2] + "/" : "",
            locale: Locale.name
        }));
        if (is_return) {
            return locale;
        } else {
            Locale.ui.page = locale;
        }
    },
    load: {
        fields: function(namespaces) {
            var locale_fields = Helpers.loadJSON("/config/locale/fields/" + Locale.name + ".json");
            var custom_fields = Helpers.loadJSON("/config/locale/fields/custom.json");
            for (var i = 0; i < namespaces.length; i++) {
                if (typeof Locale.fields[namespaces[i]] == "undefined") {
                    Locale.fields[namespaces[i]] = { };
                }
                for (var field in locale_fields[namespaces[i]]) {
                    var is_custom_set = typeof custom_fields[namespaces[i]] != "undefined" && typeof custom_fields[namespaces[i]][field] != "undefined";
                    Locale.fields[namespaces[i]][field] = is_custom_set ? custom_fields[namespaces[i]][field] : locale_fields[namespaces[i]][field];
                }
            }
        },
        ui: function() {
            Locale.ui.common = Helpers.loadJSON("/config/locale/ui/common/" + Locale.name + ".json");
        }
    }
};

var Page = {
    path: null,
    templates: {},
    fields: {},
    tableFormat: {
        "text": "input",
        "integer": "input",
        "float": "input",
        "complex": "input",
        "enum": "select",
        "date": "datetime",
        "none": "none"
    },
    theme: {
        name: $.cookie('theme') ? $.cookie('theme') : "default",
        colors: {
            pie_stroke: {
                "default": "#c5c5c5",
                "dark": "#fff"
            },
            stack_grid: {
                "default": "#000",
                "dark": "#fff"
            }
        },
        setHandler: function() {
            $(document.body).on("click", "header .theme", function() {
                $.cookie('theme', Page.theme.name == "default" ? "dark" : "default");
                location.reload();
                return false;
            })
        }
    },
    loadFieldsConfig: function(namespaces) {
        var fields = Helpers.loadJSON("/config/fields.json");
        if (ERPScan.data == null) {
            ERPScan.data = {};
        }
        ERPScan.data.fields = {};
        for (var i = 0; i < namespaces.length; i++) {
            Page.fields[namespaces[i]] = {};
            for (var field in fields[namespaces[i]]) {
                Page.fields[namespaces[i]][field] = ERPScan.data.fields[field] = fields[namespaces[i]][field];
            }
        }
    },
    setCustomURL: function(url) {
        localStorage.setItem("erpscan-url", url);
        location.reload();
    },
    requestCustomURL: function(current_url) {
        new Popup({
            class: "request-url",
            title: "Data source URL",
            content: "<input type='text' name='request-url' style='vertical-align:middle;height:17px;width:200px;margin-right:5px;' value='" + current_url + "' /> <button class='request-url-submit' style='vertical-align:middle;'>OK</button><p><a>By default: <a href='#' class='set-default-data-source'><i>" + location.host + "</i></a></i></p>",
            width: 350,
            height: 80,
            close: function() {
                $(".popup.request-url .request-url-submit").off("click");
                $(".popup.request-url input[name=\"request-url\"]").off("keyup");
                $(".popup.request-url .set-default-data-source").off("keyup");
            }
        }).show();
        $(".popup.request-url .set-default-data-source").on("click", function () {
            $(".popup.request-url input[name=\"request-url\"]").val($(this).text());
        });
        $(".popup.request-url .request-url-submit").on("click", function(){
            Page.setCustomURL($(".popup.request-url input[name=\"request-url\"]").val());
        });
        $(".popup.request-url input[name=\"request-url\"]").on("keyup", function(event){
            if (event.keyCode == 13) {
                Page.setCustomURL($(".popup.request-url input[name=\"request-url\"]").val());
            }
        });
    },
    load: function() {
        if (localStorage.getItem("erpscan-url") == null) {
            Page.requestCustomURL(location.host);
        }
        Page.theme.setHandler();
        Page.Render.init();
    }
};

var Header = {
    height: 0,
    templates: {
        theme: '<a href="#" class="erpscan-instance-url">{url}</a><div class="version">ERPScan 5.0 step 2 b2</div><a href="#" class="theme"></a>',
        logo: '<div class="logo"></div>',
        menu: {
            wrap: '<nav class="main">{content}</nav>',
            item: '<a href="{url}" class="{class}">{text}</a>'
        },
        submenu: {
            wrap: '<nav class="sub">{content}</nav>',
            item: '<a href="{url}" class="{class}">{text}</a>'
        },
        tabs: {
            wrap: '<div class="tabs">{content}</div>',
            item: '<a href="{url}" class="{class}">{text}</a>',
            add: '<a href="#" class="add"></a>',
            content: '<section class="content tab-{tab_name}">{content}</section>',
            loading: '<div class="loading"></div>'
        },
        title: '<div class="title"><h1>{title}</h1><h2>{description}</h2></div>',
        disposition: '<div class="disposition"><a href="#" class="left"></a><a href="#" class="right"></a><a href="#" class="top"></a><a href="#" class="bottom"></a></div>',
        filters: '<div class="filters"><div class="filter-inputs"><a href="#" class="add"></a></div></div>'
    },
    tabsAdd: function(tabs) {
        for (var i = 0; i < tabs.length; i++) {
            if ($("header .tabs a." + tabs[i]).length == 0) {
                var tab = Header.templates.tabs.item.format({
                    "url": "#",
                    "class": tabs[i],
                    "text": Locale.ui.common.tabs[Page.Render.path[0]][Page.Render.path[1]][tabs[i]]
                });
                $("header .tabs").append(tab);
                $("header .tabs a:last-child").attr("data-tab", tabs[i]);
                var $section = $("section.content");
                $section = $($section[$section.length - 1]);
                $section.after(Header.templates.tabs.content.format({
                    tab_name: tabs[i],
                    content: Header.templates.tabs.loading
                }));
            }
        }
    },
    tabsRemove: function(tabs) {
        for (var i = 0; i < tabs.length; i++) {
            var $tab = $("header .tabs a." + tabs[i]);
            if ($tab.hasClass("active")) {
                $tab.removeClass("active");
                $("header .tabs a").eq(0).addClass("active");
            }
            $("section.content.tab-" + tabs[i]).remove();
            $tab.remove();
        }
    },
    tabSelect: function(name, params) {
        params = params || {};
        $("header .tabs a").removeClass("active");
        $("header .tabs a." + name).addClass("active");
        $("section.content").hide();
        var $tab = $("section.content.tab-" + name);
        if (!$tab.html()) {
            $tab.html(Header.templates.tabs.loading);
        }
        $tab.show();
        $("header .title h1").html(params.title || Locale.ui.page.title);
        $("header .title h2").html(params.description || Locale.ui.page.description);
    },
    load: function(current_page, option) {
        var items = Helpers.loadJSON("/config/menu.json");
        option = option || { };
        option.type = option.type || false;
        var menu = '';
        for (var item in items.menu) {
            menu += Header.templates.menu.item.format({
                "url": items.menu[item],
                "class": item == current_page[0] ? 'active' : '',
                "text": Locale.ui.common.menu[item]
            });
        }
        menu = Header.templates.menu.wrap.format({ content: menu });
        var submenu = '';
        for (var item in items.submenu[current_page[0]]) {
            submenu += Header.templates.submenu.item.format({
                "url": items.submenu[current_page[0]][item],
                "class": item == current_page[1] ? 'active' : '',
                "text": Locale.ui.common.submenu[current_page[0]][item]
            });
        }
        submenu = Header.templates.submenu.wrap.format({ content: submenu });
        var tabs = '';
        if (items.tabs && option.tabs !== false) {
            for (var item in items.tabs[current_page[0]][current_page[1]]) {
                tabs += Header.templates.tabs.item.format({
                    "url": items.tabs[current_page[0]][current_page[1]][item],
                    "class": item == current_page[2] ? 'active ' + item : item,
                    "text": Locale.ui.common.tabs[current_page[0]][current_page[1]][item]
                });
            }
            tabs = Header.templates.tabs.wrap.format({ content: tabs });
        }
        var title = Header.templates.title.format({
            title: Locale.ui.page.title,
            description: Locale.ui.page.description
        });
        if (option.filters === true) {
            title += Header.templates.filters;
        }
        if (option.disposition === true) {
            title = Header.templates.disposition + title;
        }
        var $header = $("header"),
            theme = Header.templates.theme.format({
                url: localStorage.getItem("erpscan-url") || window.location.host
            });
        $header.html(theme + Header.templates.logo + menu + submenu + tabs + title);
        document.title = Locale.ui.page.description + " - " + Locale.ui.page.title + " - ERPScan ATP";
        Header.height = $header.height();
        Header.handlers();
    },
    handlers: function() {
        $("header .erpscan-instance-url").on("click", function() {
            Page.requestCustomURL($(this).text());
            return false;
        });
        $(window).keydown(function(event) {
            if (event.altKey && event.shiftKey && event.which == 69) {
                $(window).jGravity({
                    target: '.logo',
                    weight: 30,
                    depth: 5,
                    drag: true
                });
                $(".logo").css({marginLeft: 0}).mousemove().before("<div class='logo' style='opacity: 0.0;'></div>");
            }
        });
    }
};

function Popup(params) {
    Popup.counter++;
    this.params = params;
    this.selector = '.popup #popup-' + Popup.counter;
    this.shadowSelector = '#popupShadow-' + Popup.counter;
    this.templates = {
        wrap: '<div class="wrap" id="popup-{id}"><a href="#" class="close"></a><div class="content">{content}</div></div>',
        shadow: '<div id="popupShadow-{id}" class="popup-shadow {class}"></div>',
        confirmMessage: '<span class="exit-popup_text">{message}</span>',
        actions: '<div class="actions-bar"><button class="no default">No</button><button class="yes">Yes</button></div>'
    };
    this.show = function(){
        var block = new Block({
            title: this.params.title,
            class: this.params.class + ' popup' || "popup",
            content: this.templates.wrap.format({
                content: this.params.content,
                id: Popup.counter
            })
        }).getHTML();
        $(document.body).append(this.templates.shadow.format({
            id: Popup.counter,
            class: this.params.class ? this.params.class + "Shadow" : ''
        }) + block);
        this.setSize();
        this.handlers();
    };
    this.setSize = function() {
        var $popupBlock = $(this.selector).parents(".block");
        $popupBlock.css({
            marginTop: Math.round(-$popupBlock.outerHeight() / 2),
            marginLeft: Math.round(-$popupBlock.outerWidth() / 2)
        });
    };
    this.close = function(){
        if(this.params.close) {
            this.params.close();
        }
        $(this.selector).closest('.block').remove();
        $(this.shadowSelector).remove();
        this.clearEvents();
    };
    this.showConfirmPopup = function(){
        var o = this,
            confirmClosePopup = new Popup({
                class: 'confirmClosePopup',
                title: Locale.ui.common.confirmationClosePopup.title,
                content: this.templates.confirmMessage.format({
                    message: this.params.confirmation ? this.params.confirmation : Locale.ui.common.confirmationClosePopup.defaultMessage
                }) + this.templates.actions
            });
        confirmClosePopup.show();
        $(document.body).on('click', confirmClosePopup.selector + " .yes", function(){
            confirmClosePopup.close();
            o.close();
        });
        $(document.body).on('click', confirmClosePopup.selector + " .no", function(){
            confirmClosePopup.close();
        });
    };
    this.clearEvents = function(){
        $(document.body).off("click", this.selector + " .close, " + this.shadowSelector);
        $(document.body).off('click', this.selector + " .no");
        $(document.body).off('click', this.selector + " .yes");
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on("click", this.selector + " .close, " + this.shadowSelector, function(e){
            if (o.params.confirmation) {
                o.showConfirmPopup();
            } else {
                o.close();
            }
        });
    };
}

Popup.counter = 0;

function PopupConfirm(params) {
    this.params = params;
    this.popup = null;
    this.templates = {
        buttons: '<div class="buttons"><button class="default cancel">Cancel</button><button class="confirm">Confirm</button></div>',
        text: '<div class="text">{content}</div>'
    };
    this.handlers = function() {
        var o = this;
        var closeCallback = function() {
            o.popup.close();
            $(window).off("keydown");
            $(o.popup.selector).find(".buttons button.cancel").off("click");
            $(o.popup.selector).find(".buttons button.confirm").off("click");
        };
        $(window).keydown(function(event) {
            if (event.which == 27) {
                closeCallback();
            }
            if (event.which == 13) {
                closeCallback();
                o.params.callback();
            }
        });
        $(this.popup.selector).find(".buttons button.cancel").on("click", function() {
            closeCallback();
        });
        $(this.popup.selector).find(".buttons button.confirm").on("click", function() {
            closeCallback();
            o.params.callback();
        });
    };
    this.show = function() {
        this.popup = new Popup({
            class: "confirm-action",
            title: this.params.title || "Confirm action",
            content: this.templates.text.format({ content: this.params.text }) + this.templates.buttons
        });
        this.popup.show();
        $(this.popup.selector).find(".confirm").focus();
        this.handlers.call(this);
    }
}

function Block(params){
    Block.counter++;
    this.id = Block.counter;
    this.params = params;
    this.templates = {
        block: '<div id="block-{id}" class="block {class}">'+
        '<iframe class="blockResize"></iframe>'+
        '<h3>{title}</h3>'+
        '<div class="block-content">{content}</div>'+
        '</div>',
        actionsBlock: '<div class="action-events">{actions}</div>',
        actionsItem: '<div class="action" ><a href="#" class="link" data-action="{dataAction}">{action}</a></div>'
    };
    this.getHTML = function(){
        var o = this,
            block = '';
        block += this.templates.block.format({
            id: this.id,
            class: this.params.class ? this.params.class : '',
            title: this.params.title ? this.params.title : '',
            content: this.params.content ? this.params.content : ''
        });
        return block;
    };
}

Block.counter = 0;

function Loader() {
    this.fields = function(pagePath) {
        
    }
}

function AbstractPage() { }

AbstractPage.prototype = {
    section: "section.content",
    handlers: {},
    objects: {},
    data: {},
    addHandlers: function(handlers) {
        var o = this;
        $.each(handlers, function(handlerId, handler) {
            if (o.handlers[handlerId] != null) {
                $(document.body).off(handler.action, handler.selector);
            }
            $(document.body).on(handler.action, handler.selector, handler.callback);
            o.handlers[handlerId] = handler;
        });
    },
    removeHandlers: function(handlers) {
        var o = this;
        if(typeof handlers == "undefined" || handlers.length == 0) {
            var handlers = [];
            $.each(o.handlers, function(handlerId) {
                handlers.push(handlerId);
            });
        }
        handlers.forEach(function(handlerId) {
            $(document.body).off(o.handlers[handlerId].action, o.handlers[handlerId].selector);
        })
    },
    clearHandlers: function(handlers) {
        var o = this;
        if(typeof handlers == "undefined" || handlers.length == 0) {
            var handlers = [];
            $.each(o.handlers, function(handlerId) {
                handlers.push(handlerId);
            });
        }
        handlers.forEach(function(handlerId) {
            $(document.body).off(o.handlers[handlerId].action, o.handlers[handlerId].selector);
        })
    },
    preLoad: function (params) {
        this.data.texts = Locale.ui.page["content"];
        this.data.templates = ERPScan.loader.templates.load();
        this.data.fields = ERPScan.loader.fields();
        ERPScan.data = this.data;
        var o = this,
            counter = 0,
            totalQueries = ((params.entities == null ? 0 : params.entities.length) + (params.dictionaries != null ? 1 : 0));
        if (params.dictionaries != null) {
            ERPScan.API.loadDictionaries(params.dictionaries, {
                callback: function (result) {
                    o.data.dictionaries = ERPScan.data.dictionaries = result;
                    if (++counter == totalQueries) {
                        params.loadedCallback.call(o);
                    }
                }
            });
        }
        if (params.entities != null) {
            for (var i = 0; i < params.entities.length; i++) {
                ERPScan.API.query(params.entities[i], "map", {
                    callback: function(i) {
                        return function(result) {
                            o.data[params.entities[i]] = result["data"]["result"];
                            if (++counter == totalQueries) {
                                params.loadedCallback.call(o);
                            }
                        }
                    }(i)
                });
            }
        }
        if (params.dictionaries == null && params.entities == null) {
            params.loadedCallback.call(o);
        }
    },
    setDictionary: function(type, values) {
        this.data.dictionaries[type] = ERPScan.data.dictionaries[type] = values;
    },
    createFormValidator: function(formParams) {
        var validationRules = {};
        for (var field in this.data.fields[formParams.fieldsSet]) {
            validationRules[field] = this.data.fields[formParams.fieldsSet][field]["rules"] || {};
            if (formParams.errorsValidation[field] != null) {
                $.extend(validationRules[field], formParams.errorsValidation[field]);
            }
        }
        var config = {
            form : formParams.selector,
            validate : validationRules
        };
        var modulesLoaded = !Helpers.objIsEmpty($.formUtils.loadedModules);
        $.validate({
            modules : 'date, jsconf, security, location, date, file',
            onModulesLoaded : function() {
                $.setupValidation(config);
            }
        });
        if (modulesLoaded) {
            $.setupValidation(config);
        }
    }
};

function AbstractList() { };

AbstractList.prototype = {
    conversionFields: function(result, rules) {
        var data = [];
        for (var i = 0; i < result.length; i++) {
            var index = data.push({}) - 1;
            for (var field in rules) {
                if (typeof result[i][field] == "undefined") {
                    data[index][field] = rules[field].call(this, result[i], field, i);
                }
            }
            for (var field in result[i]) {
                var isset = typeof rules[field] != "undefined",
                    skip = isset && rules[field] == null;
                if (skip) {
                    continue;
                }
                data[index][field] = isset ? rules[field].call(this, result[i], field, i) : result[i][field];
            }
        }
        return data;
    },
    table: null
}

function Tooltip(params){
    Tooltip.counter++;
    this.params = params;
    this.id = Tooltip.counter;
    this.$tooltip = null;
    this.templates = {
        tooltip: "<div class='tooltip' id='tooltip-{id}'>{text}</div>"
    };
    this.show = function(){
        if(this.params.text != "undefined") {
            var tooltipHTML = this.templates.tooltip.format({
                text: this.params.text,
                id: this.id
            });
            $(document.body).append(tooltipHTML);
            this.$tooltip = $(document.body).find("#tooltip-" + this.id);
            this.addFollowEvent();
            this.addRemoveEvent();
        }
    };
    this.addRemoveEvent = function(){
        var o = this;
        $(this.params.e.target).on("mouseleave", function(){
            o.$tooltip.fadeOut(500, function(){
                o.$tooltip.remove();
            });
        });
    };
    this.addFollowEvent = function(){
        var o = this;
        $(this.params.e.target).on("mousemove", function(e){
            o.$tooltip.css({
                left: e.clientX + 15,
                top: e.clientY + 10
            });
        });
    };
}
Tooltip.counter = 0;