Date.prototype.getMonthName = function(){
    var month_names = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
    return month_names[this.getMonth()];
};

Date.prototype.getMonthAbbr = function(){
    var month_abbrs = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
    return month_abbrs[this.getMonth()];
};

Date.prototype.getDayFull = function(){
    var days_full = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
    return days_full[this.getDay()];
};

Date.prototype.getDayAbbr = function(){
    var days_abbr = ['Sun', 'Mon', 'Tue', 'Wed', 'Thur', 'Fri', 'Sat'];
    return days_abbr[this.getDay()];
};

Date.prototype.getDayOfYear = function() {
    var onejan = new Date(this.getFullYear(), 0, 1);
    return Math.ceil((this - onejan) / 86400000);
};

Date.prototype.getDaySuffix = function() {
    var d = this.getDate();
    var sfx = ["th", "st", "nd", "rd"];
    var val = d % 100;
    return (sfx[(val-20)%10] || sfx[val] || sfx[0]);
};

Date.prototype.getWeekOfYear = function() {
    var onejan = new Date(this.getFullYear(), 0, 1);
    return Math.ceil((((this - onejan) / 86400000) + onejan.getDay()+1)/7);
};

Date.prototype.isLeapYear = function(){
    var yr = this.getFullYear();
    if ((parseInt(yr)%4) == 0) {
        if (parseInt(yr)%100 == 0){
            if (parseInt(yr)%400 != 0)
                return false;
            if (parseInt(yr)%400 == 0)
                return true;
        }
        if (parseInt(yr)%100 != 0)
            return true;
    }
    if ((parseInt(yr)%4) != 0)
        return false;
};

Date.prototype.getMonthDayCount = function() {
    var month_day_counts = [31, this.isLeapYear() ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    return month_day_counts[this.getMonth()];
};

Date.prototype.format = function(dateFormat){
    dateFormat = dateFormat.split("");
    var date = this.getDate(),
        month = this.getMonth(),
        hours = this.getHours(),
        minutes = this.getMinutes(),
        seconds = this.getSeconds();
    var date_props = {
        d: date < 10 ? '0'+date : date,
        D: this.getDayAbbr(),
        j: this.getDate(),
        l: this.getDayFull(),
        S: this.getDaySuffix(),
        w: this.getDay(),
        z: this.getDayOfYear(),
        W: this.getWeekOfYear(),
        F: this.getMonthName(),
        m: month < 9 ? '0'+(month+1) : month+1,
        M: this.getMonthAbbr(),
        n: month+1,
        t: this.getMonthDayCount(),
        L: this.isLeapYear() ? '1' : '0',
        Y: this.getFullYear(),
        y: this.getFullYear()+''.substring(2,4),
        a: hours > 12 ? 'pm' : 'am',
        A: hours > 12 ? 'PM' : 'AM',
        g: hours % 12 > 0 ? hours % 12 : 12,
        G: hours > 0 ? hours : "12",
        h: hours % 12 > 0 ? hours % 12 : 12,
        H: hours < 10 ? '0' + hours : hours,
        i: minutes < 10 ? '0' + minutes : minutes,
        s: seconds < 10 ? '0' + seconds : seconds
    };
    var date_string = "";
    for(var i = 0;i < dateFormat.length; i++) {
        var f = dateFormat[i];
        if (f.match(/[a-zA-Z]/g)) {
            date_string += date_props[f] ? date_props[f] : '';
        } else {
            date_string += f;
        }
    }
    return date_string;
};

String.prototype.formatCheckVariable = function(variable) {
    return new RegExp("\{" + variable + "\}", "gm").test(this);
};

String.prototype.format = function(replacement, prefix) {
    var format = this,
        match,
        prefix = prefix || "";
    for (var i = 0; match = new RegExp("\\{" + prefix + "(\\d+|\\w+)?\\}", "gm").exec(format); i++) {
        var key = match[1];
        if (key) {
            format = format.replace(new RegExp("\\{" + prefix + key + "\\}", "gm"), replacement[key] ? replacement[key] : "");
        }
    }

    return format;
};

String.prototype.localizeAutomatic = function(replacement) {
    return this.format(replacement, "text::");
};

String.prototype.localize = function(replacement) {
    return this.format(replacement, "text:manually::");
};

String.prototype.sprintf = function() {
    var new_string = this;
    for (var i = 0; i < arguments.length; i++)
        new_string = new_string.replace(/%[b|c|d|e|u|f|o|s|x|X]/, arguments[i]);
    return new_string;
};

String.prototype.setParamURI = function(values) {
    var uri = this.parseURI();
    for (var name in values)
        uri.params[name] = values[name];
    return uri.href + $.param(uri.params);
};

String.prototype.checkParamURI = function(values) {
    var test = true;
    for (var name in values)
        test = test && this.indexOf(encodeURIComponent(name) + "=" + encodeURIComponent(values[name])) != -1;
    return test;
};

String.prototype.lineEndToBR = function() {
    return this.replace(/\n/g, "<br />");
}

String.prototype.parseURI = function() {
    if (this.indexOf("?") == 0)
        return null;
    var arrURLParams = { };
    var arrParams = this.split("?");
    var href = arrParams[0] + "?";
    var arrParams = arrParams[1].split("&");
    for (var i = 0; i < arrParams.length; i++) {
        var sParam =  arrParams[i].split("=");
        sParam[0] = decodeURIComponent(sParam[0]);
        sParam[1] = decodeURIComponent(sParam[1]).replace(/\+/g, " ");
        if (sParam[0].indexOf("[]") != -1) {
            if (!Array.isArray(arrURLParams[sParam[0]]))
                arrURLParams[sParam[0]] = [];
            arrURLParams[sParam[0]].push(sParam[1]);
        } else {
            arrURLParams[sParam[0]] = decodeURIComponent(sParam[1]).replace(/\+/g, " ");
        }
    }
    return { href: href, params: arrURLParams };
};

String.prototype.encodeHTML = function() {
    return this.replace(/[\u00A0-\u9999<>\&]/gim, function(i) {
        return '&#'+i.charCodeAt(0)+';';
    });
};

String.prototype.decodeHTML = function() {
    var result = $('<div/>').html(this).text();
    result = result.replace(/<!\-*\[CDATA\[(.*?)\]\]\-*>/g, "$1");
    return result;
};

String.prototype.stripHTMLTags = function() {
    var tmp = document.createElement("DIV");
    tmp.innerHTML = this;
    return tmp.textContent||tmp.innerText;
};

Number.prototype.getSignificantDigits = function() {
    var fraction = this.toString().split(".");
    return fraction.length == 1 ? 0 : fraction[1].length;
};

jQuery.fn.extend({
    changeSelector: function (change_function) {
        change_function.call(this);
        return $('<div>').append($(this).clone()).html();
    },
    getWrap: function () {
    return $('<div>').append($(this).clone()).html();
}
});

var Helpers = {
    generateUUID: function () {
        var d = new Date().getTime(),
            uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = (d + Math.random() * 16) % 16 | 0;
                d = Math.floor(d / 16);
                return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
            });
        return uuid;
    },
    cloneObject: function (obj) {
        var key, clone = {};
        for (key in obj) {
            if (obj.hasOwnProperty(key)) {
                clone[key] = obj[key];
            }
        }
        return clone;
    },
    deepcloneObject: function (obj) {
        if (typeof obj != "object") {
            return obj;
        }
        var copy = obj.constructor();
        for (var key in obj) {
            if (typeof obj[key] == "object") {
                copy[key] = Helpers.deepcloneObject(obj[key]);
            } else {
                copy[key] = obj[key];
            }
        }
        return copy;
    },
    objIsEmpty: function (o) {
        for (var p in o) {
            if (o.hasOwnProperty(p)) {
                return false;
            }
        }
        return true;
    },
    randomProperty: function (obj) {
        var result;
        var count = 0;
        for (var prop in obj)
            if (Math.random() < 1 / ++count)
                result = prop;
        return result;
    },
    numberProperty: function (o) {
        var number = 0;
        for (var p in o) {
            if (o.hasOwnProperty(p)) {
                number++;
            }
        }
        return number;
    },
    getSingleProperty: function (o) {
        for (var p in o) {
            if (o.hasOwnProperty(p)) {
                return p;
            }
        }
        return null;
    },
    objToArray: function (obj, option) {
        option = option || {};
        var values = [];
        for (var key in obj) {
            var number = values.push(obj[key]) - 1;
            if (option.key_add) {
                values[number][option.key_field_name] = key;
            }
        }
        return values;
    },
    fullscreen: {
        isset: function () {
            return document.fullscreen || document.mozFullScreen || document.webkitIsFullScreen || document.msFullscreenElement;
        },
        exit: function () {
            if (document.exitFullscreen)
                document.exitFullscreen();
            else if (document.webkitCancelFullScreen)
                document.webkitCancelFullScreen();
            else if (document.mozCancelFullScreen)
                document.mozCancelFullScreen();
            else if (document.msExitFullscreen)
                document.msExitFullscreen();
        },
        run: function () {
            var el = document.documentElement;
            var rfs = el.requestFullScreen || el.webkitRequestFullScreen || el.mozRequestFullScreen || el.msRequestFullscreen;
            rfs.call(el);
        },
        toggle: function () {
            Helpers.fullscreen[Helpers.fullscreen.isset() ? "exit" : "run"]();
        },
        change: function (callback) {
            document.addEventListener("fullscreenchange", callback, false);
            document.addEventListener("mozfullscreenchange", callback, false);
            document.addEventListener("webkitfullscreenchange", callback, false);
            document.addEventListener("msfullscreenchange", callback, false);
        }
    },
    workspace: {
        width: function () {
            return document.compatMode == 'CSS1Compat' && !window.opera ? document.documentElement.clientWidth : document.body.clientWidth;
        },
        height: function () {
            return document.compatMode == 'CSS1Compat' && !window.opera ? document.documentElement.clientHeight : document.body.clientHeight;
        }
    },
    randomNumber: function (params) {
        params = params || {};
        params.distribution = params.distribution || "uniform";
        params.precision = params.precision || 0;
        var result;
        switch (params.distribution) {
            case "uniform":
                params.min = params.min || 0;
                params.max = params.max || 0;
                result = Math.round(params.min + Math.random() * (params.max - params.min)) + (params.precision == 0 ? 0 : Math.random().toFixed(params.precision)) * 1;
                break;
            case "exponential":
                params.diam = -params.diam || -1;
                result = (params.mean * Math.exp(-params.mean * Math.random() * (-params.diam))).toFixed(params.precision) * 1;
                break;
        }
        return result;
    },
    randomString: function (length, alphabet) {
        var text = "";
        alphabet = alphabet || "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        for (var i = 0; i < length; i++)
            text += alphabet.charAt(Math.floor(Math.random() * alphabet.length));
        return text;
    },
    colorHexToRGB: function (hexColor) {
        var matches = hexColor.match(/^#([0-9a-f]{6})$/i)[1];
        if (matches) {
            return [
                parseInt(matches.substr(0, 2), 16),
                parseInt(matches.substr(2, 2), 16),
                parseInt(matches.substr(4, 2), 16)
            ];
        } else {
            return false;
        }
    },
    colorRGBParse: function (RGBColor) {
        var matches = RGBColor.match(/^rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$/i);
        if (matches) {
            return [matches[1], matches[2], matches[3]];
        } else {
            return false;
        }
    },
    getColorTransparency: function (overlay, opacity, background) {
        overlay = overlay.indexOf("#") != -1 ? Helpers.colorHexToRGB(overlay) : Helpers.colorRGBParse(overlay);
        background = background.indexOf("#") != -1 ? Helpers.colorHexToRGB(background) : Helpers.colorRGBParse(background);
        overlay[0] = Math.round(overlay[0] * opacity + (1 - opacity) * background[0]);
        overlay[1] = Math.round(overlay[1] * opacity + (1 - opacity) * background[1]);
        overlay[2] = Math.round(overlay[2] * opacity + (1 - opacity) * background[2]);
        return "rgb({read},{blue},{green})".format({
            read: overlay[0],
            blue: overlay[1],
            green: overlay[2]
        });
    },
    getColorRestore: function (overlay, opacity, background) {
        overlay = overlay.indexOf("#") != -1 ? Helpers.colorHexToRGB(overlay) : Helpers.colorRGBParse(overlay);
        background = background.indexOf("#") != -1 ? Helpers.colorHexToRGB(background) : Helpers.colorRGBParse(background);
        overlay[0] = Math.round((overlay[0] - (1 - opacity) * background[0]) / opacity);
        overlay[1] = Math.round((overlay[1] - (1 - opacity) * background[1]) / opacity);
        overlay[2] = Math.round((overlay[2] - (1 - opacity) * background[2]) / opacity);
        return "rgb({read},{blue},{green})".format({
            read: overlay[0],
            blue: overlay[1],
            green: overlay[2]
        });
    },
    escapeHtml: function(text) {
    return text
        .replace(/&/g, "&amp;")
        .replace(/</g, "&lt;")
        .replace(/>/g, "&gt;")
        .replace(/"/g, "&quot;")
        .replace(/'/g, "&#039;");
    },
    loadJSON: function (url, data) {
        var result = null;
        $.ajax({
            url: url,
            data: data || false,
            async: false,
            dataType: "JSON",
            success: function (response) {
                result = response;
            }
        });
        return result;
    },
    getRandom: function (array) {
        var number = Helpers.randomNumber({min: 0, max: array.length - 1});
        return array[number];
    },
    aggregatePropsValue: function (obj) {
        var result = {};
        for (var i = 0; i < obj.length; i++) {
            $.each(obj[i], function (key, val) {
                if (result[key]) {
                    result[key] += val;
                } else {
                    result[key] = val;
                }
            });
        }
        return result;
    },
    refreshFlag: true,
    setBlockHeight: function (section) {
        var o = this,
            windowHeight = window.innerHeight,
            blocks = {
                block: section.find('.block'),
                table: section.find('.table'),
                charts: section.find('.charts')
            };
        $.each(blocks, function (key, val) {
            if (val[0]) {
                for (var i = 0; i < val.length; i++) {
                    var block = $(val[i]),
                        blockBoxSizing = block.css('boxSizing'),
                        blockHeight = block.outerHeight(),
                        blockPadding = parseInt(block.css('paddingBottom')) + parseInt(block.css('paddingTop')),
                        blockBorderTop = parseInt(block.css('borderTop')),
                        blockOffsetTop = block.offset().top,
                        sectionMarginBottom = parseInt(section.css('margin-bottom'));
                    if (blockBoxSizing === 'content-box') {
                        block.css('maxHeight', windowHeight - blockOffsetTop - sectionMarginBottom - blockPadding - blockBorderTop - 1);
                    } else if (blockBoxSizing === 'border-box') {
                        block.css('maxHeight', windowHeight - blockOffsetTop - sectionMarginBottom - 1);
                    }
                }
            }
        });
        $(window).off('resize');
        $(window).on("resize", function () {
            Helpers.setBlockHeight(section);
        })
    },
    firstLetterToUpperCase: function (str) {
        return str.charAt(0).toUpperCase() + str.substring(1)
    },
    aggregateDataByCount: function(count, data, prop){
        var sum = 0;
        for (var i = 0; i < count; i++) {
            if(data[i]) {
                sum += data[i][prop];
            }
        }
        return sum;
    },
    convertListToTree: function(data, property, parentProperty) {
        var result = [],
            temp = {},
            i = data.length, j=i;
        while(i--) {
            temp[data[i][property]] = data[i];
        }
        for(i = 0; i<j; i++){
            if(data[i][parentProperty] === null){
                result.push(data[i]);
                continue;
            }
            var parent = temp[data[i][parentProperty]];
            if(!parent.children) {
                parent.children = [];
            }
            parent.children.push(data[i])
        }
        return result
    },
    convertObjectToDataAttribute: function(obj){
        var dataAttr = "";
        $.each(obj, function(key, val){
            dataAttr += "data-" + key + "='"+ val +"' ";
        });
        return dataAttr;
    },
    getRandomColor: function cc(){
        var r=Math.floor(Math.random() * (150)) + 40;
        var g=Math.floor(Math.random() * (150)) + 40;
        var b=Math.floor(Math.random() * (150)) + 40;
        var c='#' + r.toString(16) + g.toString(16) + b.toString(16);
        return c;
    }
};