window.ERPScan = {};

var Theme = "default";

function Loader() {
    this.homePage = "/monitor/issues";
    this.modulesFile = "/media/config/modules.json";
    this.add = {
        CSS: function(name) {
            var baseDir = "/media/css/";
            var link = document.createElement("link");
            link.type = "text/css";
            link.rel = "stylesheet";
            if (name.indexOf("{theme}") != -1) {
                name = name.replace("{theme}", Theme);
            }
            link.href = baseDir + name + ".css";
            document.getElementsByTagName("head")[0].appendChild(link);
        },
        JS: function(name) {
            var baseDir = "/media/js/";
            var script = document.createElement('script');
            script.src = baseDir + name + ".js";
            script.async = false;
            document.head.appendChild(script);
        }
    };
    this.modules = {
        JS: {
            "elasticsearch": ["elasticsearch/elasticsearch", "erpscan/elasticsearch"],
            "jquery": ["jquery/jquery", "jquery/jquery.cookie"],
            "jquery-form-validator": ["jquery-form-validator/jquery.form-validator.min"],
            "jquery-form-styler": ["jquery-form-styler/jquery.formstyler.min"],
            "form": ["jquery-form-validator/jquery.form-validator.min"],
            "jquery-ui": [
                "jquery-ui/jquery-ui.min",
                "jquery-ui/jquery-ui-timepicker-addon"
            ],
            "tablesorter": [
                "tablesorter/js/jquery.tablesorter",
                "tablesorter/js/jquery.tablesorter.widgets",
                "tablesorter/js/widgets/widget-filter-formatter-jui",
                "tablesorter/addons/pager/jquery.tablesorter.pager",
                "tablesorter/js/parsers/parser-input-select",
                "erpscan/table"
            ],
            "flot": [
                "flot/jquery.flot.min",
                "flot/jquery.flot.pie.min",
                "flot/jquery.flot.time.min",
                "flot/jquery.flot.stack.min",
                "flot/jquery.flot.orderBars",
                "flot/jquery.flot.symbol.min",
                "flot/jquery.flot.resize.min",
                "erpscan/dashboard"
            ],
            "filter": ["erpscan/filter"],
            "vis": ["visjs/vis"],
            "prism": ["prism/prism"],
            "code-mirror": ["code-mirror/lib/codemirror", "code-mirror/mode/javascript/javascript", "code-mirror/mode/sql/sql", "code-mirror/addon/edit/matchbrackets"],
            "erpscan": ["erpscan/api"],
            "tree": ["erpscan/tree"],
            "treetable": ["treetable/jquery.treetable"],
            "matrix": ["erpscan/matrix"],
            "alert": ["erpscan/alert"],
            "block": ["erpscan/block"],
            "tabs": ["erpscan/tabs"],
            "gravity": ["gravity/gravity"]
        },
        CSS: {
            "jquery-ui": [
                "jquery-ui/jquery-ui.min",
                "jquery-ui/jquery-ui.structure.min",
                "jquery-ui/jquery-ui-timepicker-addon",
                "jquery-ui/jquery-ui.theme.{theme}"
            ],
            "jquery-form-styler": [],
            "code-mirror": ["code-mirror/codemirror", "code-mirror/theme/mdn-like"],
            "form": ["jquery-form-styler/jquery.formstyler"],
            "tree": [ "erpscan/tree" ],
            "matrix": [ "erpscan/matrix" ],
            "alert": [ "erpscan/alert" ],
            "prism": ["prism/prism"],
            "tablesorter": [
                "tablesorter/addons/pager/jquery.tablesorter.pager",
                "tablesorter/theme.{theme}"
            ],
            "treetable": ["treetable/jquery.treetable"]
        }
    };
    this.general = {
        JS: ["general", "helpers"],
        CSS: ["basis", "themes/theme.{theme}"]
    };
    this.getTheme = function() {
        var matches = document.cookie.match(new RegExp("(?:^|; )" + "theme".replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + "=([^;]*)"));
        return matches ? decodeURIComponent(matches[1]) : undefined;
    };
    this.fieldsOld = function(section) {
        var fields = Helpers.loadJSON("/config/fields.json");
        return fields[section];
    };
    this.fields = function(callback) {
        return Helpers.loadJSON("/config/fields/{module}/{page}/{tab}.json".format({
            module: Page.path.module,
            page: Page.path.page,
            tab: Page.path.tab || "general"
        }));
    };
    this.templates = {
        dir: "/media/templates",
        XMLToObject: function($templates, templatesObject) {
            var o = this;
            $templates.find("> group").each(function() {
                var $this = $(this);
                templatesObject[$this.attr("name")] = {};
                if ($this.find("> group, > template").length > 0) {
                    o.XMLToObject($this, templatesObject[$this.attr("name")]);
                }
            });
            $templates.find("> template").each(function() {
                var $this = $(this);
                templatesObject[$this.attr("name")] = {
                    __group: false,
                    content: $.trim($this.html()).replace(/>\s+</g,'><'),
                    force: $this.attr("force") != "false"
                };
            });
        },
        localize: function(template, texts) {
            var textCurrentLevel = {};
            for (var item in texts) {
                if (typeof texts[item] == "object" && typeof template[item] != "undefined") {
                    template[item] = this.localize(template[item], texts[item]);
                } else {
                    textCurrentLevel[item] = texts[item];
                }
            }
            if (template["__group"] === false && Helpers.numberProperty(textCurrentLevel) != 0) {
                template["content"] = template["force"] === false ? template["content"] : template["content"].localizeAutomatic(textCurrentLevel).toString();
            }
            return template;
        },
        normalize: function(templates) {
            for (var item in templates) {
                if (templates[item]["__group"] !== false) {
                    this.normalize(templates[item]);
                } else {
                    templates[item] = templates[item]["content"];
                }
            }
        },
        load: function() {
            var o = this,
                templates;
            $.ajax({
                async: false,
                url: "{templatesDir}/{moduleName}/{pageName}/{tabName}.xml".format({
                    templatesDir: this.dir,
                    moduleName: Page.path["module"],
                    pageName: Page.path["page"],
                    tabName: Page.path["tab"] || "general"
                }),
                dataType: "XML",
                success: function(result) {
                    var templatesObject = {};
                    o.XMLToObject($(result).find("> templates"), templatesObject);
                    templates = o.localize(templatesObject, Locale.ui.page["content"]);
                    o.normalize(templates);
                }
            })
            return templates;
        }
    };
    this.loadModulesList = function() {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', this.modulesFile, false);
        xhr.send();
        return JSON.parse(xhr.responseText);
    };
    this.run = function() {
        Theme = this.getTheme() || Theme;
        try {
            var modules = this.loadModulesList(),
                i, j,
                page = location.pathname != "/" ? location.pathname.split("/") : this.homePage.split("/"),
                modules_general = modules["general"],
                modules_pages = modules["pages"][page[1]][page[2]];
        } catch(e) {
            location.href = "/404.html";
        }
        modules_pages = typeof page[3] == "undefined" || typeof modules_pages[page[3]] == "undefined" ? modules_pages["general"] : modules_pages[page[3]];
        modules = modules_general.concat(modules_pages);
        for (i = 0; i < modules.length; i++) {
            if (this.modules.CSS[modules[i]]) {
                for (j = 0; j < this.modules.CSS[modules[i]].length; j++) {
                    this.add.CSS("modules/" + this.modules.CSS[modules[i]][j]);
                }
            }
            if (this.modules.JS[modules[i]]) {
                for (j = 0; j < this.modules.JS[modules[i]].length; j++) {
                    this.add.JS("modules/" + this.modules.JS[modules[i]][j]);
                }
            }
        }
        for (i = 0; i < this.general.CSS.length; i++) {
            this.add.CSS(this.general.CSS[i]);
        }
        for (i = 0; i < this.general.JS.length; i++) {
            this.add.JS(this.general.JS[i]);
        }
        var scriptsPath = "renders/" + page[1] + "/" + page[2] + "/" + (typeof page[3] == "undefined" ? "general" : page[3]),
            stylesPath = "pages/" + page[1] + "/" + page[2];
        this.add.CSS(stylesPath);
        this.add.JS(scriptsPath);
        window.onload = function() {
            Page.path = {
                module: page[1],
                page: page[2],
                tab: page[3] || null
            };
            Page.load();
        };
    }
}

ERPScan.loader = new Loader();
ERPScan.loader.run();