var Filters = {
    selector: ".filters .filter-inputs",
    addData: function(name, title, values) {
        for (var i = 0; i < values.length; i++)
            values[i][1] = true;
        Filters.data[name] = { title: title, values: values }
    },
    data: { },
    templates: {
        field_list: ' <select class="fields-list">%s</select>',
        field_list_option: '<option value="%s">%s</option>',
        input: '<div class="field" data-name="%s"><div class="labels"></div><input type="text" placeholder="%s" /><a href="#" class="remove"></a></div>',
        label: '<div class="label" data-number="%d"><span class="label-text">%s</span><a href="#" class="label-remove"></a></div>',
        value_list: '<div class="value-list" data-field="%s">%s</div>',
        value_list_option: '<a href="#" class="value-list-item" data-number="%d">%s</a>'
    },
    $field: function(name, element) {
        return $(Filters.selector + " .field[data-name=\"" + name + "\"]" + (element ? " " + element : ""));
    },
    add: function(field, option) {
        option = option || false;
        option.type = option.type || false;
        $(Filters.selector + " .fields-list").before(Filters.templates.input.sprintf(field, Filters.data[field].title));
        if ($(Filters.selector + " .fields-list option").length > 2)
            $(Filters.selector + " .add").css({display: "inline-block"});
        $(Filters.selector + " .fields-list option[value=\"" + field + "\"]").remove();
        Filters.data[field].used = true;
        if (option.type == "datetime")
            Filters.$field(field, "input").datetimepicker();
    },
    remove: function(field) {
        var $field = Filters.$field(field, "input");
        $field.next(".remove").remove();
        Filters.$field(field, ".labels").remove();
        $field.remove();
        $(Filters.selector + " .add").css({display: "inline-block"});
        $(Filters.selector + " .fields-list").append(Filters.templates.field_list_option.sprintf(field, Filters.data[field].title));
        Filters.data[field].used = false;
    },
    fieldList: {
        generate: function() {
            var select = Filters.templates.field_list_option.sprintf(0, " -- Choice field -- ");
            for (var field in Filters.data) {
                if (!Filters.data[field].used) {
                    select += Filters.templates.field_list_option.sprintf(field, Filters.data[field].title);
                }
            }
            select = Filters.templates.field_list.sprintf(select);
            $(Filters.selector + " .add").before(select);
        },
        hide: function() {
            $(Filters.selector + " .add").css({display: 'inline-block'});
            $(Filters.selector + " .fields-list").hide();
        },
        show: function() {
            $(Filters.selector + " .add").hide();
            $(Filters.selector + " .fields-list").show();
        }
    },
    valueList: {
        show: function (field) {
            var $field = Filters.$field(field);
            var options = '';
            for (var i = 0; i < Filters.data[field].values.length; i++) {
                if (Filters.data[field].values[i][1]) {
                    options += Filters.templates.value_list_option.sprintf(i, Filters.data[field].values[i][0]);
                }
            }
            var exist_option = options != '';
            if (exist_option) {
                options = Filters.templates.value_list.sprintf(field, options);
                $field.prepend(options).addClass("active");
                Filters.$field(field, ".value-list").width(Filters.$field(field, "input").outerWidth() - 2);
            }
            Filters.$field(field, ".value-list").show();
        },
        hide: function(field) {
            if (field) {
                Filters.$field(field, ".value-list").remove();
                Filters.$field(field).removeClass("active");
            }
        }
    },
    labels: {
        add: function(field, value, number) {
            number = number === undefined ? -1 : number;
            var $labels = Filters.$field(field, ".labels");
            $labels.append(Filters.templates.label.sprintf(number, value));
            Filters.$field(field, "input").css({paddingLeft: ($labels.outerWidth() + 7) + "px"}).val("");
            Filters.valueList.hide(field);
            Filters.valueList.show(field);
            if (number != -1)
                Filters.data[field].values[number][1] = false;
        },
        remove: function($label) {
            var $labels = $label.parent(".labels");
            var number = $label.data("number");
            $label.remove();
            var field = $labels.parent(".field").data("name");
            Filters.$field(field, "input").css({paddingLeft: ($labels.outerWidth() + 7) + "px"});
            if (number != -1)
                Filters.data[field].values[number][1] = true;
            Filters.valueList.hide(field);
            Filters.valueList.show(field);
        }
    },
    handlers: function() {
        $(Filters.selector + " .add").on("click", function() {
            Filters.fieldList.show();
            return false;
        })
        $(document.body).on("click", Filters.selector + " .field .remove", function() {
            var field = $(this).parent(".field").data("name");
            Filters.valueList.hide(field);
            Filters.remove(field);
            return false;
        })
        $(document.body).on("mouseenter", Filters.selector + " .field", function() {
            var field = $(this).data("name");
            Filters.$field(field, ".remove").addClass("showed");
        })
        $(document.body).on("mouseleave", Filters.selector + " .field", function() {
            var field = $(this).data("name");
            Filters.$field(field, ".remove").removeClass("showed");
        })
        $(document.body).on("mousedown", Filters.selector + " .field", function() {
            var $this = $(this);
            var $field_active = $(Filters.selector + " .field.active").not(this);
            if ($field_active.length > 0) {
                var field = $field_active.data("name");
                Filters.valueList.hide(field);
                $field_active.removeClass("active");
            }
            if (!$this.hasClass("active"))
                $this.addClass("active");
        })
        $(document.body).on("mousedown", function(e) {
            var $this = $(e.target);
            var $field_active = $(Filters.selector + " .field.active");
            if ($field_active.length > 0 && $this.filter(".field.active").length == 0 && $this.parents(".field.active").length == 0) {
                $field_active.removeClass("active");
                var field = $field_active.data("name");
                Filters.valueList.hide(field);
            }
            if ($(".fields-list:visible").length > 0 && $this.filter(Filters.selector + " .fields-list").length == 0)
                Filters.fieldList.hide();
        })
        $(document.body).on("click", Filters.selector + " .value-list .value-list-item", function() {
            var $this = $(this);
            var field = $this.parents(".field").data("name");
            Filters.labels.add(field, $this.text(), $this.data("number"));
            Filters.valueList.hide(field);
            return false;
        })
        $(document.body).on("focus", Filters.selector + " input", function() {
            var field = $(this).parent(".field").data("name");
            Filters.valueList.show(field);
        })
        $(document.body).on("keyup", Filters.selector + " input", function() {
            var $this = $(this), value;
            if ((event.keyCode == 0xA || event.keyCode == 0xD) && (value = $this.val())) {
                var field = $this.parent(".field").data("name");
                Filters.labels.add(field, value);
            }
        })
        $(document.body).on("click", Filters.selector + " .labels .label .label-remove", function() {
            var $this = $(this);
            Filters.labels.remove($this.parent(".label"));
            return false;
        })
        $(document.body).on("change", Filters.selector + " .fields-list", function() {
            var $this = $(this), field;
            if ((field = $this.val()) == 0)
                return;
            Filters.fieldList.hide();
            Filters.add(field);
        })
    }
}

var Chart = {
    counter: 1,
    table: null,
    templates: {
        legend: '<div style="background: %s;" class="color"></div>' +
        '<div class="text">%s</div>',
        to_save: '<select class="chart-save">' +
        '<option value="0"> -- Choice dashboard -- </option>' +
        '<option>June \'15 events</option>' +
        '<option>Amsterdam</option>' +
        '<option>Misconfigurations</option>' +
        '</select>',
        dispositions: {
            wrap: '<div class="wrap">%s</div>',
            charts: '<div class="charts %s">%s</div>',
            events: '<div class="events %s">%s</div>',
            cell: '<div class="charts-cell col-%d">%s</div>'
        },
        charts: {
            chart: '<div class="chart %s">' +
            '<div class="chart-actions"><a href="#" class="add"></a><a href="#" class="remove"></a></div>' +
            '<h3>%s</h3> ' +
            '<div id="chart-%d" data-field="%s" class="chart-container"></div> ' +
            '<div class="legend"></div> ' +
            '</div>'
        }
    },
    handlersOn: false,
    redrawing_timeout: null,
    legendFormatter: function(label, params) {
        return Chart.templates.legend.sprintf(params.color, label);
    },
    setTransposition: function() {
        var disposition = $.cookie("events-disposition") || "left";
        var $wrap = $(".wrap");
        var $charts = $wrap.find(".charts");
        var $events = $wrap.find(".events");
        var charts = $charts.get(0);
        $charts.detach();
        switch(disposition) {
            case "top":
                $events.after(charts);
                break;
            case "left":
                $events.after(charts);
                break;
            case "right":
                $events.before(charts);
                break;
            case "bottom":
                $events.before(charts);
                break;
        }
        $events.removeAttr("class").addClass("events").addClass(disposition);
        $charts.removeAttr("class").addClass("charts").addClass(disposition);
        Chart.reDrawing();
    },
    generateChart: function(option, number) {
        var content = "";
        switch (option.type) {
            case "Pie":
                content = Chart.templates.charts.chart.sprintf((option.removable ? 'removable ' : '') + 'pie', option.title, number, option.field ? option.field : "");
                break;
            case "Bar":
                content = Chart.templates.charts.chart.sprintf((option.removable ? 'removable ' : '') + 'bar', option.title, number, option.field ? option.field : "");
                break;
            case "Table":
                content = Table.generate(number, option);
                break;
        }
        return Chart.templates.dispositions.cell.sprintf(option.columns, content);
    },
    generate: function(charts, events, selector) {
        selector = selector || "section.content";
        events = events || false;
        var charts_table = '',
            events_table = '';
        for (var i = 0; i < charts.length; i++) {
            charts_table += Chart.generateChart(charts[i], i);
        }
        var disposition = $.cookie("events-disposition") || "left";
        charts_table = Chart.templates.dispositions.charts.sprintf(disposition, charts_table);
        if (events) {
            events.option.fixed = true;
            Chart.table = new Table(events);
            content = Chart.table.getHTML();
            events_table = Chart.templates.dispositions.events.sprintf(disposition, content);
        }
        disposition = $.cookie("events-disposition") || "left";
        var content;
        switch(disposition) {
            case "top":
                content = events_table + charts_table;
                break;
            case "left":
                content = events_table + charts_table;
                break;
            case "right":
                content = charts_table + events_table;
                break;
            case "bottom":
                content = charts_table + events_table;
                break;
        }
        $("header .disposition ." + disposition).addClass("active");
        content = Chart.templates.dispositions.wrap.sprintf(content);
        $(selector).append(content);
        for (var i = 0; i < charts.length; i++) {
            charts[i].option = charts[i].option || { };
            charts[i].option.number = i;
            Chart[charts[i].type].build(charts[i].data, charts[i].option);
        }
        if (events) {
            events.option = events.option || { };
            Chart.table.build();
        }
        //Chart.setSizes();
    },
    setSizes: function() {
        var $chart = $(".charts .charts-cell");
        for (var i = 0; i < $chart.length; i++) {
            var $current_chart = $chart.eq(i);
            if ($current_chart.hasClass("col-2")) {
                var $chart_next_1 = $chart.eq(i + 1);
                var height = Math.max($current_chart.height(), $chart_next_1.height());
                $current_chart.height(height);
                $chart_next_1.height(height);
                i = i + 1;
            } else if ($current_chart.hasClass("col-3")) {
                var $chart_next_1 = $chart.eq(i + 1);
                var $chart_next_2 = $chart.eq(i + 2);
                var height = Math.max($current_chart.height(), $chart_next_1.height(), $chart_next_2.height());
                $current_chart.height(height);
                $chart_next_1.height(height);
                $chart_next_2.height(height);
                i = i + 2;
            }
        }
    },
    segmentHighlight: function($chart, number, option) {
        option = option || { };
        option.inverse = option.inverse || false;
        var chart = $chart.find(".chart-container").data("plot");
        var data = chart.getData();
        if ($chart.hasClass("pie")) {
            data[number - 1].color = Helpers.getColorTransparency(data[number - 1].color, 0.5, '#ffffff');
        } else if ($chart.hasClass("bar")) {
            data[number - 1].bars.fillColor = { colors: [ { opacity: 0.7 }, { opacity: 0.7 } ] };
        }
        chart.setData(data);
        chart.draw();
    },
    segmentUnhighlight: function($chart, number, option) {
        option = option || { };
        option.inverse = option.inverse || false;
        var chart = $chart.find(".chart-container").data("plot");
        var data = chart.getData();
        if ($chart.hasClass("pie")) {
            data[number - 1].color = Helpers.getColorRestore(data[number - 1].color, 0.5, '#ffffff');
        } else if ($chart.hasClass("bar")) {
            data[number - 1].bars.fillColor = { colors: [ { opacity: 0.4 }, { opacity: 0.4 } ] };
        }
        chart.setData(data);
        chart.draw();
    },
    Line: {
        templates: {
            label: '<div style="color: #ffffff;font-size: 13px; text-align: center;"><div>%s</div><div style="margin-top: 6px;">%s</div></div>'
        },
        labelFormatter: function(label, params) {
            return Chart.Line.templates.label.sprintf(label, params.percent.toFixed(2) + '%');
        },
        build: function(data, option) {
            var $chart = $("#chart-" + option.number);
            $chart.plot(data, {
                series: { stack: 0 },
                yaxis: {
                    axisLabel: 'Value',
                    axisLabelUseCanvas: true,
                    axisLabelFontSizePixels: 12,
                    axisLabelFontFamily: 'Verdana, Arial, Helvetica, Tahoma, sans-serif',
                    axisLabelPadding: 5
                },
                legend: {
                    show: true,
                    labelFormatter: Chart.legendFormatter,
                    container: $chart.next(".legend"),
                    noColumns: 0
                },
                grid: {
                    hoverable: true,
                    clickable: true,
                    borderWidth: 0,
                    color: Page.theme.colors.stack_grid[Page.theme.name]
                }
            })
        }
    },
    Pie: {
        templates: {
            label: '<div style="color: #ffffff;font-size: 13px; text-align: center;"><div>%s</div><div style="margin-top: 6px;">%s</div></div>'
        },
        maxLengthLabel: 50,
        labelFormatter: function(label, params) {
            return Chart.Pie.templates.label.sprintf(label, params.percent.toFixed(2) + '%');
        },
        build: function(data, option) {
            for (var i = 0; i < data.length; i++) {
                if (data[i].label.length > Chart.Pie.maxLengthLabel) {
                    data[i].label = data[i].label.substr(0, Chart.Pie.maxLengthLabel) + "...";
                }
            }
            var $chart = $("#chart-" + option.number);
            $chart.plot(data, {
                series: {
                    pie: {
                        show: true,
                        innerRadius: 0.4,
                        radius: 1,
                        label: {
                            show: true,
                            radius: 0.7,
                            formatter: Chart.Pie.labelFormatter,
                            threshold: 0.12
                        },
                        stroke: { color: Page.theme.colors.pie_stroke[Page.theme.name] }
                    }
                },
                grid: { hoverable: true, clickable: true },
                legend: {
                    show: true,
                    labelFormatter: Chart.legendFormatter,
                    container: $chart.next(".legend"),
                    noColumns: 0
                }
            })
        }
    },
    Bar: {
        build: function(data, option) {
            var $chart = $("#chart-" + option.number);
            for (var i = 0; i < option.ticks.length; i++) {
                if (option.ticks[i].length > Chart.Pie.maxLengthLabel) {
                    option.ticks[i] = option.ticks[i].substr(0, Chart.Pie.maxLengthLabel) + "...";
                }
            }
            for (var i = 0; i < data.length; i++) {
                if (data[i].label.length > Chart.Pie.maxLengthLabel) {
                    data[i].label = data[i].label.substr(0, Chart.Pie.maxLengthLabel) + "...";
                }
            }
            $chart.plot(data, {
                series: { stack: 0 },
                xaxis: {
                    ticks: option.ticks,
                    axisLabel: 'Tags',
                    axisLabelUseCanvas: true,
                    axisLabelFontSizePixels: 12,
                    axisLabelFontFamily: 'Verdana, Arial, Helvetica, Tahoma, sans-serif',
                    axisLabelPadding: 5,
                    color: "transparent"
                },
                yaxis: {
                    axisLabel: 'Value',
                    axisLabelUseCanvas: true,
                    axisLabelFontSizePixels: 12,
                    axisLabelFontFamily: 'Verdana, Arial, Helvetica, Tahoma, sans-serif',
                    axisLabelPadding: 5
                },
                legend: {
                    show: true,
                    labelFormatter: Chart.legendFormatter,
                    container: $chart.next(".legend"),
                    noColumns: 0
                },
                grid: {
                    hoverable: true,
                    clickable: true,
                    borderWidth: 0,
                    color: Page.theme.colors.stack_grid[Page.theme.name]
                }
            })
        }
    },
    legendItemSelect: function($chart, number) {
        var $labels = $chart.find(".legendLabel");
        $labels.addClass("off");
        $labels.eq(number).removeClass("off");
    },
    legendItemUnselect: function($chart) {
        var $labels = $chart.find(".legendLabel");
        $labels.removeClass("off");
    },
    segmentSelect: function($chart, item) {
        var field;
        if (!(field = $chart.data("field")))
            return;
        var $chart_block = $chart.parent(".chart"), number;
        if (number = $chart.data("selected")) {
            Table.filterClear($(".events table.tablesorter"), field);
            Chart.segmentUnhighlight($chart_block, number);
            $chart.data("selected", null);
            Chart.legendItemUnselect($chart_block);
        }
        if (!number || number != item.seriesIndex + 1) {
            Table.filterApply($(".events table.tablesorter"), field, item.series.label);
            Chart.segmentHighlight($chart_block, item.seriesIndex + 1);
            $chart.data("selected", item.seriesIndex + 1);
            Chart.legendItemSelect($chart_block, item.seriesIndex);
        }
    },
    reDrawing: function () {
        $(".charts .chart .chart-container").each(function() {
            var $this = $(this);
            var chart = $this.data("plot");
            chart.resize();
            chart.setupGrid();
            chart.draw();
        })
    },
    handlers: function() {
        if (this.handlersOn) {
            return;
        }
        $(document.body).on("click", ".chart .add, .events .add", function() {
            var $this = $(this);
            if ($this.hasClass("added")) {
                $this.removeClass("added");
                return false;
            }
            $this.hide().after(Chart.templates.to_save);
            $this.next(".chart-save").trigger('click');
            return false;
        });
        $(document.body).on("change", ".chart-save", function() {
            var $this = $(this);
            if ($this.find("option:selected").val() == 0)
                return false;
            $this.prev(".add").addClass("added").show();
            $this.remove();
        });
        $(document.body).on("click", function(e) {
            var $this = $(e.target);
            if ($(".chart-save").length > 0 && $this.filter(".chart-save").length == 0) {
                $(".chart-save").remove();
                $(".chart .add, .events .add").show();
            }
            if ($(".table-settings").length > 0 && $this.filter(".table-settings").length == 0 && $this.parents(".table-settings").length == 0) {
                $(".table-settings").remove();
            }
            if ($(".table .action-events .action a.link.active").length > 0) {
                Table.hideVariants($(".table .action-events .action a.link.active"));
            }
        });
        $(document.body).on("mouseenter", ".chart .legend .legendLabel", function() {
            var $this = $(this);
            var $chart = $this.parents(".chart");
            var number = Math.round($this.index() / 2);
            Chart.segmentHighlight($chart, number);
        });
        $(document.body).on("mouseleave", ".chart .legend .legendLabel", function() {
            var $this = $(this);
            var $chart = $this.parents(".chart");
            var number = Math.round($this.index() / 2);
            Chart.segmentUnhighlight($chart, number);
        });
        $(document.body).on("click", ".table .link.enable", function() {
            var $this = $(this);
            var $table_block = $this.parents(".table");
            if ($this.hasClass("on")) {
                Table.unLink($table_block);
            } else {
                Table.link($table_block);
            }
            return false;
        });
        $(document.body).on("click", ".chart-container .pieLabel", function() {
            if (!$.cookie("events_linked"))
                return;
            var $this = $(this);
            var $chart = $this.parents(".chart-container");
            var number = $this.attr("id").substr(8) * 1;
            var label = $this.find(".text").text();
            Chart.segmentSelect($chart, { seriesIndex: number, series: { label: label } });
        });
        $(document.body).on("click", ".chart .legend .legendLabel", function() {
            if (!$.cookie("events_linked"))
                return;
            var $this = $(this);
            var $chart = $this.parents(".chart").find(".chart-container");
            var number = Math.round($this.index() / 2) - 1;
            var label = $this.find(".text").text();
            Chart.segmentSelect($chart, { seriesIndex: number, series: { label: label } });
        });
        $(".chart-container").bind("plotclick", function (event, pos, item) {
            if (!$.cookie("events_linked"))
                return;
            var $chart;
            if (item && ($chart = $(event.target)))
                Chart.segmentSelect($chart, item);
        }).bind("plothover", function(event, pos, item) {
            document.body.style.cursor = item ? 'pointer' : 'default';
        });
        $(document.body).on("click", ".disposition .left", function() {
            $.cookie('events-disposition', 'left');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            Chart.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .top", function() {
            $.cookie('events-disposition', 'top');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            Chart.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .right", function() {
            $.cookie('events-disposition', 'right');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            Chart.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .bottom", function() {
            $.cookie('events-disposition', 'bottom');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            Chart.setTransposition();
            return false;
        });
        $(window).resize(Chart.reDrawing);
        this.handlersOn = true;
    }
};

/*
 $.datepicker._gotoToday = function(id) {
 $(id).datepicker('setDate', 0).datepicker('hide').change().blur();
 };
 */