function Filter(name, params) {
    this.$selector = null;
    this.number = Filter.number++;
    this.name = name;
    this.data = { };
    this.params = params || null;
    this.addData = function(name, title, values) {
        for (var i = 0; i < values.length; i++) {
            values[i][1] = true;
        }
        Filters.data[name] = { title: title, values: values }
    };
    this.templates = {
        field_list: ' <select class="fields-list">%s</select>',
        field_list_option: '<option value="%s">%s</option>',
        input: '<div class="field" data-number="{number}" data-name="{name}"><div class="value-list">{variants}</div><div class="labels"></div><input type="text" placeholder="{title}" /></div>',
        label: '<div class="label" data-number="{number}"><span class="label-text">{text}</span><a href="#" class="label-remove"></a></div>',
        value_list_option: '<a href="#" class="value-list-item" data-number="{number}">{text}</a>'
    };
    this.build = function() {
        this.$selector = $(".field[data-number=\"" + this.number + "\"]");
        if (typeof this.params.type != "undefined" && this.params.type == "datetime") {
            this.$selector.find("input").datetimepicker();
        }
        this.handlers();
    };
    this.getHTML = function() {
        var variants = "";
        for (var i = 0; i < this.params.variants.length; i++) {
            variants += this.templates.value_list_option.format({
                number: i,
                text: this.params.variants[i]
            });
        }
        var html = this.templates.input.format({
            number: this.number,
            name: this.name,
            variants: variants,
            title: Locale.ui.common["filters"]["placeholder"].format({
                title: Locale.fields[this.params.fields.namespace][this.params.fields.type][this.name]
            })
        });
        return html;
    };
    this.update = function() {
        if (typeof this.params.callback != "undefined") {
            var values = [];
            this.$selector.find(".labels .label").each(function() {
                values.push($(this).find(".label-text").text());
            });
            this.params.callback(values);
        }
    };
    this.addLabel = function($item, title) {
        var o = this;
        var $labels = this.$selector.find(".labels");
        $labels.append(this.templates.label.format({
            number: $item != null ? $item.data("number") : -1,
            text: title
        }));
        this.$selector.find("input").val("").css({paddingLeft: ($labels.outerWidth() + 7) + "px"});
        if ($item != null) {
            $item.addClass("hidden");
        }
        this.$selector.find(".labels .label .label-remove").off("click").on("click", function() {
            o.removeLabel($(this).parent(".label"));
            return false;
        });
        this.update();
    };
    this.removeLabel = function($label) {
        var number = $label.data("number"),
            $labels = this.$selector.find(".labels"),
            field = $labels.parent(".field").data("name");
        $label.remove();
        this.$selector.find("input").css({paddingLeft: ($labels.outerWidth() + 5) + "px"});
        if (number != -1) {
            this.$selector.find(".value-list .value-list-item[data-number=\"" + number + "\"]").removeClass("hidden");
        }
        this.update();
    };
    this.handlers = function() {
        var o = this;
        this.$selector.find("input").on("focus", function() {
            if (!o.$selector.hasClass("active") && o.$selector.find(".value-list .value-list-item:not(.hidden)").length > 0) {
                o.$selector.addClass("active");
                o.$selector.find(".value-list").show();
            }
        });
        this.$selector.find("input").on("blur", function() {
            if (o.$selector.hasClass("active")) {
                o.$selector.removeClass("active");
                o.$selector.find(".value-list").hide();
                var $target = $(window.event.relatedTarget);
                if ($target.length != 0 && $target.parents(".field[data-number=\"" + o.number + "\"]").length > 0 && $target.hasClass("value-list-item")) {
                    o.addLabel.call(o, $target, $target.text());
                }
            }
        });
        this.$selector.find("input").on("keyup", function() {
            var $this = $(this);
            if ((event.keyCode == 0xA || event.keyCode == 0xD) && $this.val()) {
                o.addLabel(null, $this.val());
            }
        });
    }
}

Filter.number = 1;