function Matrix(params) {
    Matrix.id++;
    this.id = Matrix.id;
    this.params = params;
    this.columnGroupsColor = this.params.colGroupsColor ? this.params.colGroupsColor : ['#6C98DA', '#F7941E', '#5B9E8C', '#ABAD25', '#CC6FC5'];
    this.templates = {
        matrix: "<div id='matrix-{id}' class='matrix {class}'>{tables}</div>",
        table: "<div class='{class}'>{body}</div>",
        row: "<div class='row'>{cells}</div>",
        cell: "<div {data} class='cell {class}'>{content}</div>",
        colGroup: "<div class='colGroup {class}' data-name='{name}' >{content}</div>",
        rowGroup: "<div class='rowGroup' data-name='{name}'>{content}</div>",
        groupCell: "<div class='groupCell' style='background-color: {color}; border-color: {color}'>{groupName}</div>",
        headTextBox: "<div class='matrixHeadTextBox'>{text}</div>",
        headTextLink: "<a class='matrixHeadTextBox' href='#'>{text}</a>",
        firstCellHeads: "<div class='matrixLine'></div><div class='firstCellPart colsName'>{colsName}</div><div class='firstCellPart rowsName'>{rowsName}</div>"
    };
    this.getColHeadsPart = function(){
        var o = this,
            colorIndex = 0,
            colNamesArr = [],
            colNames = this.params.colNames,
            cells = this.templates.cell.format({class: "firstCell", content: this.templates.firstCellHeads.format({colsName: this.params.bottomHeadling, rowsName: this.params.topHeadling})}),
            rows = "",
            html = "";
        if(typeof colNames["_first"] != "undefined") {
            colNames["_first"].forEach(function(colName){
                cells += o.templates.cell.format({
                    content: colName["content"],
                    class: colName["class"]
                });
            });
        }
        if(Object.keys(colNames).length == 1 && typeof colNames["main"] != "undefined") {
            rows = cells;
        } else {
            rows = o.templates.colGroup.format({
                name: "empty",
                content: cells,
                class: "firstGroup"
            });
        }
        $.each(colNames, function(key, val){
            if(key == "_first") {
                return true;
            }
            colNamesArr = [];
            cells = "";
            if(Object.keys(colNames).length > 1 || typeof colNames["main"] == "undefined") {
                cells += o.templates.groupCell.format({
                    groupName: o.templates.headTextLink.format({text: key}),
                    color: o.columnGroupsColor[colorIndex]
                });
            }
            val.forEach(function(nameItem){
                if(colNamesArr.indexOf(nameItem[0]["content"]) == -1) {
                    colNamesArr.push(nameItem[0]["content"]);
                    cells += o.templates.cell.format({
                        data: typeof nameItem[0]["dataAttr"] != "undefined" ? Helpers.convertObjectToDataAttribute(nameItem[0]["dataAttr"]) : "",
                        class: "",
                        content: o.params.colLinks ? o.templates.headTextLink.format({text: nameItem[0]["content"]}) : o.templates.headTextBox.format({text: nameItem[0]["content"]})
                    });
                }
            });
            if(Object.keys(colNames).length < 1 && typeof colNames["main"] != "undefined") {
                rows += cells;
            } else {
                rows += o.templates.colGroup.format({
                    name: key,
                    content: cells
                });
            }
            colorIndex = colorIndex == o.columnGroupsColor.length - 1  ? 0 : colorIndex + 1;
        });
        html = this.templates.table.format({
            body: rows,
            class: "colHeads"
        });
        return html;
    };
    this.getRowHeadsPart = function(){
        var o = this,
            rowNamesArr = [],
            rowNames = this.params.rowNames,
            rowGroups = "",
            rows = "",
            html = "";

        $.each(rowNames, function(key, val){
            rowNamesArr = [];
            rows = "";
            val.forEach(function(nameItems){
                if(rowNamesArr.indexOf(nameItems[0]["content"]) == -1) {
                    rowNamesArr.push(nameItems[0]["content"]);
                    nameItems.forEach(function(item){
                        rows += o.templates.cell.format({
                            data: typeof item["dataAttr"] != "undefined" ? Helpers.convertObjectToDataAttribute(item["dataAttr"]) : "",
                            class: "",
                            content: o.params.rowLinks ? o.templates.headTextLink.format({text: item["content"]}) : o.templates.headTextBox.format({text: item["content"]})
                        })
                    });
                }
            });
            rowGroups += o.templates.rowGroup.format({
                name: key,
                content: o.templates.row.format({cells: rows})
            });
        });
        html = this.templates.table.format({
            body: rowGroups,
            class: "rowHeads"
        });
        return html;
    };
    this.getDataPart = function(){
        var o = this,
            html,
            rows = "",
            cells = "",
            haveCellData = false,
            colNamesArr = [],
            rowNamesArr = [],
            data = this.params.data,
            rowNamesObj = this.params.rowNames,
            colNamesObj = this.params.colNames;
        $.each(rowNamesObj, function(rowGroup, rowNames){
            rowNamesArr = [];
            for(var i = 0; i < rowNames.length; i++) {
                if(rowNamesArr.indexOf(rowNames[i][0]["content"]) == -1) {
                    rowNamesArr.push(rowNames[i][0]["content"]);
                    cells = "";
                    $.each(colNamesObj, function(colGroup, colNames){
                        if(colGroup == "_first") {
                            return true;
                        }
                        colNamesArr = [];
                        for(var j = 0; j < colNames.length; j++) {
                            if(colNamesArr.indexOf(colNames[j][0]["content"]) == -1) {
                                colNamesArr.push(colNames[j][0]["content"]);
                                haveCellData = false;
                                $.each(data, function(key, val){
                                    if(val["rowGroup"] == rowGroup && val["rowName"] == rowNames[i][0]["content"] && val["colGroup"] == colGroup && val["colName"] == colNames[j][0]["content"]) {
                                        cells += o.templates.cell.format({
                                            data: typeof val["dataAttr"] != "undefined" ? Helpers.convertObjectToDataAttribute(val["dataAttr"]) : "",
                                            class: typeof val["class"] != "undefined" ? val["class"] : "",
                                            content: val["content"]
                                        });
                                        haveCellData = true;
                                        return false;
                                    }
                                });
                                if (!haveCellData) {
                                    cells += o.templates.cell.format({
                                        data: "",
                                        class: "",
                                        content: ""
                                    });
                                }
                            }
                        }
                    });
                    rows += o.templates.row.format({
                        cells: cells
                    });
                }
            }
        });

        html = this.templates.table.format({
            body: rows,
            class: "dataTable"
        });
        return html;
    };
    this.getHTML = function() {
        var o = this,
            dataPart = this.getDataPart(),
            rowHeadsPart = this.getRowHeadsPart(),
            colHeadsPart = this.getColHeadsPart();

        var html = this.templates.matrix.format({
            id: this.id,
            class: params.class ?  params.class : '',
            tables: colHeadsPart + rowHeadsPart + dataPart
        });
        this.handlers();
        return html;
    };
    this.setColGroupBorders = function(){
        var o = this,
            $matrix = $("#matrix-" + this.id),
            $dataPartRows = $matrix.find(".dataTable .row"),
            $colGroupItems,
            $dataPartRowCells,
            colOffset,
            colorIndex,
            colorsArr = [],
            $colGroups = $matrix.find(".colHeads .colGroup").not(":first");
        $.each($dataPartRows, function(){
            $dataPartRowCells = $(this).find(".cell");
            $dataPartRowCells.filter(":last").addClass("borderRight");
            colOffset = 0;
            colorIndex = 0;
            $.each($colGroups, function(){
                $colGroupItems = $(this).find(".cell");
                $dataPartRowCells[colOffset].classList.add("borderLeft");
                $($dataPartRowCells[colOffset]).css({
                    borderLeftColor: o.columnGroupsColor[colorIndex]
                });
                if(colOffset > 0) {
                    $dataPartRowCells[colOffset-1].classList.add("noBorderRight");
                }
                colorsArr.push(o.columnGroupsColor[colorIndex]);
                colOffset += $colGroupItems.length;
                colorIndex = colorIndex == o.columnGroupsColor.length - 1 ? 0 : colorIndex + 1;
            });
            $dataPartRowCells.filter(":last").css({
                borderRightColor: colorsArr[colorsArr.length - 1]
            });
        });
        colorIndex = 0;
        $.each($colGroups, function(){
            $colGroupItems = $(this).find(".cell");
            $colGroupItems.filter(":first").addClass("borderLeft");
            $colGroupItems.css({
                borderLeftColor: o.columnGroupsColor[colorIndex]
            });
            colorIndex = colorIndex == o.columnGroupsColor.length -1  ? 0 : colorIndex + 1;
        });
        $colGroups.filter(":last").find(".cell:last").addClass("borderRight");
        $colGroups.filter(":last").find(".cell:last").css({
            borderRightColor: colorsArr[colorsArr.length - 1]
        });
        $dataPartRows.filter(":last").find(".cell").addClass("borderBottom");
        var $lastRowCell;
            colorIndex = -1;
        $.each($dataPartRows.filter(":last").find(".cell"), function(){
            $lastRowCell = $(this);
            if($lastRowCell.hasClass("borderLeft")) {
                colorIndex = colorIndex == o.columnGroupsColor.length -1  ? 0 : colorIndex + 1;
            }
            $lastRowCell.css({
                borderBottomColor: o.columnGroupsColor[colorIndex]
            });
        });
    };
    this.setSize = function (){
        var $matrix = $("#matrix-" + this.id),
            $rowHeads = $matrix.find(".rowHeads"),
            $colHeads = $matrix.find(".colHeads"),
            $firstCell = $colHeads.find(".firstCell"),
            $dataTable = $matrix.find(".dataTable"),
            $colGroups = $colHeads.find(".colGroup").not(":first"),
            $firstColGroup = $colHeads.find(".firstGroup"),
            width;
        $firstColGroup.find(".cell").css({
            height: $colHeads.outerHeight()
        });
        $colHeads.find(".colGroup").not(".firstGroup").find(".cell").css({
            width: $dataTable.find(".cell").outerWidth()
        });
        $.each($colHeads.find(".firstGroup .cell"), function(i){
            width = $(this).outerWidth();
            $.each($rowHeads.find(".row"), function(){
                $($(this).find(".cell")[i]).outerWidth(width);
            });
        });
        $rowHeads.css({
            top: $colHeads.outerHeight()
        });
        $dataTable.css({
            top: $colHeads.outerHeight(),
            left: $firstColGroup.length > 0 ? $firstColGroup.outerWidth() : $firstCell.outerWidth()
        });
        if($colGroups.length > 0) {
            var colGroup;
            $.each($colGroups, function(){
                colGroup = $(this);
                colGroup.find(".groupCell").outerWidth(colGroup.find(".cell").length * colGroup.find(".cell").outerWidth());
            });
        }
        $firstCell.find(".matrixLine").css({
            transform: "rotate("+ (Math.atan($firstCell.outerHeight() / $firstCell.outerWidth()) * 45) +"deg)"
        });
    };
    this.adjustPosition = function(e, startPosition, newPosition) {
        var topMoveVal = e.clientY - startPosition.y0,
            leftMoveVal = e.clientX - startPosition.x0,
            $rowHeads = this.closest(".matrix").find(".rowHeads"),
            $colHeads = this.closest(".matrix").find(".colHeads"),
            padding = 20;
        if(this.height() + padding + $colHeads.height() > this.closest(".sod-matrix").height()) {
            if (topMoveVal + startPosition.tableTop > $colHeads.height()) {
                newPosition.top = $colHeads.height();
                newPosition.rowHeadsTop = $colHeads.height();
            } else if (topMoveVal + startPosition.tableTop < -(startPosition.maxTop + padding)) {
                newPosition.top = -(startPosition.maxTop + padding);
                newPosition.rowHeadsTop = -(startPosition.maxTop + padding);
            }
        } else {
            newPosition.top = $colHeads.height();
            newPosition.rowHeadsTop = $colHeads.height();
        }

        if(this.width() + padding + $rowHeads.width() > this.closest(".sod-matrix").width()) {
            if(startPosition.tableLeft + leftMoveVal >= $rowHeads.width()) {
                newPosition.left = $rowHeads.width();
                newPosition.colHeadsLeft = 0;
            } else if(startPosition.tableLeft + leftMoveVal <= -(startPosition.maxLeft + padding)) {
                newPosition.left = -(startPosition.maxLeft + padding);
                newPosition.colHeadsLeft = -(startPosition.maxLeft + padding + $rowHeads.width());
            }
        } else {
            newPosition.left = $rowHeads.width();
            newPosition.colHeadsLeft = 0;
        }
            return newPosition;
    };
    this.getStartPosition = function(e){
        var $colHeads = this.closest(".matrix").find(".colHeads");
        return {
            x0: e.clientX,
            y0: e.clientY,
            tableTop: parseInt(this.css("top")),
            tableLeft: parseInt(this.css("left")),
            colHeadsTop: parseInt($colHeads.css("left")),
            maxLeft: this.width() - this.closest(".matrix").width(),
            maxTop: this.height() - (this.closest(".sod-matrix").outerHeight() - $colHeads.height()),
            padding: 20
        };
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on("mousedown", "#matrix-" + this.id + " .dataTable", function(e){
            var $this = $(this),
                $rowHeads = $this.closest(".matrix").find(".rowHeads"),
                $colHeads = $this.closest(".matrix").find(".colHeads");
            if(e.which == 1) {
                e.preventDefault();
                var startPosition = o.getStartPosition.call($this, e);
                $this.on("mousemove", function(e){
                    var newPosition = {
                        left: startPosition.tableLeft + e.clientX - startPosition.x0,
                        top: startPosition.tableTop + e.clientY - startPosition.y0
                    };
                    newPosition.rowHeadsTop = newPosition.top;
                    newPosition.colHeadsLeft = newPosition.left - $rowHeads.width();

                    newPosition = o.adjustPosition.call($this, e, startPosition, newPosition);
                    $this.css("left", newPosition.left);
                    $this.css("top", newPosition.top);
                    $rowHeads.css("top", newPosition.rowHeadsTop);
                    $colHeads.css("left", newPosition.colHeadsLeft);
                });
                $this.on("mouseleave", function(){
                    $this.off("mousemove");
                    $this.off("mouseleave");
                });
                $this.on("mouseup", function(){
                    $this.off("mousemove");
                });
            }
        });
        $(document.body).on("wheel", "#matrix-" + this.id, function(e){
            var $this = $(this),
                $rowHeads = $this.find(".rowHeads"),
                $colHeads = $this.find(".colHeads"),
                $dataTable = $this.find(".dataTable"),
                $colGroups = $colHeads.find(".colGroup").not(":first"),
                cell = $dataTable.find(".cell"),
                val = 0;
            if(cell.outerHeight() + (-e.originalEvent.deltaY/10) <= 80 && cell.outerHeight() + (-e.originalEvent.deltaY/10) >= 20) {
                val = e.originalEvent.deltaY > 0 ? -10 : 10;
                $rowHeads.find(".cell").css({
                    height: parseInt(cell.css("height")) + val
                });
                $colHeads.find(".colGroup").not(".firstGroup").find(".cell").css({
                    width: parseInt(cell.css("width")) + val
                });
                $dataTable.find(".cell").css({
                    width: parseInt(cell.css("width")) + val,
                    height: parseInt(cell.css("height")) + val
                });
                $dataTable.css({
                    left: parseInt($dataTable.css("left")) + (val*$dataTable.find("row:first .cell").length),
                    top: parseInt($dataTable.css("top")) + val*$dataTable.find("row").length
                });
                if($colGroups.length > 0) {
                    var $colGroup;
                    $.each($colGroups, function(){
                        $colGroup = $(this);
                        $colGroup.find(".groupCell").outerWidth($colGroup.find(".groupCell").outerWidth() + ($colGroup.find(".cell").length * val));
                    })
                }
            }
            if(typeof o.params.scaleCallback != "undefined") {
                o.params.scaleCallback.call(this);
            }
        });
    };
}
Matrix.id = 0;

function MatrixOld(params) {
    this.params = params;
    this.templates = {
        wrap: '<div class="blocks"></div>',
        matrix: "<div class='block {style}'><h3>{headling}</h3><table class='{tableClass}'>{colGroups}<tbody>{table}</tbody></table></>",
        tbody: "<tbody class='{class}'><tr class='rowGroupName'><td>{rowGroupName}</td>{disabledCells}</tr>{tbody}</tbody>",
        tr: "<tr>{cell}</tr>",
        td: "<td class='{cellClass}'>{content}</td>",
        tdGroup: "<td class='{class}' colspan='{colspan}' style='background-color: {backgroundColor}'>{content}</td>",
        colGroup: "<colgroup class='{groupClass}' style='border-color: {borderColor}' span='{span}'>"
    };
    this.getHTML = function() {
        var o = this,
            data = this.params.data,
            firstCellContent = this.params.firstCellContent,
            colNames = this.params.colNames,
            rowNames = this.params.rowNames,
            columnGroupsData = this.params.colGroups,
            rowGroupsData = this.params.rowGroups,
            columnGroupsColor = this.params.colGroupsColor ? this.params.colGroupsColor : ['#6C98DA', '#F7941E', '#5B9E8C', '#ABAD25', '#CC6FC5'],
            colGroupColorNum = 0,
            sectionName = this.params.sectionName,
            colGroups ="",
            rowGroups = {},
            colGroupNum = 1,
            rows = '',
            cells = '';
        cells = this.templates.td.format({cellClass: '', content: ""});

        /* colgroups */
        if(columnGroupsData) {
            colGroups = this.templates.colGroup.format({span: 1, borderColor: 'none', groupClass: 'firstColGroup'});
            $.each(columnGroupsData, function(key, val) {
                colGroups += o.templates.colGroup.format({span: val, borderColor: columnGroupsColor[colGroupColorNum], groupClass: 'mat-colgroup' + colGroupNum});
                cells += o.templates.tdGroup.format({class: 'colGroupHeader',colspan: val, backgroundColor: columnGroupsColor[colGroupColorNum],  content: key});
                colGroupColorNum >= columnGroupsColor.length - 1 ? colGroupColorNum = 0 : colGroupColorNum++;
                colGroupNum > 5 ? colGroupNum = 1 : colGroupNum++;
            });
            rows = this.templates.tr.format({cell :cells});
        }

        /* headlings */
        cells = this.templates.td.format({cellClass: 'firstCell', content: firstCellContent ? firstCellContent : ""});
        for (var i = 0; i < colNames.length; i++) {
            cells += this.templates.td.format({cellClass: '', content: colNames[i]});
        }
        rows += this.templates.tr.format({cellClass: '', cell :cells});

        /* data */
        for (var i = 0; i < rowNames.length; i++) {
            cells = this.templates.td.format({cellClass: 'rowNameCell', content: rowNames[i]});
            for (var j = 0; j < colNames.length; j++) {
                if (data[i] && data[i][j]) {
                    cells += this.templates.td.format({
                        cellClass: this.params.cellClass ? this.params.cellClass[i][j] : '',
                        content: data[i][j].cell
                    });
                } else {
                    cells += this.templates.td.format({
                        cellClass: '',
                        content: ""
                    });
                }
            }
            /* rowGroups */
            if (rowGroupsData) {
                if (rowGroups[rowGroupsData[i]]) {
                    rowGroups[rowGroupsData[i]] += this.templates.tr.format({cell: cells});
                } else {
                    rowGroups[rowGroupsData[i]] = this.templates.tr.format({cell: cells});
                }
            } else {
                rows += this.templates.tr.format({cell: cells});
            }
        }
        if (rowGroupsData) {
            $.each(rowGroups, function(key, val) {
                var disabledCells = '';
                for(var i = 0; i < colNames.length; i++) {
                    disabledCells += o.templates.td.format({cellClass: 'rowGroupName', content: ''});
                }
                rows += o.templates.tbody.format({
                    class: '',
                    rowGroupName: key,
                    disabledCells: disabledCells,
                    tbody: val
                });
            });
        }

        var html = this.templates.matrix.format({
            headling: params.headling ? params.headling : '',
            tableClass: params.tableClass ?  params.tableClass : '',
            colGroups: colGroups ? colGroups : '',
            table: rows,
            style: params.style ? params.style : ''
        });
        return html;
    }
};