function Tree($selector, data, params) {
    this.params = params || { };
    this.$selector = $selector;
    this.data = data;
    this.filters = {};
    this.templates = {
        wrap: "<div class='tree'>{tree}</div>",
        filter: "<div class='filters'></div>",
        item_data: " data-{type}='{value}'",
        item: "<div class='tree-item level-{level}{additional_classes}' {data_attributes}>{content}</div>",
        text: "<div class='tree-item-content'>{controls}<span class='text'>{text}</span></div>",
        nodes: "<div class='nodes'>{nodes}</div>"
    };
    this.getTraceTree = function ($element, indexes) {
        var $item = $element.parents(".tree-item");
        indexes.push($item.index());
        if ($item.parents(".tree-item").length > 0) {
            return this.getTraceTree($item, indexes);
        }
        return indexes;
    };
    this.addItem = function (element) {
        var data_attributes = [];
        for (var dataItem in element.data) {
            data_attributes.push(
                this.templates.item_data.format({
                    type: dataItem,
                    value: element.data[dataItem]
                })
            );
        }
        var item = this.templates.item.format({
            level: element.level,
            additional_classes: " shown expanded" + (element.checkbox ? " with_checkbox" : ""),
            data_attributes: data_attributes.join(" "),
            content: this.templates.text.format({
                controls: (element.expandable === false ? '' : '<a href="#" class="expand"></a>') + (element.checkbox ? "<input type='checkbox' />" : ""),
                text: element.text
            })
        });
        return item;
    };
    this.addItemAfter = function ($item, element) {
        var newItem = this.addItem(element),
            $newItem = $item.parent(".tree-item").before(newItem);
        this.clear();
        this.handlers();
        this.lastNodesDetect($newItem.parents(".nodes"));
        return $newItem.prev();
    };
    this.addItemBefore = function ($item, element) {
        var newItem = this.addItem(element),
            $newItem = $item.parent(".tree-item").before(newItem);
        this.clear();
        this.handlers();
        this.lastNodesDetect($newItem.parents(".nodes"));
        return $newItem.prev();
    };
    this.addItemPrepend = function ($item, element) {
        if ($item.next(".nodes").length == 0) {
            $item.before(this.templates.nodes.format({
                nodes: this.addItem(element)
            }));
        } else {
            $item.next(".nodes").prepend(this.addItem(element));
        }
        this.clear();
        this.handlers();
        var $nodes = $item.parent(".tree-item").find("> .nodes");
        return $nodes.find("> .tree-item:first-child");
    };
    this.addItemAppend = function ($item, element) {
        if ($item.next(".nodes").length == 0) {
            $item.after(this.templates.nodes.format({
                nodes: this.addItem(element)
            }));
        } else {
            $item.next(".nodes").append(this.addItem(element));
        }
        this.clear();
        this.handlers();
        var $nodes = $item.parent(".tree-item").find("> .nodes");
        this.lastNodesDetect($nodes);
        return $nodes.last();
    };
    this.buildWalk = function(data, level) {
        var items = "";
        for (var i = 0; i < data.length; i++) {
            var text = this.templates.text.format({
                controls: (data[i].nodes ? "<a href='#' class='expand'></a>" : "") + (data[i].checkbox ? "<input type='checkbox' />" : ""),
                text: data[i].text
            });
            var nodes = "";
            if (typeof data[i].nodes != "undefined" && data[i].nodes.length != 0) {
                nodes = this.buildWalk(data[i].nodes, level + 1);
                nodes = this.templates.nodes.format({ nodes: nodes });
            }
            var classes = data[i].hidden === true ? " hidden" : " shown";
            if (data[i].expanded === true) {
                classes += " expanded";
            }
            if (data[i].checkbox === true) {
                classes += " with_checkbox";
            }
            var data_attrs = this.templates.item_data.format({
                type: "name",
                value: (data[i].text + "").stripHTMLTags()
            });
            if (typeof data[i].data != "undefined") {
                for (var data_item in data[i].data) {
                    data_attrs += this.templates.item_data.format({
                        type: data_item,
                        value: data[i].data[data_item]
                    });
                }
            }
            items += this.templates.item.format({
                level: data[i].level != null ? data[i].level : level,
                additional_classes: classes,
                data_attributes: data_attrs,
                content: text + nodes
            });
        }
        return items;
    };
    this.lastNodesDetect = function($nodes) {
        var o = this,
            $nodes = $nodes || this.$selector,
            $items = $nodes.find("> .tree-item");
        $items.removeClass("last");
        var $last_node = $nodes.find("> .tree-item.shown").last();
        $last_node.addClass("last");
        $items.each(function() {
            var $this = $(this);
            if ($this.find("> .nodes").length > 0) {
                o.lastNodesDetect($this.find("> .nodes"));
            }
        });

    };
    this.filtering = function(name, values) {
        if (values.length == 0) {
            this.$selector.find(".tree-item[data-type=\"" + name + "\"]").removeClass("hidden").addClass("shown");
            this.lastNodesDetect(this.$selector.find("> .tree"), 1);
            return;
        }
        this.$selector.find(".tree-item[data-type=\"" + name + "\"]").removeClass("shown").addClass("hidden");
        for (var i = 0; i < values.length; i++) {
            this.$selector.find(".tree-item[data-name*=\"" + values[i] + "\"][data-type=\"" + name + "\"]").each(function() {
                var $this = $(this);
                $this.removeClass("hidden").addClass("shown");
            })
        }
        this.lastNodesDetect(this.$selector.find("> .tree"));
    };
    this.collectFilterInfo = function() {
        var o = this;
        this.$selector.find(".tree-item[data-type]").each(function() {
            var $this = $(this),
                name = $this.attr("data-type");
            if (typeof o.filters[name] == "undefined") {
                o.filters[name] = [];
            }
            var $element = $this.find("> .tree-item-content > .text").clone();
            $element.children().remove();
            var value = $element.text();
            if (o.filters[name].indexOf(value) == -1) {
                o.filters[name].push(value);
            }
        })
    };
    this.getFilters = function () {
        return this.filters;
    };
    this.build = function() {
        var html = this.buildWalk(this.data, 1);
        var tree = this.templates.wrap.format({
            tree: html
        });
        this.$selector.html(tree);
        if (params.with_filters === true) {
            this.collectFilterInfo();
            this.$selector.prepend(this.templates.filter);
        }
        this.lastNodesDetect(this.$selector.find("> .tree"));
    };
    this.mark = function($item) {
        $item.toggleClass("checked");
        var state = $item.hasClass("checked");
        $item.find(".tree-item.shown.with_checkbox").each(function() {
            var $this = $(this);
            $this[state ? "addClass" : "removeClass"]("checked");
            var $checkbox = $this.find("> .tree-item-content input[type=\"checkbox\"]");
            if ($checkbox.length > 0 && $checkbox[0].disabled == false) {
                $checkbox[0].indeterminate = false;
                $checkbox[0].checked = state;
            }
        });
        $item.parents(".tree-item").each(function() {
            var $this = $(this);
            var checked_length = $this.find("> .nodes .tree-item.shown.with_checkbox.checked").length;
            var is_checked_all = checked_length == $this.find("> .nodes .tree-item.shown.with_checkbox").length;
            $this[is_checked_all ? "addClass" : "removeClass"]("checked");
            var checkbox = $this.find("> .tree-item-content input[type=\"checkbox\"]")[0];
            if (typeof checkbox != "undefined") {
                checkbox["checked"] = is_checked_all;
                checkbox["indeterminate"] = checked_length > 0 && !is_checked_all;
            }
        });
    };
    this.expand = function($item) {
        var $multiLineBlocks = $item.find("> .tree-item-content .multiLineBlock"),
            multiLineBlocksEpanded = $multiLineBlocks.hasClass("collapsed");
        if (multiLineBlocksEpanded) {
            $multiLineBlocks.removeClass("collapsed");
        }
        $item.find("> .nodes").slideToggle(function() {
            $item.toggleClass("expanded");
            if (!multiLineBlocksEpanded) {
                $multiLineBlocks.addClass("collapsed");
            }
        });
    };
    this.expandAll = function() {
        this.$selector.find(".tree-item").addClass("expanded");
        this.$selector.find(".tree-item > .nodes").show();
    };
    this.collapseAll = function() {
        this.$selector.find(".tree-item").removeClass("expanded");
        this.$selector.find(".tree-item > .nodes").hide();
    };
    this.handlers = function() {
        var o = this;
        this.$selector.find(".tree .tree-item .tree-item-content .text").on("click", function (event) {
            var $target = $(event.target);
            if ($target.is("a")) {
                return true;
            }
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            var $nodes = $item.find(".nodes");
            if ($nodes.length > 0 && o.params.expandWhenClick !== false) {
                o.expand($item);
            } else if ($item.hasClass("with_checkbox") && o.params.customChecking !== true) {
                o.mark.call(o, $item);
                var checkbox = $item.find(".tree-item-content > input[type=\"checkbox\"]")[0];
                checkbox.indeterminate = false;
                checkbox.checked ^= 1;
            }
        });
        if (this.params.customChecking !== true) {
            this.$selector.find(".tree .tree-item.with_checkbox .tree-item-content input[type=\"checkbox\"]").on("change", function () {
                var $this = $(this);
                var $item = $this.parent(".tree-item-content");
                o.mark.call(o, $item.parent(".tree-item"));
            });
        };
        this.$selector.find(".tree .tree-item .expand").on("click", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            o.expand($item);
            return false;
        });
    };
    this.clear = function() {
        this.$selector.find(".tree .tree-item .tree-item-content .text").off("click");
        this.$selector.find(".tree .tree-item.with_checkbox .tree-item-content input[type=\"checkbox\"]").off("change");
        this.$selector.find(".tree .tree-item .expand").off("click");
    };
    this.init = function() {
        this.build();
        this.handlers();
    }
}