Page.Render = {
    init: function() {
        var page = ['assess', 'assets', 'groups'];
        Locale.set("en", { fields: ["landscape"] });
        Locale.loadPage(page);
        Header.load(page, { type: "landscape" });
        new Landscape().init();
    }
};

function Landscape() {
    this.table = null;
    this.data = null;
    this.templates = {
        no_entry: "<p>" + Locale.ui.page.list.no_entry + "</p>",
        wrap: "<div class='blocks'><div class='block col-2'><h3>" + Locale.ui.page.editor.title + "</h3><div class='editor'>{editor_block}</div></div></div>",
        default: "<p>" + Locale.ui.page.editor.default + "</p>",
        editor: "<div><h4>" + Locale.ui.page.editor.section.info + "</h4><div class='fields'>{fields}</div><h4>" + Locale.ui.page.editor.section.assets + "</h4><ul class='assets'>{assets}</ul>{asset_new_button}<div class='buttons'>{buttons}</div><input type='hidden' name='current_id' value='{current_id}'></div>",
        buttons: {
            edit: "<button class='save'>" + Locale.ui.page.editor.buttons.save + "</button><button class='remove default'>" + Locale.ui.page.editor.buttons.remove + "</button><a href='#' class='close'>" + Locale.ui.page.editor.buttons.close + "</a>",
            create: "<button class='save'>" + Locale.ui.page.editor.buttons.create + "</button><a href='#' class='close'>" + Locale.ui.page.editor.buttons.cancel + "</a>"
        },
        create: "<div class='create-button'><button>" + Locale.ui.page.editor.buttons.new + "</button></div>",
        field: "<div class='field'><div class='title'>{title}:</div><div class='input'><input type='text' name='{name}' placeholder='{placeholder}' value='{value}' /></div></div>",
        tag: "<div class='tag'><span class='text'>{text}</span><a href='#' class='remove'></a></div>",
        asset: "<li><span class='sid'>{sid}</span><a href='#' class='remove'></a></li>",
        asset_new: "<select name='assets'>{options}</select>",
        asset_new_item: "<option value='{value}'>{text}</option>",
        tag_popup: "<div class='field'><div class='name'>Tag title:</div><div class='input'><input type='text' name='tag' maxlength='32' /></div></div><div class='buttons'><button class='save-tag'>Save</button></div>",
        error_popup: "<div>{text}</div><div class='buttons'><button class='error-ok'>" + Locale.ui.page.editor.error_close + "</button></div>"
    };
    this.tableLoad = function() {
        this.data = Helpers.loadJSON("/data/groups.json");
        var params = {
            title: "List",
            data: Helpers.objToArray(this.data, { key_add: true, key_field_name: "id" }),
            option: {
                expandable: false,
                select: true,
                fields: "landscape",
                ticks: {
                    'id': 0,
                    'title': 1,
                    'description': 2,
                    'business_value': 3
                },
                sorting: {
                    0: false,
                    1: false,
                    2: false,
                    3: false
                },
                count_one_page: 50
            }
        };
        this.table = new Table(params);
        var html = this.table.getHTML(),
            section = $('section.content'),
            blocks = section.find('.blocks'),
            table = blocks.find(".table");
        table.remove();
        blocks.append(html);
        table.find(".actions-bar").prepend(this.templates.create);
        if (Helpers.objIsEmpty(this.data)) {
            table.find("table.tablesorter").after(this.templates.no_entry);
            table.find("table.tablesorter, .blocks .table .pager").remove();
            return;
        }
        this.table.build();
        Helpers.setBlockHeight(section);
        var $opened_landscape = $(".editor input[name=\"id\"]");
        if ($opened_landscape.length > 0) {
            var o = this;
            $(".blocks .table table.tablesorter tbody tr").each(function() {
                var $td = $(this).find("td").eq(o.table.getColumnNumber("id"));
                var id = $td.text();
                if (id == $opened_landscape.val()) {
                    $td.parent("tr").addClass("active");
                    return false;
                }
            })
        }
    };
    this.editorHide = function() {
        $(".editor").html(this.templates.default);
    };
    this.editorShow = function(id) {
        var fields = "";
        for (var field in Locale.ui.page.editor.fields) {
            var value = id != 0 ? this.data[id][field] : "";
            fields += this.templates.field.format({
                title: Locale.ui.page.editor.fields[field],
                name: field,
                placeholder: Locale.ui.page.editor.fields[field],
                value: value || ""
            });
        }
        var assets_new = this.templates.asset_new_item.format({
            value: 0,
            text: Locale.ui.page.editor.assets.list
        }),
            i = 1;
        for (var asset in this.assets) {
            assets_new += this.templates.asset_new_item.format({
                value: i++,
                text: asset
            });
        }
        assets_new = this.templates.asset_new.format({
            options: assets_new
        });
        var assets = "";
        if (id != 0 && this.data[id].assets && this.data[id].assets instanceof Array) {
            for (var i = 0; i < this.data[id].assets.length; i++) {
                assets += this.templates.asset.format({
                    sid: this.data[id].assets[i]
                });
            }
        }
        var buttons_type = id == 0 ? "create" : "edit";
        var html = this.templates.editor.format({
            fields: fields,
            assets: assets,
            asset_new_button: assets_new,
            buttons: this.templates.buttons[buttons_type],
            current_id: id
        });
        $(".editor").html(html);
    };
    this.tagPaste = function(tag) {
        $(".editor .add_tag").before(this.templates.tag.format({
            text: tag
        }));
        Popup.close();
    };
    this.tagAdd = function() {
        var o = this;
        Popup.new({
            class: "add-tag add-popup",
            title: Locale.ui.page.editor.tags.adding_popup,
            content: o.templates.tag_popup,
            width: 350,
            height: 80,
            open: function() {
                $(".add-tag .input input[name=\"tag\"]").focus().on("keyup", function(e){
                    if (e.keyCode == 13) {
                        o.tagPaste.call(o, $(this).val());
                    }
                });
                $(".add-tag .buttons .save-tag").on("click", function(){
                    var tag = $(".add-tag .input input[name=\"tag\"]").val();
                    if (tag) {
                        o.tagPaste.call(o, tag);
                    }
                })
            },
            close: function() {
                $(".add-tag .input input[name=\"tag\"]").off("keyup");
                $(".add-tag .buttons .save-tag").off("click");
            }
        });
    };
    this.tagRemove = function() {
        $(this).parent(".tag").remove();
    };
    this.assetPaste = function($option) {
        var sid = $option.text();
        $(".editor .assets").append(this.templates.asset.format({
            sid: sid
        }));
        $(".editor select[name=\"assets\"] option")[0].selected = true;
        $option.hide();
    };
    this.assetRemove = function() {
        var $this = $(this);
        var sid = $this.text();
        $(".editor select[name=\"assets\"] option:contains(" + sid + ")").show();
        $this.parent("li").remove();
    };
    this.save = function(o) {
        var id = $(".editor input[name=\"id\"]").val();
        var title = $(".editor input[name=\"title\"]").val();
        var type_error = false;
        if (!title)
            type_error = "title";
        if (!id)
            type_error = "id";
        if (type_error) {
            Popup.new({
                class: "adding-error",
                title: Locale.ui.page.editor.error,
                content: o.templates.error_popup.format({
                    text: Locale.ui.page.editor.errors[type_error]
                }),
                width: 350,
                height: 60
            });
            return;
        };
        var $button = $(this);
        if ($button.hasClass("disable"))
            return false;
        $button.html(Locale.ui.page.editor.buttons.sending).addClass("disable");
        var tags = [];
        $(".editor .tags .tag").each(function() {
            tags.push($(this).find(".text").text());
        });
        var assets = [];
        $(".editor .assets li").each(function() {
            assets.push($(this).find(".sid").text());
        });
        var current_id = $(".editor input[name=\"current_id\"]").val();
        var params = {
            current_id: current_id,
            id: id,
            title: title,
            business_value: $(".editor input[name=\"business_value\"]").val(),
            description: $(".editor input[name=\"description\"]").val(),
            tags: tags,
            assets: assets
        };
        $.ajax({
            url: "http://172.16.2.43:8080/edit_landscape",
            dataType: "JSON",
            crossDomain: true,
            type: "POST",
            data: params,
            success: function() {
                $button.html(Locale.ui.page.editor.buttons.save).removeClass("disable");
                if (current_id == 0) {
                    o.editorHide.call(o);
                }
                o.tableLoad();
            }
        })
    };
    this.remove = function(o) {
        var $button = $(this);
        if ($button.hasClass("disable"))
            return false;
        $button.html(Locale.ui.page.editor.buttons.removing).addClass("disable");
        var params = {
            current_id: $(".editor input[name=\"current_id\"]").val()
        };
        $.ajax({
            url: "http://172.16.2.43:8080/edit_landscape",
            dataType: "JSON",
            crossDomain: true,
            type: "POST",
            data: params,
            success: function() {
                $button.html(Locale.ui.page.editor.buttons.remove).removeClass("disable");
                o.editorHide.call(o);
                o.tableLoad();
            }
        })
    };
    this.assetsLoad = function() {
        this.assets = Helpers.loadJSON("/data/assets.json");
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", ".blocks table.tablesorter tbody tr", function() {
            var $this = $(this);
            $(".blocks table.tablesorter tbody tr").removeClass("active");
            $this.addClass("active");
            var id = $this.find("td").eq(o.table.getColumnNumber("id")).text().trim();
            o.editorShow(id);
        });
        $(document.body).on("click", ".editor .add_tag", function() {
            o.tagAdd.call(o);
            return false;
        });
        $(document.body).on("click", ".editor .tags .tag .remove", function() {
            o.tagRemove.call(this);
            return false;
        });
        $(document.body).on("change", ".editor select[name=\"assets\"]", function() {
            o.assetPaste.call(o, $(this).find("option:selected"));
            return false;
        });
        $(document.body).on("click", ".editor .assets .remove", function() {
            o.assetRemove.call(this);
            return false;
        });
        $(document.body).on("click", ".editor .buttons .close", function() {
            $(".blocks table.tablesorter tbody tr").removeClass("active");
            o.editorHide.call(o);
            return false;
        });
        $(document.body).on("click", ".editor .landscape-new, .table .actions-bar .create-button button", function() {
            $(".blocks table.tablesorter tbody tr").removeClass("active");
            o.editorShow(0);
            return false;
        });
        $(document.body).on("click", ".editor .save", function() {
            o.save.call(this, o);
            return false;
        });
        $(document.body).on("click", ".editor .buttons .remove", function() {
            if (confirm(Locale.ui.page.editor.buttons.remove_confirm)) {
                o.remove.call(this, o);
            }
        })
        $(document.body).on("click", ".adding-error .error-ok", function() {
            Popup.close();
        })
    };
    this.init = function() {
        $("section.content").html(this.templates.wrap.format({
            editor_block: this.templates.default
        })).addClass("groups");
        this.assetsLoad();
        this.tableLoad();
        this.handlers();
    };
}