Page.Render = {
    init: function() {
        var page = ['assess', 'assets', 'inventory'];
        Locale.set("en", { fields: ["inventory"] });
        Locale.loadPage(page);
        Header.load(page, { tabs: true });
        new Inventory().init();
    }
};

function Inventory() {
    this.scans = [];
    this.templates = {
        wrap: "<div class='blocks'><div class='block services-tree col-2'><h3>" + Locale.ui.page["settings"]["title"] + "</h3><div class='block-content'>" + Locale.ui.page["settings"]["no_select"] + "</div></div></div></div>",
        all_button: '<div class="all-button"><button>' + Locale.ui.page["buttons"]["all"] + '</button></div>',
        service_disable_wrap: '<span class="disabled">{text}</span>'
    };
    this.servicesTree = [];
    this.services = {
        service: {},
        appserver: {}
    };
    this.systems = [];
    this.preLoad = function(callback) {
        var o = this,
            counter = 0,
            total_queries = 1;
        $.ajax({
            url: "/data/services.xml",
            method: "GET",
            dataType: "XML",
            success: function(result) {
                $("services > module", result).each(function() {
                    var $this = $(this);
                    o.systems[$this.attr("id")] = $this.attr("name");
                });
                $("services > appserver", result).each(function() {
                    var $this = $(this);
                    o.services.appserver[$this.attr("id")] = {
                        name: $this.attr("name"),
                        versions: $this.attr("versions"),
                        has_instance: $this.attr("has_instance"),
                        services: []
                    };
                });
                $("services > service", result).each(function() {
                    var $this = $(this);
                    var base = $this.find("base").text().split(",");
                    var service_obj = {
                        name: base[3],
                        appserver: base[1],
                        isClientable: base[4],
                        isAlwaysAnonim: base[5],
                        isAlwaysNotAnonim: base[6],
                        userCanChangePort: base[7],
                        port: base[8],
                        isGateway: base[9],
                        WebRfcPort: base[10]
                    };
                    o.services.appserver[base[1]].services.push(service_obj);
                    o.services.service[$this.attr("id")] = service_obj;
                });
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
    };
    this.buildTable = function() {
        var params = {
            title: "List",
            data: this.scans,
            option: {
                expandable: false,
                select: true,
                fields: "inventory",
                ticks: {
                    'id': 0,
                    'created': 1,
                    'hosts': 2,
                    'status': 3,
                    'progress': 4
                },
                filters: {
                    "status": "select",
                    "progress": "select",
                    "created": "datetime"
                },
                sort_default: { column: 0, order: "asc" },
                count_one_page: 100
            }
        };
        this.table = new Table(params);
        var html = this.table.getHTML(),
            section = $('section.content.inventory'),
            blocks = section.find('.blocks');
        blocks.append(html);
        blocks.find(".block").eq(1).addClass("col-2");
        this.table.build();
        Helpers.setBlockHeight(section);
        $(".table .chart-actions").before(this.templates.all_button);
    };
    this.loadScans = function() {
        var o = this;
        ERPScanAPI.query("/engine/ps/run/list", function(result) {
            $("listruns > prscanrun", result).each(function() {
                var $this = $(this),
                    progress = $this.find("progress").text(),
                    date_array = $this.find("indate").text().split("."),
                    date_obj = new Date(date_array[1] + "." + date_array[0] + "." + date_array[2]);
                o.scans.push({
                    id: $this.find("id").text(),
                    created: {
                        value: date_obj.format('d/m/y H:i'),
                        attr: { name: 'ts', value: date_obj.getTime() }
                    },
                    hosts: $this.find("hosts").text(),
                    status: progress == "100.0" ? "Finished" : "In progress",
                    progress: progress + "%"
                });
            });
            o.buildView.call(o);
        }, {
            pagesize: 100,
            page: 1
        })
    };
    this.buildView = function() {
        $("section.content").html(this.templates.wrap);
        this.buildTable();
    };
    this.displayServicesTree = function(id) {
        var o = this;
        $(".blocks .block.services-tree .block-content").html(Header.templates.tabs.loading);
        o.servicesTree = [];
        ERPScanAPI.query("/engine/ps/results/" + id, function(result) {
            $("resultsgroup > sid", result).each(function() {
                var $this = $(this);
                var sid_index = o.servicesTree.push({
                        expanded: true,
                        nodes: [],
                        disabled_services: 0,
                        all_services: 0,
                        text: $this.attr("id"),
                        data: {
                            filter: "sid"
                        }
                    }) - 1;
                $this.find("host").each(function() {
                    var $this = $(this);
                    var host_index = o.servicesTree[sid_index].nodes.push({
                            expanded: true,
                            nodes: [],
                            disabled_services: 0,
                            all_services: 0,
                            text: $this.attr("id"),
                            data: {
                                filter: "ip"
                            }
                        }) - 1;
                    $this.find("appserver").each(function() {
                        var $this = $(this),
                            systemId = $this.attr("systemid") ? " [" + $this.attr("systemid") + "]" : "",
                            appserver_index = o.servicesTree[sid_index].nodes[host_index].nodes.push({
                                    expanded: true,
                                    nodes: [],
                                    disabled_services: 0,
                                    all_services: 0,
                                    text: $this.attr("name") + systemId,
                                    data: {
                                        filter: "appserver"
                                    }
                                }) - 1,
                            clients_index,
                            services_index,
                            services_node = false,
                            clients_node = false;
                        $this.find("services service").each(function() {
                            var $this = $(this);
                            if (services_node === false) {
                                services_index = o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes.push({
                                        expanded: true,
                                        nodes: [],
                                        text: "Services",
                                        disabled_services: 0,
                                        all_services: 0
                                    }) - 1;
                                services_node = true;
                            }
                            var name = $this.attr("name") + " [" + $this.find("port").attr("value") + "]",
                                text = $this.attr("id") ? name : o.templates.service_disable_wrap.format({ text: name });
                            if (!$this.attr("id")) {
                                o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].disabled_services++;
                                o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].disabled_services++;
                                o.servicesTree[sid_index].nodes[host_index].disabled_services++;
                                o.servicesTree[sid_index].disabled_services++;
                            }
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].all_services++;
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].all_services++;
                            o.servicesTree[sid_index].nodes[host_index].all_services++;
                            o.servicesTree[sid_index].all_services++;
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].nodes.push({
                                text: text,
                                data: {
                                    filter: "service"
                                }
                            });
                        });
                        $this.find("clients client").each(function() {
                            var $this = $(this);
                            if (clients_node === false) {
                                clients_index = o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes.push({
                                    expanded: true,
                                    nodes: [],
                                    text: "Clients"
                                }) - 1;
                                clients_node = true;
                            }
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[clients_index].nodes.push({
                                text: "Client " + $this.attr("number"),
                                data: {
                                    filter: "client"
                                }
                            });
                        });
                        if (typeof services_index != "undefined" && o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].disabled_services == o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].all_services) {
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].text = o.templates.service_disable_wrap.format({
                                text: o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].nodes[services_index].text
                            });
                        }
                        if (typeof services_index != "undefined" && o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].disabled_services == o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].all_services) {
                            o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].text = o.templates.service_disable_wrap.format({
                                text: o.servicesTree[sid_index].nodes[host_index].nodes[appserver_index].text
                            });
                        }
                    });
                    if (typeof host_index != "undefined" && o.servicesTree[sid_index].nodes[host_index].disabled_services == o.servicesTree[sid_index].nodes[host_index].all_services) {
                        o.servicesTree[sid_index].nodes[host_index].text = o.templates.service_disable_wrap.format({
                            text: o.servicesTree[sid_index].nodes[host_index].text
                        });
                    }
                });
                if (typeof sid_index != "undefined" && o.servicesTree[sid_index].disabled_services == o.servicesTree[sid_index].all_services) {
                    o.servicesTree[sid_index].text = o.templates.service_disable_wrap.format({
                        text: o.servicesTree[sid_index].text,
                        data: {
                            filter: "sid"
                        }
                    });
                }
            });
            o.treeObj = new Tree($(".blocks .block.services-tree .block-content"), o.servicesTree, {
                expandWhenClick: true,
                with_filters: true,
                fields: "inventory"
            });
            o.treeObj.init();
            var filters = o.treeObj.getFilters();
            for (var filter in filters) {
                var filter_obj = new Filter(filter, {
                    fields: "inventory",
                    variants: filters[filter],
                    callback: function(filter) {
                        return function(values) {
                            o.treeObj.filtering(filter, values);
                        }
                    }(filter)
                });
                $(".block.services-tree .filters").append(filter_obj.getHTML());
                filter_obj.build();
            }
        })
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", ".blocks .table table.tablesorter tbody tr", function() {
            var $this = $(this);
            var id = $this.find("td").eq(o.table.getColumnNumber("id")).text();
            o.displayServicesTree.call(o, id);
            return false;
        });
    };
    this.init = function() {
        $("section.content").html(Header.templates.tabs.loading).addClass("inventory");
        this.preLoad(this.loadScans);
        this.handlers();
    }
}