Page.Render = {
    init: function() {
        var page = ['assess', 'assets', 'landscape'];
        Locale.set("en", { fields: ["landscape"] });
        Locale.loadPage(page);
        Header.load(page, { type: "landscape" });
        new Landscape().init();
    }
};

function Landscape() {
    this.templates = {
        no_entry: "<p>" + Locale.ui.page.list.no_entry + "</p>",
        wrap: "<div class='blocks'><div class='block editor col-2'><h3>" + Locale.ui.page.editor.title + "</h3>{editor}</div></div>",
        default: "<p class='no_select'>" + Locale.ui.page.editor.default + "</p>",
        editor: '<div class="tree-block">{tree}</div><div class="parameters-block">{parameters}</div>',
        buttons: {
            edit: "<button class='save'>" + Locale.ui.page.editor.buttons.save + "</button><button class='remove default'>" + Locale.ui.page.editor.buttons.remove + "</button><a href='#' class='close'>" + Locale.ui.page.editor.buttons.close + "</a>",
            create: "<button class='save'>" + Locale.ui.page.editor.buttons.create + "</button><a href='#' class='close'>" + Locale.ui.page.editor.buttons.cancel + "</a>"
        },
        create: "<div class='create-button'><button>" + Locale.ui.page.editor.buttons.new + "</button></div>",
        value_types: {
            input: '<input type="text" name="{name}" value="{value}" />',
            textarea: '<textarea name="{name}">{value}</textarea>'
        },
        field: "<div class='field'><div class='title'>{title}:</div><div class='input'><input type='text' name='{name}' placeholder='{placeholder}' value='{value}' /></div></div>",
        info: {
            "id": {
                type: "input"
            },
            "title": {
                type: "input"
            },
            "description": {
                type: "textarea"
            }
        }
    };
    this.systems = [];
    this.services = {
        service: {},
        appserver: {}
    };
    this.users = [];
    this.preLoad = function(callback) {
        var o = this,
            counter = 0,
            total_queries = 2;
        ERPScanAPI.query("/engine/dictionary/users", function(result) {
            $("listusers user", result).each(function() {
                var $this = $(this),
                    userID = $this.find("id").text(),
                    user = {
                        login: $this.find("login").text()
                    };
                o.users[userID] = user;
            });
            counter++;
            if (counter == total_queries) {
                callback.call(o);
            }
        });
        $.ajax({
            url: "/data/services.xml",
            method: "GET",
            dataType: "XML",
            success: function(result) {
                $("services > module", result).each(function() {
                    var $this = $(this);
                    o.systems[$this.attr("id")] = $this.attr("name");
                });
                $("services > appserver", result).each(function() {
                    var $this = $(this);
                    o.services.appserver[$this.attr("id")] = {
                        name: $this.attr("name"),
                        versions: $this.attr("versions"),
                        has_instance: $this.attr("has_instance"),
                        services: []
                    };
                });
                $("services > service", result).each(function() {
                    var $this = $(this);
                    var base = $this.find("base").text().split(",");
                    var service_obj = {
                        name: base[3],
                        appserver: base[1],
                        isClientable: base[4],
                        isAlwaysAnonim: base[5],
                        isAlwaysNotAnonim: base[6],
                        userCanChangePort: base[7],
                        port: base[8],
                        isGateway: base[9],
                        WebRfcPort: base[10]
                    };
                    o.services.appserver[base[1]].services.push(service_obj);
                    o.services.service[$this.attr("id")] = service_obj;
                });
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
    };
    this.conversionCreds = function(name) {
        var o = this,
            tree,
            systemsTree = [];
        ERPScanAPI.query("/engine/landscapes/list", function(result) {
            var $this = $("listlandscapes > landscape", result).eq(0);
            tree = JSON.parse($this.find("data").text());
            var systemsTreeAuxiliary = {};
            for (var i = 0; i < tree.length; i++) {
                var customdata = JSON.parse(tree[i]["customdata"]);
                var settings = JSON.parse(customdata["settings"]);
                var index;
                var cred_id = "";
                if (!systemsTreeAuxiliary[settings.sid]) {
                    index = systemsTree.push({
                        text: '<span class="name">' + settings.sid + '</span> <a href=\"#\" class=\"rename-sid\"></a>',
                        data: {
                            info: settings.sid + ""
                        }
                    });
                    systemsTreeAuxiliary[settings.sid] = {
                        index: index - 1,
                        ip: {}
                    };
                }
                var sid = systemsTreeAuxiliary[settings.sid];
                if (!sid.ip[settings.ip]) {
                    systemsTree[sid.index].expanded = true;
                    if (typeof systemsTree[sid.index].nodes == "undefined") {
                        systemsTree[sid.index].nodes = [];
                    }
                    index = systemsTree[sid.index].nodes.push({
                        text: '<span class="name">' + settings.ip + '</span> <a href=\"#\" class=\"setting-ip\"></a>',
                        data: {
                            info: settings.ip + ""
                        }
                    });
                    sid.ip[settings.ip] = {
                        index: index - 1,
                        appserver: {}
                    };
                }
                var ip = sid.ip[settings.ip];
                var instance = "<span class='instance'></span>";
                if (settings.realInstance != -1) {
                    instance = " [<span class='instance'>" + (settings.realInstance < 10 ? "0" + settings.realInstance : settings.realInstance) + "</span>] ";
                }
                if (!ip.appserver[settings.appserver + instance]) {
                    systemsTree[sid.index].nodes[ip.index].expanded = true;
                    if (typeof systemsTree[sid.index].nodes[ip.index].nodes == "undefined") {
                        systemsTree[sid.index].nodes[ip.index].nodes = [];
                    }
                    index = systemsTree[sid.index].nodes[ip.index].nodes.push({
                        text: "<span class='name'>" + o.services.appserver[settings.appserver].name + "</span>" + instance + "<a href=\"#\" class=\"test-connect\"></a> <a href=\"#\" class=\"setting-appserver\"></a>",
                        data: {
                            info: [
                                settings.appserver,
                                o.services.appserver[settings.appserver].name,
                                instance * 1
                            ].join(",")
                        }
                    });
                    ip.appserver[settings.appserver + instance] = {
                        index: index - 1,
                        service: {},
                        clients: {}
                    };
                }
                var appserver = ip.appserver[settings.appserver + instance];
                if (!appserver.service[settings.service]) {
                    systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].expanded = true;
                    if (typeof systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes == "undefined") {
                        systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes = [
                            {
                                text: "Services",
                                nodes: []
                            }
                        ];
                    }
                    systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[0].expanded = true;
                    index = systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[0].nodes.push({
                        text: o.services.service[settings.service].name + " [ " + tree[i].port + " ] <a href=\"#\" class=\"setting-service\"></a>",
                        data: {
                            info: [
                                o.services.service[settings.service].name,
                                tree[i]["port"],
                                tree[i]["protocolversion"],
                                settings["anonim"] == "true" ? 1 : 0
                            ].join(",")
                        }
                    });
                    appserver.service[settings.service] = {
                        index: index - 1
                    };
                } else {
                    systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[0].nodes[appserver.service[settings.service].index].info += "," + cred_id;
                }
                if (customdata.clientid && typeof appserver.clients[customdata.clientid] == "undefined") {
                    if (!systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1]) {
                        systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1] = {
                            text: "Clients",
                            nodes: []
                        };
                    }
                    var client = "Client: " + customdata.clientid;
                    systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1].expanded = true;
                    if (typeof systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1].nodes == "undefined") {
                        systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1].nodes = [];
                    }
                    systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[1].nodes.push({
                        text: client,
                        data: {
                            info: cred_id
                        }
                    });
                    appserver.clients[customdata.clientid] = true;
                }
            }
        }, {}, "", false);
        return systemsTree;
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", ".blocks table.tablesorter tbody tr", function() {
            var $this = $(this);
            var id = $this.find("td").eq(o.table.getColumnNumber("id")).text();
            o.editorShow.call(o, id);
            return false;
        });
    };
    this.editorShow = function(id) {
        id = id.encodeHTML();
        var systemsTree = this.conversionCreds(id),
            fields = {
                "id": "ams1",
                "title": "Landscape 1",
                "descriotion": "HQ and Finance department"
            };
        for (var field in this.templates.info) {
            var field_obj = this.templates.info[field],
                type = field_obj.type,
                value = fields[field];
            fields += this.templates.field.format({
                title: Locale.ui.page["editor"]["fields"][field],
                name: this.templates.value_types[type].format({
                    name: field,
                    value: value
                })
            });
        }
        var creds = "<button class='default add-credentials'>Add Credentials</button>";
        $(".blocks .block.editor .save-button").remove();
        $(".blocks .block.editor h3").before(this.templates.save);
        $(".blocks .block.editor p.no_select").after(this.templates.editor.format({
            tree: "",
            parameters: ";;"
        })).remove();
        this.treeObj = new Tree($(".blocks .block.editor .tree-block"), systemsTree, { expandWhenClick: true });
        this.treeObj.init();

    };
    this.loadLandscapes = function() {
        this.data = Helpers.loadJSON("/data/landscape.json");
        var params = {
            title: "List",
            data: Helpers.objToArray(this.data, { key_add: true, key_field_name: "id" }),
            option: {
                expandable: false,
                select: true,
                fields: "landscape",
                ticks: {
                    'id': 0,
                    'title': 1,
                    'description': 2
                },
                sort_default: { column: 0, order: "asc" },
                count_one_page: 50,
            }
        };
        this.table = new Table(params);
        var html = this.table.getHTML(),
            section = $('section.content.landscape'),
            blocks = section.find('.blocks'),
            table = blocks.find(".table");
        table.remove();
        blocks.append(html);
        table.find(".actions-bar").prepend(this.templates.create);
        if (Helpers.objIsEmpty(this.data)) {
            table.find("table.tablesorter").after(this.templates.no_entry);
            table.find("table.tablesorter, .blocks .table .pager").remove();
            return;
        }
        this.table.build();
        Helpers.setBlockHeight(section);
    };
    this.init = function() {
        $("section.content").html(this.templates.wrap.format({
            editor: this.templates.default
        })).addClass("landscape");
        this.preLoad(this.loadLandscapes);
        this.handlers();
    };
}