Page.Render = {
    init: function() {
        var fields = ["assets", "credentials"],
            locale = "en";
        Locale.set(locale, { fields: fields });
        Locale.loadPage([Page.path.module, Page.path.page, Page.path.tab]);
        Header.load([Page.path.module, Page.path.page, Page.path.tab], { tabs: true });
        new AssetManager().init();
    }
};

function AssetManager() {
    this.objects = {};
    this.entityNameTransformation = {
        "DB Name": "dbname",
            "Portal": "site_name",
            "Client": "client"
    };
    this.buildView = function() {
        $(this.section).html(this.data.templates.general.wrap);
    };
    this.init = function() {
        $(Page.section).html(Header.templates.tabs.loading);
        var o = this;
        this.preLoad({
            entities: ["appservers", "connectionmethods", "appserverproperties"],
            dictionaries: AssetManager.List.dictionaryFields,
            loadedCallback: function () {
                for (var appserver in o.data.appservers) {
                    for (var i = 0; i < o.data.appservers[appserver].properties.length; i++) {
                        var propertyName = o.data.appservers[appserver].properties[i].name;
                        o.data.appservers[appserver].properties[i].name = o.entityNameTransformation[propertyName];
                    }
                }
                for (var item in o.data.appserverproperties) {
                    o.data.appserverproperties[item].name = o.entityNameTransformation[o.data.appserverproperties[item].name];
                }
                o.buildView();
                o.objects.list = new AssetManager.List(o);
                o.objects.list.load();
            }
        });
    }
}

AssetManager.prototype = Object.create(AbstractPage.prototype);
AssetManager.prototype.constructor = AssetManager;

AssetManager.List = function(__parent) {
    this.__parent = __parent;
    this.objects = {};
    this.item = null;
    this.conversionFieldRules = {
        "__addition": function(fields, name) {
            return "<a href=''></a>";
        },
        "severity": function(fields, name)  {
            return fields[name];
        },
        "sid":  function(fields, name) {
            return fields[name];
        },
        "class":  function(fields, name) {
            return fields[name];
        },
        "type": function(fields, name) {
                return fields[name];
            }
    };
    this.show = function(data) {
        var params = {
            title: "",
            data: data,
            option: {
                namespace: "assets",
                type: "asset",
                sort_default: { column: 0, order: "asc" },
                expandable: false
            }
        };
        this.objects.table = new Table(params);
        var html = this.objects.table.getHTML(),
            $table = $(html),
            $block = $("section.content .blocks .block.table .block-content");
        $block.attr("id", $table.attr("id")).html($table.find(".actions-bar").getWrap() + $table.find(".tablesorter").getWrap());
        this.objects.table.build();
        this.handlers();
        Helpers.setBlockHeight($(Page.section));
    };
    this.load = function() {
        var o = this;
        ERPScan.API.query("assets", "list", {
            callback: function(result) {
                result = o.conversionFields.call(o, result["data"]["result"], o.conversionFieldRules);
                o.show.call(o, result);
            }
        });
    };
    this.selectItem = function(o) {
        var $this = $(this),
            number = $this.data("number");
        $(".block.editor h3").html(o.__parent.data.texts["general"]["wrap"]["editingAsset"]);
        $(".blocks .block.table table.tablesorter tbody tr").removeClass("active");
        $this.addClass("active");
        o.item = new AssetManager.Item(o.objects.table.params.data[number], o.__parent);
        o.item.load();
        return false;
    };
    this.createItem = function(o) {
        if ($(".block.editor .buttons button.cancel").length > 0) {
            return false;
        }
        $(".blocks .block.table table.tablesorter tbody tr.active").removeClass("active");
        $(".block.editor h3").html(o.__parent.data.texts["general"]["wrap"]["creatingAsset"]);
        new AssetManager.Item(null, o.__parent).load();
        $(".block.editor .buttons button.save").text(o.__parent.data.texts["item"]["actionButtons"]["create"]).removeClass("save").addClass("create");
        $(".block.editor .buttons button.delete").text(o.__parent.data.texts["item"]["actionButtons"]["cancel"]).removeClass("delete").addClass("cancel");
        o.item = null;
        return false;
    };
    this.handlers = function() {
        var o = this;
        this.__parent.addHandlers({
            "AssetManager.List.selectItem": {
                action: "click",
                selector: ".block.assets-list table.tablesorter tbody tr",
                callback: function() {
                    o.selectItem.call(this, o);
                }
            },
            "AssetManager.List.createItem": {
                action: "click",
                selector: ".block.assets-list .new-asset, .block.editor .new-asset",
                callback: function() {
                    o.createItem(o);
                    return false;
                }
            }
        });
    }
};

AssetManager.List.dictionaryFields = {
    "asset": ["severity", "class", "type"]
};

AssetManager.List.prototype = Object.create(AbstractList.prototype);
AssetManager.List.prototype.constructor = AssetManager.List;

AssetManager.Item = function(item, __parent) {
    this.__parent = __parent;
    this.__sharedFields = ["__parent", "data", "objects", "settings", "utility"];
    this.data = {
        item: item || {},
        defaultSettings: {},
        templates: __parent.data.templates["item"],
        texts: __parent.data.texts["item"],
        fields: __parent.data.fields,
        dictionaries: __parent.data.dictionaries
    };
    this.objects = {};
    this.settings = {
        SIDDefault: "UNKNOWN",
        RPSDefault: "—",
        instanceDefault: "00",
        monitoringTabTempContent: this.__parent.data.texts["general"]["underConstruction"],
        forms: {
            common: {
                selector: "#assetEditor",
                fieldsSet: "asset",
                errorsValidation: this.data.texts["commonInfoBlock"]["validation"]
            },
            host: {
                selector: "#assetTreeAddingHost",
                fieldsSet: "host",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            appServer: {
                selector: "#assetTreeAddingAppServer",
                fieldsSet: "appServer",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            service: {
                selector: "#assetTreeAddingService",
                fieldsSet: "service",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            client: {
                selector: "#assetTreeAddingClient",
                fieldsSet: "client",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            instance: {
                selector: "#assetTreeSettingAppServerInstance",
                fieldsSet: "instance",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            dbname: {
                selector: "#assetTreeAddingDatabase",
                fieldsSet: "dbname",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            site_name: {
                selector: "#assetTreeAddingPortal",
                fieldsSet: "site_name",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            servicePort: {
                selector: "#assetTreeSettingServicePort",
                fieldsSet: "servicePort",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
            appServerVersion: {
                selector: "#assetTreeSettingAppServerVersion",
                fieldsSet: "appServerVersion",
                errorsValidation: this.data.texts["treeBlock"]["validation"]
            },
        }
    };
    this.utility = {
        credRowCounter: 1
    };
    this.__sharingFields = function() {
        for (var childObject in AssetManager.Item.prototype) {
            this[childObject].__common = this;
            for (var i = 0; i < this.__sharedFields.length; i++) {
                this[childObject][this.__sharedFields[i]] = this[this.__sharedFields[i]];
            }
        }
    };
    this.create = function ($button) {
        var o = this,
            values = this.collector.run();
        $button.addClass("disabled").text(this.data.texts["actionButtons"]["creating"]);
        ERPScan.API.query("assets", "add", {
            type: "POST",
            params: {
                data: JSON.stringify(values)
            },
            callback: function (response) {
                $button.removeClass("disabled").text("Save");
                $button.next().removeClass("cancel").addClass("delete").text("Delete");
                var listObj = o.__parent.objects.list;
                var asset = listObj.conversionFields.call(o, [ response["data"]["result"] ], listObj.conversionFieldRules),
                    index = listObj.objects.table.params.data.push(asset[0]) - 1,
                    row = Table.generateRow.call(listObj.objects.table, index),
                    $table = $(".block.assets-list table.tablesorter tbody");
                $table.prepend(row);
                listObj.objects.table.obj.trigger("update");
                $table.find("tr[data-number=\"" + index + "\"]").click();
            }
        })
    };
    this.delete = function ($button, id) {
        var o = this;
        $button.attr("disabled", "disabled").text("Deleting...");
        ERPScan.API.query("assets", "delete", {
            params: {
                id: id
            },
            callback: function () {
                $(".blocks .block.table table.tablesorter tbody tr.active").fadeOut(function () {
                    $(this).remove();
                });
                $(".blocks .block.editor .block-content").html(o.__parent.data.texts["general"]["wrap"]["assetSelectOrCreate"]);
            }
        })
    };
    this.save = function ($button) {
        var values = this.collector.run(),
            o = this;
        $button.addClass("disabled").text(this.data.texts["actionButtons"]["saving"]);
        ERPScan.API.query("assets", "edit", {
            type: "POST",
            params: {
                data: JSON.stringify(values)
            },
            callback: function (response) {
                $button.removeClass("disabled").text("Save");
                var listObj = o.__parent.objects.list,
                    asset = listObj.conversionFields.call(o, [ response["data"]["result"] ], listObj.conversionFieldRules),
                    $trSelected = $(".block.assets-list table.tablesorter tbody tr.active"),
                    trNumber = $trSelected.attr("data-number");
                listObj.objects.table.params.data[trNumber] = asset[0];
                var row = Table.generateRow.call(listObj.objects.table, trNumber);
                $trSelected.after(row).next("tr").click();
                $trSelected.remove();
                listObj.objects.table.obj.trigger("update");
            }
        })
    };
    this.createTabs = function(appServersTree) {
        this.objects["tabs"] = new Tabs({
            container: $(".block.editor .block-content"),
            locale: {
                info: this.data.texts["editorForm"]["infoTab"],
                appserversTree: this.data.texts["editorForm"]["treeTab"],
                creds: this.data.texts["editorForm"]["credentialsTab"],
                monitoring: this.data.texts["editorForm"]["monitoringTab"]
            },
            tabs: {
                info: this.data.templates["commonInfoBlock"]["wrap"].format({ content: this.commonGetInfo(this.data.item) }),
                appserversTree: this.data.templates["treeBlock"]["wrap"].format({ content: appServersTree.tree.length == 0 ? "" : "" }),
                creds: this.data.templates["credentialsBlock"]["general"],
                monitoring: this.settings.monitoringTabTempContent
            }
        });
    };
    this.load = function() {
        this.__sharingFields();
        var o = this,
            isNewAsset = Helpers.objIsEmpty(this.data.item),
            mode = isNewAsset ? "add" : "edit";
        if (!isNewAsset && this.data.item["structure"] != null) {
            this.tree.fillSIDs();
        } else {
            this.data.item["rps"] = this.settings.RPSDefault;
            this.data.item["structure"] = [ { "sid": this.settings.SIDDefault } ];
        }
        var appServersTree = this.tree.createStructure(this.data.item["structure"]);
        this.tree.addNewElementLinks(appServersTree.tree[0], appServersTree.tree[0].data.type);
        this.createTabs(appServersTree);
        $(".blocks .block.editor .block-content").html(this.data.templates["editorForm"].format({
            mode: mode,
            content: this.objects["tabs"].getHTML(),
            buttons: this.data.templates["actionButtons"]
        }));
        this.objects["tabs"].build();
        this.tree.build(appServersTree);
        this.credentials.build();
        this.__parent.createFormValidator(this.settings.forms.common);
        this.handlers();
    };
    this.commonGetInfo = function(asset) {
        var fields = "";
        for (var field in this.data.fields["asset"]) {
            var fieldObj = this.data.fields["asset"][field],
                fieldType = fieldObj["fieldType"],
                value = "";
            switch (fieldType) {
                case "select":
                    value = this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                        number: 0,
                        text: this.data.texts["commonInfoBlock"]["selectFirstValue"].format({
                            field: this.data.texts["commonInfoBlock"]["fields"][field]
                        })
                    });
                    for (var i = 0; i < this.data.dictionaries["asset"][field].length; i++) {
                        var option = this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                            number: i + 1,
                            text: this.data.dictionaries["asset"][field][i]
                        });
                        if (this.data.dictionaries["asset"][field][i] == asset[field]) {
                            option = $(option).attr("selected", "selected").getWrap();
                        }
                        value += option;
                    }
                    fieldType = "select";
                    break;
                default:
                    value = asset[field];
                    break;
            }
            fields += this.data.templates["commonInfoBlock"]["field"].format({
                name: this.data.texts["commonInfoBlock"]["fields"][field],
                value: this.data.templates["commonInfoBlock"]["valueTypes"][fieldType].format({
                    name: field,
                    value: value || ""
                })
            });
        }
        return fields;
    };
    this.changeSID = function(o) {
        var $this = $(this),
            $item = $(".block.editor .block-content > div[data-name=\"appserversTree\"] .tree > .tree-item.level-1"),
            sid = $this.val();
        if (sid == "") {
            sid = this.settings.SIDDefault;
        }
        $item.attr("data-value", sid).attr("data-id", sid).find("> .tree-item-content > .text").text(sid);
    };
    this.handlers = function () {
        var o = this;
        this.__parent.addHandlers({
            "AssetManager.Item.changeSID": {
                action: "change",
                selector: ".block.editor div[data-name=\"info\"] input[name=\"sid\"]",
                callback: function() {
                    o.changeSID.call(this, o);
                }
            },
            "AssetManager.Item.delete": {
                action: "click",
                selector: ".block.editor .buttons button.delete",
                callback: function() {
                    var $this = $(this),
                        rowNumber = $(".blocks .block.table table.tablesorter tbody tr.active").data("number"),
                        assetId = o.__parent.objects.list.objects.table.params.data[rowNumber].id;
                    new PopupConfirm({
                        title: o.data.texts["confirmations"]["delete"]["title"],
                        text: o.data.texts["confirmations"]["delete"]["content"],
                        callback: function () {
                            o.delete($this, assetId);
                        }
                    }).show();
                }
            },
            "AssetManager.Item.cancelCreating": {
                action: "click",
                selector: ".block.editor .buttons button.cancel",
                callback: function() {
                    new PopupConfirm({
                        title: o.data.texts["confirmations"]["cancel"]["title"],
                        text: o.data.texts["confirmations"]["cancel"]["content"],
                        callback: function () {
                            $(".blocks .block.editor .block-content").html(o.__parent.data.texts["general"]["wrap"]["assetSelectOrCreate"]);
                            $(".blocks .block.editor > h3").html(o.__parent.data.texts["general"]["wrap"]["assetEditor"]);
                        }
                    }).show();
                }
            },
            "AssetManager.Item.save": {
                action: "submit",
                selector: "#assetEditor",
                callback: function() {
                    var action = o.__parent.objects.list.item == null ? "create" : "save";
                    o[action].call(o, $(".block.editor .buttons button[class=\"{action}\"]".format({ action: action })));
                    return false;
                }
            }
        });
    };
};

AssetManager.Item.prototype.collector = {
    data: null,
    commonInfo: function ($common, assetObj) {
        assetObj.sid = $common.find("input[name=\"sid\"]").val();
        assetObj.severity = $common.find("input[name=\"severity\"]").val();
        assetObj.description = $common.find("textarea[name=\"description\"]").val();
        assetObj.owner = $common.find("input[name=\"owner\"]").val();
        assetObj.class = $common.find("select[name=\"class\"] option:selected").text();
        assetObj.type = $common.find("select[name=\"type\"] option:selected").text();
        return assetObj;
    },
    structure: function ($structure, assetObj, assetStructureItem) {
        var o = this;
        $structure.each(function () {
            var $this = $(this);
            assetStructureItem["host"] = $this.data("id");
            assetStructureItem["os"] = "Windows NT";
            if ($this.find("> .nodes > .tree-item[data-type]").length > 0) {
                $this.find("> .nodes > .tree-item[data-type]").each(function () {
                    var $this = $(this);
                    assetStructureItem["appServer"] = $this.data("id");
                    assetStructureItem["appServerVersion"] = $this.attr("data-version");
                    assetStructureItem["instance"] = $this.attr("data-instance");
                    $this.find("> .nodes > .tree-item[data-type]").each(function () {
                        var $this = $(this);
                        if ($this.find("> .nodes > .tree-item[data-type]").length > 0) {
                            $this.find("> .nodes > .tree-item[data-type]").each(function () {
                                var $this = $(this),
                                    type = $this.data("type");
                                if (type == "service") {
                                    assetStructureItem["service"] = $this.data("id");
                                    assetStructureItem["port"] = $this.attr("data-port") * 1;
                                    assetStructureItem["appServerProperty"] = null;
                                    assetStructureItem["appServerPropertyValue"] = null;
                                    assetStructureItem["selected"] = $this.attr("data-selected") == 1;
                                    assetObj.structure.push(Helpers.cloneObject(assetStructureItem));
                                    delete assetStructureItem["service"];
                                    delete assetStructureItem["port"];
                                } else {
                                    assetStructureItem["appServerProperty"] = $this.data("id");
                                    assetStructureItem["appServerPropertyValue"] = $this.data("value");
                                    assetStructureItem["service"] = null;
                                    assetObj.structure.push(Helpers.cloneObject(assetStructureItem));
                                    delete assetStructureItem["appServerProperty"];
                                    delete assetStructureItem["appServerPropertyValue"];
                                }
                            })
                        }
                    })
                })
            }
        });
        return assetObj;
    },
    credentials: function ($connections, assetObj) {
        $connections.each(function () {
            var $this = $(this),
                fields = {
                    target: $this.find("td.appserver-cell span[data-name=\"appserver\"]").data("value"),
                    targetVersion: $this.find("td.appserver-cell span[data-name=\"appserverVersion\"]").data("value"),
                    connectionMethod: $this.find("td.method-cell span[data-name=\"connectionMethod\"]").data("value"),
                    appServerParams: null,
                    connectorParams: null
                },
                $appServerParams = $this.find("td.connector_params-cell span[data-source=\"appServer\"]"),
                $connectorParams = $this.find("td.connector_params-cell span[data-source=\"connector\"]");
            if ($appServerParams.length > 0) {
                fields.appServerParams = [];
                $appServerParams.each(function () {
                    $this = $(this);
                    fields.appServerParams.push({
                        id: $this.data("id"),
                        value: $this.data("value")
                    });
                });
            }
            if ($connectorParams.length > 0) {
                fields.connectorParams = [];
                $connectorParams.each(function () {
                    $this = $(this);
                    fields.connectorParams.push({
                        id: $this.data("id"),
                        name: $this.data("name"),
                        value: $this.data("value")
                    });
                });
            }
            assetObj.connections.push(fields);
        });
        return assetObj;
    },
    run: function () {
        var $rowActive = $(".blocks .block.table table.tablesorter tbody tr.active"),
            rowNumber = $rowActive.length > 0 ? $rowActive.data("number") : null,
            assetId = $rowActive.length > 0 ? this.__parent.objects.list.objects.table.params.data[rowNumber].id : 0,
            $editor = $(".blocks .block.editor"),
            $common = $editor.find(".block-content > div[data-name=\"info\"]"),
            $structure = $editor.find(".block-content > div[data-name=\"appserversTree\"] .tree > .tree-item > .nodes > .tree-item[data-type]"),
            $connections = $editor.find(".block-content > div[data-name=\"creds\"] table.tablesorter tbody tr"),
            assetObj = {
                structure: [],
                connections: []
            };
        if ($(".block.editor .block-content .buttons button.create").length > 0) {
            assetObj.id = null;
        } else if ($rowActive.length > 0) {
            assetObj.id = assetId;
        }
        assetObj = this.commonInfo($common, assetObj);
        assetObj = this.structure($structure, assetObj, { });
        assetObj = this.credentials($connections, assetObj, { });
        return assetObj;
    }
};

AssetManager.Item.prototype.tree = {
    data: null,
    build: function(appServersTree) {
        var o = this;
        this.objects["tree"] = new Tree($(".block.editor .block-content > div[data-name=\"appserversTree\"] .tree-block"), appServersTree.tree, {
            expandWhenClick: true,
            with_filters: true,
            customChecking: true
        });
        this.objects["tree"].init();
        this.objects["tree"].expandAll();
        this.toggleStateSelected("hide");
        this.checkSelected();
        this.objects["tree"].lastNodesDetect($(".tree-block .tree"));
        $(".block-content-tab.appserversTree .tree-block .tree-item.level-5").each(function() {
            var $this = $(this);
            if ($this.data("type") == "service") {
                $this.find("> .tree-item-content").append(o.data.templates["treeBlock"]["servicePort"].format({
                    port: $this.data("port")
                }));
            }
        });
        $(".block-content-tab.appserversTree .tree-block .tree-item.level-3[data-type]").each(function() {
            var $this = $(this),
                appServerId = $this.data("id"),
                appServerObj = o.__parent.data.appservers[appServerId],
                appServerVersionId = $this.data("version"),
                appServerVersionObj = o.getAppserverVersionName(appServerObj, $this.data("version"));
            if (appServerVersionId != 0) {
                $this.find("> .tree-item-content").append(o.data.templates["treeBlock"]["appServerVersion"].format({
                    version: appServerVersionObj,
                    versionId: appServerVersionId
                }));
            }
        });
        var filters = this.objects["tree"].getFilters(),
            skipFilters = ["services", "clients", "sid", "dbnames", "sitenames"];
        for (var filter in filters) {
            if (skipFilters.indexOf(filter) != -1) {
                continue;
            }
            var filter_obj = new Filter(filter, {
                fields: {
                    namespace: "assets",
                    type: "asset"
                },
                variants: filters[filter],
                callback: function(filter) {
                    return function(values) {
                        o.objects.tree.filtering(filter, values);
                    }
                }(filter)
            });
            $(".block-content > div[data-name=\"appserversTree\"] .filters").append(filter_obj.getHTML());
            filter_obj.build();
        }
        $(".block-content > div[data-name=\"appserversTree\"] .filters").prepend(this.data.templates["treeBlock"].resetSatesDefault + this.data.templates["treeBlock"].viewSwitch);
        this.handlers();
        $.formUtils.addValidator({
            name: "nodeAssetTreeAlreadyExist",
            validatorFunction: function(value, $el, config, language, $form) {
                return o.checkAlreadyAddedElement.call(o, $el, value);
            }
        });
    },
    fillSIDs: function() {
        if (this.data.item["structure"].length > 0) {
            for (var i = 0; i < this.data.item["structure"].length; i++) {
                this.data.item["structure"][i]["sid"] = this.data.item["sid"];
            }
        } else {
            this.data.item["structure"][0] = {
                "sid": this.data.item["sid"]
            }
        }
    },
    newElementControlsHandling: {
        "host": function () {
            return this.data.templates["treeBlock"]["newElementControls"]["host"];
        },
        "appServer": function () {
            var template = this.data.templates["treeBlock"]["newElementControls"]["appServer"],
                values = "";
            for (var appServer in this.__parent.data.appservers) {
                values += this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                    number: appServer,
                    text: this.__parent.data.appservers[appServer].name
                })
            }
            return template.format({
                appServers: values
            })
        },
        "service": function ($element) {
            var template = this.data.templates["treeBlock"]["newElementControls"]["service"],
                values = "",
                appServerId = $element.parent(".text").parent(".tree-item-content").parent(".tree-item").parent(".nodes").parent(".tree-item").data("id");
            for (var service in this.__parent.data.appservers[appServerId]["services"]) {
                values += this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                    number: service,
                    text: this.__parent.data.appservers[appServerId]["services"][service].name
                })
            }
            return template.format({
                services: values
            })
        },
        "client": function () {
            return this.data.templates["treeBlock"]["newElementControls"]["client"];
        },
        "dbname": function () {
            return this.data.templates["treeBlock"]["newElementControls"]["dbname"];
        },
        "site_name": function () {
            return this.data.templates["treeBlock"]["newElementControls"]["site_name"];
        }
    },
    addingElements: {
        "host": function ($element, type, $fields) {
            var text = $fields.find("input[name=\"host\"]").val(),
                $itemContent = $element.parent(".text").parent(".tree-item-content");
            var $newItem = this.objects.tree.addItemAfter($itemContent, {
                text: text + this.data.templates["treeBlock"]["removeElement"],
                data: {
                    value: text,
                    type: type,
                    id: text,
                    selected: 0
                },
                checkbox: true,
                level: this.getLevelNumberTree(type)
            });
            var $checkbox = $newItem.find("> .tree-item-content > input[type=\"checkbox\"]");
            $checkbox[0].disabled = "disabled";
            return {
                text: this.data.templates["treeBlock"].newElement.format({
                    type: "appServer"
                }).localize({
                    type: this.data.texts["treeBlock"]["newElement"]["types"]["appServer"]
                }),
                id: null,
                $newItem: $newItem
            }
        },
        "appServer": function ($element, type, $fields) {
            var appServer = $fields.find("select[name=\"appServer\"] option:selected"),
                appServerId = appServer.val(),
                appServerVersionId = $fields.find("select[name=\"appServerVersion\"]").val() || null,
                instance = $fields.find("input[name=\"instance\"]").val(),
                $itemContent = $element.parent(".text").parent(".tree-item-content");
            var appServerObj = this.__parent.data.appservers[appServerId],
                $appserver = this.objects.tree.addItemBefore($itemContent, {
                    text: appServerObj.name,
                    data: {
                        value: appServerObj.name,
                        type: "appServer",
                        version: appServerVersionId != null ? appServerVersionId : "",
                        instance: appServerObj["instantiable"] ? instance : null,
                        id: appServerId,
                        selected: 0
                    },
                    checkbox: true,
                    expandable: true,
                    level: 3
                });
            if (appServerVersionId != null) {
                var appServerVersionName = $fields.find("select[name=\"appServerVersion\"] option:selected").text();
                $appserver.find("> .tree-item-content > .text").append(this.data.templates["treeBlock"].appServerVersion.format({
                    version: appServerVersionName,
                    versionId: appServerVersionId
                }));
            };
            if (appServerObj["instantiable"]) {
                $appserver.find("> .tree-item-content > .text").append(this.data.templates["treeBlock"].appServerInstance.format({
                    instance: instance
                }));
            }
            $appserver.find("> .tree-item-content > .text").append(this.data.templates["treeBlock"]["removeElement"]);
            var $checkbox = $appserver.find("> .tree-item-content > input[type=\"checkbox\"]");
            $checkbox[0].disabled = "disabled";
            return {
                text: this.data.templates["treeBlock"].newElement.format({
                    type: "service"
                }).localize({
                    type: this.data.texts["treeBlock"]["newElement"]["types"]["service"]
                }),
                id: appServerId,
                $newItem: $appserver
            }
        },
        "entity": function ($element, type, $fields, text, id, entity) {
            var $itemContent = $element.parent(".text").parent(".tree-item-content"),
                groupName = typeof this.entityGroups[entity] == "undefined" ? "services" : this.entityGroups[entity],
                $entities = $itemContent.parent(".tree-item").parent(".nodes").find("> .tree-item[data-type=\"" + groupName + "\"]"),
                level = this.getLevelNumberTree("entity");
            if ($entities.length == 0) {
                $entities = this.objects.tree.addItemAfter($itemContent, {
                    text: this.__parent.data.texts["item"]["treeBlock"]["groups"][groupName],
                    data: {
                        value: groupName,
                        type: groupName,
                        id: groupName,
                        selected: 1
                    },
                    checkbox: true,
                    level: level
                });
                var $checkbox = $entities.find("> .tree-item-content > input[type=\"checkbox\"]");
            }
            var $entity = this.objects.tree.addItemAppend($entities.find("> .tree-item-content"), {
                text: text,
                data: {
                    value: text,
                    type: "service",
                    id: id,
                    selected: 1
                },
                checkbox: true,
                expandable: false,
                level: 5
            }).find("> .tree-item").last();
            this.objects.tree.mark($entity);
            $entity.find("> .tree-item-content > input[type=\"checkbox\"]")[0].checked = true;
            var $text = $entity.find("> .tree-item-content > .text");
            if (entity == "service") {
                var $servicePort = $fields.find("input[name=\"port\"]"),
                    servicePort = $servicePort.val();
                $text.append(this.data.templates["treeBlock"][$servicePort.attr("disabled") ? "servicePortDisabled" : "servicePort"].format({
                    port: servicePort
                }) + this.data.templates["treeBlock"]["settingElement"] + this.data.templates["treeBlock"]["removeElement"]);
                $entity.attr("data-portmask", $servicePort.attr("data-portmask"));
                $entity.attr("data-port", servicePort);
                $text.find("input[name=\"port\"]").attr("disabled", "disabled");
            } else {
                $text.append(this.data.templates["treeBlock"]["removeElement"]);
            }
            $entity.parents(".tree-item").each(function() {
                var $checkbox = $(this).find("> .tree-item-content > input[type=\"checkbox\"]");
                if ($checkbox.length != 0) {
                    $checkbox[0].disabled = false;
                }
            })
            return {
                text: text,
                id: id,
                $newItem: $entities
            }
        },
        "service": function ($element, type, $fields) {
            var option = $fields.find("select[name=\"service\"] option:selected"),
                text = option.text(),
                id = option.val();
            if (id == 0) {
                return null;
            }
            return this.addingElements.entity.call(this, $element, type, $fields, text, id, "service");
        },
        "site_name": function ($element, type, $fields) {
            var value = $fields.find("input[name=\"site_name\"]").val();
            if (value == "") {
                return null;
            }
            return this.addingElements.entity.call(this, $element, type, $fields, value, this.getAppServerPropertyUIDByName("site_name"), "site_name");
        },
        "dbname": function ($element, type, $fields) {
            var value = $fields.find("input[name=\"dbname\"]").val();
            if (value == "") {
                return null;
            }
            return this.addingElements.entity.call(this, $element, type, $fields, value, this.getAppServerPropertyUIDByName("dbname"), "dbname");
        },
        "client": function ($element, type, $fields) {
            var value = $fields.find("input[name=\"client\"]").val();
            if (value == "") {
                return null;
            }
            return this.addingElements.entity.call(this, $element, type, $fields, value, this.getAppServerPropertyUIDByName("client"), "client");
        }
    },
    hieracrhyTree: [
        {
            element: "sid"
        },
        {
            element: "host"
        },
        {
            element: "appServer",
            getNameByUID: function(credential) {
                return this.__parent.data.appservers[credential["appServer"]]["name"]
            }
        },
        {
            element: "entityGroup",
            detect: function(credential) {
                if (credential["appServerProperty"] != null) {
                    var entity = this.__parent.data.appserverproperties[credential["appServerProperty"]].name;
                    if (credential["appServerProperty"] != null) {
                        return {
                            value: this.entityGroups[entity]
                        };
                    }
                }
                return {
                    value: "services"
                };
            }
        },
        {
            element: "entity",
            detect: function(credential) {
                var o = this;
                if (credential["appServerProperty"] != null) {
                    return {
                        value: credential["appServerProperty"],
                        has_uid: true,
                        getNameByUID: function(credential) {
                            return credential["appServerPropertyValue"]
                        }
                    };
                }
                return {
                    value: credential["service"],
                    has_uid: true,
                    getNameByUID: function(credential) {
                        return o.__parent.data.appservers[credential["appServer"]]["services"][credential["service"]].name
                    }
                };
            }
        }
    ],
    removingTexts: {
        entityGroup: "Are you sure you want to disable all items in the group?",
        entity: "Are you sure you want to delete the item?"
    },
    entityGroups: {
        "dbname": "dbnames",
        "site_name": "site_names",
        "client": "clients",
        "service": "services"
    },
    checkAlreadyAddedElement: function($element, value) {
        var type = this.data["$addingElementContext"].attr("data-type"),
            $addingItem = this.data["$addingElementContext"].parent(".text").parent(".tree-item-content").parent(".tree-item"),
            $searchSource = $addingItem.parent(".nodes");
        if (this.entityGroups[type] != null) {
            $searchSource = $searchSource.find("> .tree-item[data-type=\"" + this.entityGroups[type] + "\"] > .nodes");
        }
        return $searchSource.find("> .tree-item[data-id=\"" + value + "\"], > .tree-item[data-value=\"" + value + "\"]").length == 0;
    },
    checkSelected: function () {
        var $entities = $(".tree-block .tree-item.level-5[data-selected=\"1\"]"),
            o = this;
        $entities.each(function() {
            var $this = $(this);
            o.objects.tree.mark($this);
            $this.find("> .tree-item-content > input[type=\"checkbox\"]")[0].checked = true;
        })
    },
    getAppServerPropertyUIDByName: function(name) {
        for (var appserverpropery in this.__parent.data.appserverproperties) {
            if (this.__parent.data.appserverproperties[appserverpropery].name == name) {
                return this.__parent.data.appserverproperties[appserverpropery].id;
            }
        }
        return null;
    },
    toggleStateSelected: function (state) {
        $(".tree-block .tree-item[data-type][data-selected=\"0\"]").removeClass("shown hidden").addClass(state == "hide" ? "hidden" : "shown");
        $(".tree-block .tree-item .new-element[data-type=\"service\"], .tree-block .tree-item .new-element[data-type=\"appServer\"]").each(function() {
            if (state == "hide") {
                $(this).parent(".text").parent(".tree-item-content").parent(".tree-item").removeClass("hidden").addClass("shown");
            } else {
                $(this).parent(".text").parent(".tree-item-content").parent(".tree-item").addClass("hidden").removeClass("shown");
            }
        });
        this.objects.tree.lastNodesDetect($(".tree-block .tree"));
    },
    createStructure: function(assetStructure) {
        var o = this,
            appServersTree = [],
            appServersTreeMap = {},
            hierarchyCreate = function(elements, elementsMap, cred, level) {
                var current_hierarchy = o.hieracrhyTree[level - 1];
                if (typeof cred[current_hierarchy.element] == "undefined" && current_hierarchy.element != "entityGroup" && current_hierarchy.element != "entity") {
                    return;
                }
                var hasDetectFunc = typeof current_hierarchy.detect != "undefined",
                    elementObj = hasDetectFunc ? current_hierarchy.detect.call(o, cred) : current_hierarchy,
                    credValue = hasDetectFunc ? elementObj.value : cred[elementObj.element];
                if (typeof elementsMap[credValue] == "undefined") {
                    var text = typeof elementObj.getNameByUID != "undefined" ? elementObj.getNameByUID.call(o, cred) : credValue,
                        element = {
                            text: text
                        };
                    if (current_hierarchy.element != "entityGroup") {
                        var element_type = elementObj.element;
                        if (current_hierarchy.element == "entity") {
                            if (cred["appServerProperty"] == null) {
                                element_type = null;
                            } else {
                                element_type = o.__parent.data.appserverproperties[cred["appServerProperty"]].name;
                            }
                        }
                        if (element_type == null) {
                            element_type = "service";
                        }
                        element.data = {
                            id: credValue,
                            type: element_type,
                            value: text,
                            selected: cred["selected"] ? 1 : 0
                        };
                        if (current_hierarchy.element == "appServer") {
                            var appServerObj = o.__parent.data.appservers[cred["appServer"]],
                                lastVersion = appServerObj["versions"].length == 0 ? null : appServerObj["versions"][appServerObj["versions"].length - 1];
                            element.data.version = lastVersion ? lastVersion.id : "";
                            if (lastVersion != null) {
                                element.text += o.data.templates["treeBlock"].appServerVersion.format({
                                    version: lastVersion ? lastVersion.name : "",
                                    versionId: lastVersion ? lastVersion.id : ""
                                });
                            }
                            if (appServerObj["instantiable"]) {
                                element.text += o.data.templates["treeBlock"].appServerInstance.format({
                                    instance: cred["instance"] || o.settings.instanceDefault
                                });
                                element.data.instance = cred["instance"] || o.settings.instanceDefault;
                            }
                        }
                        if (element_type == "service") {
                            var serviceObj = o.__parent.data.appservers[cred["appServer"]]["services"][cred["service"]];
                            element.data.port = cred["port"];
                            element.data.portmask = serviceObj["portMask"];
                            var isPortMutable = serviceObj["isPortMutable"];
                            element.text += o.data.templates["treeBlock"][!isPortMutable ? "servicePortDisabled" : "servicePort"].format({
                                port: cred["port"]
                            });
                        }
                        if (element_type == "service") {
                            element.text += o.data.templates["treeBlock"]["settingElement"];
                        }
                        element.checkbox = element_type != "sid";
                        if (current_hierarchy.element != "sid") {
                            element.text += o.data.templates["treeBlock"]["removeElement"];
                        }
                    } else {
                        element.data = {
                            type: element.text
                        };
                        element.checkbox = true;
                        element.text = o.data.texts["treeBlock"]["groups"][element.text];
                    }
                    elementsMap[credValue] = {
                        index: elements.push(element) - 1
                    };
                }
                if (o.hieracrhyTree.length > level) {
                    if (typeof elementsMap[credValue].nodes == "undefined") {
                        elementsMap[credValue].nodes = {};
                    }
                    if (typeof elements[elementsMap[credValue].index].nodes == "undefined") {
                        elements[elementsMap[credValue].index].nodes = [];
                    }
                    hierarchyCreate(elements[elementsMap[credValue].index].nodes, elementsMap[credValue].nodes, cred, level + 1);
                }
            };
        for (var i = 0; i < assetStructure.length; i++) {
            hierarchyCreate(appServersTree, appServersTreeMap, assetStructure[i], 1);
        }
        return {
            tree: appServersTree,
            treeMap: appServersTreeMap
        };
    },
    getNextLevelNameTree: function (currentLevelName) {
        for (var i = 0; i < this.hieracrhyTree.length; i++) {
            if (this.hieracrhyTree[i].element == currentLevelName && typeof this.hieracrhyTree[i + 1] != "undefined") {
                return this.hieracrhyTree[i + 1].element;
            }
        }
        return null;
    },
    getLevelNumberTree: function (levelName) {
        for (var i = 0; i < this.hieracrhyTree.length; i++) {
            if (this.hieracrhyTree[i].element == levelName) {
                return i;
            }
        }
        return null;
    },
    addNewElementLinks: function (appserversTree, parentType) {
        if (parentType == "entityGroup") {
            return;
        }
        var type = this.getNextLevelNameTree(parentType),
            i;
        if (appserversTree.nodes.length > 0) {
            for (i = 0; i < appserversTree.nodes.length; i++) {
                this.addNewElementLinks(appserversTree.nodes[i], type);
            }
        }
        if (type == "entityGroup") {
            appserversTree.nodes.push({
                text: this.data.templates["treeBlock"].newElement.format({
                    type: "service"
                }).localize({
                    type: this.data.texts["treeBlock"]["newElement"]["types"]["service"]
                })
            });
            var appServerProperties = this.__parent.data.appservers[appserversTree.data.id]["properties"];
            for (i = 0; i < appServerProperties.length; i++) {
                var appServerPropertyObj = appServerProperties[i];
                if (appServerPropertyObj.name == "instance") {
                    continue;
                }
                appserversTree.nodes.push({
                    text: this.data.templates["treeBlock"].newElement.format({
                        type: appServerPropertyObj.name
                    }).localize({
                        type: this.data.texts["treeBlock"]["newElement"]["types"][appServerPropertyObj.name]
                    })
                });
            }
        } else {
            appserversTree.nodes.push({
                text: this.data.templates["treeBlock"].newElement.format({
                    type: type
                }).localize({
                    type: this.data.texts["treeBlock"]["newElement"]["types"][type]
                })
            });
        }
    },
    getAppserverVersionName: function(appserverObj, versionId) {
        for (var i = 0; i < appserverObj["versions"].length; i++) {
            if (appserverObj["versions"][i]["id"] == versionId) {
                return appserverObj["versions"][i]["name"];
            }
        }
        return null;
    },
    getNextLevelPossibleItems: function (type, id) {
        if (type == "appServer") {
            var level = this.getLevelNumberTree("entityGroup"),
                entityGroups = [ {
                    element: "service",
                    level: level
                } ];
            for (var i = 0; i < this.__parent.data.appservers[id].properties.length; i++) {
                var propertyName = this.__parent.data.appservers[id].properties[i].name;
                if (propertyName == "instance") {
                    continue;
                }
                entityGroups.push({
                    element: propertyName,
                    level: level
                });
            }
            return entityGroups;
        } else {
            for (var i = 0; i < this.hieracrhyTree.length; i++) {
                if (this.hieracrhyTree[i].element == type && typeof this.hieracrhyTree[i + 1] != "undefined") {
                    return [ {
                        element: this.hieracrhyTree[i + 1].element,
                        level: this.getLevelNumberTree(this.hieracrhyTree[i + 1].element)
                    } ];
                }
            }
        }
        return null;
    },
    newTreeElementPopupAdding: function($element, type) {
        var elementObj = this.addingElements[type].call(this, $element, type, $(".popup.add-entity .field"));
        if (elementObj == null) {
            return 1;
        }
        var newPossibleItems = this.getNextLevelPossibleItems(type, elementObj.id);
        if (newPossibleItems != null && elementObj.$newItem != null) {
            for (var i = 0; i < newPossibleItems.length; i++) {
                var $itemContent = elementObj.$newItem.find("> .tree-item-content");
                this.objects.tree.addItemAppend($itemContent, {
                    text: this.data.templates["treeBlock"].newElement.format({
                        type: newPossibleItems[i].element
                    }).localize({
                        type: this.data.texts["treeBlock"]["newElement"]["types"][newPossibleItems[i].element]
                    }),
                    checkbox: false,
                    expandable: false,
                    level: newPossibleItems[i].level + 1
                });
            }
        }
        return 0;
    },
    newTreeElementPopup: function ($element) {
        var o = this,
            type = $element.data("type");
        var popupObj = new Popup({
            class: "add-entity",
            title: "Adding " + this.data.texts["treeBlock"]["newElement"]["types"][type],
            content: this.newElementControlsHandling[type].call(o, $element)
        });
        popupObj.show();
        this.__parent.createFormValidator(this.settings.forms[type]);
        this.data["$addingElementContext"] = $element;
        $(".popup.add-entity button.add-entity-cancel").on("click", function () {
            popupObj.close("add-entity");
            return false;
        });
        $(".popup.add-entity .tree-adding-element").on("submit", function () {
            var isAdded = o.newTreeElementPopupAdding.call(o, $element, type) == 0;
            if (isAdded) {
                popupObj.close("add-entity");
            }
            return false;
        });
    },
    clearEmptyGroup: function() {
        var o = this;
        $(".tree-block .tree .tree-item.level-4[data-type]").each(function () {
            var $this = $(this);
            if ($this.find("> .nodes").length == 0 || $this.find("> .nodes > .tree-item.shown").length == 0) {
                o.objects.tree.mark($this);
                $this.removeClass("shown").addClass("hidden").attr("data-selected", 0);
            }
        });
        $(".tree-block .tree .tree-item.level-3[data-type]").each(function () {
            var $this = $(this);
            if ($this.find("> .nodes").length == 0 || $this.find("> .nodes > .tree-item.shown[data-type]").length == 0) {
                o.objects.tree.mark($this);
                $this.removeClass("shown").addClass("hidden").attr("data-selected", 0);
            }
        })
    },
    setServicePort: function($link) {
        var settingServicePortPopup = new Popup({
            class: "setting-service-port",
            title: "Setting service port",
            content: this.data.templates["treeBlock"].settingPort.format({
                port: $link.text()
            }),
            width: 470,
            height: 100
        });
        settingServicePortPopup.show();
        this.__parent.createFormValidator(this.settings.forms["servicePort"]);
        $(".popup.setting-service-port button.setting-service-port-cancel").on("click", function () {
            settingServicePortPopup.close("setting-service-port");
            return false;
        });
        $(".popup.setting-service-port .tree-setting-service-port").on("submit", function () {
            var port = $(".popup.setting-service-port .field:first-child .value input[name=\"port\"]").val();
            $link.html(port);
            $link.parent(".service-port").parent(".tree-item-content").parent(".tree-item").attr("data-port", port);
            settingServicePortPopup.close("setting-service-port");
            return false;
        });
    },
    setAppserverVersion: function($link) {
        var $item = $link.parent(".appserver-version").parent(".text").parent(".tree-item-content").parent(".tree-item"),
            id = $item.data("id"),
            versions = "";
        for (var i = 0; i < this.__parent.data.appservers[id].versions.length; i++) {
            var versionObj = this.__parent.data.appservers[id].versions[i];
            versions += this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                number: versionObj.id,
                text: versionObj.name
            });
        }
        var settingAppserverVersionPopup = new Popup({
            class: "setting-appserver-version",
            title: "Setting application server version",
            content: this.data.templates["treeBlock"].settingAppserverVersion.format({
                versions: versions
            }),
            width: 470,
            height: 100,
            close: function() {
                $(".popup.setting-appserver-version .field:first-child .value input[type=\"text\"]:first-child").off("keyup");
            }
        });
        settingAppserverVersionPopup.show();
        this.__parent.createFormValidator(this.settings.forms["appServerVersion"]);
        var $popup = $(".popup.setting-appserver-version");
        $popup.find("select[name=\"appServerVersion\"] option[value=\"" + $link.attr("data-id") + "\"]")[0].selected = true;
        $popup.find("button.setting-appserver-version-cancel").on("click", function () {
            settingAppserverVersionPopup.close("setting-appserver-version");
            return false;
        });
        $popup.find(".tree-setting-appserver-version").on("submit", function () {
            var $version = $(".popup.setting-appserver-version select[name=\"appServerVersion\"]"),
                value = $version.val();
            $link.attr("data-id", value).html($version.find("option:selected").text());
            $item.attr("data-version", value);
            settingAppserverVersionPopup.close("setting-appserver-version");
            return false;
        });
    },
    setServicePortsByInstance: function($appServer, instanceOld, instance) {
        $appServer.find(".tree-item[data-type=\"service\"]").each(function() {
            var $this = $(this),
                $port = $(this).find("> .tree-item-content .service-port"),
                $portLink = $port.find(".set-service-port"),
                portMask = $this.attr("data-portmask"),
                portAccordingInstance = portMask.replace(/NN/g, instanceOld);
            if (portAccordingInstance == $portLink.text()) {
                $portLink.html(portMask.replace(/NN/g, instance));
            }
        })
    },
    setAppserverInstance: function($link) {
        var $item = $link.parent(".appserver-instance").parent(".text").parent(".tree-item-content").parent(".tree-item"),
            instance = $link.text(),
            o = this;
        var settingAppserverInstancePopup = new Popup({
            class: "setting-appserver-instance",
            title: "Setting application server instance",
            content: this.data.templates["treeBlock"].settingAppserverInstance.format({
                instance: ""
            }),
            width: 470,
            height: 100
        });
        settingAppserverInstancePopup.show();
        this.__parent.createFormValidator(this.settings.forms["instance"]);
        var $popup = $(".popup.setting-appserver-instance");
        $popup.find("input[name=\"instance\"]").val(instance);
        $popup.find("button.setting-appserver-instance-cancel").on("click", function () {
            settingAppserverInstancePopup.close("setting-appserver-instance");
            return false;
        });
        $popup.find(".tree-setting-appserver-instance").on("submit", function () {
            var $instance = $(".popup.setting-appserver-instance input[name=\"instance\"]");
            $link.html($instance.val());
            $item.attr("data-instance", $instance.val());
            o.setServicePortsByInstance($item, instance, $instance.val());
            settingAppserverInstancePopup.close("setting-appserver-instance");
            return false;
        });
    },
    showAppServerAdvancedFields: function($appservers) {
        var appServerId = $appservers.val(),
            $fieldAppServerVersions = $("#assetTreeAddingAppServer .field.appServerVersions"),
            $fieldAppServerInstance = $("#assetTreeAddingAppServer .field.appServerInstance");
        if (!appServerId) {
            $fieldAppServerVersions.addClass("hidden");
            $fieldAppServerInstance.addClass("hidden");
            return;
        }
        var appServerObj = this.__parent.data.appservers[appServerId],
            versions = appServerObj["versions"],
            values = "";
        if (versions.length == 0) {
            $fieldAppServerVersions.addClass("hidden");
            return false;
        }
        for (var i = 0; i < versions.length; i++) {
            values += this.data.templates["commonInfoBlock"]["valueTypes"]["option"].format({
                number: versions[i].id,
                text: versions[i].name
            })
        }
        var $versions = $("#assetTreeAddingAppServer").find("select[name=\"appServerVersion\"]");
        $versions.find("option[value!=\"\"]").remove();
        $versions.append(values);
        $fieldAppServerVersions.removeClass("hidden");
        if (appServerObj["instantiable"]) {
            $fieldAppServerInstance.removeClass("hidden").find("input[name=\"instance\"]").val(this.settings.instanceDefault);
        } else {
            $fieldAppServerInstance.addClass("hidden").find("input[name=\"instance\"]").val("");
        }
    },
    showServicePortInput: function($services) {
        var serviceId = $services.val(),
            $fieldServicePort = $("#assetTreeAddingService .field.servicePort"),
            $item = this.data["$addingElementContext"].parent(".text").parent(".tree-item-content").parent(".tree-item"),
            $appServerItem = $item.parent(".nodes").parent(".tree-item"),
            appServerId = $appServerItem.attr("data-id"),
            serviceObj = this.__parent.data.appservers[appServerId]["services"][serviceId],
            portDefault = serviceObj["portMask"].replace(/NN/g, $appServerItem.attr("data-instance"));
        $fieldServicePort.removeClass("hidden");
        var $input = $fieldServicePort.find("input[name=\"port\"]");
        $input.val(portDefault).attr("data-portmask", serviceObj["portMask"]);
        if (!serviceObj.isPortMutable) {
            $input.attr("disabled", "disabled");
        } else {
            $input.removeAttr("disabled");
        }
    },
    checkEmptyAndDisabling: function($item) {
        var disabledItems = 0,
            itemsNumber = $item.find("> .nodes > .tree-item[data-type]").length;
        $item.find("> .nodes > .tree-item").each(function() {
            var $this = $(this),
                $checkbox = $this.find("> .tree-item-content > input[type=\"checkbox\"]");
            if ($checkbox.length != 0 && $checkbox[0].disabled == true) {
                disabledItems++;
            }
        });
        if (itemsNumber == 0 || itemsNumber == disabledItems) {
            var $checkbox = $item.find("> .tree-item-content input[type=\"checkbox\"]");
            if ($checkbox.length > 0) {
                $checkbox[0].checked = false;
                $checkbox[0].indeterminate = false;
                $checkbox[0].disabled = true;
                this.checkEmptyAndDisabling($item.parent(".nodes").parent(".tree-item"));
            }
        }
    },
    elementRemove: function($link) {
        var $item = $link.parent(".text").parent(".tree-item-content").parent(".tree-item"),
            $parent = $item.parent(".nodes").parent(".tree-item");
        $item.remove();
        if ($parent.find("> .nodes > .tree-item").length == 0) {
            for (var group in this.entityGroups) {
                if ($parent.attr("data-type") == this.entityGroups[group]) {
                    var $nextParent = $parent.parent(".nodes").parent(".tree-item");
                    $parent.remove();
                    this.checkEmptyAndDisabling($nextParent);
                    return;
                }
            }
        }
    },
    goToCredsSetFilters: function(appServerObj, appServerVersionId, serviceObj) {
        var listObj = this.__parent.objects.list,
            tableObj = listObj.item.credentials.objects.table,
            appServerColumnNumber = tableObj.getColumnNumber("appServer"),
            connectionMethodColumnNumber = tableObj.getColumnNumber("connectionMethod"),
            $filterCells = $(".block.editor .block-content .credentianls-table .tablesorter-filter-row").find("td"),
            $appServerFilterSelect = $filterCells.eq(appServerColumnNumber).find("select"),
            $connectionMethodFilterSelect = $filterCells.eq(connectionMethodColumnNumber).find("select"),
            connectionsByEntity = this.__common.credentials.data.item["connectionsByEntity"],
            connectionMethod = connectionsByEntity[appServerObj["id"]][appServerVersionId]["services"][serviceObj["id"]],
            appServerFullName = appServerObj["name"] + " " + this.getAppserverVersionName(appServerObj, appServerVersionId),
            $appServerFilter = $appServerFilterSelect.find("option[value=\"" + appServerFullName + "\"]"),
            $connectionMethodFilter = $connectionMethodFilterSelect.find("option[value=\"" + connectionMethod + "\"]"),
            appServerExist = $appServerFilter.length > 0,
            connectionMethodExist = $connectionMethodFilter.length > 0;
        if (appServerExist) {
            $appServerFilter[0].selected = true;
            setTimeout(function() {
                $appServerFilterSelect.css({color: "#FF4500"});
                $appServerFilterSelect.animate({color: "#000000"}, 2500);
            }, 1000);
        }
        if (connectionMethodExist) {
            $connectionMethodFilter[0].selected = true;
            setTimeout(function() {
                $connectionMethodFilterSelect.css({color: "#FF4500"});
                $connectionMethodFilterSelect.animate({color: "#000000"}, 2500);
            }, 1000);
        }
    },
    goToCredsNew: function(appServerObj, appServerVersionId, serviceObj) {
        var listObj = this.__parent.objects.list,
            tableObj = listObj.item.credentials.objects.table,
            $filterCells = $(".block.editor .block-content .credentianls-table .tablesorter-filter-row").find("td"),
            appServerColumnNumber = tableObj.getColumnNumber("appServer"),
            connectionMethodColumnNumber = tableObj.getColumnNumber("connectionMethod"),
            $appServerFilterSelect = $filterCells.eq(appServerColumnNumber).find("select"),
            $connectionMethodFilterSelect = $filterCells.eq(connectionMethodColumnNumber).find("select"),
            newCred = listObj.item.credentials.newEmptyCred();
        $(".credentianls-table table.tablesorter tbody").prepend(newCred);
        $appServerFilterSelect.find("option[value=\"\"]")[0].selected = true;
        $connectionMethodFilterSelect.find("option[value=\"\"]")[0].selected = true;
        tableObj.obj.trigger('update');
        return;
    },
    goToCreds: function($item) {
        $(".block.editor .block-content .block-tabs .block-tab[data-name=\"creds\"]").click();
        var $appServer = $item.parents(".tree-item[data-type=\"appServer\"]"),
            appServerId = $appServer.attr("data-id"),
            appServerObj = this.__parent.data.appservers[appServerId],
            appServerVersionId = $appServer.attr("data-version"),
            serviceId = $item.attr("data-id"),
            serviceObj = appServerObj["services"][serviceId],
            connectionsByEntity = this.__common.credentials.data.item["connectionsByEntity"];
        if (connectionsByEntity[appServerId] == null || connectionsByEntity[appServerId][appServerVersionId] == null || connectionsByEntity[appServerId][appServerVersionId]["services"][serviceId] == null) {
            this.goToCredsNew(appServerObj, appServerVersionId, serviceObj);
        } else {
            this.goToCredsSetFilters(appServerObj, appServerVersionId, serviceObj);
        }
    },
    handlers: function () {
        var o = this;
        this.__parent.addHandlers({
            "AssetManager.Item.tree.newElement": {
                action: "click",
                selector: ".block.editor .tree .new-element",
                callback: function() {
                    var $this = $(this);
                    o.newTreeElementPopup($this);
                    return false;
                }
            },
            "AssetManager.Item.tree.switchSelected": {
                action: "change",
                selector: ".tree-block .only-selected-switch input[type=\"checkbox\"]",
                callback: function() {
                    var $this = $(this);
                    o.toggleStateSelected($this.filter(":checked").length > 0 ? "hide" : "show");
                    o.objects.tree.lastNodesDetect($(".tree-block .tree"));
                }
            },
            "AssetManager.Item.tree.check": {
                action: "click",
                selector: ".block.editor .tree-block .tree-item .tree-item-content input[type=\"checkbox\"], .tree-block .tree-item.level-5 .tree-item-content .text",
                callback: function() {
                    var $this = $(this),
                        $item = $this.parent(".tree-item-content").parent(".tree-item"),
                        $checkbox = $item.find("> .tree-item-content input[type=\"checkbox\"]"),
                        currentState = $this.is("input[type=\"checkbox\"]") ? !$checkbox[0].checked : $checkbox[0].checked;
                    $item.attr("data-selected", currentState ? 0 : 1);
                    o.objects.tree.mark($item);
                    $checkbox[0].checked = !currentState;
                }
            },
            "AssetManager.Item.tree.setPort": {
                action: "click",
                selector: ".block.editor .tree-block .tree-item .set-service-port",
                callback: function() {
                    var $this = $(this);
                    o.setServicePort.call(o, $this);
                    return false;
                }
            },
            "AssetManager.Item.tree.setAppserverVersion": {
                action: "click",
                selector: ".tree-block .tree-item .set-appserver-version",
                callback: function() {
                    var $this = $(this);
                    o.setAppserverVersion.call(o, $this);
                    return false;
                }
            },
            "AssetManager.Item.tree.setInstance": {
                action: "click",
                selector: ".block.editor .tree-block .tree-item .set-appserver-instance",
                callback: function() {
                    var $this = $(this);
                    o.setAppserverInstance.call(o, $this);
                    return false;
                }
            },
            "AssetManager.Item.tree.appServerSelected": {
                action: "change",
                selector: "#assetTreeAddingAppServer select[name=\"appServer\"]",
                callback: function() {
                    var $this = $(this);
                    o.showAppServerAdvancedFields.call(o, $this);
                    return false;
                }
            },
            "AssetManager.Item.tree.serviceSelected": {
                action: "change",
                selector: "#assetTreeAddingService select[name=\"service\"]",
                callback: function() {
                    var $this = $(this);
                    o.showServicePortInput.call(o, $this);
                    return false;
                }
            },
            "AssetManager.Item.tree.elementRemove": {
                action: "click",
                selector: ".block.editor .tree-block .tree-item > .tree-item-content > .text .element-remove",
                callback: function() {
                    var $this = $(this);
                    new PopupConfirm({
                        title: o.data.texts["treeBlock"]["confirmations"]["remove"]["title"],
                        text: o.data.texts["treeBlock"]["confirmations"]["remove"]["content"],
                        callback: function () {
                            o.elementRemove.call(o, $this);
                        }
                    }).show();
                    return false;
                }
            },
            "AssetManager.Item.tree.elementSetCreds": {
                action: "click",
                selector: ".block.editor .tree-block .tree-item > .tree-item-content > .text .element-setting",
                callback: function() {
                    var $this = $(this);
                    o.goToCreds($this.parent(".text").parent(".tree-item-content").parent(".tree-item"));
                    return false;
                }
            }
        });
    }
};

AssetManager.Item.prototype.credentials = {
    cellClasses: {
        "appServer": "appserver-cell creds-cell",
        "connectionMethod": "method-cell creds-cell",
        "connectorParams": "connector_params-cell creds-cell-complex",
        "actions": "actions-cell"
    },
    connectorFieldTypes: {
        default: "text",
        password: "password"
    },
    buildConnectionsMap: function() {
        var connectionsMap = {},
            connections = this.data.item["connections"];
        for (var i = 0; i < connections.length; i++) {
            var connection = connections[i];
            connectionsMap[connection["target"]] = connectionsMap[connection["target"]] || {};
            var connectionByAppServers = connectionsMap[connection["target"]];
            connectionByAppServers[connection["targetVersion"]] = connectionByAppServers[connection["targetVersion"]] || {
                "services": {}
            };
            var connectionByAppServerVersions = connectionByAppServers[connection["targetVersion"]],
                serviceId = connection["connectionMethodObject"]["appService"]["id"];
            connectionByAppServerVersions["services"][serviceId] = connection["connectionMethodObject"]["name"];
        }
        return connectionsMap;
    },
    collectFilterValues: function() {
        var values = {
            "appServer": [],
            "connectionMethod": []
        },
            connections = this.data.item["connections"];
        for (var i = 0; i < connections.length; i++) {
            var connection = connections[i];
            if (values["appServer"].indexOf(connection["appServer"]["name"]) == -1) {
                values["appServer"].push(connection["appServer"]["name"] + " " + this.__common.tree.getAppserverVersionName(connection["appServer"], connection["targetVersion"]));
            }
            if (values["connectionMethod"].indexOf(connection["connectionMethodObject"]["name"]) == -1) {
                values["connectionMethod"].push(connection["connectionMethodObject"]["name"]);
            }
        }
        return values;
    },
    build: function() {
        this.data.item["connectionsByEntity"] = this.buildConnectionsMap();
        var dictionary = this.collectFilterValues();
        this.__parent.setDictionary("creds", dictionary);
        var creds = this.data.item["connections"] != null && this.data.item["connections"].length != 0 ? this.generateRowsHTML(this.data.item["connections"]) : [],
            params = {
                title: "List",
                data: creds,
                option: {
                    namespace: "assets",
                    type: "creds",
                    sort_default: { column: 0, order: "asc" }
                }
            };
        this.objects.table = new Table(params);
        var $tableContainer = $(".credentianls-table"),
            html = this.objects.table.getHTML(),
            $table = $(html),
            id = $table.attr("id"),
            o = this;
        $tableContainer.attr("id", id).html($table.find(".tablesorter").getWrap()).before($table.find(".actions-bar").getWrap());
        this.objects.table.build();
        this.objects.table.clickEventOff();
        $tableContainer.find("tbody tr").addClass("filled-in stringed").each(function () {
            for (var field in Locale.fields["assets"]["creds"]) {
                var colNumber = o.objects.table.getColumnNumber(field);
                $(this).find("td").eq(colNumber).addClass(o.cellClasses[field] + " filled");
            }
        });
        this.handlers();
    },
    generateRowsHTML: function(connections) {
        var data = [];
        for (var i = 0; i < connections.length; i++) {
            data.push(this.generateRowHTML(connections[i]));
        }
        return data;
    },
    generateRowHTML: function(data) {
        var credsHTML = {
                connectorParams: "",
                appServer: "",
                connectionMethod: ""
            },
            templates = this.data.templates["credentialsBlock"];
        data.connectorParams = data.connectorParams || [];
        for (var i = 0; i < data.connectorParams.length; i++) {
            var fieldObj = data.connectorParams[i],
                content = this.connectorFieldTypes[fieldObj["name"]] == "password" ? "**********" : fieldObj["value"];
            credsHTML.connectorParams += templates.rowItem.format({
                content: templates.rowItemTitle.format({
                    content: Locale.fields["credentials"][fieldObj["name"]]
                })
                + ': ' +
                templates.rowItemValue.format({
                    content: content,
                    value: fieldObj["value"],
                    name: fieldObj["name"],
                    source: "connector",
                    id: fieldObj["id"]
                })
            });
        }
        data.appServerParams = data.appServerParams || [];
        for (var i = 0; i < data.appServerParams.length; i++) {
            var fieldObj = data.appServerParams[i],
                content = this.connectorFieldTypes[fieldObj["name"]] == "password" ? "**********" : fieldObj["value"];
            credsHTML.connectorParams += templates.rowItem.format({
                content: templates.rowItemTitle.format({
                    content: Locale.fields["credentials"][fieldObj["name"]]
                }) + ': ' +
                templates.rowItemValue.format({
                    content: content,
                    value: fieldObj["value"],
                    name: fieldObj["name"],
                    source: "appServer",
                    id: fieldObj["id"]
                })
            });
        }
        var appserverObj = this.__parent.data.appservers[data.target],
            versionName = this.__common.tree.getAppserverVersionName(appserverObj, data.targetVersion),
            connectionMethodObj = this.__parent.data.connectionmethods[data.connectionMethod];
        credsHTML.appServer = templates.rowItem.format({
                content: templates.rowItemValue.format({
                    content: appserverObj["name"],
                    value: data.target,
                    name: "appserver",
                    source: "",
                    id: data.target
                })
            }) + templates.rowItem.format({
                content: templates.rowItemValue.format({
                    content: " " + versionName,
                    value: data.targetVersion,
                    name: "appserverVersion",
                    source: "",
                    id: data.targetVersion
                })
            });
        credsHTML.connectionMethod = templates.rowItem.format({
            content: templates.rowItemValue.format({
                content: connectionMethodObj["name"],
                value: data.connectionMethod,
                name: "connectionMethod",
                source: "",
                id: data.connectionMethod
            })
        });
        credsHTML.actions = this.data.templates["credentialsBlock"].actions.edit + this.data.templates["credentialsBlock"].actions.test + this.data.templates["credentialsBlock"].actions.remove;
        return credsHTML;
    },
    collectAppServerFromTree: function() {
        var appServers = {};
        $(".block.editor .tree-block .tree-item[data-type=\"appServer\"]").each(function() {
            var $this = $(this);
            appServers[$this.attr("data-id")] = $(this).attr("data-value");
        })
        return appServers;
    },
    collectAppServerVersionsFromTree: function(appServerId) {
        var appServerVersions = {},
            o = this;
        $(".block.editor .tree-block .tree-item[data-type=\"appServer\"][data-id=\"" + appServerId + "\"]").each(function() {
            var $this = $(this),
                appServerVersionName = o.__common.tree.getAppserverVersionName(o.__parent.data.appservers[$this.attr("data-id")], $this.attr("data-version"));
            appServerVersions[$this.attr("data-version")] = appServerVersionName;
        })
        return appServerVersions;
    },
    collectServicesFromTree: function(appServerId) {
        var services = [],
            o = this;
        $(".block.editor .tree-block .tree-item[data-type=\"appServer\"][data-id=\"" + appServerId + "\"] .tree-item[data-type=\"service\"]").each(function() {
            var $this = $(this),
                serviceId = $this.attr("data-id");
            services.push(serviceId);
        });
        return services;
    },
    newEmptyCred: function() {
        var appServersAvailable = this.collectAppServerFromTree(),
            appservers_html = "";
        for (var appserver in appServersAvailable) {
            appservers_html += this.data.templates["credentialsBlock"].option.format({
                value: appserver,
                text: appServersAvailable[appserver]
            });
        }
        var row = this.data.templates["credentialsBlock"].row.format({
            number: this.utility.credRowCounter++,
            appserver: this.data.templates["credentialsBlock"].appServerSelect.format({
                appservers: appservers_html,
                versions: this.data.templates["credentialsBlock"].appServerVersionDefault
            }),
            method: this.data.templates["credentialsBlock"].dash,
            connector_params: this.data.templates["credentialsBlock"].dash,
            actions: this.data.templates["credentialsBlock"].actions.save + this.data.templates["credentialsBlock"].actions.remove
        });
        return row;
    },
    setValue: function($row, name, value, type) {
        type = type || "select";
        switch (type) {
            case "select":
                var $select = $row.find("select[name=\"" + name + "\"]"),
                    $option = $select.find("option[value=\"" + value + "\"]");
                $option[0].selected = true;
                this.data.defaultSettings[name] = {
                    id: value
                };
                break;
            case "text":
                $row.find("input[name=\"" + name + "\"]").val(value);
                this.data.defaultSettings[name] = {
                    value: value
                };
                break;
        }
    },
    setValues: function($row, data) {
        this.setValue($row, "appserver", data["appserver"]["id"]);
        this.showAppserverVersionList($row, data["appserver"]["id"]);
        this.setValue($row, "appserverVersion", data["appserverVersion"]["id"]);
        this.showMethodsList($row, data["appserver"]["id"], data["appserverVersion"]["id"]);
        this.setValue($row, "connectionMethod", data["connectionMethod"]["id"]);
        delete data["appserver"];
        delete data["appserverVersion"];
        delete data["connectionMethod"];
        var fields = [];
        for (var field in data) {
            fields.push({
                id: data[field]["id"],
                name: field
            });
        }
        this.showConnectionBlocks($row, fields);
        for (var field in data) {
            this.setValue($row, field, data[field]["value"], "text");
        }
    },
    pasteValues: function ($row, credsHTML) {
        $row.find(".appserver-cell").html(credsHTML.appServer);
        $row.find(".method-cell").html(credsHTML.connectionMethod);
        $row.find(".connector_params-cell").html(credsHTML.connectorParams);
    },
    showMethodsList: function($row, appserver, version) {
        var services = this.collectServicesFromTree(appserver);
        var methods_html = "",
            connectionMethods = {};
        for (var i = 0; i < this.__parent.data.appservers[appserver]["connectionMethods"].length; i++) {
            var connectionMethod = this.__parent.data.appservers[appserver]["connectionMethods"][i];
            if (services.indexOf(connectionMethod["appService"]["id"]) != -1) {
                connectionMethods[connectionMethod["id"]] = connectionMethod["name"];
            }
        }
        for (var connectionMethod in connectionMethods) {
            methods_html += this.data.templates["credentialsBlock"].option.format({
                value: connectionMethod,
                text: connectionMethods[connectionMethod]
            });
        }
        methods_html = this.data.templates["credentialsBlock"].fields.method.format({
            col_number: 1,
            "text::selectMethod": this.data.texts["credentialsBlock"]["fields"]["method"]["selectMethod"],
            methods: methods_html
        });
        $row.find(".method-cell").html(methods_html);
        this.hideConnectionBlocks($row);
        $row.find(".method-cell select").removeAttr("disabled");
    },
    hideMethodsList: function($row) {
        $row.find("td.method-cell").html(this.data.templates["credentialsBlock"].dash);
        $row.find("td.connector_params-cell").html(this.data.templates["credentialsBlock"].dash);
    },
    hideConnectionBlocks: function($row) {
        $row.find(".connector_params-cell").html(this.data.templates["credentialsBlock"].dash);
    },
    showConnectionBlocks: function($row, fields) {
        var o = this,
            connectionBlockContent = "";
        for (var i = 0; i < fields.length; i++) {
            var inputName = fields[i]["name"],
                typeInput = "textField";
            if (inputName == "password") {
                typeInput = "passwordField";
            }
            connectionBlockContent += o.data.templates["credentialsBlock"].fields[typeInput].format({
                name: inputName,
                value: "",
                col_number: 1,
                id: fields[i]["id"],
                source: "connector",
                placeholder: Locale.fields["credentials"][inputName]
            });
            $row.find(".connector_params-cell").html(connectionBlockContent);
        }
    },
    loadConnectorFields: function($row, appserver, version, connectionMethod, callback) {
        $row.find(".connector_params-cell").addClass("loading").html(Header.templates.tabs.loading);
        ERPScan.API.query("connectorfields", "find", {
            type: "POST",
            params: {
                appserver: appserver,
                connectionmethod: connectionMethod,
                version: version
            },
            callback: function (response) {
                callback(response);
                $row.find(".connector_params-cell").removeClass("loading");
            }
        });
    },
    hideAppserverVersionList: function($row) {
        $row.find("select[name=\"appserverVersion\"]").html(this.data.templates["credentialsBlock"].appServerVersionDefault).attr("disabled", "disabled");
        $row.find("td.method-cell").html(this.data.templates["credentialsBlock"].dash);
        $row.find("td.location-cell").html(this.data.templates["credentialsBlock"].dash);
        $row.find("td.account-cell").html(this.data.templates["credentialsBlock"].dash);
    },
    showAppserverVersionList: function($row, appserver) {
        var versions = this.collectAppServerVersionsFromTree(appserver),
            versions_html = "";
        for (var version in versions) {
            versions_html += this.data.templates["credentialsBlock"].option.format({
                value: version,
                text: versions[version]
            });
        }
        if (!Helpers.objIsEmpty(versions)) {
            $row.find("select[name=\"appserverVersion\"]").html(this.data.templates["credentialsBlock"].appServerVersionDefault + versions_html).removeAttr("disabled");
        } else {
            $row.find("select[name=\"appserverVersion\"]").html(this.data.templates["credentialsBlock"].option.format({
                value: -1,
                text: "Default"
            })).attr("disabled", "disabled");
            this.showMethodsList($row, appserver, "default");
        }
    },
    collectValues: function ($row) {
        var values = {
            connectorParams: [],
            appServerParams: []
        };
        $row.find(".connector_params-cell input[data-source=\"connector\"]").each(function () {
            var $this = $(this);
            values.connectorParams.push({
                id: $this.data("id"),
                name: $this.attr("name"),
                value: $this.val()
            })
        });
        $row.find(".connector_params-cell input[data-source=\"appServer\"]").each(function () {
            var $this = $(this);
            values.appServerParams.push({
                id: $this.data("id"),
                name: $this.attr("name"),
                value: $this.val()
            })
        });
        values.connectionMethod = $row.find("select[name=\"connectionMethod\"]").val();
        values.target = $row.find("select[name=\"appserver\"]").val();
        values.targetVersion = $row.find("select[name=\"appserverVersion\"]").val();
        return values;
    },
    setState: function ($row, state) {
        switch (state) {
            case "text":
                $row.find(".appserver-cell, .method-cell, .connector_params-cell").addClass("filled");
                $row.addClass('stringed');
                $row.find(".action-cancel").removeClass("action-cancel").addClass("action-remove").text("Remove");
                $row.find(".action-save").removeClass("action-save").addClass("action-edit").text("Edit").after(this.data.templates["credentialsBlock"].actions.test);
                break;
            case "form":
                $row.find(".appserver-cell, .method-cell, .connector_params-cell").removeClass("filled");
                $row.removeClass('stringed');
                $row.find(".action-remove").addClass("action-cancel").removeClass("action-remove").text("Cancel");
                $row.find(".action-save").addClass("action-save").removeClass("action-edit").text("Save");
                break;
        }
    },
    handlers: function () {
        var o = this;
        this.__parent.addHandlers({
            "AssetManager.Item.credentials.addCredential": {
                action: "click",
                selector: ".block.editor .add-credentials",
                callback: function () {
                    var $this = $(this),
                        newCred = o.newEmptyCred.call(o);
                    $(".credentianls-table table.tablesorter tbody").prepend(newCred);
                    o.objects.table.obj.trigger('update');
                    return false;
                }
            },
            "AssetManager.Item.credentials.appServerSelect": {
                action: "change",
                selector: ".block.editor .credentianls-table select[name=\"appserver\"]",
                callback: function () {
                    var $this = $(this);
                    o.hideConnectionBlocks.call(o, $this.parents("tr"));
                    o.hideMethodsList.call(o, $this.parents("tr"));
                    if ($this.val() == 0) {
                        o.hideAppserverVersionList.call(o, $this.parents("tr"));
                        return false;
                    }
                    var $row = $this.parents("tr");
                    o.showAppserverVersionList.call(o, $row, $row.find("select[name=\"appserver\"]").val());
                    return false;
                }
            },
            "AssetManager.Item.credentials.connectionMethodSelect": {
                action: "change",
                selector: ".credentianls-table select[name=\"connectionMethod\"]",
                callback: function () {
                    var $this = $(this);
                    if ($this.val() == 0) {
                        o.hideConnectionBlocks.call(o, $this.parents("tr"));
                        return false;
                    }
                    var $row = $this.parents("tr");
                    o.loadConnectorFields($row, $row.find("select[name=\"appserver\"]").val(), $row.find("select[name=\"appserverVersion\"]").val(), $row.find("select[name=\"connectionMethod\"]").val(), function (response) {
                        o.showConnectionBlocks.call(o, $row, response["data"]["result"]);
                    });
                    return false;
                }
            },
            "AssetManager.Item.credentials.appServerVersionSelect": {
                action: "change",
                selector: ".credentianls-table select[name=\"appserverVersion\"]",
                callback: function () {
                    var $this = $(this);
                    if ($this.val() == 0) {
                        o.hideMethodsList.call(o, $this.parents("tr"));
                        return false;
                    }
                    var $row = $this.parents("tr");
                    o.showMethodsList.call(o, $row, $row.find("select[name=\"appserver\"]").val(), $this.val());
                    return false;
                }
            },
            "AssetManager.Item.credentials.actionRemove": {
                action: "click",
                selector: ".credentianls-table .action-remove",
                callback: function () {
                    $(this).parents("tr").remove();
                    o.objects.table.obj.trigger('update');
                }
            },
            "AssetManager.Item.credentials.actionCancel": {
                action: "click",
                selector: ".credentianls-table .action-cancel",
                callback: function () {
                    var $row = $(this).parents("tr");
                    o.setValues($row, o.data.defaultSettings);
                    var values = o.collectValues($row),
                        row = o.generateRowHTML.call(o, values);
                    o.pasteValues($row, row);
                    o.setState($row, "text");
                }
            },
            "AssetManager.Item.credentials.actionEdit": {
                action: "click",
                selector: ".credentianls-table .action-edit",
                callback: function () {
                    var $this = $(this);
                    var $row = $this.parents("tr"),
                        data = {};
                    $row.find(".field_val").each(function() {
                        var $this = $(this);
                        data[$this.data("name")] = {
                            id: $this.data("id"),
                            value: $this.data("value")
                        };
                    });
                    $row.html($(o.newEmptyCred()).html()).removeClass("stringed");
                    o.setValues.call(o, $row, data);
                    o.setState.call(o, $row, "form");
                }
            },
            "AssetManager.Item.credentials.actionTest": {
                action: "click",
                selector: ".credentianls-table .action-test",
                callback: function () {
                    var $this = $(this),$row = $this.parents("tr");
                }
            },
            "AssetManager.Item.credentials.actionSave": {
                action: "click",
                selector: ".credentianls-table .action-save",
                callback: function () {
                    var $this = $(this),
                        $row = $this.parents("tr"),
                        values = o.collectValues($row);
                    var row = o.generateRowHTML.call(o, values);
                    o.pasteValues($row, row);
                    o.setState($row, "text");
                }
            }
        });
    }
};