function SoD() {
    this.templates = {
        table: "<div class='table-wrapper'><table class='sod-matrix'>%s</table></div>",
        tr: "<tr>%s</tr>",
        td: "<td>%s</td>",
        select: "<select name='%s'>%s</select>",
        option: "<option value='%d'>%s</option>",
        no_entry: '<div class="no_entry">' + Locale.ui.page["no_entry"] + '</div>',
        policy_title: '<h3>%s</h3>',
        risk_description: '<input type="text" name="risk_description" placeholder="' + Locale.ui.page["fields"]["risk_description"] + '" />',
        priority: '<input type="text" name="priority" placeholder="' + Locale.ui.page["fields"]["priority"] + '" />',
        actions: '<button class="add-row default">' + Locale.ui.page["add_row"] + '</button><button class="save">' + Locale.ui.page["save"] + '</button>'
    };
    this.risks = ["-- Select risk --", "High", "Medium", "Low", "No"];
    this.policies = ["-- Select policy --", "Policy 1", "Policy 2", "Policy 3", "Policy 4" ,"Policy 5", "Policy 6"];
    this.buildTable = function(data) {
        var params = {
            title: "Business processes",
            data: data,
            option: {
                ticks: {
                    '__addition': 0,
                    'business_process': 1,
                    'risk_name': 2,
                    'risk_description': 3,
                    'priority': 4,
                    'function1_name': 5,
                    'function2_name': 6,
                    'function3_name': 7,
                    'function4_name': 8,
                    'function5_name': 9
                },
                sorting: {
                    0: false,
                    1: false,
                    2: false,
                    3: false,
                    4: false,
                    5: false,
                    6: false,
                    7: false,
                    8: false,
                    9: false
                },
                filtering: false,
                sort_default: { column: 2, order: "desc" }
            }
        };
        var table_obj = new Table(params),
            html = table_obj.getHTML(),
            section = $('section.content');
        section.html(html);
        table_obj.build();
        $(document.body).off("click", ".table .tablesorter tr");
        $("section.content .table .actions-bar .action-events").html(this.templates.actions).show();
        Helpers.setBlockHeight(section);
    };
    this.generateRow = function() {
        var options_bp = "";
        for (var i = 0; i < this.business_processes.length; i++) {
            options_bp += this.templates.option.sprintf(i, this.business_processes[i]);
        }
        options_bp = this.templates.select.sprintf("business_process", options_bp);
        var options_f = [];
        for (var k = 0; k < 5; k++) {
            options_f[k] = k != 0 ? this.templates.option.sprintf(0, Locale.ui.page["no_function"]) : "";
            for (var i = 1; i <= this.functions.length; i++) {
                options_f[k] += this.templates.option.sprintf(i, this.functions[i - 1]);
            }
            options_f[k] = this.templates.select.sprintf("business_function_" + (k + 1), options_f[k]);
        }
        var options_risk = "";
        for (var i = 0; i < this.risks.length; i++) {
            options_risk += this.templates.option.sprintf(i, this.risks[i]);
        }
        options_risk = this.templates.select.sprintf("risk", options_risk);
        return {
            '__addition': '<a href="#" class="remove-row"></a>',
            'business_process': options_bp,
            'risk_name': options_risk,
            'risk_description': this.templates.risk_description,
            'priority': this.templates.priority,
            'function1_name': options_f[0],
            'function2_name': options_f[1],
            'function3_name': options_f[2],
            'function4_name': options_f[3],
            'function5_name': options_f[4]
        };
    };
    this.loadTable = function(policy) {
        $("section.content").html(Header.templates.tabs.loading);
        var o = this;
        $.ajax({
            url: "/data/sod_policies_n_way.json",
            method: "GET",
            dataType: "JSON",
            success: function(result) {
                o.business_processes = result["business_processes"];
                o.functions = result["functions"];
                var data = [];
                for (var i = 0; i < 3; i++) {
                    data.push(o.generateRow());
                }
                o.buildTable.call(o, data);
            }
        });
    };
    this.loadPoliciesList = function() {
        var options_policies = "";
        for (var i = 0; i < this.policies.length; i++) {
            options_policies += this.templates.option.sprintf(i, this.policies[i]);
        }
        options_policies = this.templates.select.sprintf("policies", options_policies);
        $("header .title h2").after(options_policies);
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("change", "header .title select", function() {
            var $this = $(this);
            var policy_number = $this.val();
            if (policy_number == 0) {
                $("section.content").html(o.templates.no_entry);
                return false;
            }
            o.loadTable($this.find("option").eq(policy_number).text());
            return false;
        });
        $(document.body).on("change", "section.content .sod-matrix select", function() {
            var $this = $(this);
            var risk_number = $this.val();
            var i = $this.parents("td").index();
            var j = $this.parents("tr").index();
            $("section.content .sod-matrix tr:eq(" + i + ") td:eq(" + j + ")").find("select option").eq(risk_number)[0].selected = true;
            return false;
        });
        $(document.body).on("click", "section.content .add-row", function() {
            var $this = $(this);
            var row = o.generateRow();
            var row_html = "";
            for (var field in row) {
                row_html += "<td" + (field == "__addition" ? " class=\"toggle-info\"" : "") + ">" + row[field] + "</td>";
            }
            row_html = "<tr>" + row_html + "</tr>";
            $(".table table tbody").append(row_html);
            $('.table table').trigger('update');
            return false;
        });
        $(document.body).on("click", ".table .tablesorter .toggle-info a", function() {
            $(this).parents("tr").remove();
            $('.table table').trigger('update');
            return false;
        })
    };
    this.init = function() {
        $("section.content").html(this.templates.no_entry).addClass("sod_n_way");
        this.loadPoliciesList();
        this.handlers();
    };
}

Page.Render = {
    init: function(tab) {
        var page = ['assess', 'policies', "N_way_sod_matrix"];
        Locale.set("en", { fields: ["policies"] });
        Locale.loadPage(page);
        Header.load(page, { tabs: true, type: "landscape" });
        new SoD().init();
    }
};