Jobs.AccessControl = function(obj, class_name) {
    this.path = ['assess', 'projects', 'sod'];
    this.parentObj = obj;
    this.BPEditorTreeElements = {
        "businessProcess": {
            level: 1,
            hasChildren: true,
            child: "businessFunction",
            addingText: {
                "DEFAULT": {
                    text: "Add business process"
                }
            },
            fields: {
                "BPName": {
                    type: "text",
                    source: ["name", "{locale}"]
                }
            },
            structureChild: "businessRoles",
            structure: {
                "type": "SOD",
                "name" : {
                    "{locale}": "{BPName}"
                },
                "businessRoles": []
            }
        },
        "businessFunction": {
            level: 2,
            hasChildren: true,
            child: "businessAction",
            addingText: {
                "DEFAULT": {
                    text: "Add business function"
                }
            },
            fields: {
                "BFName": {
                    type: "text",
                    source: ["name", "{locale}"]
                }
            },
            structureChild: "actions",
            structure: {
                "name" : {
                    "{locale}": "{BFName}"
                },
                "actions": []
            }
        },
        "businessAction": {
            level: 3,
            hasChildren: true,
            child: "authObjectsGroup",
            addingText: {
                "DEFAULT": {
                    text: "Add business action"
                }
            },
            fields: {
                "BAType": {
                    type: "select",
                    callback: function() {
                        var $context = $(this);
                        $context.parent(".BATypeContainer").next(".BANameContainer")[$context.val() == 4 ? "hide" : "show"]();
                    },
                    source: ["type"]
                },
                "BAName": {
                    type: "text",
                    source: ["name"]
                },
                "BADescription": {
                    type: "textarea",
                    source: ["description", "{locale}"]
                },
                "BARootGroupType": {
                    type: "select",
                    source: ["BARootGroupType"]
                }
            },
            structure: {
                "type": "{BAType}",
                "name" : "{BAName}",
                "description" : {
                    "{locale}": "{BADescription}"
                },
                "authObjects": function($context) {
                    return this.generateCortege($context);
                }
            }
        },
        "authObjectsGroup": {
            level: 4,
            hasChildren: true,
            child: "authObject",
            addingText: {
                "DEFAULT": {
                    text: "Add object",
                    type: "authObject"
                },
                "OR": {
                    text: "Add group auth objects (OR operator)",
                    callback: function($item) {
                        $item.find("> .tree-item-content .text select[name=\"groupingType\"]").find("option").eq(2).attr("selected", "selected");
                    }
                },
                "AND": {
                    text: "Add group auth objects (AND operator)",
                    callback: function($item) {
                        $item.find("> .tree-item-content .text select[name=\"groupingType\"]").find("option").eq(1).attr("selected", "selected");
                    }
                }
            },
            fields: {
                "AOGroupType": {
                    type: "select"
                }
            },
            "groupType": {
                type: "text"
            }
        },
        "authObject": {
            level: 5,
            hasChildren: true,
            child: "attribute",
            addingText: {
                "DEFAULT": {
                    text: "Add object"
                },
                "OR": {
                    text: "Add group auth objects (OR operator)",
                    type: "authObjectsGroup"
                },
                "AND": {
                    text: "Add group auth objects (AND operator)",
                    type: "authObjectsGroup"
                }
            },
            fields: {
                "AOName": {
                    type: "text"
                }
            }
        },
        "attribute": {
            level: 6,
            hasChildren: false,
            addingText: {
                "DEFAULT": {
                    text: "Add attribute"
                }
            },
            fields: {
                "AOAttributeName": {
                    type: "text"
                },
                "AOAttributeValue": {
                    type: "text"
                }
            }
        }
    };
    this.section = $('.tab-' + class_name);
    this.data = {};
    this.locale = null;
    this.controls = {};
    this.rulesGroupObjects = {
        "&": "AND",
        "|": "OR"
    };
    this.templates = {
        editorTreeElements: {
            authObjectsGroup: {
                item: '<span class="AOGroupTypeContainer">Grouping by <b>{AOGroupType}</b></span>'
            },
            authObject: {
                item: '<span class="AONameContainer"><b>{AOName}</b></span>'
            },
            attribute: {
                item: '<span class="AOAttributeNameContainer">{AOAttributeName}</span>: <span class="AOAttributeValueContainer"><b>{AOAttributeValue}</b></span>'
            }
        },
        showResults: "<a class='showResults' href='#'>{text}</a>",
        resultRow: "<div class='resultsRow' data-parent-number='{number}'><td class='toggleInfo preset'></td><td colspan='100'>{content}</td></div>",
        block_content: {
            description: '<p><b>{id_name}</b>: {id_value}</p><p><b>{description_name}:</b></p><p>{description_value}</p><p>{authorization_value}</p><p><b>{threat_name}:</b></p><p>{threat_value}</p>',
            solution: '<p><b>{solution_name}:</b></p><p>{solution_value}</p><p><b>{links_name}:</b></p><p>{links_value}</p>',
            issues: '<select class="entityTypes">'+
            '<option value="CriticalUserIssue">{users}</option>'+
            '<option value="CriticalRoleIssue">{roles}</option>'+
            '<option value="CriticalProfileIssue">{profiles}</option>'+
            '</select>' +
            '<div class="entitiesTable CriticalUserIssue empty" style="display: none"><div class="loading"><img src="/media/images/loading.gif" /></div></div>' +
            '<div class="entitiesTable CriticalRoleIssue empty" style="display: none"><div class="loading"><img src="/media/images/loading.gif" /></div></div>' +
            '<div class="entitiesTable CriticalProfileIssue empty" style="display: none"><div class="loading"><img src="/media/images/loading.gif" /></div></div>',
            stats: '<p><b>{stats_name}:</b></p><p>{stats_value} ms</p>'
        },
        loading: "<div class='loading'></div>",
        object: "<div class='objects-list'><div class='object-type'>{object_type}:</div><div class='object-name'><a href='#'>{object_name}</a></div></div>",
        authList: "<ul>{list}</ul>",
        authItem: "<li>{item}</li>",
        table: '<div id="{id}">{table}</div>'
    };
    this.getResultsStructure = function (){
        var o = this,
            params = {
                scanId: o.parentObj.data.scan.id
            };
        ERPScan.query("resultsStructure", "get", {
            type: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            params: JSON.stringify(params),
            callback: function(response){
                var convertedResults = o.convertResultsForTreetable(response["data"]["result"]);
                o.data.resultsStructure = convertedResults;
                o.buildCategoriesTable(convertedResults);
            }
        });
    };
    this.conversionFieldsRules = {
        "resultsStructureTable": [
            {
                name: "name",
                getVal: function(fields){
                    return {
                        value: fields["name"]["en_US"],
                        attr: {
                            name: "structure_item_id",
                            value: fields["policyStructureId"]
                        }
                    }
                }
            },
            {
                name: "count",
                getVal: function(fields){
                    return fields["resultCount"]
                }
            },
            {
                name: "users",
                getVal: function(fields){
                    return fields["statistic"]["CriticalUserIssue"] || "0"
                }
            },
            {
                name: "roles",
                getVal: function(fields){
                    return fields["statistic"]["CriticalRoleIssue"] || "0"
                }
            },
            {
                name: "profiles",
                getVal: function(fields){
                    return fields["statistic"]["CriticalProfileIssue"] || "0"
                }
            }
        ],
        "resultsTable": [
            {
                name: "id",
                getVal: function(fields){
                    return fields["id"]
                }
            },
            {
                name: "actionType",
                getVal: function(fields){
                    var address = fields["address"],
                        businessProcessObj = this.getObjectWithPropertyVal(this.parentObj.data["projectPolicyStructure"]["structure"], "id", address[0]),
                        businessRoleObj = this.getObjectWithPropertyVal(businessProcessObj["businessRoles"], "id", address[1]),
                        actionObj = this.getObjectWithPropertyVal(businessRoleObj["actions"], "id", address[2]);
                    return actionObj["type"]
                }
            },
            {
                name: "actionName",
                getVal: function(fields){
                    var adress = fields["address"],
                        businessProcessObj = this.getObjectWithPropertyVal(this.parentObj.data["projectPolicyStructure"]["structure"], "id", adress[0]),
                        businessRoleObj = this.getObjectWithPropertyVal(businessProcessObj["businessRoles"], "id", adress[1]),
                        actionObj = this.getObjectWithPropertyVal(businessRoleObj["actions"], "id", adress[2]);
                    return actionObj["name"]
                }
            },
            {
                name: "sid",
                getVal: function(fields){
                    return fields["projectResult"]["credential"]["sid"]
                }
            },
            {
                name: "users",
                getVal: function(fields){
                    return fields["statistic"]["CriticalUserIssue"]
                }
            },
            {
                name: "projectResult",
                getVal: function(fields){
                    return fields["projectResult"]
                }
            },
            {
                name: "sid",
                getVal: function(fields){
                    return fields["projectResult"]["credential"]["sid"]
                }
            },
            {
                name: "client",
                getVal: function(fields){
                    // return fields["projectResult"]["credential"]["customParameters"][0]["value"]
                }
            },
            {
                name: "action",
                getVal: function(fields){
                    var adress = fields["address"],
                        businessProcessObj = this.getObjectWithPropertyVal(this.parentObj.data["projectPolicyStructure"]["structure"], "id", adress[0]),
                        businessRoleObj = this.getObjectWithPropertyVal(businessProcessObj["businessRoles"], "id", adress[1]),
                        actionObj = this.getObjectWithPropertyVal(businessRoleObj["actions"], "id", adress[2]);
                    return actionObj
                }
            }
        ],
        "CriticalUserIssue": [
            {
                name: "userName",
                getVal: function(fields){
                    return fields["userObject"]["userName"]
                }
            },
            {
                name: "aggregatedRisk",
                getVal: function(fields){
                    return fields["aggregatedRisk"]
                }
            }
        ],
        "CriticalRoleIssue": [
            {
                name: "userName",
                getVal: function(fields){
                    return fields["userObject"]["userName"]
                }
            },
            {
                name: "aggregatedRisk",
                getVal: function(fields){
                    return fields["aggregatedRisk"]
                }
            }
        ],
        "CriticalProfileIssue": [
            {
                name: "userName",
                getVal: function(fields){
                    return fields["userObject"]["userName"]
                }
            },
            {
                name: "aggregatedRisk",
                getVal: function(fields){
                    return fields["aggregatedRisk"]
                }
            }
        ]
    };
    this.getObjectWithPropertyVal = function (arr, prop, val){
        var object;
        arr.forEach(function (obj){
            if (obj[prop] == val) {
                object = obj;
                return false;
            }
        });
        return object;
    };
    this.filterNeededFields = function (resultItem, namespace){
        var o = this,
            hierarchyElem = {};
        this.conversionFieldsRules[namespace].forEach(function(field){
            hierarchyElem[field["name"]] = field.getVal.call(o, resultItem);
        });
        return hierarchyElem;
    };
    this.convertResultsForTreetable = function(results) {
        var o = this,
            obj = {},
            rowId = 'type',
            convertedData = [];
        createHierarchy(results, 1, rowId);
        function createHierarchy(hierarchyElem, level, rowId) {
            hierarchyElem.forEach(function (resultItem) {
                obj = o.filterNeededFields(resultItem, "resultsStructureTable");
                obj["_id"] = rowId + '_' + level;
                if (rowId != "type") {
                    obj["_parent"] = rowId;
                }
                convertedData.push(obj);
                if (resultItem["children"].length > 0) {
                    createHierarchy(resultItem["children"], 1, obj["_id"]);
                } else {
                    convertedData.push({
                        "_id": obj["_id"] + "_" + 1,
                        "_parent": obj["_id"],
                        "name": "_delete"
                    });
                }
                level++;
            });
        }
        return convertedData;
    };
    this.buildCategoriesTable = function (data){
        var params = {
            title: this.locale.accessControl["tableTitle"],
            data: data,
            ticks: {"__addition": true},
            option: {
                deleteFields: [],
                expandable: false,
                namespace: "projects",
                type: "sodAccessControlHierarchy",
                tree: true,
                treeColumn: 1
            }
        };
        this.controls.categoriesTable = new Table(params);
        var html = this.controls.categoriesTable.getHTML();
        this.section.append(html);
        this.controls.categoriesTable.build(params.data, params.option);
        this.handlers();
        this.section.find("table.treetable td:contains('_delete')").closest("tr").remove();
    };
    this.getResults = function ($row){
        var o = this;
        $row.find(".resultsRow").append(this.templates.loading);
        $row.find("a.showResults").addClass("active").text(this.locale["accessControl"]["hideResults"]);
        ERPScan.query("resultsSOD", "list", {
            params: {
                scanId: o.parentObj.data.scan.id,
                structureItemId: $row.find("td[data-structure_item_id]").data("structure_item_id")
            },
            callback: function(response) {
                if(response["data"]["result"].length > 0) {
                    var convertedResults = o.convertResultsForTable(response["data"]["result"], "resultsTable");
                    o.buildResultsTable(convertedResults, $row);
                    o.resizeResultRow($row);
                }
            }
        });
    };
    this.convertResultsForTable = function (results, namespace){
        var o = this,
            convertedResults = [];
        results.forEach(function (result){
            convertedResults.push(o.filterNeededFields(result, namespace));
        });
        return convertedResults;
    };
    this.buildResultsTable = function (results, $row){
        var params = {
            data: results,
            option: {
                pager: false,
                expandable: false,
                namespace: "projects",
                type: "sodAccessControl"
            }
        };
        this.controls.resultsTable = new Table(params);
        var html = this.controls.resultsTable.getHTML();
        $row.find(".loading").remove();
        $row.find(".resultsRow").append(html);
        this.controls.resultsTable.build();
        if(typeof this.data["results"] == "undefined") {
            this.data["results"] = {};
        }
        this.data["results"][this.controls.resultsTable.number] = results;
    };
    this.buildBlocks = function() {
        var descriptionBlock = new Block({
            title: this.locale["accessControl"]["descriptionBlock"]["title"],
            content: this.locale["accessControl"]["descriptionBlock"]["empty"],
            class: "checkDescription"
        }).getHTML();
        this.section.html(descriptionBlock);
    };
    this.correctSequenceBrackets = function(str) {
        var counter = 0;
        for (var i = 0; i < str.length; i++) {
            if (str[i] == '(') {
                counter++;
            } else if (str[i] == ')') {
                counter--;
            }
            if (counter < 0) {
                return false;
            }
        }
        return counter == 0;
    };
    this.clearUnnecessaryBrackets = function(authObjects) {
        var correctSequenceBrackets;
        while (correctSequenceBrackets = this.correctSequenceBrackets(authObjects)) {
            if (authObjects.substr(0,1) == "(" && authObjects.substr(authObjects.length - 1, 1) == ")") {
                authObjects = authObjects.substr(1, authObjects.length - 2);
            } else {
                break;
            }
        }
        if (!correctSequenceBrackets) {
            authObjects = "(" + authObjects + ")";
        }
        return authObjects;
    };
    this.showIssues = function(resultInfo) {
        var o = this,
            checkInfo = this.parentObj.info.checks["d7199939-856c-473b-af9b-af594b84e276"];
        var description_content = this.templates.block_content.description.format({
            id_name: this.locale.accessControl.descriptionBlock.content.description.id,
            id_value: checkInfo.name,
            description_name: this.locale.accessControl.descriptionBlock.content.description.description,
            description_value: checkInfo.description["en_US"].replace(/\[TRANSACTION\]/g, resultInfo["actionName"]),
            authorization_value: "<div class='auth-objects-tree'></div>",
            threat_name: this.locale.accessControl.descriptionBlock.content.description.threat,
            threat_value: checkInfo.threat["en_US"].replace(/\[TRANSACTION\]/g, resultInfo["actionName"]).replace(/\[ACTIONNAME\]/g, resultInfo["action"]["description"]["en_US"])
        });
        var solution_content = this.templates.block_content.solution.format({
            solution_name: this.locale.accessControl.descriptionBlock.content.solution.solution,
            solution_value: checkInfo.solution["en_US"].replace(/\[TRANSACTION\]/g, resultInfo["actionName"]),
            links_name: this.locale.accessControl.descriptionBlock.content.solution.links,
            links_value: checkInfo.links["en_US"]
        });
        var issues_content = this.templates.block_content.issues.format({
            users: this.locale.accessControl.descriptionBlock.tabs.issues.entitySelect.users,
            roles: this.locale.accessControl.descriptionBlock.tabs.issues.entitySelect.roles,
            profiles: this.locale.accessControl.descriptionBlock.tabs.issues.entitySelect.profiles
        });
        var stats = "";
        var content = new Tabs({
            container: this.section.find(" .checkDescription .block-content"),
            locale: this.locale.accessControl.descriptionBlock.tabs,
            tabs: {
                issues: issues_content,
                description: description_content,
                solution: solution_content,
                stats: stats
            }
        }).getHTML();
        this.section.find(" .checkDescription .block-content").html(content);
        this.section.find(" .checkDescription h3").html(checkInfo["title_" + Locale.name]);
        $(".block.checkDescription .block-content > div[data-name=\"description\"] > ul").remove();
        if(resultInfo.action.authObjects != null) {
            authObjects = resultInfo.action.authObjects.replace(/\[/g, "(").replace(/]/g, ")");
            var hierarchy = this.clearUnnecessaryBrackets(authObjects);
            hierarchy = this.cortegeToHierarchy(hierarchy);
            this.cortegeHierarchyNormalize(hierarchy);
            var nodes = this.BTTreeCortegeToStructureWalk(hierarchy["elements"], [], 1);
            if (nodes.length > 1) {
                var rootNode = {
                    checkbox: false,
                    expanded: true,
                    level: 0,
                    nodes: nodes,
                    text: this.templates.editorTreeElements.authObjectsGroup.item.format({
                        "AOGroupType": this.rulesGroupObjects[hierarchy.type]
                    })
                };
                nodes = [rootNode];
            }
            var tree = new Tree($(".block.checkDescription .block-content > div[data-name=\"description\"] .auth-objects-tree"), nodes, {
                expandWhenClick: true
            });
            tree.init();
        }
    };
    this.loadIssues = function(entity) {
        var o = this;
        ERPScan.query("queries", "execute", {
            params: {
                query: "from {entityType} where result.id = '{resultId}'".format({
                    resultId: this.data.resultInfo.projectResult.id,
                    entityType: entity
                })
            },
            callback: function(response) {
                if(!o.data.issues) {
                    o.data.issues = {};
                }
                o.data.issues[entity] = response;
                o.buildEntitiesTable(response, entity);
            }
        });
    };
    this.showObjects = function($row, issueInfo, entity) {
        /*
         var objects_html = this.templates.object.format({object_type: issueInfo.object.objectType, object_name: issueInfo.object.objectName}),
         fields = [],
         tempObj = {};
         $.each(issueInfo.fields, function(key, val){
         tempObj = {};
         tempObj['name'] = key;
         tempObj['value'] = val;
         fields.push(tempObj);
         });
         */
        this.controls[entity].showAllFields($row);
        //$row.find('.addition-field .arrow').after(objects_html);
        $row.removeClass("loading");
    };
    this.BTTreeCortegeToStructureWalk = function (hierarchy, parent, level) {
        for (var i = 0; i < hierarchy.length; i++) {
            var itemType = "authObject",
                nodes = [];
            if (hierarchy[i].type == ",") {
                itemType = "authObject";
            }
            if (hierarchy[i]["AOName"] == "" || hierarchy[i]["value"] == "") {
                itemType = "authObjectsGroup";
            }
            if (typeof hierarchy[i]["elements"] == "undefined") {
                itemType = "attribute";
            }
            var params = this.BPEditorTreeElements[itemType];
            var text = this.templates.editorTreeElements[itemType].item;
            var templater = {};
            for (var field in params["fields"]) {
                templater[field] = hierarchy[i][field];
            }
            text = text.format(templater);
            parent.push({
                checkbox: false,
                expanded: true,
                text: text + "",
                level: params["level"] + 1
            });
            if (hierarchy[i].elements != null && hierarchy[i].elements.length != 0) {
                parent[parent.length - 1].nodes = this.BTTreeCortegeToStructureWalk(hierarchy[i].elements, [], level + 1).concat(nodes);
            }
        }
        return parent;
    };
    this.cortegeHierarchyNormalize = function (hierarchy) {
        var elementsNumber = hierarchy["elements"].length;
        for (var i = 0; i < elementsNumber; i++) {
            if (hierarchy["elements"][i]["elements"].length == 0) {
                hierarchy["elements"][i].parent["AOAttributeName"] = hierarchy["elements"][i].parent["AOName"];
                hierarchy["elements"][i].parent["AOAttributeValue"] = hierarchy["elements"][i].value;
                delete hierarchy["elements"][i].parent.value;
                delete hierarchy["elements"][i].parent["AOName"];
                delete hierarchy["elements"][i].parent.parent;
                delete hierarchy["elements"];
                return;
            } else {
                if (hierarchy["elements"][i].type != "&" && hierarchy["elements"][i].type != "|") {
                    hierarchy["elements"][i]["AOName"] = hierarchy["elements"][i].value;
                    delete hierarchy["elements"][i].value;
                }
                if (hierarchy["elements"][i].type == "&") {
                    hierarchy["elements"][i]["AOGroupType"] = "AND";
                }
                if (hierarchy["elements"][i].type == "|") {
                    hierarchy["elements"][i]["AOGroupType"] = "OR";
                }
                this.cortegeHierarchyNormalize(hierarchy["elements"][i]);
            }
            delete hierarchy.parent;
        }
    };
    this.cortegeToHierarchy = function (authObjects) {
        var ast = {
                parent: null,
                elements: []
            },
            root = {
                elements: [],
                parent: ast
            };
        ast.elements.push(root);
        var pointer = root,
            buffer = "";
        for (var i = 0; i < authObjects.length; i++) {
            switch (authObjects[i]) {
                case "(":
                    pointer.value = buffer;
                    buffer = "";
                    var item = {
                        elements: [],
                        parent: pointer
                    };
                    pointer.elements.push(item);
                    pointer = item;
                    break;
                case ")":
                    if (pointer.value == null) {
                        pointer.value = buffer;
                    }
                    pointer = pointer.parent;
                    buffer = "";
                    break;
                case ",":
                    var item = {
                        elements: [],
                        parent: pointer.parent
                    };
                    pointer.parent.type = ",";
                    pointer.parent.elements.push(item);
                    pointer = item;
                    break;
                case "|":
                    var item = {
                        elements: [],
                        parent: pointer.parent
                    };
                    pointer.parent.type = "|";
                    pointer.parent.elements.push(item);
                    pointer = item;
                    break;
                case "&":
                    var item = {
                        elements: [],
                        parent: pointer.parent
                    };
                    pointer.parent.type = "&";
                    pointer.parent.elements.push(item);
                    pointer = item;
                    break;
                default:
                    buffer += authObjects[i];
                    break;
            }
        }
        return ast;
    };
    this.buildEntitiesTable = function(data, entity) {
        if (data.length == 0) {
            this.section.find(".checkDescription .entitiesTable").hide();
            this.section.find('.checkDescription div[data-name="issues"] .entitiesTable.'+ entity).html("<p style='text-align: center; font-weight: bold;'>No issues</p>").show();
            return;
        }
        var params = {
            data: this.convertResultsForTable(data, entity),
            ticks: {"__addition": true},
            option: {
                expandable: false,
                namespace: "projects",
                type: entity
            }
        };
        this.controls[entity] = new Table(params);
        var $table = $(this.controls[entity].getHTML()),
            tableId = $table.attr('id'),
            $tables = this.section.find(".checkDescription .entitiesTable");
        $tables.hide();
        this.section.find('.checkDescription div[data-name="issues"] .entitiesTable.'+ entity)
            .html(this.templates.table.format({id: tableId, table: $table.find('table')[0].outerHTML}))
            .attr("data-entity", entity)
            .removeClass("empty")
            .show();
        this.controls[entity].build();
    };
    this.resizeResultRow = function($row) {
        var $resultsRow = $row.find(".resultsRow"),
            cellPaddingBtm = 0;
        $resultsRow.css({
            width: $row.width() - parseInt($resultsRow.css("paddingLeft")) - parseInt($resultsRow.css("paddingRight"))
        });
        if($resultsRow.is(":visible")) {
            cellPaddingBtm = $resultsRow.height() + $resultsRow.position().top;
        }
        $row.find("td.toggle-info").css({
            paddingBottom: cellPaddingBtm
        });
    };
    this.insertResultsRow = function($row) {
        var o = this;
        var level = $row.data("tt-id").substr($row.data("tt-id").indexOf("_")).split("_").length - 1;
        $row.find("td.toggle-info").each(function() {
            var $this = $(this);
            if ($this.parent("tr").attr("data-tt-parent-id") == null) {
                return;
            }
            $this.html(o.templates.resultRow.format({
                number: $row.attr("data-number"),
                content: o.templates.showResults.format({
                    text: o.locale["accessControl"]["showResults"].format({
                        number: o.data.resultsStructure[$row.data("number")]["count"]
                    })
                })
            }))
        });
        $row.find(".resultsRow").css({
            paddingLeft: $row.find(".toggle-info").outerWidth() + ($row.find("span.indenter").width() * level)
        });
        this.resizeResultRow($row);
    };
    this.removeResultsRow = function($row){
        var o = this,
            table = $row.closest("table"),
            $resultRow = $row.find(".resultsRow"),
            tableId = $resultRow.find(".table").attr("id");
        $resultRow.find(".table").remove();
        if(typeof tableId != "undefined") {
            delete o.data["results"][tableId.substr(tableId.indexOf("-") + 1, tableId.length)];
        }
        $row.find("a.showResults").removeClass("active").text(this.locale["accessControl"]["showResults"].format({
                number: o.data.resultsStructure[$row.data("number")]["count"]
            })
        );
        this.resizeResultRow($row);
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", o.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tbody tr", function(e) {
            var $this = $(this),
                $target = $(e.target),
                entity = $target.closest('div[data-name="issues"]').find('select.entityTypes option:checked').val();

            if ($target.hasClass(".addition-field") || $target.parents(".addition-field").length > 0) {
                return false;
            }
            if ($this.hasClass("active")) {
                o.controls[entity].hideAllFields($this);
            } else {
                o.showObjects.call(o, $this, o.data.issues[entity][$this.data("number")], entity);
            }
            return false;
        });
        $(document.body).on("click", o.section.selector + " .resultsRow table.tablesorter tbody tr", function() {
            var $this = $(this),
                table = $this.closest('.table'),
                tableId = table.attr("id"),
                rows = table.find('tr');
            o.data.resultInfo = o.data["results"][tableId.substr(tableId.indexOf("-") + 1, tableId.length)][$this.data("number")];
            rows.removeClass("active");
            $this.toggleClass("active");
            $this.toggleClass("leaf");
            o.showIssues(o.data.resultInfo);
            o.loadIssues("CriticalUserIssue");
        });
        $(document.body).on("mousedown", o.section.selector + " .table table.tablesorter tbody tr.branch", function(e) {
            var $this = $(this),
                $resultsRow = $this.find(".resultsRow"),
                target = $(e.target),
                expanded = $this.hasClass("expanded");
            if (target.is(".resultsRow table.tablesorter tbody tr td")) {
                return false;
            }
            if(target.is($this.find(".showResults"))) {
                if (target.hasClass("active")) {
                    o.removeResultsRow($this);
                } else {
                    o.getResults($this.closest("tr"));
                }
                return false;
            }
            if (target.parent(".indenter").length == 0) {
                o.section.find(".table table.tablesorter").treetable(expanded ? "collapseNode" : "expandNode", $this.data("ttId"));
            }
            if(expanded) {
                $resultsRow.hide();
            } else {
                if($resultsRow.length == 0) {
                    o.insertResultsRow($this);
                } else {
                    $resultsRow.show();
                }
            }
            o.resizeResultRow($this);
        });
        $(document.body).on("change", o.section.selector + " .checkDescription select.entityTypes", function() {
            var $this = $(this),
                $tables = o.section.find(".checkDescription .entitiesTable"),
                $activeTable = $tables.filter("." + $this.val());
            if($activeTable.hasClass('empty')) {
                o.loadIssues($this.val());
            }
            $tables.hide();
            $activeTable.show();
        });
        $(o.section.selector + ' iframe.blockResize')[0].contentWindow.onresize = function() {
            var activeRow = $(o.section.selector + ' .table table tbody tr.active'),
                additionField = activeRow.find('.addition-field');
            additionField.outerWidth(activeRow.width());
        };
    };
    this.clearEvents = function() {
        $(document.body).off("click", this.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tbody tr");
        $(document.body).off("click", this.section.selector + " .resultsRow table.tablesorter tbody tr");
        $(document.body).off("mousedown", this.section.selector + " .table table.tablesorter tbody tr.branch");
        $(document.body).off("change", this.section.selector + " .checkDescription select.entityTypes");
        $(this.section.selector + ' iframe.blockResize')[0].contentWindow.onresize = null;
        this.section.empty();
    };
    this.clear = function() {
        this.clearEvents();
        this.section.empty();
    };
    this.load = function() {
        this.buildBlocks();
        this.getResultsStructure();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    }
};