Jobs.CodeIssuesByType = function(obj, class_name) {
    this.path = ['assess', 'projects', 'code_vulnerability'];
    this.parentObj = obj;
    this.section = $('.tab-' + class_name);
    this.data = {};
    this.locale = null;
    this.controls = {};
    this.templates = {
        showResults: "<a class='showResults' href='#'>{text}</a>",
        resultRow: "<div class='resultsRow' data-parent-number='{number}'><td class='toggleInfo preset'></td><td colspan='100'>{content}</td></div>",
        block_content: {
            description: '<p><b>{id_name}</b>: {id_value}</p><p><b>{description_name}:</b></p><p>{description_value}</p><p><b>{threat_name}:</b></p><p>{threat_value}</p>',
            solution: '<p><b>{solution_name}:</b></p><p>{solution_value}</p><p><b>{links_name}:</b></p><p>{links_value}</p>',
            issues: '{issues}',
            stats: '<p><b>{stats_name}:</b></p><p>{stats_value} ms</p>'
        },
        loading: "<div class='loading'></div>",
        field: "<div class='field'><div class='name'>{name}</div><div class='value'>{value}</div></div>",
        object: "<div class='objects-list'><div class='object-type'>{object_type}:</div><div class='object-name' data-id='{objectId}'><a href='#'>{object_name}</a></div></div>",
        objectReview: '<div class="issue-list"><select class="current-issue"><option value="1">Issue 1</option><option value="2">Issue 2</option><option value="3">Issue 3</option></select></div></div><div class="block-tabs"><a href="#" data-name="code" class="block-tab active">Code</a><a href="#" data-name="description" class="block-tab">Description</a><a href="#" data-name="solution" class="block-tab">Solution</a><a href="#" data-name="stats" class="block-tab">Stats</a></div><div class="block-content"><div class="block-content-tab code active"><pre class="line-numbers language-{language}" data-line="{highlightLines}" data-start="{startLine}"><code>{code}</code></pre></div><div class="block-content-tab description">{descriptionFields}<hr /><div class="description-block textBlock"><h4>Description</h4><div class="indent">{description}</div></div><div class="threat-block textBlock"><h4>Threat</h4><div class="indent">{threat}</div></div></div><div class="block-content-tab solution"><div class="threat-block textBlock"><h4>Solution</h4><div class="indent">{solution}</div><h4>Links</h4><div class="indent">{links}</div></div></div><div class="block-content-tab stats">stats</div></div>'
    };
    this.getResultsStructure = function (){
        var o = this,
            params = {
                scanId: o.parentObj.data.scan.id
            };
        this.section.append(this.templates.loading);
        ERPScan.query("resultsStructure", "get", {
            type: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            params: JSON.stringify(params),
            callback: function (response){
                var convertedResults = o.convertResultsForTreetable(response["data"]["result"]);
                o.buildCategoriesTable(convertedResults);
            }
        });
    };
    this.conversionFieldsRules = {
        "resultsStructureTable": [
            {
                name: "name",
                getVal: function(fields){
                    return {
                        value: fields["name"]["en_US"],
                        attr: {
                            name: "structure_item_id",
                            value: fields["id"]
                        }
                    }
                }
            },
            {
                name: "resultCountOverall",
                getVal: function(fields){
                    return fields["resultCountOverall"];
                }
            }
        ],
        "resultsTable": [
            {
                name: "id",
                getVal: function(fields){
                    return fields["id"];
                }
            },
            {
                name: "projectResult",
                getVal: function(fields){
                    return fields["projectResult"];
                }
            },
            {
                name: "checkId",
                getVal: function(fields){
                    return this.parentObj.info.checks[fields["check"]]["name"];
                }
            },
            {
                name: "issues",
                getVal: function(fields){
                    return fields["aggregatedStatus"]["positive"] + "";
                }
            },
            {
                name: "exploitable",
                getVal: function(fields){
                    return "";
                }
            },
            {
                name: "sid",
                getVal: function(fields){
                    return fields["projectResult"]["credential"]["sid"];
                }
            },
            {
                name: "host",
                getVal: function(fields){
                    return fields["projectResult"]["credential"]["host"];
                }
            },
            {
                name: "appServer",
                getVal: function(fields){
                    return this.parentObj.info.appservers[fields["projectResult"]["credential"]["appServer"]["id"]]["name"]
                }
            },
            {
                name: "service",
                getVal: function(fields){
                    return this.parentObj.info.appservers[fields["projectResult"]["credential"]["appServer"]["id"]]["services"][fields["projectResult"]["credential"]["service"]["id"]]["name"]
                }
            },
            {
                name: "check",
                getVal: function(fields){
                    return fields["check"]
                }
            }
        ]
    };
    this.filterNeededFields = function (resultItem, namespace){
        var o = this,
            hierarchyElem = {};
        this.conversionFieldsRules[namespace].forEach(function(field){
            hierarchyElem[field["name"]] = field.getVal.call(o, resultItem);
        });
        return hierarchyElem;
    };
    this.convertResultsForTreetable = function(results) {
        var o = this,
            obj = {},
            rowId = 'type',
            convertedData = [];
        createHierarchy(results, 1, rowId);
        function createHierarchy(hierarchyElem, level, rowId) {
            hierarchyElem.forEach(function (resultItem) {
                obj = o.filterNeededFields(resultItem, "resultsStructureTable");
                obj["_id"] = rowId + '_' + level;
                if (rowId != "type") {
                    obj["_parent"] = rowId;
                }
                convertedData.push(obj);
                if (resultItem["children"].length > 0) {
                    createHierarchy(resultItem["children"], 1, obj["_id"]);
                } else {
                    convertedData.push({
                        "_id": obj["_id"] + "_" + 1,
                        "_parent": obj["_id"],
                        "name": "_delete"
                    });
                }
                level++;
            });
        }
        return convertedData;
    };
    this.buildCategoriesTable = function (data){
        var params = {
            title: this.locale["resultsTableTitle"],
            data: data,
            ticks: {"__addition": true},
            option: {
                deleteFields: [],
                expandable: false,
                namespace: "projects",
                type: "resultsCategories",
                tree: true,
                treeColumn: 1
            }
        };
        this.controls.categoriesTable = new Table(params);
        var html = this.controls.categoriesTable.getHTML();
        this.section.find(".loading").remove();
        this.section.append(html);
        this.controls.categoriesTable.build(params.data, params.option);
        this.handlers();
        this.section.find("table.treetable td:contains('_delete')").closest("tr").remove();
    };
    this.getResults = function ($row){
        var o = this;
        $row.find(".resultsRow").append(this.templates.loading);
        $row.find("a.showResults").addClass("active").text(this.locale["hideResults"]);
        ERPScan.query("results", "list", {
            params: {
                scanId: o.parentObj.data.scan.id,
                structureItemId: $row.find("td[data-structure_item_id]").data("structure_item_id"),
                limit: 10
            },
            callback: function(response) {
                if(response["data"]["result"].length > 0) {
                    var convertedResults = o.convertResultsForTable(response["data"]["result"], "resultsTable");
                    o.buildResultsTable(convertedResults, $row);
                    o.resizeResultRow($row);
                }
            }
        });
    };
    this.convertResultsForTable = function (results, namespace){
        var o = this,
            convertedResults = [];
        results.forEach(function (result){
            convertedResults.push(o.filterNeededFields(result, namespace));
        });
        return convertedResults;
    };
    this.buildResultsTable = function (results, $row){
        var params = {
            data: results,
            option: {
                pager: false,
                expandable: false,
                namespace: "projects",
                type: "result",
                count_one_page: 10
            }
        };
        this.controls.resultsTable = new Table(params);
        var html = this.controls.resultsTable.getHTML();
        $row.find(".loading").remove();
        $row.find(".resultsRow").append(html);
        this.controls.resultsTable.build();
        if(typeof this.data["results"] == "undefined") {
            this.data["results"] = {};
        }
        this.data["results"][this.controls.resultsTable.number] = results;
    };
    this.buildBlocks = function() {
        var descriptionBlock = new Block({
            title: this.locale.descriptionBlock.title,
            content: this.locale.descriptionBlock.empty,
            class: "checkDescription"
        }).getHTML();
        this.section.html(descriptionBlock);
    };
    this.showIssues = function(resultInfo) {
        var params = {
            ticks: {"__addition": true},
            title: "Issue list",
            data: this.parentObj.conversionFields(this.data.issues, this.parentObj.conversionFieldRules.issues[this.data.issues[0]["$type$"]]),
            option: {
                expandable: false,
                namespace: "projects",
                type: this.data.issues[0]["$type$"],
                count_one_page: 50
            }
        };
        this.controls.issues = new Table(params);
        var $table = $(this.controls.issues.getHTML()),
            table_id = $table.attr("id"),
            issues = $table.find("table")[0].outerHTML;
        var checkInfo = this.parentObj.info.checks[resultInfo.check];
        var description_content = this.templates.block_content.description.format({
            id_name: this.locale.descriptionBlock.content.description.id,
            id_value: checkInfo["name"],
            description_name: this.locale.descriptionBlock.content.description.description,
            description_value: checkInfo["description"]["en_US"],
            threat_name: this.locale.descriptionBlock.content.description.threat,
            threat_value: checkInfo["threat"]["en_US"]
        });
        var solution_content = this.templates.block_content.solution.format({
            solution_name: this.locale.descriptionBlock.content.solution.solution,
            solution_value: checkInfo["solution"]["en_US"],
            links_name: this.locale.descriptionBlock.content.solution.links,
            links_value: checkInfo["links"]["en_US"]
        });
        var issues_content = this.templates.block_content.issues.format({
            issues: "<div class='' id='" + table_id + "'>" + issues + "</div>"
        });
        var stats_content = this.templates.block_content.stats.format({
            stats_name: this.locale.descriptionBlock.content.stats.runtime,
            stats_value: ""
        });
        var tabs = new Tabs({
            container: this.section.find(" .checkDescription .block-content"),
            locale: this.locale.descriptionBlock.tabs,
            tabs: {
                issues: issues_content,
                description: description_content,
                solution: solution_content,
                stats: stats_content
            }
        });
        var content = tabs.getHTML();
        this.section.find(" .checkDescription .block-content").html(content);
        this.section.find(" .checkDescription h3").html(checkInfo["title_" + Locale.name]);
        this.controls.issues.build();
    };
    this.loadIssues = function(resultInfo) {
        var o = this;
        this.section.find(".checkDescription .block-content").html(this.templates.loading);
        ERPScan.query("issues", "list", {
            params: {
                resultId: resultInfo["projectResult"].id
            },
            callback: function(result) {
                o.data.issues = result.data.result;
                if(result.data.total == 0) {
                    o.section.find(".block.checkDescription .block-content").html("No results");
                } else {
                    o.showIssues.call(o, resultInfo);
                }
            }
        });
    };
    this.showObjects = function($row, issueInfo) {
        var objects_html = this.templates.object.format({
                object_type: issueInfo["$type$"],
                object_name: issueInfo.object.objectName,
                objectId: issueInfo.object.objectId
            }),
            fields = [],
            tempObj = {};
        $.each(issueInfo.fields, function(key, val){
            tempObj = {};
            tempObj['name'] = key;
            tempObj['value'] = val;
            fields.push(tempObj);
        });
        this.controls.issues.showAllFields($row);
        $row.find('.addition-field .arrow').after(objects_html);
        $row.removeClass("loading");
    };
    this.resizeResultRow = function($row) {
        var $resultsRow = $row.find(".resultsRow"),
            cellPaddingBtm = 0;
        $resultsRow.css({
            width: $row.width() - parseInt($resultsRow.css("paddingLeft")) - parseInt($resultsRow.css("paddingRight"))
        });
        if($resultsRow.is(":visible")) {
            cellPaddingBtm = $resultsRow.height() + $resultsRow.position().top;
        }
        $row.find("td.toggle-info").css({
            paddingBottom: cellPaddingBtm
        });
    };
    this.insertResultsRow = function($row) {
        var level = $row.data("tt-id").substr($row.data("tt-id").indexOf("_")).split("_").length - 1;
        $row.find("td.toggle-info").html(this.templates.resultRow.format({
            number: $row.attr("data-number"),
            content: this.templates.showResults.format({text: this.locale.showResults})
        }));
        $row.find(".resultsRow").css({
            paddingLeft: $row.find(".toggle-info").outerWidth() + ($row.find("span.indenter").width() * level)
        });
        this.resizeResultRow($row);
    };
    this.removeResultsRow = function($row){
        var o = this,
            table = $row.closest("table"),
            $resultRow = $row.find(".resultsRow"),
            tableId = $resultRow.find(".table").attr("id");
        $resultRow.find(".table").remove();
        if(typeof tableId != "undefined") {
            delete o.data["results"][tableId.substr(tableId.indexOf("-") + 1, tableId.length)];
        }
        $row.find("a.showResults").removeClass("active").text(this.locale["showResults"]);
        this.resizeResultRow($row);
    };
    this.loadObject = function(id) {
        var o = this;
        ERPScan.query("objects", "list", {
            callback: function(response) {
                o.showObject.call(o, response["data"]["result"][0]);
            },
            params: {
                id: id
            }
        });
    };
    this.showObject = function(data) {
        var startLine = 532,
            number = 30,
            o = this,
            checkObj = this.parentObj.info.checks[o.data.resultInfo.check],
            fields = this.templates.field.format({
                    name: "Name:",
                    value: checkObj["common"]["name"]
                }) + this.templates.field.format({
                    name: "Exploitable:",
                    value: checkObj["info"]["vulnerabilityInfo"] != null && checkObj["info"]["vulnerabilityInfo"]["exploitsMask"] != null ? "Possible" : "Impossible"
                }) + this.templates.field.format({
                    name: "Risk:",
                    value: Locale.fields["projects"]["issues"]["risk"][ERPScan.dictionaries["issue"]["aggregatedRisk"][checkObj["info"]["criticality"]]]
                }) + this.templates.field.format({
                    name: "Criticality:",
                    value: Locale.fields["projects"]["issues"]["risk"][ERPScan.dictionaries["issue"]["aggregatedRisk"][checkObj["info"]["criticality"]]]
                }) + this.templates.field.format({
                    name: "Probability:",
                    value: Locale.fields["projects"]["issues"]["risk"][ERPScan.dictionaries["issue"]["aggregatedRisk"][checkObj["info"]["probability"]]]
                }) + this.templates.field.format({
                    name: "Risk type:",
                    value: checkObj["info"]["riskType"]
                });
        var popupObj = new Popup({
            class: "object",
            title: data["objectName"],
            content: this.templates.objectReview.format({
                language: "xml",
                highlightLines: "2-6, 9, 16, 19, 27-32",
                startLine: startLine,
                code: data["fields"]["code"]["value"].encodeHTML(),
                descriptionFields: fields,
                description: checkObj["view"]["description"][Locale.name],
                threat: checkObj["view"]["threat"][Locale.name],
                solution: checkObj["view"]["solution"][Locale.name],
                links: checkObj["view"]["links"][Locale.name]
            })
        });
        popupObj.show();
        Prism.highlightAll();
        $(".popup.object pre .line-highlight").eq(0).addClass("dataFlow");
        $(".popup.object pre .line-highlight").eq(1).addClass("highlighted");
        $(".popup.object pre .line-highlight").eq(2).addClass("highlighted deleted");
        $(".popup.object pre .line-highlight").eq(3).addClass("deleted");
        $(".popup.object pre .line-highlight").eq(4).addClass("autopatched");
        $(".popup.object pre .line-numbers-rows span:lt(32):gt(25)").html("<span class='action-symbol'>+</span>").addClass("no-number");
        $(".popup.object pre .line-numbers-rows span:eq(15)").html("<span class='action-symbol highlighted'>-</span>");
        $(".popup.object pre .line-numbers-rows span:eq(8)").html("<span class='action-symbol highlighted'></span>");
        $(".popup.object pre .line-numbers-rows span:eq(20)").html("<span class='action-symbol'>-</span>");
        var widthNumbersBlock = 35,
            digitNumbers = Math.ceil(Math.log(startLine + number + 1) / Math.log(10));
        widthNumbersBlock += digitNumbers * 7;
        $(".popup.object pre .line-numbers-rows").css("left", -widthNumbersBlock + "px");
        $(".popup.object pre.line-numbers").css("padding-left", widthNumbersBlock + "px");
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", o.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tbody tr", function(e) {
            var $this = $(this),
                $target = $(e.target);
            if ($target.hasClass(".addition-field") || $target.parents(".addition-field").length > 0) {
                return false;
            }
            if ($this.hasClass("active")) {
                o.controls.issues.hideAllFields($this);
            } else {
                o.showObjects.call(o, $this, o.data.issues[$this.data("number")]);
            }
            if ($target.is(".object-name a")) {
                o.loadObject($target.data("id"));
                return false;
            }
            return false;
        });
        $(document.body).on("click", o.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tr.active .toggle-info a", function(e) {
            var $tr =  $(this).parents("tr.active"),
                $target = $(e.target);

            if ($target.is(".object-name a")) {
                o.loadObject($target.data("id"));
                return false;
            }
            o.controls.issues.hideAllFields($tr);
            return false;
        });
        $(document.body).on("click", o.section.selector + " .resultsRow table.tablesorter tbody tr", function() {
            var $this = $(this),
                table = $this.closest('.table'),
                tableId = table.attr("id"),
                rows = table.find('tr');
            o.data.resultInfo = o.data["results"][tableId.substr(tableId.indexOf("-") + 1, tableId.length)][$this.data("number")];
            rows.removeClass("active");
            $this.toggleClass("active");
            $this.toggleClass("leaf");
            o.loadIssues(o.data.resultInfo);
        });
        $(document.body).on("mousedown", o.section.selector + " .table table.tablesorter tbody tr.branch", function(e) {
            var $this = $(this),
                $resultsRow = $this.find(".resultsRow"),
                target = $(e.target),
                expanded = $this.hasClass("expanded");
            if (target.is(".resultsRow table.tablesorter tbody tr td")) {
                return false;
            }
            if(target.is($this.find(".showResults"))) {
                if (target.hasClass("active")) {
                    o.removeResultsRow($this);
                } else {
                    o.getResults($this.closest("tr"));
                }
                return false;
            }
            if (target.parent(".indenter").length == 0) {
                o.section.find(".table table.tablesorter").treetable(expanded ? "collapseNode" : "expandNode", $this.data("ttId"));
            }
            if(expanded) {
                $resultsRow.hide();
            } else {
                if($resultsRow.length == 0) {
                    o.insertResultsRow($this);
                } else {
                    $resultsRow.show();
                }
            }
            o.resizeResultRow($this);
        });
        $(document.body).on("click", o.sectionName + " .checkDescription .block-tabs .block-tab", function() {
            var $this = $(this);
            var name = $this.data("name");
            $(o.sectionName + " .checkDescription .block-content-tab").removeClass("active");
            $(o.sectionName + " .checkDescription .block-content-tab." + name).addClass("active");
            $(o.sectionName + " .checkDescription .block-tabs .block-tab").removeClass("active");
            $this.addClass("active");
            return false;
        });
        $(document.body).on("click", ".popup.object .block-tabs .block-tab", function() {
            var $this = $(this);
            var name = $this.data("name");
            $(".popup.object .block-content-tab").removeClass("active");
            $(".popup.object .block-content-tab." + name).addClass("active");
            $(".popup.object .block-tabs .block-tab").removeClass("active");
            $this.addClass("active");
            return false;
        });
        $(o.section.selector + ' .checkDescription iframe.blockResize')[0].contentWindow.onresize = function() {
            var activeRow = o.section.find('.checkDescription [data-name="issues"] tbody tr.active'),
                additionField = activeRow.find('.addition-field');
            additionField.outerWidth(activeRow.width());
        };
        $(document.body).on("click", o.section.selector + " .line-highlight.autopatched", function() {
            var $this = $(this);
        });
    };
    this.clearEvents = function(){
        $(this.section.selector + ' iframe.blockResize')[0].contentWindow.onresize = null;
        $(document.body).off("click", this.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tbody tr");
        $(document.body).off("click", this.section.selector + " .block.checkDescription [data-name='issues'] table.tablesorter tr.active .toggle-info a");
        $(document.body).off("click", this.section.selector + " .resultsRow table.tablesorter tbody tr");
        $(document.body).off("mousedown", this.section.selector + " .table table.tablesorter tbody tr.branch");
        $(document.body).off("click", this.section.selector + " .line-highlight.autopatched");
    };
    this.clear = function() {
        this.clearEvents();
        this.section.empty();
    };
    this.load = function() {
        this.buildBlocks();
        this.getResultsStructure();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    }
};