Page.Render = {
    path: ['assess', 'projects', 'all'],
    init: function() {
        var fields = ["projects"];
        Page.loadFieldsConfig(fields);
        Locale.set("en", { fields: fields });
        Locale.loadPage(Page.Render.path);
        Header.load(Page.Render.path, { tabs: true });
        new Jobs.All().init();
        var scripts_additional = ["tests", "results", "statistic", "sap_notes", "code_type", "code_program", "access_control", "sod_matrix", "sod_table", "sod_graph", "sod_conflicts", "sod_conflicts_matrix", "sod_roles_by_actions", "information"];
        for (var i = 0; i < scripts_additional.length; i++) {
            ERPScan.loader.add.JS("renders/" + Page.Render.path[0] + "/" + Page.Render.path[1] + "/" + scripts_additional[i]);
        }
    }
};

var Jobs = { };

Jobs.All = function() {
    this.controls = {};
    this.section = $("section.content.tab-all");
    this.sectionName = '.tab-all';
    this.info = {
        checks: {},
        policies: {},
        users: {},
        appservers: {}
    };
    this.data = {};
    this.tabs = {
        "VM": {
            vulnerabilities: "Results",
            misconfigurations: "Results",
            statistic: "Statistic"
        },
        "CODE": {
            codeIssuesByType: "CodeIssuesByType",
            codeIssuesByProgram: "CodeIssuesByProgram",
            statistic: "Statistic"
        },
        "SOD": {
            accessControl: "AccessControl",
            SODRisksMatrix: "SoDMatrix",
            SODRisksTable: "SoDTable",
            // sod_graph: "SoDGraph",
            sod_conflicts_by_users_table: "SoDConflictsByUsersTable",
            sod_conflicts_by_users_matrix: "SoDConflictsByUsersMatrix",
            sod_roles_by_actions: "SoDRolesByActions"
        },
        "ALL": {
            before: {
                test: "Test"
            },
            after: {
                // information: "Information"
            }
        }
    };
    this.activeTabs = {};
    this.enumeratedField = {
        "project": ["type", "state", "system"],
        "scan": ["type"],
        "asset": ["type", "class"],
        "result": ["aggregatedStatus", "aggregatedRisk"],
        "issue": ["aggregatedStatus", "runStatus", "userOpinionStatus", "aggregatedRisk", "riskSource", "riskUserOpinion"]
    };
    this.templates = {
        no_entry: "<p>" + Locale.ui.page.list.no_entry + "</p>",
        loading_scans: "<div class='scans'><p>" + Locale.ui.page.list.loading + "</p></div>",
        create: "<div class='create-button'><button>" + Locale.ui.page.editor.buttons.new + "</button></div>",
        loading: "<div class='loading'></div>",
        statistic: "<div class='statistic'>{fields}</div>",
        statistic_field: "<div class='field'><div class='name'>{name}</div><div class='value'>{value}</div></div>"
    };
    this.conversionFieldRules = {
        "accessControl": {
            "check_id": function(fields, name) {
                return {
                    value: this.info.checks[fields["check"]]["UUID"],
                    attr: { name: 'check_id', value: fields["check"] }
                };
            },
            "check_name": function(fields, name) {
                return this.info.checks[fields["check"]]["title_" + Locale.name];
            },
            "aggregatedStatus": function (fields, name) {
                return fields[name]["negative"] > 0 ? "negative" : "positive";
            },
            "aggregatedRisk": function(fields, name) {
                var max_risk = 0,
                    max_risk_type = null;
                for (var risk in fields[name]) {
                    if (fields[name][risk] > max_risk) {
                        max_risk = max_risk;
                        max_risk_type = risk;
                    }
                }
                return max_risk_type;
            },
            "issues": function (fields, name) {
                return {
                    value: fields["aggregatedStatus"]["negative"],
                    attr: {
                        name: "result_id",
                        value: fields["id"]
                    }
                };
            },
            "sid": function (fields, name) {
                return fields["credential"]["sid"];
            },
            "host": function (fields, name) {
                return fields["credential"]["host"];
            },
            "appServer": function (fields, name) {
                return {
                    value: this.info.appservers[fields["credential"]["appServer"]]["name"],
                    attr: {
                        name: "id",
                        value: fields["credential"]["appServer"]
                    }
                };
            },
            "entity": function (fields, name) {
                var entity = Helpers.getSingleProperty(fields["credential"]["customParameters"]);
                if (entity == null) {
                    entity = "service";
                }
                var entity_name = Locale.fields['projects']["projects_result"][entity],
                    value;

                if (entity == "service") {
                    value = entity_name + ": " + this.info.appservers[fields["credential"]["appServer"]]["services"][fields["credential"]["service"]]["name"];
                    entity += "|" + fields["credential"]["service"];
                } else {
                    value = entity_name + ": " + fields["credential"]["customParameters"][entity];
                    entity += "|" + fields["credential"]["customParameters"][entity];
                }
                return {
                    value: value,
                    attr: {
                        name: "type",
                        value: entity
                    }
                }
            },
            "exploitable": function (fields, name) {
                return fields[name];
            }
        },
        "codeIssues": {
            "program_name": function(fields, name){
                return fields["i.fields.abapProgram.programName"];
            },
            "check_id": function(fields, name) {
                return {
                    value: this.info.checks[fields["r.check"]]["UUID"],
                    attr: { name: 'check_id', value: fields["r.check"] }
                };
            },
            "check_name": function(fields, name) {
                return this.info.checks[fields["r.check"]]["title_" + Locale.name];
            },
            "aggregatedStatus": function (fields, name) {
                return fields["r.aggregatedStatus"]["negative"] > 0 ? "negative" : "positive";
            },
            "aggregatedRisk": function(fields, name) {
                var max_risk = 0,
                    max_risk_type = null;
                for (var risk in fields["r.aggregatedRisk"]) {
                    if (fields["r.aggregatedRisk"][risk] > max_risk) {
                        max_risk = max_risk;
                        max_risk_type = risk;
                    }
                }
                return max_risk_type;
            },
            "issues": function (fields, name) {
                return {
                    value: fields["r.aggregatedStatus"]["negative"],
                    attr: {
                        name: "result_id",
                        value: fields["r.id"]
                    }
                };
            },
            "sid": function (fields, name) {
                return fields["r.credential"]["sid"];
            },
            "host": function (fields, name) {
                return fields["r.credential"]["host"];
            },
            "appServer": function (fields, name) {
                return {
                    value: this.info.appservers[fields["r.credential"]["appServer"]]["name"],
                    attr: {
                        name: "id",
                        value: fields["r.credential"]["appServer"]
                    }
                };
            },

            "entity": function (fields, name) {
                var entity = Helpers.getSingleProperty(fields["r.credential"]["customParameters"]);
                if (entity == null) {
                    entity = "service";
                }
                var entity_name = Locale.fields['projects']["projects_result"][entity],
                    value;

                if (entity == "service") {
                    value = entity_name + ": " + this.info.appservers[fields["r.credential"]["appServer"]]["services"][fields["r.credential"]["service"]]["name"];
                    entity += "|" + fields["r.credential"]["service"];
                } else {
                    value = entity_name + ": " + fields["r.credential"]["customParameters"][entity];
                    entity += "|" + fields["r.credential"]["customParameters"][entity];
                }
                return {
                    value: value,
                    attr: {
                        name: "type",
                        value: entity
                    }
                }
            },
            "exploitable": function (fields, name) {
                return 'true';
            }
        },
        "jobs": {
            "__check": function(){
                return '<input type="checkbox" class="table-mark">';
            },
            "scansCount": function(fields, name) {
                return fields[name].toString();
            },
            "author": function(fields, name) {
                return typeof this.info.users[fields[name]] != "undefined" ? this.info.users[fields[name]].login : "[Deleted user]";
            },
            "created": function(fields, name) {
                var created = new Date(fields[name]).format('d/m/y H:i');
                return {
                    value: created,
                    attr: { name: 'ts', value: fields[name] }
                };
            },
            "state": function(fields, name) {
                return {
                    value: Locale.ui.page.fields.state[fields[name]],
                    attr: { name: "state", value: fields[name] }
                };
            },
            "name": function(fields, name) {
                return {
                    value: fields[name],
                    attr: { name: "id", value: fields["id"] }
                };
            },
            "type": function(fields, name) {
                return {
                    value: Locale.ui.page.fields.type[fields[name]],
                    attr: { name: "type", value: fields[name] }
                };
            },
            "statistic": function(fields, name) {
                if (fields[name] == null) {
                    return "";
                }
                var statistic_html = "",
                    statistic = fields[name][fields["type"]];
                for (var statistic_field in statistic) {
                    var value = statistic[statistic_field].value;
                    switch(statistic[statistic_field].format) {
                        case "percent":
                            value = value + "%";
                            break;
                    }
                    statistic_html += this.templates.statistic_field.format({
                        name: Locale.fields["projects"]["projects_list_statistic"][statistic_field] + ":",
                        value: value
                    });
                }
                return {
                    value: this.templates.statistic.format({ fields: statistic_html }),
                    attr: { name: "cell", value: 'statistic' }
                }
            }
        },
        "scans": {
            "__check": function(){
                return '<input type="checkbox" class="simple-table_mark">';
            },

            "number": function(fields, name) {
                return {
                    value: ++this.data.scansCount,
                    attr: { name: 'id', value: fields.id}
                }
            },
            "state": function(fields, name) {
                return {
                    value:  Locale.ui.page.fields.state[fields[name]] ? Locale.ui.page.fields.state[fields[name]] : '' ,
                    attr: { name: "state", value:  fields[name] }
                };
            },
            "finishDate": function(fields, name) {
                var finished = fields[name] == null ? "—" : new Date(fields[name]).format('d/m/y H:i');
                return {
                    value: finished,
                    attr: { name: 'ts', value: fields[name] }
                }
            },
            "startDate": function(fields, name) {
                var created;
                if(fields[name] == null) {
                    created = new Date().format('d/m/y H:i');
                } else {
                    created = new Date(fields[name]).format('d/m/y H:i');
                }
                return {
                    value: created,
                    attr: { name: 'ts', value: fields[name] }
                };
            },
            "readiness": function(fields, name) {
                return fields[name].toFixed(2);
            },
            "statistic": function(fields, name) {
                if (fields[name] != null) {
                    var statistic_html = "",
                        statistic = fields[name][fields["type"]];
                    for (var statistic_field in statistic) {
                        var value = statistic[statistic_field].value;
                        switch(statistic[statistic_field].format) {
                            case "percent":
                                value = value + "%";
                                break;
                        }
                        statistic_html += this.templates.statistic_field.format({
                            name: Locale.fields["projects"]["projects_list_statistic"][statistic_field] + ":",
                            value: value
                        });
                    }
                    return {
                        value: this.templates.statistic.format({ fields: statistic_html }),
                        attr: { name: "cell", value: 'statistic' }
                    }
                } else {
                    return '';
                }
            }
        },
        "issues": {
            "AbapProgramIssue": {
                "codePatch": function(fields, name){
                    return fields[name];
                },
                "aggregatedRisk": function(fields, name){
                    return fields[name];
                },
                "programName": function(fields, name){
                    return fields["abapProgramObject"]["programName"];
                },
                "author": function(fields, name){
                    return fields["abapProgramObject"]["author"];
                }
            },
            "CriticalUserIssue": {
                "aggregatedRisk": function(fields, name){
                    return fields[name];
                },
                "userName": function(fields, name){
                    return fields["userObject"]["userName"];
                },
                "password": function(fields, name){
                    return fields["password"];
                },
                "uniqueId": function(fields, name){
                    return fields["userObject"]["name"];
                }
            },
            "VulnerableComponentIssue": {
                "aggregatedRisk": function(fields, name){
                    return fields[name];
                },
                "sapNoteNumber": function(fields, name){
                    return fields[name];
                },
                "uniqueId": function(fields, name){
                    return fields["componentObject"]["name"];
                },
                "componentName": function(fields, name){
                    return fields["componentObject"][name];
                },
                "componentVersion": function(fields, name){
                    return fields["componentObject"][name];
                }
            },
            "SapnoteIssue": {
                "aggregatedRisk": function(fields, name){
                    return fields[name];
                },
                "uniqueId": function(fields, name){
                    return fields["sapnoteNumberObject"]["name"];
                },
                "applicationArea": function(fields, name){
                    return fields["sapnoteNumberObject"][name];
                },
                "description": function(fields, name){
                    return fields["sapnoteNumberObject"][name]
                },
                "release": function(fields, name){
                    return fields["sapnoteNumberObject"][name];
                },
                "sapNoteNumber": function(fields, name){
                    return fields["sapnoteNumberObject"][name];
                },
                "probability": function(fields, name){
                    return fields["sapnoteNumberObject"][name];
                }
            },
            "testIssueType": {
                "aname": function(fields, name){
                    return fields["fields"]["aname"];
                },
                "bname": function(fields, name){
                    return fields["fields"]["bname"];
                }
            },
            "CriticalConnectionIssue": {
                "aggregatedRisk": function(fields, name){
                    return fields[name];
                },
                "uniqueId": function(fields, name){
                    return fields["connectionObject"]["name"];
                },
                "connName": function(fields, name){
                    return fields["connectionObject"][name];
                },
                "connType": function(fields, name){
                    return fields["connectionObject"][name];
                },
                "hostDest": function(fields, name){
                    return fields["connectionObject"][name];
                },
                "hostSource": function(fields, name){
                    return fields["connectionObject"][name];
                }
            }
        },
        "asset" : {
            "sid" : function(fields){
                return {
                    value: fields.sid,
                    attr: { name: "uid", value: fields.id }
                }
            }
        },
        "policyStructures" : {
            "name" : function(fields){
                return {
                    value: fields.name,
                    attr: { name: "uid", value: fields.id }
                }
            }
        },
        "policyRules" : {
            "name" : function(fields){
                return {
                    value: fields.name,
                    attr: { name: "uid", value: fields.id }
                }
            }
        }
    };
    this.queries = {
        sapnoteIssues: function(scanId) {
            return {
                title: "Sapnote issues",
                id: "sapnoteIssues",
                queryObj: {
                    "from": [
                        {
                            entity: "Issue"
                        }
                    ],
                    "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.issueType = :type2",
                    "fields": [
                        "i.vulnerableComponent.status",
                        "i.sapnote.status"
                    ],
                    "parameters": [
                        {
                            name: "type1",
                            value: "sapnote"
                        },
                        {
                            name: "type2",
                            value: "vulnerableComponent"
                        },
                        {
                            name: "scanId",
                            value: scanId
                        }
                    ]
                }
            }
        }
    };
    this.loadJobs = function() {
        var o = this;
        ERPScan.API.query("projects", "list", {
            callback: function(response) {
                o.data.projects = response["data"]["result"];
                var result = o.conversionFields.call(o, response["data"]["result"], o.conversionFieldRules.jobs);
                o.showJobs.call(o, result);
            }
        });
    };
    this.showJobs = function(data) {
        var actions = {};
        actions[Locale.ui.page.actions.start] = [];
        actions[Locale.ui.page.actions.share] = [];
        actions[Locale.ui.page.actions.edit] = [];
        actions[Locale.ui.page.actions.delete] = [];
        actions[Locale.ui.page.actions.hidden] = [];
        var params = {
            ticks: {"__check": true, "__addition": true},
            actions: actions,
            title: "Job list",
            data: data,
            option: {
                expandable: false,
                namespace: "projects",
                type: "project",
                sort_default: { column: 2, order: "desc" }
            }
        };
        this.controls.projectsTable = new Table(params);
        var html = this.controls.projectsTable.getHTML(),
            section = $("section.content");
        section.html(this.templates.create + html);
        this.controls.projectsTable.build();
        this.handlers(this.controls.projectsTable.number);
        Helpers.setBlockHeight(section);
    };
    this.loadScans = function(o) {
        var $this = $(this);
        if ($this.hasClass("active"))
            return false;
        $this.addClass("loading");
        var id = $this.find("td").eq(o.controls.projectsTable.getColumnNumber("name")).data("id"),
            type = $this.find("td").eq(o.controls.projectsTable.getColumnNumber("type")).data("type");
        ERPScan.API.query("scans", "list", {
            params: {
                projectId: id
            },
            callback: function(responce) {
                o.data.scanList = responce["data"]["result"];
                o.data.scansCount = 0;
                for (var i = 0; i < o.data.scanList.length; i++) {
                    o.data.scanList[i]["type"] = type;
                }
                var result = o.conversionFields.call(o, o.data.scanList, o.conversionFieldRules.scans);
                if(result.length === 0) {
                    o.controls.alert_obj.toggleShow('This project does not contain the scan');
                } else {
                    o.showScans.call(o, $this, result);
                }
            }
        });
    };
    this.showScans = function($row, scans) {
        this.controls.projectsTable.showAllFields($row, {
            title: "Scans",
            content: this.templates.loading_scans
        });
        var html = Locale.ui.page["no_scans"];
        if (scans.length != 0) {
            var actions = {};
            actions[Locale.ui.page.actions.resume] = [];
            actions[Locale.ui.page.actions.interrupt] = [];
            actions[Locale.ui.page.actions.delete] = [];
            this.controls.simpleTable_obj = new SimpleTable({
                actions: actions,
                ticks: {"__check": true},
                data: scans,
                option: {
                    namespace: 'projects',
                    type: 'scanList'
                }
            });
            html = this.controls.simpleTable_obj.init().getHTML();
        }
        var $scans = $row.find(".scans");
        $scans.html(html);
        if (scans.length == 0) {
            $scans.addClass("no_scans");
        }
        $row.removeClass("loading");

        /* change scans state from LocalStorage */
        var storageData = JSON.parse(localStorage.getItem('scansPseudoStates')),
            rows = $scans.find('.simple-table .simple-table-row');
        if (storageData) {
            $.each(rows, function(){
                var row = $(this),
                    id = row.find('.simple-table-cell[data-id]').attr('data-id');
                $.each(storageData, function(key){
                    if (id === this.id) {
                        var stateCell = row.find('.simple-table-cell[data-state]');
                        if (stateCell.attr('data-state') == this.prevState) {
                            stateCell.text(this.pseudoState);
                            stateCell.attr('data-state', this.pseudoState.toUpperCase());
                        } else {
                            storageData.splice(key);
                        }
                    }
                });
                if (storageData.length === 0) {
                    localStorage.removeItem('scansPseudoStates');
                } else {
                    localStorage.setItem('scansPseudoStates', JSON.stringify(storageData));
                }
            });
        }
    };
    this.setScanInfo = function($scan) {
        var $td = $scan.parents("tr.active").find("td"),
            $scanInfo = $scan.find(".simple-table-cell").eq(1),
            projectType = $td.eq(this.controls.projectsTable.getColumnNumber("type")).data('type'),
            projectID = $td.eq(this.controls.projectsTable.getColumnNumber("name")).data("id"),
            projectName = $td.eq(this.controls.projectsTable.getColumnNumber("name")).text(),
            projectAuthor = $td.eq(this.controls.projectsTable.getColumnNumber("author")).text(),
            scanNum = $scan.find(".simple-table-cell[data-id]").text(),
            system = $td.eq(this.controls.projectsTable.getColumnNumber("system")).text();
        this.data.scan = {
            projectID: projectID,
            projectName: projectName,
            projectAuthor: projectAuthor,
            projectType: projectType,
            number: $scanInfo.text(),
            id: $scanInfo.data("id"),
            scanNum: scanNum,
            system: system
        };
    };
    this.projectTypes = {
        "VM": "audit",
        "CODE": "audit",
        "SOD": "sod"
    };
    this.getAssetsForScan = function(){
        var o = this;
        ERPScan.API.query("assets", "list", {
            params: {
                id: this.data.scanList[this.data.scan.number - 1]["target"]["values"]
            },
            callback: function(response) {
                o.data.scan.assets = response.data.result;
                o.data.scan.appServersTree = o.buildAppserversTree(o.projectTypes[o.data.scan.projectType]);
                var tabForToggle = $("header .tabs a")[2].getAttribute("data-tab");
                o.toggleTab(tabForToggle, o.tabs[o.data.scan.projectType][tabForToggle]);
            }
        });
    };
    this.buildAppserversTree = function(projType){
        var o = this,
            tree = [],
            map = {},
            hierarchy = {
                "audit": [
                    {
                        elemType: "host",
                        getElements: function (structureItem){
                            return [structureItem["host"]]
                        }
                    },
                    {
                        elemType: "appServer",
                        getElements: function (structureItem){
                            return [o.info.appservers[structureItem["appServer"]]["name"]]
                        }
                    },
                    {
                        elemType: "entitiesGroup",
                        getElements: function (structureItem){
                            var entitiesGroups = [];
                            entitiesGroups.push("Services");
                            if(structureItem["appServerProperty"] != null) {
                                entitiesGroups.push(o.info.appserverproperties[structureItem["appServerProperty"]]["name"]);
                            }
                            return entitiesGroups;
                        }
                    },
                    {
                        elemType: "entity",
                        getElements: function (structureItem, parent){
                            if(parent == "Services") {
                                return [o.info.appservers[structureItem["appServer"]]["services"][structureItem["service"]]["name"]];
                            } else {
                                return [structureItem["appServerPropertyValue"]];
                            }
                        }
                    }
                ],
                "sod": [
                    {
                        elemType: "client",
                        getElements: function (structureItem){
                            if(typeof o.info.appserverproperties[structureItem["appServerProperty"]] != "undefined" && o.info.appserverproperties[structureItem["appServerProperty"]]["name"] == "client") {
                                return [structureItem["appServerPropertyValue"]]
                            }
                        }
                    }
                ]
            };
        this.data.scan.assets.forEach(function(asset){
            if(typeof map[asset["sid"]] == "undefined") {
                tree.push({
                    data: {
                        id: asset["sid"],
                        type: "sid"
                    },
                    nodes: [],
                    text: asset["sid"]
                });
                map[asset["sid"]] = {
                    index: tree.length - 1,
                    nodes: {}
                };
            }
            createHierarchy(0, map[asset["sid"]]["nodes"], tree[map[asset["sid"]]["index"]]["nodes"], asset);
        });
        function pushHierarchyElem(treeElem, currentElem, type, mapElem, parent){
            treeElem.push({
                data: {
                    id: currentElem,
                    type: type == "entitiesGroup" ? null : (parent == "Services" ? "service" : type)
                },
                text: currentElem
            });
            mapElem[currentElem] = {
                index: treeElem.length - 1
            };
        }
        function createHierarchy(level, mapElem, treeElem, asset, parent){
            if(asset["structure"].length > 0) {
                asset["structure"].forEach(function(structureItem){
                    var currentElems = hierarchy[projType][level].getElements(structureItem, parent);
                    if(typeof currentElems == "undefined") {
                        return true;
                    }
                    currentElems.forEach(function(elem){
                        if(typeof mapElem[elem] == "undefined") {
                            pushHierarchyElem(treeElem, elem, hierarchy[projType][level]["elemType"], mapElem, parent);
                            if (hierarchy[projType][level]["elemType"] == "entity") {
                                treeElem[mapElem[elem]["index"]]["data"]["id"] = structureItem["appServerProperty"];
                            }
                        }
                        if(level < hierarchy[projType].length - 1) {
                            treeElem[mapElem[elem]["index"]]["nodes"] = [];
                            mapElem[elem]["nodes"] = {};
                            createHierarchy(level + 1, mapElem[elem]["nodes"], treeElem[mapElem[elem]["index"]]["nodes"], asset, elem);
                        }
                    })
                });
            }
        }
        $.each(tree, function(key, val){
            if(val["nodes"].length == 0) {
                delete val["nodes"];
                val["data"]["empty"] = true;
            }
        });
        return tree;
    };
    this.plotsConfig = {
        plotTypes: {
            "VM": {
                "Misconfigurations": ["aggregatedStatus", "aggregatedRisk", "criticality", "probability"],
                "Vulnerabilities": ["aggregatedStatus", "aggregatedRisk", "criticality", "probability"],
                "sapnote": ["installed"],
                "vulnerableComponent": ["installed"]
            },
            "CODE": {
                "checks": ["aggregatedStatus", "aggregatedRisk"],
                "issues": ["aggregatedStatus", "aggregatedRisk"]
            }
        },
        criticality: {
            //sort by criticality
            "aggregatedStatus": ["negative", "positive"],
            "aggregatedRisk": ["critical", "high", "medium", "low", "unknown"],
            "criticality": ["critical", "high", "medium", "low", "no"],
            "probability": ["critical", "high", "medium", "low", "no"],
            "installed": ["uninstalled","installed"]
        },
        convertList: {
            criticality : {
                0: "no",
                1: "low",
                2: "medium",
                3: "high",
                4: "critical"
            },
            probability : {
                0: "no",
                1: "low",
                2: "medium",
                3: "high",
                4: "critical"
            },
            installed: {
                "Success": "installed",
                "Fail": "uninstalled"
            },
            sapnote: ["Success", "Fail"],
            vulnerableComponent: ["Success", "Fail"]
        }
    };
    this.getDataForLinearPlots = function(){
        var o = this;
        ERPScan.API.query("results", "list", {
            params: {
                projectId: o.data.scan.projectID
            },
            callback: function(response) {
                var data = {};
                $.each(response["data"]["result"], function(){
                    if (!data[this.scanId]) {
                        data[this.scanId] = [];
                    }
                    data[this.scanId].push(this);
                });
                o.data.scan.projectResultsByScan = data;
                var k = 1;//plug
                $.each(data, function(key, val){
                    o.convertDataForPlots(val, "linearPlots", k);
                    k++;
                });
                o.convertDataForPlots(o.data.scan.results, "circlePlots", 1);
            }
        });
    };
    this.convertDataForPlots = function(result, entity, k/*plug*/) {
        var o = this,
            scanId = result[0].scanId,
            tempObj = {},
            data = {};
        if(!o.data.scan.aggregatedResults) {
            o.data.scan.aggregatedResults = {};
        }
        if(!o.data.scan.aggregatedResults[entity]) {
            o.data.scan.aggregatedResults[entity] = {};
        }
        if(!o.data.scan.aggregatedResults.tempData) {
            o.data.scan.aggregatedResults.tempData = {};
        }
        $.each(o.plotsConfig.plotTypes[o.data.scan.projectType], function(key, val){
            data[key] = {};
            tempObj = {};
            for (var i = 0; i < val.length; i++) {
                tempObj[val[i]] = []
            }
            data[key] = tempObj;
        });
        if (o.data.scan.projectType == "VM") {
            var checkClass,
                tempObj = {
                    criticality: {},
                    probability: {},
                    aggregatedRisk: {},
                    aggregatedStatus: {},
                    sapnote: {},
                    vulnerableComponent: {}
                },
                tempProp = '',
                sapNoteQuery = this.queries.sapnoteIssues(o.data.scan.id);
            $.each(result, function(num, obj){
                checkClass = o.info.checksMapByPolicy[o.data.scan.projectType][obj.check];
                if (typeof checkClass != "undefined") {
                    $.each(o.plotsConfig.convertList.criticality, function(key, val){
                        tempObj.criticality[val] = key == o.info.checks[obj.check]["info"].criticality ? 1 : 0;
                        tempObj.probability[val] = key == o.info.checks[obj.check]["info"].probability ? 1 : 0;
                    });
                    data[checkClass].criticality.push(Helpers.cloneObject(tempObj.criticality));
                    data[checkClass].probability.push(Helpers.cloneObject(tempObj.probability));
                    data[checkClass].aggregatedRisk.push(this.aggregatedRisk);
                    data[checkClass].aggregatedStatus.push(this.aggregatedStatus);
                }
            });
            o.data.scan.aggregatedResults.tempData[scanId] = data;
            ERPScan.API.query("queries", "sapnoteIssues" + k, {
                params: {
                    query: JSON.stringify(sapNoteQuery.queryObj)
                },
                type: "POST",
                callback: function(response) {
                    var tempData = o.data.scan.aggregatedResults.tempData[scanId];
                    $.each(response.data.result, function(){
                        for (var i = 0; i < o.plotsConfig.convertList.sapnote.length; i++) {
                            tempProp = o.plotsConfig.convertList.sapnote[i];
                            tempObj.sapnote[o.plotsConfig.convertList.installed[tempProp]] = this["i.sapnote.status"] == tempProp ? 1 : 0;
                            tempObj.vulnerableComponent[o.plotsConfig.convertList.installed[tempProp]] = this["i.vulnerableComponent.status"] == tempProp ? 1 : 0;
                        }
                        tempData.sapnote['installed'].push(Helpers.cloneObject(tempObj.sapnote));
                        tempData.vulnerableComponent['installed'].push(Helpers.cloneObject(tempObj.vulnerableComponent));
                    });
                    if (entity === 'linearPlots') {
                        o.data.scan.aggregatedResults[entity][scanId] = o.aggregateScanResults.call(o, tempData);
                    } else {
                        o.data.scan.aggregatedResults[entity]= o.aggregateScanResults.call(o, tempData);
                    }
                }
            });
        }

        if (o.data.scan.projectType == "CODE") {
            var checkProps = o.plotsConfig.plotTypes.CODE.checks;
            tempObj = {
            };

            $.each(result, function(res, metricsData){
                $.each(checkProps, function(key, val){
                    tempObj[val] = {};
                    for (var i = 0; i < o.plotsConfig.criticality[val].length; i++) {
                        tempObj[val][o.plotsConfig.criticality[val][i]] = 0;
                    }
                    for (var i = 0; i < o.plotsConfig.criticality[val].length; i++) {
                        if (metricsData[val][o.plotsConfig.criticality[val][i]] > 0) {
                            tempObj[val][o.plotsConfig.criticality[val][i]] += 1;
                            break;
                        }
                    }
                });
                data.checks.aggregatedRisk.push(tempObj.aggregatedRisk);
                data.checks.aggregatedStatus.push(tempObj.aggregatedStatus);
                data.issues.aggregatedRisk.push(this.aggregatedRisk);
                data.issues.aggregatedStatus.push(this.aggregatedStatus);
            });
            if (entity === 'linearPlots') {
                o.data.scan.aggregatedResults[entity][scanId] = o.aggregateScanResults.call(o, data);
            } else {
                o.data.scan.aggregatedResults[entity]= o.aggregateScanResults.call(o, data);
            }
        }
    };
    this.aggregateScanResults = function(data){
        var o = this,
            aggr = {};
        $.each(this.plotsConfig.plotTypes[this.data.scan.projectType], function(entity, metrics){
            if (!aggr[entity]) {
                aggr[entity] = {};
            }
            for(var i = 0; i < metrics.length; i++) {
                aggr[entity][metrics[i]] = Helpers.aggregatePropsValue(data[entity][metrics[i]]);
            }
        });
        return aggr;
    };
    this.conversionFields = function(result, rules) {
        var data = [];
        for (var i = 0; i < result.length; i++) {
            var index = data.push({}) - 1;
            for (var field in rules) {
                if (typeof result[i][field] == "undefined" && rules[field] != null) {
                    data[index][field] = rules[field].call(this, result[i], field, i);
                }
            }
            for (var field in result[i]) {
                var isset = typeof rules[field] != "undefined",
                    skip = isset && rules[field] == null;
                if (skip) {
                    continue;
                }
                data[index][field] = isset ? rules[field].call(this, result[i], field, i) : result[i][field];
            }
        }
        return data;
    };
    this.preLoad = function(callback) {
        var o = this,
            counter = 0,
            total_queries = 4;
        ERPScan.API.loadDictionaries(this.enumeratedField, {
            callback: function (response) {
                o.info.dictionaries = ERPScan.data.dictionaries = response;
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
        ERPScan.API.query("appservers", "map", {
            callback: function(response) {
                o.info.appservers = response["data"]["result"];
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
        ERPScan.API.query("appserverproperties", "map", {
            callback: function(response) {
                o.info.appserverproperties = response["data"]["result"];
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
        ERPScan.API.query("checks", "map", {
            callback: function(response) {
                o.info.checks = response["data"]["result"];
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
    };
    this.getProjectPolicyStructure = function (policyStructureId){
        var o = this;
        ERPScan.API.query("policyStructures", "get", {
            params: {
                id: policyStructureId
            },
            callback: function(response) {
                o.data["projectPolicyStructure"] = response["data"]["result"];
            }
        });
    }
    this.buildChecksMapByPolicy = function(){
        var o = this;
        function pushCheckToMap(map, level, category) {
            level.checkObjects.forEach(function(check){
                if(typeof map[check] == "undefined") {
                    map[check] = category;
                }
                if(typeof level.children != "undefined") {
                    level.children.forEach(function(levelItem){
                        pushCheckToMap(map, levelItem, category);
                    });
                }
            });
        }
        $.each(this.info.policies, function(project, policies){
            if(project != "SOD") {
                var checksMap = {};
                policies.forEach(function(policy){
                    policy.structure.forEach(function(structureItem){
                        structureItem.checkObjects.forEach(function(check){
                            if(typeof checksMap[structureItem.name] == "undefined") {
                                checksMap[check] = {};
                            }
                            checksMap[check] = structureItem.name;
                        });
                        if(typeof structureItem.children != "undefined") {
                            structureItem.children.forEach(function(level){
                                pushCheckToMap(checksMap, level, structureItem.name);
                            });
                        }
                    });
                });
                if (typeof o.info.checksMapByPolicy == "undefined") {
                    o.info.checksMapByPolicy = {};
                }
                o.info.checksMapByPolicy[project] = checksMap;
            }
        });
    };
    this.addTabs = function() {
        var resultTabs = this.tabs[this.data.scan.projectType],
            tabs = [];
        $.each(this.tabs.ALL.before, function(key){
            tabs.push(key);
        });
        $.each(resultTabs, function(key){
            tabs.push(key);
        });
        $.each(this.tabs.ALL.after, function(key){
            tabs.push(key);
        });
        this.tabsClear.call(this);
        Header.tabsAdd(tabs);
        $("header .tabs a").filter("[data-tab='accessControl'], [data-tab='SODRisksTable'], [data-tab='sod_conflicts_by_users_matrix']").addClass("borderRight");
        Helpers.setBlockHeight($('.tab-test'));
    };
    this.tabsClear = function() {
        for (var tab in this.activeTabs) {
            this.activeTabs[tab].clear();
            delete this.activeTabs[tab];
        }
        var tabs = [];
        $("header .tabs a[data-tab]").each(function() {
            tabs.push($(this).data("tab"));
        });
        Header.tabsRemove(tabs);
    };
    this.toggleTab = function(name, obj) {
        if (obj && !this.activeTabs[name]) {
            this.activeTabs[name] = new Jobs[obj](this, name || null);
            this.activeTabs[name].init();
            this.activeTabs[name].load();
        }
        Header.tabSelect(name, {
            title: obj ? this.activeTabs[name].locale.mainDescription.format({project_name: this.data.scan.projectName}) : Locale.ui.page.title,
            description: obj ? this.activeTabs[name].locale.secDescription.format({scan_number: this.data.scan.number}) : ""
        });
        var section = $(".tab-" + name);
        Helpers.setBlockHeight(section);
    };
    this.showPopup = function() {
        var o = this,
            popupRender = function() {
                o.controls.newProjPopupObj = new Popup(popupParams);
                o.controls.newProjPopupObj.show();
                var popup = $('.new_job_popup'),
                    contentBlock = popup.find('.content'),
                    actionsBar = templates.actions,
                    overlay = $('.popup-shadow');
            popup.append(actionsBar);
            contentBlock.attr('data-step', 1);
        };
        var templates = {
                br: '<br />',
                step: '<div class="step-{step}">{content}</div>',
                input: '<div class="newJob_input_descr">Name:</div><input type="text" class="{class}">',
                textarea: '<div class="newJob_input_descr">Description:</div><textarea class="{class}"></textarea>',
                jobType: '<div class="newJob_input_descr">Type:</div><select class="{class}"><option selected>Vulnerability Management</option><option>Code Scanning</option><option>SoD</option></select>',
                actions: '<div class="actions-bar"><button class="cancel default">Cancel</button><button class="prev" disabled>Back</button><button class="next" disabled>Next</button></div>'
            },
            popupParams = {
                class: 'new_job_popup',
                title: Locale.ui.page.newProjPopup.popupTitle,
                content: '',
                confirmation: Locale.ui.page.newProjPopup.closeMessage
            };

        popupParams.content = templates.input.format({placeholder: 'Job name', class: 'job_name_input'}) + templates.br;
        popupParams.content += templates.textarea.format({placeholder: 'Description', class: 'job_descr'}) + templates.br;
        popupParams.content += templates.jobType.format({class: 'job_type'});
        popupParams.content = templates.step.format({step: 1, content: popupParams.content});
        popupParams.content += templates.step.format({step: 2, content: this.templates.loading});
        popupParams.content += templates.step.format({step: 3, content: this.templates.loading});
        popupParams.content += templates.step.format({step: 4, content: this.templates.loading});

        popupRender();
    };
    this.getAssetsTable = function(container, callback) {
        var o = this;
        ERPScan.API.query("assets", "list", {
            callback: function(response) {
                o.data.assets = response["data"]["result"];
                var params = {
                    data: o.conversionFields(response["data"]["result"], o.conversionFieldRules.asset),
                    ticks: {"__tree": true},
                    option: {
                        pager: false,
                        expandable: false,
                        namespace: "projects",
                        type: "asset",
                        sort_default: { column: 1, order: "asc" }
                    }
                };
                o.controls.assetsTable = new Table(params);
                container.html(o.controls.assetsTable.getHTML());
                o.controls.assetsTable.build();
                callback();
            }
        });
    };
    this.getPoliciesStructureTable = function(projectType, container, callback) {
        var o = this;
        ERPScan.API.query("policyStructures", "list", {
            callback: function(response) {
                var params = {
                        ticks: {"__tree": true},
                        title: Locale.ui.page["table_policies_block"],
                        data: o.conversionFields(response["data"]["result"], o.conversionFieldRules.policyStructures),
                        option: {
                            pager: false,
                            expandable: false,
                            namespace: "projects",
                            type: "policyStructures",
                            sort_default: {column: 1, order: "asc"}
                        }
                    },
                    table = new Table(params);
                container.html(table.getHTML());
                table.build();
                callback();
            }
        });
    };
    this.getPolicyRulesTable = function(projectType, container, callback) {
        var o = this;
        ERPScan.API.query("policyRules", "list", {
            callback: function(response) {
                var params = {
                        ticks: {"__tree": true},
                        title: Locale.ui.page["table_policies_block"],
                        data: o.conversionFields(response["data"]["result"], o.conversionFieldRules.policyRules),
                        option: {
                            pager: false,
                            expandable: false,
                            namespace: "projects",
                            type: "policyRules",
                            sort_default: {column: 1, order: "asc"}
                        }
                    },
                    table = new Table(params);
                container.html(table.getHTML());
                table.build();
                callback();
            }
        });
    };
    this.hideTreeIconsIfStructureEmpty = function(){
        var o = this,
            assetsTable = $(".new_job_popup").find(".step-2 table"),
            rows = assetsTable.find("tbody tr"),
            row,
            assetUID;
        $.each(rows, function(){
            row = $(this);
            assetUID = row.find("td[data-uid]").data("uid");
            o.data.assets.forEach(function(asset){
                if(asset.id == assetUID && asset.structure.length == 0) {
                    row.find("td.tree-icon").removeClass("tree-icon");
                }
            });
        });
    };
    this.changePopupStep = function(step) {
        var o = this,
            types = {
            "Vulnerability Management": "VM",
            "Code Scanning": "CODE",
            "SoD": "SOD"
        },
            popup = $('.new_job_popup'),
            projType = types[popup.find('.job_type').val()],
            title = popup.children('h3'),
            content = popup.find('.content'),
            step1 = content.find('.step-1'),
            step2 = content.find('.step-2'),
            step3 = content.find('.step-3'),
            step4 = content.find('.step-4'),
            activeAsset = step2.find('.tablesorter tbody tr.active'),
            activePolicyStructure = step3.find('.tablesorter tbody tr.active'),
            activePolicyRules = step4.find('.tablesorter tbody tr.active'),
            nextButton = popup.find('button.next')[0],
            prevButton = popup.find('button.prev')[0];

        if (step === 1) {
            step2.hide();
            step1.show();
            title.text('New project');
            nextButton.disabled = false;
            prevButton.disabled = true;
            content.attr('data-step', 1);
            o.controls.newProjPopupObj.setSize();
            return false;
        }
        if (step === 2) {
            step1.hide();
            step2.show();
            step3.hide();
            title.text('Choose assets');
            nextButton.textContent = Locale.ui.page.newProjPopup.next;
            nextButton.disabled = true;
            prevButton.disabled = false;
            content.attr('data-step', 2);
            o.controls.newProjPopupObj.setSize();
            o.getAssetsTable(step2, function() {
                o.hideTreeIconsIfStructureEmpty();
                o.controls.newProjPopupObj.setSize();
            });
            return false;
        }

        if (step === 3) {
            step2.hide();
            step4.hide();
            step3.show();
            title.text('Choose policy');
            nextButton.textContent = Locale.ui.page.newProjPopup.next;
            nextButton.disabled = true;
            content.attr('data-step', 3);
            o.controls.newProjPopupObj.setSize();
            o.getPoliciesStructureTable(projType, step3, function() {
                o.controls.newProjPopupObj.setSize();
            });
            return false;
        }
        if (step === 4) {
            step3.hide();
            step4.show();
            title.text('Choose rules');
            nextButton.textContent = Locale.ui.page.newProjPopup.create;
            content.attr('data-step', 4);
            o.controls.newProjPopupObj.setSize();
            o.getPolicyRulesTable(projType, step4, function() {
                o.controls.newProjPopupObj.setSize();
            });
            return false;
        }
        if (step === 5) {
            $.getJSON('../data/project.json', function(response){
                var newProject = response,
                    assets = [],
                    policyStructure,
                    policyRules;
                $.each(activeAsset, function() {
                    assets.push($(this).find('td[data-uid]').attr('data-uid'));
                });
                $.each(activePolicyStructure, function() {
                    policyStructure = $(this).find('td[data-uid]').attr('data-uid');
                });
                $.each(activePolicyRules, function() {
                    policyRules = $(this).find('td[data-uid]').attr('data-uid');
                });
                var types = {
                    "Vulnerability Management": "VM",
                    "Code Scanning": "CODE",
                    "SoD": "SOD"
                };
                newProject.name = popup.find('.job_name_input').val();
                newProject.description = popup.find('.job_descr').val();
                newProject.type = types[popup.find('.job_type').val()];
                newProject.target = {
                    type: 'assets',
                    values: assets
                };
                newProject.checkBy = {
                    "policyStructure" : {id: policyStructure},
                    "policyRules"     : policyRules == null ? null : {id: policyRules},
                    "categories"      : null
                };
                newProject.system = 'ERP';
                newProject.schedule = {
                    "start"      : new Date().toISOString(),
                    "frequency"  : "none",
                    "end"        : null,
                    "weekdays"   : ["mon", "thu", "sun"],
                    "months"     : ["jun", "oct", "feb", "sep"],
                    "quarters"   : [1, 3]
                };
                // newProject.parameters.specific = newProject.parameters.specific.settings[projType];
                // newProject.parameters.specific["type"] = projType;
                ERPScan.API.query("projects", "add", {
                    params: {
                        data: JSON.stringify(newProject)
                    },
                    type: "POST",
                    callback: function(response) {
                        var table = $('section.tab-all .table table.tablesorter'),
                            tbody = table.find('tbody'),
                            result = response["data"]["result"];
                        result = o.conversionFields.call(o, [result], o.conversionFieldRules.jobs);
                        var rowIndex = o.controls.projectsTable.params.data.push(result[0]) - 1,
                            rowHTML = Table.generateRow.call(o.controls.projectsTable, rowIndex);
                        tbody.prepend(rowHTML);
                        /*TEMP for POC*/
                        ERPScan.API.query("projects", "run",{
                            params: {
                                id: response["data"]["result"]["id"]
                            }
                        });
                    }
                });
                o.controls.newProjPopupObj.close();
            });
        }
    };
    this.runProject = function(projectId, callback){
        var o = this;
        ERPScan.API.query("projects", "run", {
            params: {
                id: projectId
            },
            callback: function(response){
                if(o.section.find(".table .table-mark:checked").closest('tr').hasClass("active")) {
                    o.addNewScanRow(response["data"]["result"]);
                }
                callback();
            }
        });
    };
    this.addNewScanRow = function(scanData){
        var data = this.conversionFields.call(this, scanData, this.conversionFieldRules.scans),
            activeRow = this.section.find(".table .table-mark:checked").closest("tr"),
            simpleTableBody = activeRow.find(".simple-table-body"),
            simpleTableContainer = simpleTableBody.closest('.select-row'),
            rowIndex = this.controls.simpleTable_obj.data.push(data[0]) - 1,
            newScanRow = this.controls.simpleTable_obj.generateRow(rowIndex);
        simpleTableContainer.css('paddingBottom', parseInt(simpleTableContainer.css('paddingBottom')) + simpleTableBody.find('.simple-table-row').height());
        simpleTableBody.append(newScanRow);
        activeRow.find("toggle-info").html("surprise");
        activeRow.find("td[data-state]")
            .text(Locale.ui.page.fields.state[scanData[0]["state"]] ? Locale.ui.page.fields.state[scanData[0]["state"]] : scanData[0]["state"])
            .attr('data-state', scanData[0]["state"]);
    };
    this.activateProjectMenu = function($row){
        var state = $row.find("td[data-state]").attr("data-state");
        if (state === 'SCANNING' || state === 'SCHEDULED' || state === 'PAUSED') {
            $row.closest(".table").find(".actions-bar .action a[data-action='start']").addClass("disabled");
        } else {
            $row.closest(".table").find(".actions-bar .action a[data-action='start']").removeClass("disabled");
        }
    };
    this.activateScanMenu = function($row){
        var $actions = $row.closest(".simple-table").find(".simple_action-events div.action a"),
            actionsToActivate = this.scanActionStatesMap[$row.find(".simple-table-cell[data-state]").data("state")];
        $actions.addClass("disabled");
        if ($row.find("input.simple-table_mark")[0].checked == true && typeof actionsToActivate != "undefined") {
            this.activateActions($actions, actionsToActivate);
        }
    };
    this.scanActionStatesMap = {
        SCHEDULED : [],
        SCANNING: [{'pause': 'resume'}, 'interrupt'],
        STOPPED: ['delete'],
        FINISHED: ['delete'],
        INTERRUPTED: ['delete'],
        PAUSED: [{'resume': 'pause'}],
        pseudo: {
            PAUSING:'pause',
            RESUMING: 'resume',
            INTERRUPTING: 'interrupt'
        }
    };
    this.activateActions = function($actions, actionsToActivate) {
        var name,
            action;
        actionsToActivate.forEach(function(actionName){
            /* init action */
            if (typeof actionName === 'object') {
                /* Swap names */
                $.each(actionName, function(key, val){
                    action = $actions.filter('[data-action="'+ key +'"]');
                    if (action.length === 0) {
                        action = $actions.filter('[data-action="'+ val +'"]');
                        name = key;
                        /* change state */
                        action.text(Locale.ui.page.actions[name] ? Locale.ui.page.actions[name] : name);
                        action.attr('data-action', name);
                    }
                });
            } else {
                action = $actions.filter('[data-action="'+ actionName +'"]');
            }
            action.removeClass('disabled');
        });
    };
    this.changeState = function($cell, state){
        $cell.attr("data-state", state);
        $cell.text(Locale.ui.page.fields.state[state]);
    };
    this.handlers = function(id) {
        var o = this;
        $(document.body).on("click", "section.content.tab-all .table .action-events a[data-action='delete']", function (e) {
            var $this = $(this),
                $checkedProjectRow = $(".select-row.selPreset .table-mark:checked").parents("tr");
            if ($this.hasClass('disabled')) {
                return false;
            }
            ERPScan.API.query("projects", "delete", {
                params: {
                    id: $checkedProjectRow.find("td").eq(o.controls.projectsTable.getColumnNumber("name")).data("id")
                },
                callback: function(result) {
                    $(".select-row.selPreset .table-mark:checked").parents("tr").fadeOut(function() {
                        $(this).remove();
                        $(".action-events").hide();
                        $(".table  > h3").show();
                    });
                }
            });
        });
        $(document.body).on("click", "header .tabs a[data-tab]", function() {
            var tab = $(this).data("tab"),
                loaded = typeof o.activeTabs[tab] != "undefined",
                obj;
            $.each(o.tabs, function(key, val){
                if (key == 'ALL') {
                    $.each(val, function(keyAll, valAll){
                        if (valAll[tab]) {
                            obj = valAll[tab];
                        }
                    });
                }
                else if (val[tab]) {
                    obj = val[tab];
                }
            });
            o.toggleTab.call(o, tab, obj);
            return false;
        });
        $(document.body).on("click", "section.content.tab-all .table table.tablesorter tbody tr .scans .simple-table-body .simple-table-row", function(event) {
            if ($(event.target).is(".simple-table_mark")) {
                return true;
            }
            var $this = $(this),
                target = $(event.target),
                projectRow = target.closest('tr'),
                scanRows = projectRow.find(".simple-table-row"),
                projectType = projectRow.find('td[data-type]').data('type'),
                scanId = target.closest('.simple-table-row').find('.simple-table-cell[data-id]').data('id');
            if ($this.hasClass("active")) {
                return false;
            }
            scanRows.removeClass("active");
            $this.addClass("active");
            o.setScanInfo.call(o, $this);
            o.getAssetsForScan();
            o.getProjectPolicyStructure(o.data["projects"][projectRow.data("number")]["checkBy"]["policyStructure"]["id"]);
            o.addTabs.call(o);
            return false;
        });
        $(document.body).on("click", "section.content.tab-all .table table.tablesorter tr.active .toggle-info a", function() {
            var $tr =  $(this).parents("tr.active");
            o.controls.projectsTable.hideAllFields($tr);
            if ($tr.find(".scans_entry.active").length > 0) {
                o.tabsClear.call(o);
            }
            return false;
        });
        $(document.body).on("click", ".new_job_popup .actions-bar, .popup-shadow", function (e) {
            var popup = $('.new_job_popup'),
                close = popup.find('.close'),
                overlay = $('.popup-shadow'),
                target = $(e.target),
                content = popup.find('.content'),
                nextButton = popup.find('button.next'),
                prevButton = popup.find('button.prev'),
                cancelButton = popup.find('button.cancel');

            /* next step */
            if (target.is(nextButton)) {
                var step = content.attr('data-step');
                step <= 5 ? o.changePopupStep(parseInt(step) + 1) : false;
            }

            /* prev step */
            if (target.is(prevButton)) {
                var step = content.attr('data-step');
                step > 0 ? o.changePopupStep(parseInt(step) - 1) : false;
            }
            /* cancel */
            if (target.is(cancelButton)) {
                o.controls.newProjPopupObj.showConfirmPopup();
            }
        });
        /* step 1 */
        $(document.body).on("click keyup change", ".new_job_popup .content .step-1", function (e) {
            var popup = $('.new_job_popup'),
                nameField = popup.find('.job_name_input'),
                typeSelect = popup.find('.job_type'),
                selectedOption = popup.find('.job_type option:selected')[0],
                nextButton = popup.find('button.next')[0];

            /* next button disabled */
            if (nameField.val().length > 0 && !selectedOption.disabled) {
                nextButton.disabled = false;
            } else {
                nextButton.disabled = true;
            }
        });
        /* step 2 */
        $(document.body).on("click", ".new_job_popup .content .step-2", function (e) {
            var popup = $('.new_job_popup'),
                target = $(e.target),
                content = popup.find('.content'),
                assetRow = content.find('.step-2 .tablesorter tbody tr'),
                nextButton = popup.find('button.next')[0],
                row = target.closest(assetRow);

            /* tree */
            if (target.is('.tree-icon')) {
                var assetUID = row.find("td").eq(o.controls.assetsTable.getColumnNumber("sid")).data("uid"),
                    treePopupObj = new Popup({
                    class: 'treePopup',
                    title: 'Services tree',
                    content: ''
                });
                treePopupObj.show();
                var assets = [],
                    assetStructure = {};
                o.data.assets.forEach(function(asset){
                    if(asset.id == assetUID) {
                        if(asset.structure.length > 0) {
                            asset.structure.forEach(function(node){
                                assetStructure = node;
                                assetStructure.sid = asset.sid;
                            });
                            assets.push(assetStructure);
                        }
                    }
                });
                if(assets.length > 0) {
                    assets = o.buildAppServersTree(assets);
                }
                var treePopup = $('.treePopup .content');
                var treeObj = new Tree(treePopup, assets, { expandWhenClick: true });
                treeObj.init();
                treeObj.expandAll();
                treePopupObj.setSize();
                return false;
            }

            /* next button disabled */
            if (row) {
                row.hasClass('active') ? row.removeClass('active') : row.addClass('active');
                var activeRows = assetRow.filter('.active');
                if (activeRows.length > 0) {
                    nextButton.disabled = false;
                } else {
                    nextButton.disabled = true;
                }
            }
        });

        /* step 3 */
        $(document.body).on("click", ".new_job_popup .content .step-3", function (e) {
            var popup = $('.new_job_popup'),
                target = $(e.target),
                content = popup.find('.content'),
                rows = content.find('.step-3 .tablesorter tbody tr'),
                row = target.closest(rows),
                nextButton = popup.find('button.next')[0];

            /* tree */
            if (target.is('.tree-icon')) {
                var params = {
                    class: 'treePopup',
                    title: 'Policies tree',
                    content: ''
                };
                var treePopupObj = new Popup(params);
                treePopupObj.show();
                var treePopup = $('.treePopup .content');
                return false;
            }

            if (row) {
                var activeRows = rows.filter('.active');
                if (activeRows.length > 0) {
                    rows.removeClass('active');
                }
                row.hasClass('active') ? row.removeClass('active') : row.addClass('active');
                activeRows = rows.filter('.active');

                if (activeRows.length > 0) {
                    nextButton.disabled = false;
                } else {
                    nextButton.disabled = true;
                }
            }
        });
        /* step 4 */
        $(document.body).on("click", ".new_job_popup .content .step-4", function (e) {
            var popup = $('.new_job_popup'),
                target = $(e.target),
                content = popup.find('.content'),
                rows = content.find('.step-4 .tablesorter tbody tr'),
                row = target.closest(rows),
                nextButton = popup.find('button.next')[0];

            /* tree */
            if (target.is('.tree-icon')) {
                var params = {
                    class: 'treePopup',
                    title: 'Policies tree',
                    content: ''
                };
                var treePopupObj = new Popup(params);
                treePopupObj.show();
                var treePopup = $('.treePopup .content');
                return false;
            }

            if (row) {
                var activeRows = rows.filter('.active');
                if (activeRows.length > 0) {
                    rows.removeClass('active');
                }
                row.hasClass('active') ? row.removeClass('active') : row.addClass('active');
                activeRows = rows.filter('.active');
            }
        });
        /* Simple table handlers */
        var o = this;
        $(document.body).on('click', '.simple-table .action a', function(e){
            var target = $(e.target),
                activeAction = target.attr('data-action'),
                additionField = target.closest('.addition-field'),
                parentRow = additionField.closest('tr'),
                parentRowStateCell = parentRow.find('td[data-state]'),
                activeCheckbox = additionField.find('.simple-table_mark:checked'),
                activeRow = activeCheckbox.closest('.simple-table-row'),
                stateCell = activeRow.find('[data-state]'),
                state = stateCell.attr('data-state'),
                scanNumCell = activeRow.find('[data-id]'),
                scanNum = scanNumCell.html(),
                scanId = scanNumCell.attr('data-id'),
                projectName = additionField.closest('tr').find('td[data-id]').text(),
                actions = additionField.find('.action a');

            function savePseudoState(pseudoState){
                var scansPseudoStatesObj,
                    pseudoState_item = {};
                pseudoState_item.id = scanId;
                pseudoState_item.pseudoState = pseudoState;
                pseudoState_item.prevState = state;
                if (!localStorage.getItem('scansPseudoStates')) {
                    scansPseudoStatesObj = [pseudoState_item];
                } else {
                    scansPseudoStatesObj = JSON.parse(localStorage.getItem('scansPseudoStates'));
                    scansPseudoStatesObj.push(pseudoState_item);
                }
                scansPseudoStatesObj = JSON.stringify(scansPseudoStatesObj);
                localStorage.setItem('scansPseudoStates', scansPseudoStatesObj);
            }

            $.each(o.scanActionStatesMap.pseudo, function (key, val) {
                if (activeAction == val) {
                    ERPScan.API.query("scans", val.toLowerCase(), {
                        params: {
                            id: scanId
                        },
                        callback: function () {
                            savePseudoState(key);
                            stateCell.text(Locale.ui.page.fields.state[key] ? Locale.ui.page.fields.state[key] : lastScanState);
                            stateCell.attr('data-state', key);
                            actions.addClass('disabled');
                            /* alert */
                            o.controls.alert_obj.toggleShow('Scan №' + scanNum + ' of project "'+ projectName +'" will ' + key.toLowerCase());
                            parentRowStateCell.text(Locale.ui.page.fields.state[val]);
                        }
                    });
                }
            });
            if (activeAction === "delete") {
                var simpleTableRows = additionField.find('.simple-table-row').not(':first'),//without header row
                    activeRowIndex = activeRow.index();
                ERPScan.API.query("scans", "delete", {
                    params: {
                        id: scanId
                    },
                    callback: function(){
                        scanNumCell.html(o.templates.loading);
                        simpleTableRows.eq(activeRowIndex).fadeOut(function(){
                            $(this).remove();

                            /* if last row deleted */
                            if (additionField.find('.simple-table-row').not(':first').length === 0) {
                                o.controls.projectsTable.hideAllFields(parentRow);
                                parentRowStateCell.text(Locale.ui.page.fields.state['SCHEDULED'] ? Locale.ui.page.fields.state['SCHEDULED'] : 'SCHEDULED');
                                parentRowStateCell.attr('data-state', 'SCHEDULED');
                            } else {
                                var lastScanState = additionField.find('.simple-table-row:last [data-state]').attr('data-state');
                                parentRowStateCell.text(Locale.ui.page.fields.state[lastScanState] ? Locale.ui.page.fields.state[lastScanState] : lastScanState);
                                parentRowStateCell.attr('data-state', lastScanState);
                            }
                        });
                        actions.addClass('disabled');
                    }
                });
            }
        });
        $(document.body).on('click', '.simple-table', function(e){
            var target = $(e.target);
            if (target.is('.action a')) {
                e.stopPropagation();
            }
        });
        /* END Alert handlers */
        $(o.sectionName + ' iframe.blockResize')[0].contentWindow.onresize = function() {
            var activeRow = $(o.sectionName + ' .table table tbody tr.active'),
                additionField = activeRow.find('.addition-field');
            additionField.outerWidth(activeRow.width());
        };
        this.addHandlers({
            "abortLinks": {
                action: "click",
                selector: "header .tabs a.all",
                callback: function(e) {
                    e.preventDefault();
                }
            },
            "createProject": {
                action: "click",
                selector: this.section.selector + " .create-button button",
                callback: function() {
                    this.showPopup();
                }
            },
            "startProject": {
                action: "click",
                selector: this.section.selector + " .table .action-events a[data-action='start']",
                callback: function(e) {
                    var o = this,
                        activeRow = this.section.find(".table .table-mark:checked").closest('tr'),
                        toggleInfo = activeRow.find("td.toggle-info"),
                        toggleInfoHTML = toggleInfo.html();
                    toggleInfo.html(this.templates.loading);
                    $(e.target).addClass("disabled");
                    this.runProject(activeRow.find('td[data-id]').attr('data-id'), function(){
                        toggleInfo.html(toggleInfoHTML);
                        o.changeState(activeRow.find("td[data-state]"), "SCHEDULED");
                    });
                }
            },
            "toggleTabAll": {
                action: "click",
                selector: "header .tabs a.all",
                callback: function(){
                    this.toggleTab.call(this, "all");
                }
            },
            "activateProjectMenu": {
                action: "click",
                selector: this.section.selector + " .table table.tablesorter tbody tr",
                callback: function(e){
                    var $row = $(e.currentTarget),
                        target = $(e.target);
                    if (target.is(".table-mark")) {
                        e.stopPropagation();
                        var otherCheckBoxes = $row.closest("tbody").find('.table-mark:checked').not(target);
                        if(otherCheckBoxes.length > 0) {
                            otherCheckBoxes[0].checked = false;
                        }
                        this.activateProjectMenu($row);
                        e.stopImmediatePropagation();
                        return true;
                    }
                }
            },
            "activateScanMenu": {
                action: "click",
                selector: this.section.selector + " .table table.tablesorter tbody tr",
                callback: function(e){
                    var $row = $(e.currentTarget),
                        target = $(e.target);
                    if (target.is('.simple-table_mark')) {
                        var otherCheckBoxes = target.closest('.simple-table').find('.simple-table_mark').not(target);
                        if(otherCheckBoxes.length > 0) {
                            otherCheckBoxes[0].checked = false;
                        }
                        this.activateScanMenu(target.closest(".simple-table-row"));
                        e.stopImmediatePropagation();
                        return true;
                    }
                }
            },
            "showScans": {
                action: "click",
                selector: this.section.selector + " .table table.tablesorter tbody tr",
                callback: function(e){
                    var $row = $(e.currentTarget);
                    if ($row.hasClass("active")) {
                        this.controls.projectsTable.hideAllFields($row);
                        if ($row.find(".simple-table-row.active").length > 0) {
                            this.tabsClear();
                        }
                    } else {
                        o.loadScans.call($row, o);
                    }
                    return false;
                }
            }
        });
    };
    this.init = function() {
        $("section.content").addClass("tab-all");
        this.section = $("section.content.tab-all");
        $("section.content.tab-all").html(Header.templates.tabs.loading);
        this.preLoad(this.loadJobs);
        this.controls.alert_obj = new Alert();
        this.controls.alert_obj.init();
    };
};
Jobs.All.prototype = new AbstractPage();
