Page.Render = {
    path: ['assess', 'jobs', 'all'],
    init: function() {
        Locale.set("en", { fields: ["jobs_list", "jobs_list_statistic", "scan_list", "jobs_result", "jobs_sap_notes", "jobs_critical_actions"] });
        Locale.loadPage(Page.Render.path);
        Header.load(Page.Render.path, { tabs: true });
        new Jobs.All().init();
        var scripts_additional = ["tests", "results", "statistic", "sap_notes", "code_vulnerability", "access_control", "sod_matrix", "sod_table"];
        for (var i = 0; i < scripts_additional.length; i++) {
            loader.add.JS("renders/" + Page.Render.path[0] + "/" + Page.Render.path[1] + "/" + scripts_additional[i]);
        }
    }
};

var Jobs = { };

Jobs.All = function() {
    this.table = null;
    this.job = {};
    this.statistics = null;
    this.users = {};
    this.checks = {};
    this.codeVulnerabilities = {
        byType: [],
        byProgram: []
    };
    this.targetChecks = [];
    this.checkCats = {};
    this.classes = {};
    this.sodChecks = {
        fake: "SOD_000",
        users: "SOD_0002",
        roles: "SOD_0003"
    };
    this.SIDByCreds = {};
    this.services = {};
    this.data = [];
    this.scanInfo = null;
    this.systemsTree = [];
    this.systemsTreeAuxiliary = { };
    this.creds = { };
    this.types = {
        "VM": {
            tabs: ["vulnerabilities", "misconfigurations", "sap_notes"],
            classes: {
                "vulnerabilities": "Vulnerabilities",
                "misconfigurations": "Misconfigurations"
            },
            IDs: ["MISCONF", "VULNS", "SAPNOTES", "PATCHES"]
        },
        "CODE": {
            tabs: [ "code_review", "vulnerabilities_by_type", "vulnerabilities_by_program" ],
            classes: [ "Code Review" ],
            IDs: ["CODE", "CODEISSUES"]
        },
        "SOD": {
            tabs: [ "access_control", "sod_risks_matrix", "sod_risks_table", "sod_graph" ],
            classes: [ "SoD" ],
            IDs: ["SOD"]
        }
    };
    this.tabs = {};
    this.templates = {
        no_entry: "<p>" + Locale.ui.page.list.no_entry + "</p>",
        loading_scans: "<div class='scans'><p>" + Locale.ui.page.list.loading + "</p></div>",
        create: "<div class='create-button'><button>" + Locale.ui.page.editor.buttons.new + "</button></div>",
        scan_info_loading: "<span class='loading'></span>",
        statistic: "<div class='statistic'>{fields}</div>",
        statistic_field: "<div class='field'><div class='name'>{name}</div><div class='value'>{value}</div></div>"
    };
    this.tableLoad = function() {
        var actions = {};
        actions[Locale.ui.page.actions.start] = [];
        actions[Locale.ui.page.actions.share] = [];
        actions[Locale.ui.page.actions.edit] = [];
        actions[Locale.ui.page.actions.delete] = [];
        actions[Locale.ui.page.actions.hidden] = [];
        var params = {
            title: "Job list",
            data: data,
            option: {
                expandable: false,
                fields: "jobs_list",
                ticks: {
                    '__addition': 0,
                    'name': 1,
                    'created': 2,
                    'type': 3,
                    'state': 4,
                    'system': 5,
                    'count': 6,
                    'author': 7,
                    'statistic': 8
                },
                sorting: {
                    0: false,
                    2: "data-ts"
                },
                count_one_page: 50,
                filters: {
                    type: "select",
                    created: "datetime",
                    state: "select",
                    system: "select",
                    author: "select"
                },
                sort_default: { column: 2, order: "desc" }
            }
        };
        this.table = new Table(params);
        var html = this.table.getHTML();
        $("section.content").html(this.templates.create + html);
        if (data.length == 0) {
            $(".table table.tablesorter").after(this.templates.no_entry);
            $(".table table.tablesorter, .table .pager").remove();
            return;
        }
        this.table.build();
        this.handlers();
        //var $table_wrapper = $(".table-wrapper");
        //$table_wrapper.height($(window).height() - $table_wrapper.offset().top - 40);
    };
    this.showScans = function($row, scans) {
        this.table.showAllFields($row, {
            title: "Scans",
            content: this.templates.loading_scans
        });
        var html = Locale.ui.page["no_scans"];
        if (scans.length != 0) {
            var table_obj = new SimpleTable({
                data: scans,
                option: {
                    fields: "scan_list"
                }
            });
            html = table_obj.init().getHTML();
        }
        var $scans = $row.find(".scans");
        $scans.html(html);
        if (scans.length == 0) {
            $scans.addClass("no_scans");
        }
        $row.removeClass("loading");
    };
    this.buildTreeSystems = function() {
        var o = this;
        o.systemsTree = [];
        o.creds = {};
        $("credentials", o.job).each(function() {
            var $this = $(this);
            var data = JSON.parse($this.find("customdata").text());
            var settings = JSON.parse(data.settings);
            var cred_id = $this.find("id").text();
            var index;
            o.SIDByCreds[cred_id] = settings.sid;
            if (!o.systemsTreeAuxiliary[settings.sid]) {
                index = o.systemsTree.push({
                    text: settings.sid,
                    data: {
                        info: cred_id
                    }
                });
                o.systemsTreeAuxiliary[settings.sid] = {
                    index: index - 1,
                    ip: {}
                };
            } else {
                o.systemsTree[o.systemsTreeAuxiliary[settings.sid].index].info += "," + cred_id;
            }
            var sid = o.systemsTreeAuxiliary[settings.sid];
            if (!sid.ip[settings.ip]) {
                o.systemsTree[sid.index].expanded = true;
                if (typeof o.systemsTree[sid.index].nodes == "undefined") {
                    o.systemsTree[sid.index].nodes = [];
                }
                index = o.systemsTree[sid.index].nodes.push({
                    text: settings.ip,
                    data: {
                        info: cred_id
                    }
                });
                sid.ip[settings.ip] = {
                    index: index - 1,
                    appserver: {}
                };
            } else {
                o.systemsTree[sid.index].nodes[sid.ip[settings.ip].index].info += "," + cred_id;
            }
            var ip = sid.ip[settings.ip];
            if (!ip.appserver[settings.appserver]) {
                o.systemsTree[sid.index].nodes[ip.index].expanded = true;
                if (typeof o.systemsTree[sid.index].nodes[ip.index].nodes == "undefined") {
                    o.systemsTree[sid.index].nodes[ip.index].nodes = [];
                }
                index = o.systemsTree[sid.index].nodes[ip.index].nodes.push({
                    text: o.services.appserver[settings.appserver],
                    data: {
                        info: cred_id
                    }
                });
                ip.appserver[settings.appserver] = {
                    index: index - 1,
                    service: {}
                };
            } else {
                o.systemsTree[sid.index].nodes[ip.index].nodes[ip.appserver[settings.appserver].index].info += "," + cred_id;
            }
            var appserver = ip.appserver[settings.appserver];
            if (!appserver.service[settings.service]) {
                o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].expanded = true;
                if (typeof o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes == "undefined") {
                    o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes = [];
                }
                index = o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes.push({
                    text: o.services.service[settings.service],
                    data: {
                        info: cred_id
                    }
                });
                appserver.service[settings.service] = {
                    index: index - 1,
                    clients: {},
                    clients_indexes: {},
                    clients_count: 0
                };
            } else {
                o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[appserver.service[settings.service].index].info += "," + cred_id;
            }
            var service = appserver.service[settings.service];
            var client = data["clientid"] ? "Client: " + data["clientid"] : "General";
            if ((data["clientid"] && !service.clients[client]) || (service.clients_count > 0 && !data["clientid"] && !service.clients[client])) {
                o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].expanded = true;
                if (typeof o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].nodes == "undefined") {
                    o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].nodes = [];
                }
                index = o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].nodes.push({
                    text: client,
                    data: {
                        info: cred_id
                    }
                });
                service.clients_indexes[client] = index - 1;
                if (service.clients_count == 1 && service.clients["General"]) {
                    index = o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].nodes.push({
                        text: "General",
                        data: {
                            info: cred_id
                        }
                    });
                    service.clients_indexes["General"] = index - 1;
                }
            }
            if (service.clients_indexes[client]) {
                o.systemsTree[sid.index].nodes[ip.index].nodes[appserver.index].nodes[service.index].nodes[service.clients_indexes[client]].info += "," + cred_id;
            }
            service.clients[client] = data.clientid || -1;
            o.creds[cred_id] = {
                sid: settings.sid,
                client: data["clientid"] || "General"
            };
            service.clients_count++;
        });
        o.systemsTreeAuxiliary = { };
    };
    this.loadJobs = function() {
        var o = this;
        ERPScan.API.query("projects", "list", {
            callback: function(result) {
                result = o.conversionFields.call(o, result, o.conversionFieldRules.jobs);
                o.tableLoad.call(o, result);
            }
        });
    };
    this.loadScans = function(o) {
        var $this = $(this);
        if ($this.hasClass("active"))
            return false;
        $this.addClass("loading");
        var id = $this.find("td").eq(o.table.getColumnNumber("name")).data("id"),
            type = $this.find("td").eq(o.table.getColumnNumber("type")).data("type");
        ERPScan.API.query("scans", "list", {
            params: {
                project_id: id
            },
            callback: function(result) {
                o.job.scan_counter = 0;
                for (var i = 0; i < result.length; i++) {
                    result[i]["type"] = type;
                }
                result = o.conversionFields.call(o, result, o.conversionFieldRules.scans);
                o.showScans.call(o, $this, result);
            }
        });
    };
    this.loadResults = function($this) {
        var o = this;
        $(".table table.tablesorter tbody tr .scans .simple-table-row").removeClass("active");
        $this.addClass("active");
        var $td = $this.parents("tr.active").find("td"),
            $scanInfo = $this.find(".simple-table-cell").eq(0);
        var projectType = $td.eq(this.table.getColumnNumber("type")).text();
        this.scanInfo = {
            projectID: $td.eq(this.table.getColumnNumber("name")).data("id"),
            projectName: $td.eq(this.table.getColumnNumber("name")).text(),
            projectType: projectType,
            number: $scanInfo.text(),
            ID: $scanInfo.data("id"),
            types: o.types[projectType].classes
        };
        $scanInfo.html(this.templates.scan_info_loading);

        ERPScan.API.query("results", "list", {
            params: {
                scan_id: this.scanInfo.ID
            },
            callback: function(result) {
                o.scanInfo.results = o.conversionFields.call(o, result, o.conversionFieldRules.results);
                var tabs = [ { name: "test" }],
                    type = $this.parents("tr[role=row].active").find("td").eq(o.table.getColumnNumber("type")).data("type");
                for (var i = 0; i < o.types[projectType].tabs.length; i++) {
                    tabs.push({
                        name: o.types[projectType].tabs[i],
                        attr: {
                            name: "results",
                            value: o.types[projectType].tabs[i]
                        }
                    });
                }
                if (type != "sod") {
                    tabs.push({ name: "statistic" });
                }
                o.tabsClear.call(o);
                var finishCallback = function(tabs) {
                    Header.tabsAdd(tabs);
                    o.toggleTab("test", { obj: "Test" });
                    $scanInfo.html(o.scanInfo.number);
                };
                if (projectType == "Vulnerability Management") {
                    o.loadCheckInfo.call(o, finishCallback, tabs, "sap_notes");
                } else if (projectType == "Code Scanning") {
                    tabs.push({ name: "vulnerability_by_type" }); // o.codeVulnerabilities.byType
                    o.loadCheckInfo.call(o, finishCallback, tabs, "programs");
                } else if (projectType == "SoD") {
                    tabs.push({
                        name: "access_control",
                        attr: {
                            name: "sod",
                            value: "access_control"
                        }
                    });
                    tabs.push({
                        name: "sod_risks_matrix",
                        attr: {
                            name: "sod",
                            value: "sod_risks_matrix"
                        }
                    });
                    tabs.push({
                        name: "sod_risks_table",
                        attr: {
                            name: "sod",
                            value: "sod_risks_table"
                        }
                    });
                    tabs.push({
                        name: "sod_graph",
                        attr: {
                            name: "sod",
                            value: "sod_graph"
                        }
                    });
                    o.scanInfo.critical_actions = {
                        cats: { },
                        actions: [ ],
                        counter: 1,
                        matrix: { },
                        table: { }
                    };
                    o.targetChecks = [o.sodChecks.users, o.sodChecks.roles];
                    o.loadSoDActions.call(o, function() {
                        finishCallback(tabs);
                    });
                } else {
                    finishCallback(tabs);
                }
            }
        });
    };
    this.conversionFields = function(result, rules) {
        var data = [];
        for (var i = 0; i < result.length; i++) {
            var index = data.push({}) - 1;
            for (var field in rules) {
                if (typeof result[i][field] == "undefined") {
                    data[index][field] = rules[field].call(this, result[i], field);
                }
            }
            for (var field in result[i]) {
                var isset = typeof rules[field] != "undefined",
                    skip = isset && rules[field] == null;
                if (skip) {
                    continue;
                }
                data[index][field] = isset ? rules[field].call(this, result[i], field) : result[i][field];
            }
        }
        return data;
    };
    this.catsTreeWalk = function($items, item, level) {
        var o = this;
        $items.each(function() {
            var $this = $(this);
            var id = $this.attr("id");
            var name = $this.find("> name").text();
            var $child = $this.find("> item");
            item[id] = { name: name };
            if (level == 1) {
                o.classes[name] = id;
            }
            if ($child.length > 0) {
                item[id].child = {};
                o.catsTreeWalk($child, item[id].child, level + 1);
            }
        });
    };
    this.servicesBuild = function(services) {
        this.services.appserver = {};
        this.services.service = {};
        var o = this;
        services.appserver.each(function() {
            var $this = $(this);
            o.services.appserver[$this.attr("id")] = $this.attr("name");
        });
        services.service.each(function() {
            var $this = $(this);
            var params = $this.find("base").text().split(",");
            var name = params[3];
            var id = $this.attr("id");
            o.services.service[id] = name;
        });
    };
    this.preLoad = function(callback) {
        var o = this,
            counter = 0,
            total_queries = 4;
        ERPScanAPI.query("/engine/dictionary/users", function(result) {
            $("listusers user", result).each(function() {
                var $this = $(this),
                    userID = $this.find("id").text(),
                    user = {
                        login: $this.find("login").text()
                    };
                o.users[userID] = user;
            });
            counter++;
            if (counter == total_queries) {
                callback.call(o);
            }
        });
        ERPScanAPI.query("/engine/dictionary/checkobjects", function(result) {
            $("listobjects prototypecheck", result).each(function() {
                var $this = $(this),
                    id = $this.find("id").text(),
                    data = JSON.parse($this.find("data").text());
                o.checks[id] = {
                    prototype: $this.find("prototypename").text(),
                    data: data
                };
                if (id == o.sodChecks.users) {
                    o.checks[o.sodChecks.fake] = o.checks[o.sodChecks.users];
                    o.checks[o.sodChecks.users]["data"]["title_" + Locale.name] = "Access to %s";
                    o.checks[o.sodChecks.users]["data"]["description_" + Locale.name] = "<html>You can see the list of users, roles or profiles with access to the %s transaction.<br/><br/><b>Authorization values:</b><br/>Users are chosen by following authorization values: %s</html>";
                    o.checks[o.sodChecks.users]["data"]["threat_" + Locale.name] = "<html>Access to the %s transaction can do critical actions in the system, for example to %s</html>";
                    o.checks[o.sodChecks.users]["data"]["solution_" + Locale.name] = "<html>The number of users, roles and profiles with access to the %s transactions should be minimized. Roles should be assigned according to the principle of least privileges.</html>";
                }
            });
            counter++;
            if (counter == total_queries) {
                callback.call(o);
            }
        });
        $.ajax({
            url: "/data/checkCats.xml",
            method: "GET",
            dataType: "XML",
            success: function(result) {
                o.catsTreeWalk($("tree > item", result), o.checkCats, 1);
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
        $.ajax({
            url: "/data/services.xml",
            method: "GET",
            dataType: "XML",
            success: function(result) {
                o.servicesBuild({
                    appserver: $("services > appserver", result),
                    service: $("services > service", result)
                });
                counter++;
                if (counter == total_queries) {
                    callback.call(o);
                }
            }
        });
    };
    this.tabsClear = function() {
        for (var tab in this.tabs) {
            this.tabs[tab].clear();
            delete this.tabs[tab];
        }
        var tabs = ["test", "statistic", "sap_notes", "vulnerability_by_type", "access_control", "sod_risks_matrix", "sod_risks_table", "sod_graph"];
        $("header .tabs a[data-results]").each(function() {
            tabs.push($(this).data("results"));
        });
        Header.tabsRemove(tabs);
    };
    this.toggleTab = function(name, params) {
        params = params || { };
        if (params.obj && !this.tabs[name]) {
            this.tabs[name] = new Jobs[params.obj](this, params.option || null);
            this.tabs[name].init();
        }
        Header.tabSelect(name, {
            title: params.obj ? this.tabs[name].locale.description.format({ scan_number: this.scanInfo.number }) : Locale.ui.page.title,
            description: params.obj ? this.scanInfo.jobName : Locale.ui.page.description
        });
        if (params.obj && typeof this.tabs[name].afterInit != "undefined") {
            this.tabs[name].afterInit();
        }
        this.blocksResize();
    };
    this.transformCriticalActions = function() {
        for (var cat in this.scanInfo.critical_actions.cats) {
            var cat_obj = this.scanInfo.critical_actions.cats[cat];
            this.scanInfo.critical_actions.actions.push(cat_obj.data);
            for (var i = 0; i < cat_obj.actions.length; i++) {
                this.scanInfo.critical_actions.actions.push(cat_obj.actions[i]);
            }
        }
    };
    this.conversionSAPNotes = function($result) {
        var ticks = {};
        var ticks_numeric = [];
        var tick_number = 1;
        ticks['__addition'] = 0;
        $result.find("table thead th").each(function() {
            var $this = $(this);
            var tick = $this.text();
            var is_hidden = $this.attr("hidden") == "true";
            ticks[tick] = is_hidden ? -1 : tick_number++;
            ticks_numeric.push(tick);
        });
        var data = [];
        $result.find("table tbody tr").each(function() {
            var $this = $(this);
            var index = data.push({});
            var counter = 0;
            data[index - 1]["__addition"] = '<a href="#"></a>';
            $this.find("td").each(function(){
                var tick_name = ticks_numeric[counter++];
                var value = $(this).text();
                if (tick_name == "RELEASE_DATE") {
                    var date = value * 1;
                    var ts = new Date(date).format('d/m/y H:i');
                    data[index - 1][tick_name] = {
                        value: ts,
                        attr: { name: 'ts', value: date }
                    };
                } else {
                    data[index - 1][tick_name] = $(this).text();
                }
            });
        });
        return {
            ticks: ticks,
            data: data
        };
    };
    this.collectCriticalActionCats = function($result) {
        var o = this;
        $result.find("brole").each(function() {
            var $this = $(this);
            var brole_id = $this.attr("id");
            if (typeof o.scanInfo.critical_actions.cats[brole_id] == "undefined") {
                o.scanInfo.critical_actions.cats[brole_id] = {
                    data: {
                        _id: "cat_" + brole_id,
                        name: $this.attr("name"),
                        value: "",
                        sid: "",
                        client: ""
                    },
                    actions: []
                };
            }
        });
    };
    this.collectCriticalActions = function($result, checkid, credId, counter) {
        var o = this,
            check_type;
        for (var type in o.sodChecks) {
            if (o.sodChecks[type] == checkid) {
                check_type = type;
            }
        }
        this.collectCriticalActionCats($result);
        counter = counter || 1;
        $result.find("brole").each(function() {
            var $this = $(this);
            var brole_id = $this.attr("id");
            $this.find("action").each(function() {
                var $this = $(this),
                    name = $this.attr("name"),
                    transaction = $this.find("transactions").text(),
                    auth = $this.find("auth").text();
                o.scanInfo.critical_actions.cats[brole_id].actions.push({
                    _id: "entry_" + (counter++),
                    _parent: "cat_" + brole_id,
                    name: {
                        attr: {
                            name: "info",
                            value: transaction + ":" + auth + ":" + brole_id
                        },
                        value: name
                    },
                    value: $this.attr("size"),
                    sid: {
                        attr: {
                            name: "credID",
                            value: credId
                        },
                        value: o.creds[credId].sid
                    },
                    client: o.creds[credId].client
                });
            });
        });
        if ($result.find("matrix[risk]").length != 0) {
            if (typeof o.scanInfo.critical_actions.matrix[credId] == "undefined") {
                o.scanInfo.critical_actions.matrix[credId] = {};
            }
            var cred_obj = o.scanInfo.critical_actions.matrix[credId];
            $result.find("matrix[risk]").each(function() {
                var $this = $(this);
                if (typeof cred_obj[$this.attr("row") + "_" + $this.attr("col")] == "undefined") {
                    cred_obj[$this.attr("row") + "_" + $this.attr("col")] = {};
                }
                var cell = cred_obj[$this.attr("row") + "_" + $this.attr("col")];
                cell["risk"] = $this.attr("risk");
                cell["number_" + check_type] = $this.attr("count");
            });
        }
        if ($result.find("nwaymatrix").length != 0) {
            if (typeof o.scanInfo.critical_actions.table[credId] == "undefined") {
                o.scanInfo.critical_actions.table[credId] = [];
            }
            var cred_obj = o.scanInfo.critical_actions.table[credId],
                i = 0;
            $result.find("nwaymatrix tbody tr").each(function() {
                var $cells = $(this).find("td");
                if (typeof cred_obj[i] == "undefined") {
                    cred_obj[i] = {
                        business_process: $cells.eq(0).text(),
                        risk: $cells.eq(2).text(),
                        function_1: $cells.eq(3).text(),
                        function_2: $cells.eq(4).text(),
                        function_3: $cells.eq(5).text(),
                        function_4: $cells.eq(6).text(),
                        function_5: $cells.eq(7).text()
                    };
                }
                cred_obj[i]["number_" + check_type + "_conflicts"] = $cells.eq(1).text();
                i++;
            });
        }
        return counter;
    };
    this.loadSoDActions = function(callback) {
        var o = this;
        o.scanInfo.critical_actions.cats = {};
        var query_number = Helpers.numberProperty(o.creds) * o.targetChecks.length;
        var query_counter = 1;
        var collect_counter = 1;
        for (var credId in o.creds) {
            for (var i = 0; i < o.targetChecks.length; i++) {
                ERPScanAPI.query("/engine/project/result/find/" + this.scanInfo.jobID, function(credId, checkid) {
                    return function (result) {
                        collect_counter = o.collectCriticalActions($("> result", result), checkid, credId, collect_counter);
                        if (query_number == query_counter) {
                            o.transformCriticalActions.call(o);
                            callback();
                        }
                        query_counter++;
                    }
                }(credId, o.targetChecks[i]), {
                    checkid: o.targetChecks[i],
                    projectid: this.scanInfo.jobID,
                    error: false,
                    fullform: false,
                    runid: this.scanInfo.ID,
                    credid: credId
                }, callback);
            }
        }
    };
    this.loadCheckInfo = function(callback, tabs, target, sync) {
        sync = sync || false;
        var o = this,
            query_number = o.targetChecks.length,
            query_counter = 0;
        if (query_number == 0) {
            callback(tabs);
            return;
        }
        for (var i = 0; i < query_number; i++) {
            var params = {
                checkid: o.targetChecks[i],
                projectid: this.scanInfo.jobID,
                error: false,
                fullform: false,
                runid: this.scanInfo.ID
            };
            ERPScanAPI.query("/engine/project/result/find/" + this.scanInfo.jobID, function(result) {
                switch (target) {
                    case "sap_notes":
                        var sap_notes = o.conversionSAPNotes($("> result", result));
                        if (typeof o.scanInfo.SAPNotes == "undefined") {
                            o.scanInfo.SAPNotes = { data: [], ticks: [] };
                        }
                        o.scanInfo.SAPNotes.data = o.scanInfo.SAPNotes.data.concat(sap_notes.data);
                        query_counter++;
                        if (query_counter == query_number - 1) {
                            o.scanInfo.SAPNotes.ticks = sap_notes.ticks;
                            tabs.push({ name: "sap_notes" });
                            callback(tabs);
                        }
                        break;
                    case "programs":
                        query_counter++;
                        if (query_counter == query_number) {
                            tabs.push({ name: "vulnerability_by_program" });
                            callback(tabs);
                        }
                        break;
                }
            }, params, function() {
                callback(tabs);
            }, !sync);
        }
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", "section.content.tab-all .table table.tablesorter tbody tr", function() {
            var $this = $(this);
            if ($this.hasClass("active")) {
                o.table.hideAllFields($this);
                if ($this.find(".simple-table-row.active").length > 0) {
                    o.tabsClear.call(o);
                }
            } else {
                o.loadScans.call(this, o);
            }
            return false;
        });
        $(document.body).on("click", "header .tabs a.all", function() {
            o.toggleTab.call(o, "all");
            return false;
        });
        $(document.body).on("click", "header .tabs a.test", function() {
            o.toggleTab.call(o, "test", { obj: "Test" });
            return false;
        });
        $(document.body).on("click", "header .tabs a.sap_notes", function() {
            o.toggleTab.call(o, "sap_notes", { obj: "SAPNotes" });
            return false;
        });
        $(document.body).on("click", "header .tabs a.vulnerability_by_type", function() {
            var type = "vulnerability_by_type";
            var loaded = typeof o.tabs[type] != "undefined";
            o.toggleTab.call(o, type, { obj: "CodeVulnerability", option: { type: type } });
            if (!loaded) {
                o.tabs[type].load();
                o.blocksResize();
            }
            return false;
        });
        $(document.body).on("click", "header .tabs a[data-sod]", function() {
            var type = $(this).data("sod"),
                loaded = typeof o.tabs[type] != "undefined",
                obj;
            switch (type) {
                case "access_control":
                    obj = "AccessControl";
                    break;
                case "sod_risks_matrix":
                    obj = "SoDMatrix";
                    break;
                case "sod_risks_table":
                    obj = "SoDTable";
                    break;
            }
            o.toggleTab.call(o, type, { obj: obj, option: type });
            if (!loaded) {
                o.tabs[type].load();
                o.blocksResize();
            }
            return false;
        });
        $(document.body).on("click", "header .tabs a[data-results]", function() {
            var type = $(this).data("results");
            var loaded = typeof o.tabs[type] != "undefined";
            o.toggleTab.call(o, type, { obj: "Results", option: o.classes[o.scanInfo.types[type]] });
            if (!loaded) {
                o.tabs[type].load();
                o.blocksResize();
            }
            return false;
        });
        $(document.body).on("click", "header .tabs a.statistic", function() {
            o.toggleTab.call(o, "statistic", { obj: "Statistic" });
            return false;
        });
        $(document.body).on("click", "section.content.tab-all .table table.tablesorter tbody tr .scans .simple-table-body .simple-table-row", function() {
            var $this = $(this);
            if ($this.hasClass("active"))
                return false;
            o.loadResults.call(o, $this);
            return false;
        });
        $(document.body).on("click", "section.content.tab-all .table table.tablesorter tr.active .toggle-info a", function() {
            var $tr =  $(this).parents("tr.active");
            o.table.hideAllFields($tr);
            if ($tr.find(".scans_entry.active").length > 0) {
                o.tabsClear.call(o);
            }
            return false;
        });
        $(window).on("resize", this.blocksResize);
    };
    this.blocksResize = function() {
        return false;
        var resize = function() {
            var $this = $(this);
            $this.height($(window).height() - $this.offset().top - 40);
        };
        $(".table-wrapper, .system-list, .check-description .block-content").each(resize);
    };
    this.init = function() {
        $("section.content").addClass("tab-all");
        $("section.content.tab-all").html(Header.templates.tabs.loading);
        this.preLoad(this.loadJobs);
    };
};