Jobs.Information = function(obj, name){
    this.parentObj = obj;
    this.sectionName = '.tab-' + name;
    this.section = $('.tab-information');
    this.locale = null;
    this.path = ['assess', 'projects', "information"];
    this.templates = {
        textCell: '<p><span class="name">{name}: </span><span class="value">{value}</span></p>',
        select: '<select class="selectInfo">{options}</select>',
        option: '<option data-name="{name}">{content}</option>',
        tableWrapper: '<div class="tableWrapper"></div>'
    };
    this.queriesData = {};
    this.info_commonInfo = [];
    this.tablesData = {};
    this.classes = {
        1: "info_vulnerabilities",
        2: "info_misconfigurations"
    };
    this.queries = {
        VM: {
            sapnoteIssues: function(scanId) {
                return {
                    title: "Sapnote issues",
                    id: "sapnoteIssues",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.sapnote.status = :type2",
                        "fields": [
                            "i.sapnote.number",
                            "i.sapnote.criticality"
                        ],
                        "parameters": [
                            {
                                name: "type1",
                                value: "sapnote"
                            },
                            {
                                name: "type2",
                                value: "Fail"
                            },
                            {
                                name: "scanId",
                                value: scanId
                            }
                        ]
                    }
                }
            },
            componentIssues: function(scanId) {
                return {
                    title: "Component issues",
                    id: "componentIssues",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.vulnerableComponent.status = :type2",
                        "fields": [
                            "i.vulnerableComponent.componentName",
                            "i.vulnerableComponent.desctiption"
                        ],
                        "parameters": [
                            {
                                name: "type1",
                                value: "vulnerableComponent"
                            },
                            {
                                name: "type2",
                                value: "Fail"
                            },
                            {
                                name: "scanId",
                                value: scanId
                            }
                        ]
                    }
                }
            },
            allCriticalUser: function(scanId) {
                return {
                    title: "All critical users",
                    id: "allCriticalUser",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1",
                        "fields": [
                            "i.user.userName"
                        ],
                        "parameters": [
                            {
                                name: "scanId",
                                value: scanId
                            },
                            {
                                name: "type1",
                                value: "criticalUser"
                            }
                        ]
                    }
                }
            },
            usersWithSapAllRole: function(scanId) {
                return {
                    title: "Users with SAP_ALL role",
                    id: "usersWithSapAllRole",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.fields.criticalRole.role.roleName = :type2 AND i.field.criticalRole.status = :type3",
                        "fields": [
                            "i.user.userName"
                        ],
                        "parameters": [
                            {
                                name: "scanId",
                                value: scanId
                            },
                            {
                                name: "type1",
                                value: "criticalRole"
                            },
                            {
                                name: "type2",
                                value: "SAP_ALL"
                            },
                            {
                                name: "type3",
                                value: "fail"
                            }
                        ]
                    }
                }
            },
            usersWithSimplePassword: function(scanId) {
                return {
                    title: "Users with simple password",
                    id: "usersWithSimplePassword",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.fields.simplePassword.status = :type2",
                        "fields": [
                            "i.user.userName"
                        ],
                        "parameters": [
                            {
                                name: "scanId",
                                value: scanId
                            },
                            {
                                name: "type1",
                                value: "simplePassword"
                            },
                            {
                                name: "type2",
                                value: "fail"
                            }
                        ]
                    }
                }
            }
        },
        CODE: {
            vulnerabilitiesInProgram: function(scanId) {
                return {
                    title: "Vulnerabilities in program",
                    id: "vulnerabilitiesInProgram",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1",
                        "fields": [
                            "i.abapProgram.ProgramName",
                            "i.abapProgram.codeSize",
                            "i.abapProgram.author"
                        ],
                        "parameters": [
                            {
                                name: "scanId",
                                value: scanId
                            },
                            {
                                name: "type1",
                                value: "abapProgramIssue"
                            }
                        ]
                    }
                }
            }
        },
        SOD: {
            sapnoteIssues: function(scanId) {
                return {
                    title: "Sapnote issues",
                    id: "sapnoteIssues",
                    queryObj: {
                        "from": [
                            {
                                entity: "Issue"
                            }
                        ],
                        "query": "i.scanId = :scanId AND i.issueType = :type1 AND i.sapnote.status = :type2",
                        "fields": [
                            "i.sapnote.number",
                            "i.sapnote.criticality"
                        ],
                        "parameters": [
                            {
                                name: "type1",
                                value: "sapnote"
                            },
                            {
                                name: "type2",
                                value: "Fail"
                            },
                            {
                                name: "scanId",
                                value: scanId
                            }
                        ]
                    }
                }
            }
        }
    };
    this.buildView = function(){
        /* Append blocks */
        var block = new Block({
            title: this.locale.blockTitle,
            content: this.info_commonInfo,
            class: "info_commonInfo"
        }).getHTML();
        this.section.html(block);
        var o = this,
            type = Object.getOwnPropertyNames(this.tablesData)[0],
            options = this.templates.option.format({name:"", content: this.locale.selectInfo});
        /* Generate options for select */
        $.each(this.tablesData, function(name){
            options += o.templates.option.format({name: name, content: o.locale.options[name]});
        });
        options = this.templates.select.format({options: options});
        this.createTable(type, options);
    };

    this.createTable = function(type, select){
        if(type) {
            table = new Table({
                title: this.locale.tableTitle,
                data: this.tablesData[type],
                option: {
                    namespace: "projects",
                    type: type
                }
            });
            var first = select ? true : false,
                select = select ? select : this.section.find('.table .selectInfo')[0].outerHTML;
            this.section.find('.table').remove();
            this.section.append(table.getHTML());
            this.section.find('.table').prepend(select);
            if (!first) {
                this.section.find('.table .selectInfo [data-name="'+ type +'"]').attr('selected', 'true');
            }
            table.build();
            this.clearEvents();
            this.handlers();
        }
        Helpers.setBlockHeight(this.section);
    };
    this.getData = function(){
        var o = this,
            scanData = {},
            projType = this.parentObj.data.scan.projectType,
            scanId = this.parentObj.data.scan.id,
            rows = {
                name: this.parentObj.data.scan.projectName,
                author: this.parentObj.data.scan.projectAuthor,
                system: this.parentObj.data.scan.system,
                scanNum: this.parentObj.data.scan.scanNum
            };

        /* Data for all project types */
        for(var i = 0; i < this.parentObj.data.scanList.length; i++) {
            if (this.parentObj.data.scanList[i].id == scanId) {
                scanData = this.parentObj.data.scanList[i];
            }
        }
        rows['startAt'] = new Date(scanData.startDate).format('d/m/y H:i');
        rows['finishAt'] = new Date(scanData.finishDate).format('d/m/y H:i');
        rows['state'] = scanData.state;
        /* Save info_commonInfo for all types*/
        $.each(rows, function(key, val){
            o.info_commonInfo += o.templates.textCell.format({name: Locale.fields.projects.info_commonInfo[key], value: val});
        });
        /* Data depends of project type */
        var queryNum = 1;
        o.tablesData = {};
        $.each(this.queries[projType], function(){
            var query = this(scanId);
            rows = {};
            ERPScan.API.query("queries", query.id, {
                params: {
                    query: JSON.stringify(query.queryObj)
                },
                type: "POST",
                callback: function(response) {
                    o.queriesData[query.id] = response.data.result;
                    if (queryNum === Object.keys(o.queries[projType]).length) {

                        if (projType == 'VM') {
                            var allCriticalUser = o.queriesData.usersWithSapAllRole,
                                usersWithSapAllRole =  o.queriesData.usersWithSapAllRole,
                                usersWithSimplePassword = o.queriesData.usersWithSimplePassword,
                                usersWithSapAllRoleAndSimplePassword = [];

                            /* String data */
                            if(typeof scanData.statistic != 'undefined' && scanData.statistic != null) {
                                rows['vulnerabilities'] = scanData.statistic.VM.vulnerabilities.value + '%';
                                rows['misconfigurations'] = scanData.statistic.VM.misconfigurations.value + '%';
                                rows['uninstalledSap'] = scanData.statistic.VM.sapnotes.value + '%';
                                rows['securityLevel'] = scanData.statistic.VM.security_level.value + '%';
                            }
                            //TODO Total Project Execution Time(VM)
                            rows['numberUsers'] = allCriticalUser.length;
                            rows['numberUsersSapAllRole'] = usersWithSapAllRole.length;
                            rows['percentUsersSapAllRole'] = (usersWithSapAllRole.length * 100 /allCriticalUser.length).toFixed(2) + '%';
                            rows['numberUsersSimplePassword'] = usersWithSimplePassword.length;
                            rows['percentUsersSimplePassword'] = (usersWithSimplePassword.length * 100 /allCriticalUser.length).toFixed(2) + '%';


                            for (var i = 0; i < usersWithSapAllRole.length; i++) {
                                for (var j = 0; j < usersWithSimplePassword.length; j++) {
                                    if (usersWithSapAllRole[i]['user.userName'] === usersWithSimplePassword[j]['user.userName']) {
                                        usersWithSapAllRoleAndSimplePassword.push(usersWithSimplePassword[j]['user.userName']);
                                    }
                                }
                            }
                            rows['numberUsersSapAllRoleAndSimplePassword'] = usersWithSapAllRoleAndSimplePassword.length;
                            rows['percentUsersSapAllRoleAndSimplePassword'] = (usersWithSapAllRoleAndSimplePassword.length * 100 /allCriticalUser.length).toFixed(2) + '%';

                            /* Save info_commonInfo for VM*/
                            $.each(rows, function(key, val){
                                o.info_commonInfo += o.templates.textCell.format({name: Locale.fields.projects.info_commonInfo[key], value: val});
                            });

                            /* Table data for VM */
                            var resultItem,
                                check;

                            //     /* Vulnerabilities, misconfigurations */
                            // for (i in o.parentObj.data.scan.results) {
                            //     resultItem = o.parentObj.data.scan.results[i];
                            //     check = o.parentObj.info.checks[resultItem.check];
                            //     if (!o.tablesData[o.classes[check.class]]) {
                            //         o.tablesData[o.classes[check.class]] = [];
                            //     }
                            //     o.tablesData[o.classes[check.class]].push({
                            //         name: check.title_ru, //TODO поменять на common.name(после изменения checks)
                            //         criticality: check.criticality //TODO поменять на info.criticality
                            //     });
                            // }

                            /* Sapnotes, Components */
                            o.tablesData['info_sapnotes'] = o.queriesData.sapnoteIssues;
                            o.tablesData['info_components'] = o.queriesData.componentIssues;
                            o.buildView();
                        }
                        if (projType == 'CODE') {
                            var categories = {
                                info_issuesByAuthor: 'author',
                                info_issuesByProgram: 'programName'
                            },
                                confCounts = {
                                Author: [10, 100],
                                Program: [10, 100]
                            },
                                arr = [],
                                index,
                                issue;

                            /* Table data for CODE */
                                /* Issues by author, issues by program */
                            $.each(categories, function(category, property){
                                arr = [];
                                for (i in o.queriesData.vulnerabilitiesInProgram) {
                                    issue = o.queriesData.vulnerabilitiesInProgram[i];
                                    if (!o.tablesData[category]) {
                                        o.tablesData[category] = [];
                                    }
                                    if (arr.indexOf(issue[property]) == -1) {
                                        arr.push(issue[property]);
                                        index = arr.indexOf(issue[property]);
                                        o.tablesData[category][index] = {};
                                        o.tablesData[category][index][property] = issue[property];
                                        o.tablesData[category][index].vulnerabilities = 1;
                                    } else {
                                        index = arr.indexOf(issue[property]);
                                        o.tablesData[category][index].vulnerabilities++;
                                    }
                                }
                            });

                                /* Top developers and programs */
                            $.each(confCounts, function(name, counts){
                                issues = o.tablesData['info_issuesBy' + name];
                                for (i in counts) {
                                    if (!o.tablesData['info_top' + counts[i] + name + 's']) {
                                        o.tablesData['info_top' + counts[i] + name + 's'] = [];
                                    }
                                    for (var j = 0; j < parseInt(counts[i]); j++) {
                                        if (issues[j]) {
                                            o.tablesData['info_top' + counts[i] + name + 's'].push(issues[j]);
                                        }
                                    }
                                }
                            });
                            /* Sorting data */
                            $.each(o.tablesData, function (metric){
                                o.tablesData[metric].sort(function (x, y) {
                                    return (y ["vulnerabilities"] - x ["vulnerabilities"]);
                                });
                            });

                            /* String data for CODE */
                                /* Total percentage of vulnerabilities by count of programs and authors */
                            var vulnerabilities,
                                totalVulnerabilities,
                                issues;
                            $.each(confCounts, function(name, counts){
                                issues = o.tablesData['info_issuesBy' + name];
                                totalVulnerabilities = 0;
                                for (k in issues) {
                                    totalVulnerabilities += issues[k]['vulnerabilities'];
                                }
                                for (i in counts) {
                                    vulnerabilities = 0;
                                    for(var j = 0; j < parseInt(counts[i]); j++) {
                                        if(issues[j]) {
                                            vulnerabilities += issues[j]['vulnerabilities'];
                                        }
                                    }
                                    rows['percentIssues' + counts[i] + name + 's'] = vulnerabilities * 100 / totalVulnerabilities;
                                }
                            });

                            /* Save info_commonInfo for CODE*/
                            $.each(rows, function(key, val){
                                o.info_commonInfo += o.templates.textCell.format({name: Locale.fields.projects.info_commonInfo[key], value: val});
                            });
                            o.buildView();
                        }
                        if (projType == 'SOD') {
                            o.tablesData['info_sapnotes'] = o.queriesData.sapnoteIssues;
                            o.buildView();
                        }
                    }
                    queryNum++;
                }
            });
        });
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on('change',this.sectionName + ' .selectInfo', function(e){
            var target = $(e.target),
                tableName = target.find('option:checked').data('name');
            o.createTable(tableName);
        });
        $(o.sectionName + ' .table iframe.blockResize')[0].contentWindow.onresize = function() {
            var activeRow = $(o.sectionName + ' .table table tbody tr.active'),
                additionField = activeRow.find('.addition-field');
            additionField.outerWidth(activeRow.width());
        };
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    };
    this.load = function() {
        this.getData();
    };
    this.clearEvents = function(){
        $(document.body).off('change',this.sectionName + ' .selectInfo');
        $(this.sectionName + ' .table iframe.blockResize')[0].contentWindow.onresize = null;
    };
    this.clear = function() {
        this.clearEvents();
        this.section.empty();
    };
};