Jobs.SoDConflictsByUsersTable = function(obj) {
    this.parentObj = obj;
    this.locale = null;
    this.section = $(".tab-sod_conflicts_by_users_table");
    this.data = {};
    this.controls = {};
    this.path = ['assess', 'projects', 'sod'];
    this.templates = {
        loading: "<div class='loading'></div>",
        systemList: '<div class="system-list"></div>',
        link: '<a class="info" href="#">{link}</a>',
        riskMark: '<span data-risk="{risk}"></span>'
    };
    this.policyLevels = [
        {
            name: "businessProcesses",
            copiedFields: ["name", "type"]
        },
        {
            name: "businessRoles",
            copiedFields: ["name"]
        },
        {
            name: "actions",
            copiedFields: ["authObjects", "description", "name", "type"]
        }
    ];
    this.createCountOfUseQuery = function(users){
        var queriesArr = [],
            queryObj = function (transaction, username) {
                return {
                    "query": {
                    "bool": {
                        "must": [
                            {
                                "match": {
                                    "suser": username
                                }
                            },
                            {
                                "match": {
                                    "saptcode": transaction
                                }
                            }
                        ]
                    }
                }
                }
            };
        users.forEach(function(user){
            queriesArr.push({});
            queriesArr.push(queryObj(user["transaction1Name"], user["username"]));
            queriesArr.push({});
            queriesArr.push(queryObj(user["transaction2Name"], user["username"]));
        });
        return queriesArr;
    };
    this.getTransactionUseCounts = function(data){
        var o = this,
            queries = o.createCountOfUseQuery(data),
            useCountHeaderInners = this.section.find(".table thead th").filter("[data-name='useCount1'], [data-name='useCount2']").find("div.tablesorter-header-inner"),
            params = {
                searchType: "count"
            };
        useCountHeaderInners.append(this.templates.loading);
        new Elasticsearch(params).multipleQuery(queries, function(responses) {
            var rows = o.section.find(".table table tbody tr"),
                response,
                query,
                usernameCell,
                useCount1Cell,
                useCount2Cell,
                statusCell,
                responseUsername,
                responseTransaction,
                status;
            for(var i = 0; i < responses["responses"].length; i++) {
                response = responses["responses"][i];
                query = i == 0 ? queries[1] : queries[i * 2 + 1];

                responseUsername = query["query"]["bool"]["must"][0]["match"]["suser"];
                responseTransaction = query["query"]["bool"]["must"][1]["match"]["saptcode"];
                $.each(rows, function(){
                    usernameCell = $(this).find("td").eq(o.controls["conflictsTable"].getColumnNumber("username"));
                    useCount1Cell = $(this).find("td").eq(o.controls["conflictsTable"].getColumnNumber("useCount1"));
                    useCount2Cell = $(this).find("td").eq(o.controls["conflictsTable"].getColumnNumber("useCount2"));
                    statusCell = $(this).find("td").eq(o.controls["conflictsTable"].getColumnNumber("status"));
                    if(usernameCell.text() == responseUsername && responseTransaction == useCount1Cell.data("transaction")) {
                        useCount1Cell.html(o.templates.link.format({link: responses["responses"][i]["hits"]["total"]}));
                    } else if(usernameCell.text() == responseUsername && responseTransaction == useCount2Cell.data("transaction")) {
                        useCount2Cell.html(o.templates.link.format({link: responses["responses"][i]["hits"]["total"]}));
                    }
                    if(useCount1Cell.text() !== "" && useCount2Cell.text() !== "" && statusCell.text() == "") {
                        if(parseInt(useCount1Cell.text()) + parseInt(useCount2Cell.text()) == 0) {
                            status = o.parentObj.info.dictionaries["sod"]["conflictStatus"][0]
                        } else if(useCount1Cell.text() > 0 && useCount2Cell.text() == 0 || useCount2Cell.text() > 0 && useCount1Cell.text() == 0) {
                            status = o.parentObj.info.dictionaries["sod"]["conflictStatus"][1]
                        } else {
                            status = o.parentObj.info.dictionaries["sod"]["conflictStatus"][2]
                        }
                        statusCell.html(status);
                    }
                });
            }
            useCountHeaderInners.find(".loading").remove();
        });
    };
    this.conversionFieldRules = {
        "conflictsTable": {
            "ruleID": function(fields){
                var ruleId = "";
                fields["conflict"].forEach(function(conflict){
                    ruleId += conflict["businessRoleId"] + "+";
                });
                return ruleId;
            },
            "ruleName": function(fields){
                var o = this,
                    ruleName = "";
                fields["conflict"].forEach(function(conflict){
                    ruleName += o.data["SODBusinessRolesMap"][conflict["businessRoleId"]] + "+";
                });
                return ruleName.substr(0, ruleName.length - 1);
            },
            "ruleDescription": function(fields){
                var ruleDescription = "";
                this.parentObj.data.scan.policy.structure.forEach(function(structureItem){
                    structureItem["businessRoles"].forEach(function(businessRole){
                        fields["conflict"].forEach(function(conflict){
                            if(conflict["businessRoleId"] == businessRole["id"]){
                                ruleDescription = "Rule description";
                            }
                        });
                    });
                });
                return ruleDescription;
            },
            "risk": function(fields, name){
                return this.templates.riskMark.format({risk: fields["riskSeverity"].toLowerCase()}) + fields["riskSeverity"]
            },
            "username": function(fields, name){
                return fields["fields"]["userName"];
            },
            "transaction1": function(fields, name){
                return fields["conflict"][0]["actionName"]
            },
            "f1": function(fields, name){
                return fields["conflict"][0]["businessRoleId"]
            },
            "useCount1": function(fields, name){
                return {
                    value: "",
                    attr: {
                        name: "transaction",
                        value: fields["conflict"][0]["actionName"]
                    }
                }
            },
            "transaction2": function(fields, name){
                return fields["conflict"][1]["actionName"]
            },
            "f2": function(fields, name){
                return fields["conflict"][1]["businessRoleId"]
            },
            "useCount2": function(fields, name){
                return {
                    value: "",
                    attr: {
                        name: "transaction",
                        value: fields["conflict"][1]["actionName"]
                    }
                }
            },
            "status": function(fields, name){

            }
        }
    };
    this.filterIssuesByConflictsNumber = function(data){
        var filteredData = [];
        data.forEach(function(dataItem){
            if(dataItem["conflict"].length == 2) {
                filteredData.push(dataItem);
            }
        });
        return filteredData;
    };
    this.buildTable = function(data) {
        var params = {
          data: data,
          title: this.locale.conflictsTable["tableTitle"],
            ticks: {'__addition': true},
            option: {
              namespace: "projects",
              type: "conflictsTable",
              count_one_page: 50,
              sort_default: { column: 0, order: "asc" }
          }
        };
        this.controls["conflictsTable"] = new Table(params);
        var html = this.controls["conflictsTable"].getHTML(),
            section = $("section.content.tab-sod_conflicts_by_users_table");
        section.append(html);
        this.controls["conflictsTable"].build();
        var table = section.find('.table');
        Helpers.setBlockHeight(section);
    };
    this.getData = function () {
        var o = this;
        ERPScan.API.query("queries", "execute1", {
            params: {
                "from": [
                    {
                        entity: "ResultsSODConflict"
                    },
                    {
                        entity: "Issue"
                    }
                ],
                "query": "r.scanId = :scanId AND i.resultId = r.id",
                "fields": [
                    "i.riskSeverity as risk",
                    "i.user.userName as username",
                    "r.addresses as addresses"
                ],
                "parameters": [
                    {
                        name: "scanId",
                        value: o.parentObj.data.scan.scanId
                    }
                ]
            },
            type: "POST",
            callback: function(response) {
                var policy = o.parentObj.data.scan["policy"]["structure"],
                    policyMap = {};
                o.buildMapPolicyElements(policy["structure"], policyMap, 1);
                var rulesMap = o.buildMapPolicyRules(o.parentObj.data.scan["policy"]["rules"]["rules"]),
                    data = o.convertDataForTable(response.data.result, policyMap, rulesMap);
                o.buildView.call(o);
                o.buildTable.call(o, data);
                o.data["transactionUseCounts"] = o.getTransactionUseCounts(data);
            }
        });
    };
    this.buildView = function(data) {
        var block = new Block({
            title: this.locale["risksTable"].systemsTree,
            content: this.templates.systemList,
            class: "systems-tree"
        }).getHTML();
        this.section.html(block);
        this.treeObj = new Tree(this.section.find(".system-list"), this.parentObj.data.scan.appServersTree, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
    };
    this.showTransaction = function(username, data) {
        var formattedData = [];
        for (var i = 0; i < data.length; i++) {
            formattedData.push({
                "date": {
                    value: data[i]["_source"]["rt"],
                    attr: {
                        name: "ts",
                        value: new Date(data[i]["_source"]["rt"]).getTime()
                    }
                },
                "transaction": data[i]["_source"]["saptcode"],
                "SAPProgram": data[i]["_source"]["sapprogram"]
            });
        }
        var table = new Table({
            title: "",
            ticks: {"__addition": true},
            data: formattedData,
            option: {
                namespace: "projects",
                type: "transactionsByUser",
                sort_default: { column: 1, order: "desc" }
            }
        });
        $(".tansactionsByUser.popup .block-content .content").html(table.getHTML());
        table.build();
        this.controls.transactionsPopup.setSize();
    };
    this.loadTransactions = function(username, transactionCode) {
        var o = this;
        this.controls.transactionsPopup = new Popup({
            class: 'tansactionsByUser',
            title: this.locale["conflictsTable"]["transactionsPopupTitle"].format({username: username}),
            content: this.templates.loading
        });
        this.controls.transactionsPopup.show();
        new Elasticsearch().query({
            "bool": {
                "must": [
                    {
                        "match": {
                            "suser": username
                        }
                    },
                    {
                        "match": {
                            "saptcode": transactionCode
                        }
                    }
                ]
            }
        }, function(hits) {
            o.showTransaction.call(o, username, hits);
        });
    };
    this.filterIssuesByTree = function($treeItem, data){
        var filteredData = [],
            itemType = $treeItem.data("type"),
            itemVal = $treeItem.data("id"),
            $parentItem = $treeItem.parents(".tree-item");
        data.forEach(function(dataItem){
            if(dataItem["targetSystem"][itemType] == itemVal){
                filteredData.push(dataItem);
            }
        });
        if($parentItem.length > 0) {
            return this.filterIssuesByTree($parentItem, filteredData);
        } else {
            return filteredData;
        }
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on("click", this.section.selector + " .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if ($item.data("type") == null) {
                return false;
            }
            o.section.find(" .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.section.find('.table').remove();
            var filteredData = o.filterIssuesByTree($item, o.data["usersByIssues"]);
            o.buildTable(filteredData);
            return false;
        });
        $(document.body).on("click", this.section.selector + " table td a.info", function(e){
            e.stopPropagation();
            var $this = $(this),
                dataForTable = [],
                username = $this.closest("tr").find("td").eq(o.controls["conflictsTable"].getColumnNumber("username")).text(),
                transaction = $this.closest("td").data("transaction");
            for(var i = 0; i < 25; i++) {
                dataForTable.push({
                    transaction: (Math.random()*100).toFixed(0)
                });
            }
            o.loadTransactions(username, transaction);
        });
    };
    this.convertDataForTable = function(results, policyMap, rules) {
        var resultsFormatted = [];
        for (var i = 0; i < results.length; i++) {
            var businessRoles = [];
            resultsFormatted[i] = {};
            for (var j = 0; j < results[i]["addresses"].length; j++) {
                var businessRoleObj = policyMap[results[i]["addresses"][j][0]].child[results[i]["addresses"][j][1]],
                    businessRoleName = businessRoleObj["params"]["name"]["en_US"];
                businessRoles.push(businessRoleName);
                resultsFormatted[i]["function" + (j + 1) + "Name"] = businessRoleName;
                resultsFormatted[i]["transaction" + (j + 1) + "Name"] = businessRoleObj.child[results[i]["addresses"][j][2]]["params"]["name"];
            }
            var criterionArray = businessRoles.sort(),
                criterionString = criterionArray.join("|"),
                rule = rules[criterionString];
            resultsFormatted[i]["ruleID"] = criterionArray.join(" + ");
            resultsFormatted[i]["businessProcess"] = policyMap[results[i]["addresses"][0][0]].params["name"]["en_US"];
            resultsFormatted[i]["ruleDescription"] = rule["description"];
            resultsFormatted[i]["risk"] = results[i]["risk"];
            resultsFormatted[i]["ruleName"] = rule["name"];
            resultsFormatted[i]["username"] = results[i]["username"];
        }
        return resultsFormatted;
    };
    this.buildMapPolicyElements = function(item, map, level) {
        for (var i = 0; i < item.length; i++) {
            map[item[i]["id"]] = {
                params: {}
            };
            var copiedFields = this.policyLevels[level - 1].copiedFields;
            for (var j = 0; j < copiedFields.length; j++) {
                map[item[i]["id"]].params[copiedFields[j]] = item[i][copiedFields[j]];
            }
            if (level < this.policyLevels.length) {
                map[item[i]["id"]].child = {};
                this.buildMapPolicyElements(item[i][this.policyLevels[level].name], map[item[i]["id"]].child, level + 1);
            }
        }
    };
    this.buildMapPolicyRules = function(rules) {
        var hashMap = {};
        for (var i = 0; i < rules.length; i++) {
            rules[i]["criterion"] = rules[i]["criterion"].sort();
            var criterionString = rules[i]["criterion"].join("|");
            hashMap[criterionString] = rules[i];
        }
        return hashMap;
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    };
    this.load = function() {
        this.getData();
        this.handlers();
    };
    this.clearEvents = function(){
        $(document.body).off("click", this.section.selector + " .system-list .tree .tree-item .text");
        $(document.body).off("click", this.section.selector + " table td a.info");
    };
    this.clear = function() {
        this.clearEvents();
        $("section.content.tab-test").empty();
    };
};