Jobs.SoDConflictsByUsersMatrix = function(obj, name) {
    this.parentObj = obj;
    this.section = $(".tab-sod_conflicts_by_users_matrix");
    this.controls = {};
    this.path = ['assess', 'projects', "sod"];
    this.data = {};
    this.locale = null;
    this.name = name;
    this.templates = {
        loading: "<div class='loading'></div>",
        systemList: '<div class="system-list"></div>',
        link: '<a class="info" href="#">{link}</a>',
        riskMark: '<div class="{class} riskMark" data-risk="{risk}"></div>',
        usedMark: "<div class='usedMark'></div>"
    };
    this.queries = {
        "usersBySODIssues": function(){
            return {
                title: "Users by SOD issues",
                id: "usersBySODIssues",
                queryObj: {
                    "from": [
                        {
                            entity: "Issue"
                        },
                        {
                            entity: "Object"
                        }
                    ],
                    "query": "o.objectType = :type1 AND o.issueId = i.id",
                    "fields": [
                        "i.id as id",
                        "i.conflict as conflict",
                        "i.targetSystem as targetSystem",
                        "i.riskSeverity as riskSeverity",
                        "o.fields as fields"
                    ],
                    "parameters": [
                        {
                            name: "type1",
                            value: "user"
                        }
                    ]
                }
            }
        }
    };
    this.policyLevels = [
        {
            name: "businessProcesses",
            copiedFields: ["name", "type"]
        },
        {
            name: "businessRoles",
            copiedFields: ["name"]
        },
        {
            name: "actions",
            copiedFields: ["authObjects", "description", "name", "type"]
        }
    ];
    this.matrixScaleCallback = function(){
        var $this = $(this),
            $matrixCells = $this.find(".cell"),
            $tableCells = $this.find(".dataTable .cell");
        if($tableCells.outerHeight() < 30) {
            $matrixCells.removeClass("smallCell");
            $matrixCells.addClass("miniCell");
        } else if($tableCells.outerHeight() < 60) {
            $matrixCells.removeClass("mediumCell");
            $matrixCells.addClass("smallCell");
            $matrixCells.removeClass("miniCell");
        } else {
            $matrixCells.removeClass("smallCell");
            $matrixCells.addClass("mediumCell");
        }
    };
    this.buildMatrix = function(data){
        var params = {
            bottomHeadling: this.locale["conflictsMatrix"]["matrixBottomHeadling"],
            topHeadling: this.locale["conflictsMatrix"]["matrixTopHeadling"],
            colLinks: true,
            rowLinks: true,
            scaleCallback: this.matrixScaleCallback,
            colNames: data["users"],
            rowNames: data["roles"],
            data: data,
            class: 'sodConflictsMatrix'
        };
        var matrix = new Matrix(params),
            html = matrix.getHTML(),
            section = $("section.content.tab-" + this.name),
            html = new Block({
                title: this.locale["conflictsMatrix"]["matrixTitle"],
                content: html,
                class: "sod-matrix"
            }).getHTML();
        section.append(html);
        matrix.setSize();
        Helpers.setBlockHeight(section);
    };
    this.getUsersByIssues = function(){
        var o = this;
        ERPScan.API.query("queries", "usersBySODIssues", {
            params: {
                query: JSON.stringify(this.queries["usersBySODIssues"].queryObj)
            },
            type: "POST",
            callback: function(response) {
                o.data["usersByIssues"] = response["data"]["result"];
                var usedTransactionsQueries = o.createUsedTransactionsInfoQueries(o.data["usersByIssues"]);
                var appServerTreeData = o.buildSODAppServersTree(o.data["usersByIssues"]);
                o.buildView(appServerTreeData);
                o.getUsedTransactionsStatistic(usedTransactionsQueries);
            }
        });
    };
    this.getUsedTransactionInfo = function(username, transaction){
        var o = this;
        this.controls.transactionsPopup = new Popup({
            class: 'tansactionsByUser',
            title: this.locale["conflictsTable"]["transactionsPopupTitle"].format({username: username}),
            content: this.templates.loading
        });
        this.controls.transactionsPopup.show();
        new Elasticsearch().query({
            "bool": {
                "must": [
                    {
                        "match": {
                            "suser": username
                        }
                    },
                    {
                        "match": {
                            "saptcode": transaction
                        }
                    }
                ]
            }
        }, function(hits) {
            o.showTransaction.call(o, username, hits);
        });
    };
    this.showTransaction = function(username, data) {
        var formattedData = [];
        for (var i = 0; i < data.length; i++) {
            formattedData.push({
                "date": {
                    value: data[i]["_source"]["rt"],
                    attr: {
                        name: "ts",
                        value: new Date(data[i]["_source"]["rt"]).getTime()
                    }
                },
                "transaction": data[i]["_source"]["saptcode"],
                "SAPProgram": data[i]["_source"]["sapprogram"]
            });
        }
        var table = new Table({
            title: "",
            ticks: {"__addition": true},
            data: formattedData,
            option: {
                namespace: "projects",
                type: "transactionsByUser",
                sort_default: { column: 1, order: "desc" }
            }
        });
        $(".tansactionsByUser.popup .block-content .content").html(table.getHTML());
        table.build();
        this.controls.transactionsPopup.setSize();
    };
    this.getUsedTransactionsStatistic = function(queries){
        var o = this,
            params = {
            searchType: "count"
        };
        o.data["usedTransactionsQueries"] = queries;
        new Elasticsearch(params).multipleQuery(queries, function(responses) {
            o.data["dataForMatrix"] = {};
            o.data["usedTransactions"] = responses["responses"];
            o.buildMatrix(o.convertDataForMatrix(o.data["usersByIssues"]));
        });
    };
    this.createUsedTransactionsInfoQueries = function(results, policyMap){
        var queriesArr = [],
            queryObj = function (transaction, username) {
                return {
                    "query": {
                        "bool": {
                            "must": [
                                {
                                    "match": {
                                        "suser": username
                                    }
                                },
                                {
                                    "match": {
                                        "saptcode": transaction
                                    }
                                }
                            ]
                        }
                    }
                }
            };
        for (var i = 0; i < results.length; i++) {
            for (var j = 0; j < results[i]["addresses"].length; j++) {
                var businessRoleObj = policyMap[results[i]["addresses"][j][0]].child[results[i]["addresses"][j][1]];
                queriesArr.push({});
                queriesArr.push(queryObj(businessRoleObj.child[results[i]["addresses"][j][2]]["params"]["name"], results[i]["username"]));
            }
        }
        return queriesArr;
    };
    this.isCriticalActionUsed = function(username, transaction){
        var query;
        for(var i = 0; i < this.data["usedTransactionsQueries"].length; i++) {
            query = this.data["usedTransactionsQueries"][i];
            if (typeof this.data["usedTransactions"] != "undefined" && typeof query["query"] != "undefined" && query["query"]["bool"]["must"][0]["match"]["suser"] == username && query["query"]["bool"]["must"][1]["match"]["saptcode"] == transaction) {
                return this.data["usedTransactions"][(i - 1) / 2]["hits"]["total"] > 0;
            }
        }
    };
    this.convertDataForMatrix = function(data){
        var o = this,
            convertedData = [],
            obj = {},
            businessRole,
            users = [],
            roles = [];
        o.data["dataForMatrix"]["users"] = {main: []};
        o.data["dataForMatrix"]["roles"] = {main: []};
        data.forEach(function(dataItem){
            if(users.indexOf(dataItem["fields"]["userName"]) == -1) {
                users.push(dataItem["fields"]["userName"]);
                o.data["dataForMatrix"]["users"]["main"].push([{
                    content: dataItem.fields.userName,
                    dataAttr: {
                        type: "user",
                        issue: dataItem.fields.userName
                    }
            }]);
            }
            dataItem["conflict"].forEach(function(conflict){
                businessRole = o.data["SODBusinessRolesMap"][conflict["businessRoleId"]];
                if(roles.indexOf(businessRole) == -1) {
                    roles.push(businessRole);
                    o.data["dataForMatrix"]["roles"]["main"].push([{
                        content: businessRole,
                        dataAttr: {
                            type: "role",
                            id: conflict["businessRoleId"]
                        }
                    }]);
                }
                obj = {};
                obj["rowGroup"] = "main";
                obj["colGroup"] = "main";
                obj["content"] = o.templates.riskMark.format({
                    risk: dataItem["riskSeverity"].toLowerCase()
                });
                if(o.isCriticalActionUsed(dataItem["fields"]["userName"], conflict["actionName"])) {
                    obj["dataAttr"] = {
                        transaction: conflict["actionName"],
                        username: dataItem["fields"]["userName"]
                    };
                    obj["content"] += o.templates.usedMark;
                }
                obj["rowName"] = businessRole;
                obj["colName"] = dataItem["fields"]["userName"];
                convertedData.push(obj);
            })
        });
        return convertedData;
    };

    this.filterUsersByRole = function(userName){
        var o = this,
            repeatList = [],
            businessRole,
            filteredData = [];
        this.data["usersByIssues"].forEach(function(dataItem){
            if(dataItem["fields"]["userName"] == userName) {
                dataItem["conflict"].forEach(function(conflict){
                    businessRole = o.data["SODBusinessRolesMap"][conflict["businessRoleId"]];
                    if(repeatList.indexOf(businessRole) == -1) {
                        repeatList.push(businessRole);
                        filteredData.push({
                            businessRole: businessRole
                        });
                    }
                });
            }
        });
        return filteredData;
    };
    this.filterRolesByUser = function(roleId){
        var o = this,
            repeatList = [],
            filteredData = [];
        this.data["usersByIssues"].forEach(function(dataItem){
            dataItem["conflict"].forEach(function(conflict){
                if(conflict["businessRoleId"] == roleId) {
                    if(repeatList.indexOf(dataItem.fields["userName"]) == -1) {
                        repeatList.push(dataItem.fields["userName"]);
                        filteredData.push({
                            userName: dataItem.fields["userName"]
                        });
                    }
                }
            });
        });
        return filteredData;
    };
    this.buildInfoTable = function(data, $cell){
        var params = {
            data: data,
            ticks: {"__addition": true},
            option: {
                namespace: "projects",
                type: "SODConflictsMatrixInfo" + Helpers.firstLetterToUpperCase($cell.data("type")),
                sort_default: { column: 1, order: "asc" }
            }
        };
        this.controls.infoTable = new Table(params);
        return this.controls.infoTable.getHTML();
    };
    this.showInfoPopup = function(content, $cell){
        var params = {
            class: 'sod-conflicts-matrix-info',
            title: this.locale["conflictsMatrix"]["infoPopup"][$cell.data("type")].format({name: $cell.find("a.matrixHeadTextBox").text()}),
            content: content
        };
        this.controls.infoPopup = new Popup(params);
        this.controls.infoPopup.show();
        this.controls.infoTable.build();
        this.controls.infoPopup.setSize();
    };
    this.buildView = function(data) {
        var block = new Block({
            title: this.locale["risksTable"].systemsTree,
            content: this.templates.systemList,
            class: "systems-tree"
        }).getHTML();
        this.section.html(block);
        this.treeObj = new Tree(this.section.find(".system-list"), this.parentObj.data.scan.appServersTree, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
    };
    this.filterIssuesByTree = function($treeItem, data){
        var filteredData = [],
            itemType = $treeItem.data("type"),
            itemVal = $treeItem.data("id"),
            $parentItem = $treeItem.parents(".tree-item");
        data.forEach(function(dataItem){
            if(dataItem["targetSystem"][itemType] == itemVal){
                filteredData.push(dataItem);
            }
        });
        if($parentItem.length > 0) {
            return this.filterIssuesByTree($parentItem, filteredData);
        } else {
            return filteredData;
        }
    };
    this.getData = function () {
        var o = this;
        ERPScan.API.query("queries", "execute1", {
            params: {
                "from": [
                    {
                        entity: "ResultsSODConflict"
                    },
                    {
                        entity: "Issue"
                    }
                ],
                "query": "r.scanId = :scanId AND i.resultId = r.id",
                "fields": [
                    "i.riskSeverity as risk",
                    "i.user.userName as username",
                    "r.addresses as addresses"
                ],
                "parameters": [
                    {
                        name: "scanId",
                        value: o.parentObj.data.scan.scanId
                    }
                ]
            },
            type: "POST",
            callback: function(response) {
                var policy = o.parentObj.data.scan["policy"]["structure"],
                    policyMap = {};
                o.buildMapPolicyElements(policy["structure"], policyMap, 1);
                var usedTransactionsQueries = o.createUsedTransactionsInfoQueries(response.data.result, policyMap);
                o.getUsedTransactionsStatistic(usedTransactionsQueries);
                var rulesMap = o.buildMapPolicyRules(o.parentObj.data.scan["policy"]["rules"]["rules"]),
                    data = o.convertDataForMatrix(response.data.result, policyMap, rulesMap);
                o.buildView.call(o);
                o.buildMatrix.call(o, data);
            }
        });
    };
    this.convertDataForMatrix = function(results, policyMap, rules) {
        var matrixParams = {
                "roles": {
                    "main": []
                },
                "users": {
                    "main": []
                },
                "data": []
            },
            businessRolesMap = {},
            usersMap = {};
        for (var i = 0; i < results.length; i++) {
            var addresses = results[i]["addresses"];
            for (var j = 0; j < addresses.length; j++) {
                var businessRole = policyMap[addresses[j][0]].child[addresses[j][1]]["params"]["name"]["en_US"];
                if (businessRolesMap[businessRole] == null) {
                    matrixParams["roles"]["main"].push([{
                        content: businessRole,
                        dataAttr: {
                            type: "role",
                            id: addresses[j][1]
                        }
                    }]);
                    businessRolesMap[businessRole] = 1;
                }
            }
            var username = results[i]["username"];
            if (usersMap[username] == null) {
                matrixParams["users"]["main"].push([{
                    content: username,
                    dataAttr: {
                        type: "user",
                        issue: username
                    }
                }]);
                usersMap[username] = 1;
            }
        }
        for (var i = 0; i < results.length; i++) {
            var addresses = results[i]["addresses"];
            for (var j = 0; j < addresses.length; j++) {
                var businessRoleObj = policyMap[results[i]["addresses"][j][0]].child[results[i]["addresses"][j][1]],
                    businessRoleName = businessRoleObj["params"]["name"]["en_US"],
                    username = results[i]["username"],
                    transaction = businessRoleObj.child[results[i]["addresses"][j][2]]["params"]["name"],
                    isCriticalAction = this.isCriticalActionUsed(username, transaction);
                matrixParams["data"].push({
                    "content": this.templates.riskMark.format({
                        risk: results[i]["risk"]
                    }) + (isCriticalAction ? this.templates.usedMark : ""),
                    "rowGroup": "main",
                    "colGroup": "main",
                    "colName": username,
                    "rowName": businessRoleName,
                    "dataAttr": {
                        transaction: transaction,
                        username: username
                    }
                });
            }
        }
        console.log(matrixParams);
        return matrixParams;
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on("click", this.section.selector + " .sod-matrix .cell:has(a.matrixHeadTextBox)", function(){
            var $cell = $(this).closest(".cell"),
                filteredData = $cell.data("type") == "user" ? o.filterUsersByRole($cell.data("issue")) : o.filterRolesByUser($cell.data("id")),
                infoTableHTML = o.buildInfoTable(filteredData, $cell);
            o.showInfoPopup(infoTableHTML, $cell);
        });
        $(document.body).on("click", this.section.selector + " .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if ($item.data("type") == null) {
                return false;
            }
            o.section.find(" .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.section.find('.sod-matrix').remove();
            var filteredData = o.filterIssuesByTree($item, o.data["usersByIssues"]),
                dataForMatrix = o.convertDataForMatrix(filteredData);
            o.buildMatrix(dataForMatrix);
            return false;
        });
        $(document.body).on("click", this.section.selector + " .matrix .usedMark", function(e) {
            var cell = $(e.target).closest(".cell");
            o.getUsedTransactionInfo(cell.data("username"), cell.data("transaction"));
        });
    };
    this.load = function() {
        this.getData();
        this.handlers();
    };
    this.buildMapPolicyElements = function(item, map, level) {
        for (var i = 0; i < item.length; i++) {
            map[item[i]["id"]] = {
                params: {}
            };
            var copiedFields = this.policyLevels[level - 1].copiedFields;
            for (var j = 0; j < copiedFields.length; j++) {
                map[item[i]["id"]].params[copiedFields[j]] = item[i][copiedFields[j]];
            }
            if (level < this.policyLevels.length) {
                map[item[i]["id"]].child = {};
                this.buildMapPolicyElements(item[i][this.policyLevels[level].name], map[item[i]["id"]].child, level + 1);
            }
        }
    };
    this.buildMapPolicyRules = function(rules) {
        var hashMap = {};
        for (var i = 0; i < rules.length; i++) {
            rules[i]["criterion"] = rules[i]["criterion"].sort();
            var criterionString = rules[i]["criterion"].join("|");
            hashMap[criterionString] = rules[i];
        }
        return hashMap;
    };
    this.clearEvents = function(){
        $(document.body).off("click", this.section.selector + " .sod-matrix a.matrixHeadTextBox");
        $(document.body).off("click", this.sectionSelector + " .system-list .tree .tree-item .text");
    };
    this.clear = function() {
        $("section.content.tab-" + this.name + "").empty();
        this.clearEvents();
    };
};