Jobs.SoDMatrix = function(obj, name) {
    this.parentObj = obj;
    this.name = name;
    this.locale = null;
    this.path = ['assess', 'jobs', "results"];
    this.treeObj = null;
    this.table = null;
    this.sodCheck = "SOD_000";
    this.data = {};
    this.systemTreeLevels = {
        1: "sid",
        2: "ip",
        3: "appserver",
        4: "service",
        5: "client"
    };
    this.entities = {
        users: {
            check: "SOD_0002"
        },
        roles: {
            check: "SOD_0003"
        },
        profiles: {
            check: "SOD_0002"
        }
    };
    this.templates = {
        wrap: '<div class="blocks"><div class="block col-3 systems-tree"><h3>%s</h3><div class="system-list"></div></div></div>',
        table: "<div class='block col-2 sod-matrix'><div class='table-wrapper'><h3>%s</h3><table>%s</table></div></div>",
        tr: "<tr>%s</tr>",
        td: "<td class='risk-%s'>%s</td>",
        select: "<select name='risk'>%s</select>",
        option: "<option value='%d'>%s</option>",
        no_entry: '<div class="no_entry">' + Locale.ui.page["no_entry"] + '</div>',
        policy_title: '<h3>%s</h3>',
        actions: '<button class="save">' + Locale.ui.page["save"] + '</button>',
        cell: '<div class="users">Users: <b>%s</b></div><div class="roles">Roles: <b>%s</b></div>'
    };
    this.risks = {
        H: "high",
        M: "medium",
        L: "low",
        N: "no"
    };
    this.buildMatrix = function(functions, data) {
        var rows = "";
        var cells = this.templates.td.sprintf("none", "");
        for (var i = 0; i < functions.length; i++) {
            cells += this.templates.td.sprintf("none", functions[i]);
        }
        rows += this.templates.tr.sprintf(cells);
        for (var i = 0; i < functions.length; i++) {
            cells = this.templates.td.sprintf("none", functions[i]);
            for (var j = 0; j < functions.length; j++) {
                if (typeof data[i][j] != "undefined") {
                    cells += this.templates.td.sprintf(this.risks[data[i][j].risk], this.templates.cell.sprintf(data[i][j]["number_users"], data[i][j]["number_roles"]));
                } else {
                    cells += this.templates.td.sprintf("none", "");
                }
            }
            rows += this.templates.tr.sprintf(cells);
        }
        var table = this.templates.table.sprintf("Risks matrix", rows);
        $("section.content.tab-sod_risks_matrix .blocks").append(table);
    };
    this.loadCheckInfo = function(values) {
        var o = this;
        var tables = {};
        var query_number = Helpers.numberProperty(this.entities);
        var query_counter = 1;
        $(".block.check-description .block-content").html(this.templates.block_content.loading);
        for (var entity in this.entities) {
            ERPScanAPI.query("/engine/project/results/extra/" + this.parentObj.scanInfo.ID, function(entity) {
                return function(result) {
                    tables[entity] = o.generateEntitiesTable.call(o, result);
                    if (query_number == query_counter) {
                        o.pasteCheckInfo.call(o, values, tables);
                    }
                    query_counter++;
                }
            }(entity), {
                checkId: this.entities[entity].check,
                extraA: 1,
                extraB: values.brole_id,
                extraC: values.name,
                credId: values.credId
            });
        }
    };
    this.buildView = function() {
        var html = this.templates.wrap.sprintf(this.locale["systemsTree"], "");
        $("section.content.tab-" + this.name + "").html(html);
        this.treeObj = new Tree($("section.content.tab-" + this.name + " .system-list"), this.parentObj.systemsTree, { expandWhenClick: false });
        this.treeObj.init();
    };
    this.systemTreeClearEmpty = function() {
        var o = this;
        $(".system-list .tree .tree-item").each(function() {
            var creds = String($(this).data("info")).split(",");
            var for_remove = true;
            for (var i = 0; i < creds.length; i++) {
                if (o.data[creds[i]]) {
                    for_remove = false;
                }
            }
            if (for_remove) {
                $(this).remove();
            }
        });
    };
    this.loadCheckInfo = function(values) {
        var o = this;
        var tables = {};
        var query_number = Helpers.numberProperty(this.entities);
        var query_counter = 1;
        $(".block.check-description .block-content").html(this.templates.block_content.loading);
        for (var entity in this.entities) {
            ERPScanAPI.query("/engine/project/results/extra/" + this.parentObj.scanInfo.ID, function(entity) {
                return function(result) {
                    tables[entity] = o.generateEntitiesTable.call(o, result);
                    if (query_number == query_counter) {
                        o.pasteCheckInfo.call(o, values, tables);
                    }
                    query_counter++;
                }
            }(entity), {
                checkId: this.entities[entity].check,
                extraA: 1,
                extraB: values.brole_id,
                extraC: values.name,
                credId: values.credId
            });
        }
    };
    this.applyCred = function($item) {
        var o = this;
        $("section.content.tab-" + o.name + " .system-list .tree .tree-item").removeClass("active");
        $item.addClass("active");
        var creds = String($item.data("info")).split(","),
            cats = o.parentObj.scanInfo.critical_actions.cats,
            functions = [],
            data = [],
            matrix = o.parentObj.scanInfo.critical_actions.matrix[creds[0]];
        for (var cat in cats) {
            functions.push(cats[cat].data.name);
        }
        for (var i = 0; i < functions.length; i++) {
            data[i] = [];
        }
        for (var element in matrix) {
            var coordinates = element.split("_");
            data[coordinates[0] - 1][coordinates[1] - 1] = matrix[element];
        }
        console.log(functions);
        $(".block.sod-matrix").remove();
        o.buildMatrix(functions, data);
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", "section.content.tab-" + o.name + " .table table.tablesorter tbody tr.leaf", function() {
            $("section.content.tab-" + o.name + " .table table.tablesorter tr").removeClass("active");
            var $this = $(this),
                $cells = $this.find("td"),
                name = $cells.eq(o.table.getColumnNumber("name")).text(),
                info = $cells.eq(o.table.getColumnNumber("name")).data("info").split(":"),
                size = $cells.eq(o.table.getColumnNumber("value")).text(),
                credId = $cells.eq(o.table.getColumnNumber("sid")).data("credid");
            $this.toggleClass("active");
            o.loadCheckInfo.call(o, {
                name: name,
                size: size,
                transaction: info[0],
                auth: info[1],
                brole_id: info[2],
                credId: credId
            });
        });
        $(document.body).on("change", "section.content.tab-" + o.name + " .block-content-tab.additional select", function() {
            var $this = $(this);
            var $tables = $("section.content.tab-" + o.name + " .block-content-tab.additional .entities-table");
            $tables.hide();
            if ($this.val() == "0") {
                return false;
            }
            $tables.filter("." + $this.val()).show();
        });
        $(document.body).on("mousedown", "section.content.tab-" + o.name + " .table table.tablesorter tbody tr.branch", function(event) {
            if ($(event.target).parent(".indenter").length > 0) {
                return false;
            }
            var $this = $(this);
            var expanded = $this.hasClass("expanded");
            $("section.content.tab-" + o.name + " .table table.tablesorter").treetable(expanded ? "collapseNode" : "expandNode", $this.data("ttId"));
        });
        $(document.body).on("click", "section.content.tab-" + o.name + " .check-description .block-tabs .block-tab", function() {
            var $this = $(this);
            var name = $this.data("name");
            $("section.content.tab-" + o.name + " .check-description .block-content-tab").removeClass("active");
            $("section.content.tab-" + o.name + " .check-description .block-content-tab." + name).addClass("active");
            $("section.content.tab-" + o.name + " .check-description .block-tabs .block-tab").removeClass("active");
            $this.addClass("active");
            return false;
        });
        $(document.body).on("click", "section.content.tab-" + o.name + " .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if (!$item.hasClass("level-5")) {
                return false;
            }
            $("section.content.tab-" + o.name + " .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.applyCred($item);
            return false;
        });
    };
    this.clear = function() {
        $(document.body).off("click", "section.content.tab-" + this.name + " table.tablesorter tr.leaf");
        $(document.body).off("mousedown", "section.content.tab-" + this.name + " table.tablesorter tr.branch");
        $(document.body).off("click", "section.content.tab-" + this.name + " .check-description .block-tabs .block-tab");
        $(document.body).off("click", "section.content.tab-" + this.name + " .system-list .tree .tree-item .text");
        this.treeObj.clear();
        $("section.content.tab-" + this.name + "").empty();
    };
    this.load = function() {
        this.buildView();
        this.applyCred($("section.content.tab-" + this.name + " .system-list .tree .tree-item.level-5").eq(0));
        this.handlers();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    }
};