Jobs.SoDMatrix = function(obj, name) {
    this.parentObj = obj;
    this.controls = {};
    this.sectionSelector = '.tab-' + name;
    this.section = $(this.sectionSelector);
    this.path = ['assess', 'projects', "sod"];
    this.data = {};
    this.locale = null;
    this.treeObj = null;
    this.name = name;
    this.templates = {
        wrap: '<div class="block systems-tree"><h3>{headling}</h3><div class="system-list"></div></div>',
        cell: '<div class="conflictsBox"><div class="conflictsNumber" data-type="users">Users: <a href="#" data-link="showObjects"><b> {users}</b></a></div><div class="conflictsNumber" data-type="roles">Roles: <a href="#" data-link="showObjects"><b> {roles}</b></a></div><div class="conflictsNumber" data-type="profiles">Profiles: <a href="#" data-link="showObjects"><b> {profiles}</b></a></div></div>',
    };
    this.conversionFieldRules = {
        "issueObjects": {
            "appServer": null,
            "objectType": null,
            "fields": null,
            "id": null,
            "objectName": null,
            "name": function(fields){
                return fields.objectName
            }
        }
    };
    this.policyLevels = [
        {
            name: "businessProcesses",
            copiedFields: ["name", "type"]
        },
        {
            name: "businessRoles",
            copiedFields: ["name"]
        },
        {
            name: "actions",
            copiedFields: ["authObjects", "description", "name", "type"]
        }
    ];
    this.buildView = function() {
        var html = this.templates.wrap.format({headling: this.locale["risksMatrix"]["systemsTree"]});
        $("section.content.tab-" + this.name ).html(html);
        this.treeObj = new Tree($("section.content.tab-SODRisksMatrix .system-list"), this.parentObj.data.scan.appServersTree, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
    };
    this.getData = function() {
        var o = this;
        ERPScan.API.query("resultsSODConflict", "list", {
            params: {
                scanId: o.parentObj.data.scan.scanId
            },
            type: "POST",
            callback: function(response) {
                o.buildView.call(o);
                o.buildMatrix.call(o, response.data.result);
            }
        });
    };
    this.convertDataForMatrix = function(results, policyMap) {
        var matrixParams = {
            "businessRoles": {
                "main": []
            },
            "data": []
        },
            businessRolesMap = {};
        for (var i = 0; i < results.length; i++) {
            var addresses = results[i]["addresses"];
            if (addresses.length != 2) {
                continue;
            }
            matrixParams["businessRoles"]["main"] = [];
            for (var j = 0; j < addresses.length; j++) {
                var businessRole = policyMap[addresses[j][0]].child[addresses[j][1]]["params"]["name"]["en_US"];
                if (businessRolesMap[businessRole] == null) {
                    matrixParams["businessRoles"]["main"].push([{
                        "content": businessRole
                    }]);
                    businessRolesMap[businessRole] = 1;
                }
            }
        }
        for (var i = 0; i < results.length; i++) {
            matrixParams["data"].push({
                "content": this.templates.cell.format({
                    users: results[i]["statistic"]["users"],
                    roles: results[i]["statistic"]["roles"],
                    profiles: results[i]["statistic"]["profiles"]
                }),
                "rowGroup": "main",
                "colGroup": "main",
                "colName": policyMap[addresses[0][0]].child[addresses[0][1]]["params"]["name"]["en_US"],
                "rowName": policyMap[addresses[1][0]].child[addresses[1][1]]["params"]["name"]["en_US"],
                "dataAttr": {
                    "resultId": results[i]["id"],
                    "risk": "unknown"
                }
            });
            matrixParams["data"].push({
                "content": this.templates.cell.format({
                    users: results[i]["statistic"]["users"],
                    roles: results[i]["statistic"]["roles"],
                    profiles: results[i]["statistic"]["profiles"]
                }),
                "rowGroup": "main",
                "colGroup": "main",
                "rowName": policyMap[addresses[0][0]].child[addresses[0][1]]["params"]["name"]["en_US"],
                "colName": policyMap[addresses[1][0]].child[addresses[1][1]]["params"]["name"]["en_US"],
                "dataAttr": {
                    "resultId": results[i]["id"],
                    "risk": "unknown"
                }
            });
        }
        console.log(matrixParams);
        return matrixParams;
    };
    this.buildMapPolicyElements = function(item, map, level) {
        for (var i = 0; i < item.length; i++) {
            map[item[i]["id"]] = {
                params: {}
            };
            var copiedFields = this.policyLevels[level - 1].copiedFields;
            for (var j = 0; j < copiedFields.length; j++) {
                map[item[i]["id"]].params[copiedFields[j]] = item[i][copiedFields[j]];
            }
            if (level < this.policyLevels.length) {
                map[item[i]["id"]].child = {};
                this.buildMapPolicyElements(item[i][this.policyLevels[level].name], map[item[i]["id"]].child, level + 1);
            }
        }
    };
    this.buildMatrix = function(data) {
        var policy = this.parentObj.data.scan["policy"]["structure"]["structure"],
            policyMap = {};
        this.buildMapPolicyElements(policy, policyMap, 1);
        data = this.convertDataForMatrix(data, policyMap);
        var params = {
            bottomHeadling: this.locale["risksMatrix"]["matrixBottomHeadling"],
            topHeadling: this.locale["risksMatrix"]["matrixTopHeadling"],
            scaleCallback: this.matrixScaleCallback,
            colNames: data.businessRoles,
            rowNames: data.businessRoles,
            data: data.data,
            class: 'sodRisksMatrix'
        };
        var matrix = new Matrix(params),
            html = matrix.getHTML(),
            section = $("section.content.tab-" + this.name),
            html = new Block({
                title: this.locale["risksMatrix"]["matrixTitle"],
                content: html,
                class: "sod-matrix"
            }).getHTML();
        section.append(html);
        matrix.setSize();
        Helpers.setBlockHeight(section);
    };
    this.filterIssuesByConflictsNumber = function(data){
        var filteredData = [];
        data.forEach(function(dataItem){
            if(dataItem["conflict"].length == 2) {
                filteredData.push(dataItem);
            }
        });
        return filteredData;
    };
    this.matrixScaleCallback = function(){
        var $this = $(this),
            $dataTable = $this.find(".dataTable"),
            $tableCells = $dataTable.find(".cell");
        if($tableCells.outerHeight() <= 20){
            $tableCells.find(".conflictsBox").hide();
        } else if($tableCells.outerHeight() < 50) {
            $tableCells.find(".conflictsBox").show();
            $tableCells.removeClass("mediumCell");
            $tableCells.addClass("smallCell");
        } else {
            $tableCells.removeClass("smallCell");
            $tableCells.addClass("mediumCell");
        }
    };
    this.loadObjects = function(issueId, type){
        var o = this;
        ERPScan.API.query("objects", "list", {
            params: {
                issueId: issueId
            },
            callback: function(response) {
                o.data["issueObjects"] = response["data"]["result"];
                var tableHTML = o.buildObjectsTable(type);
                o.showObjectsPopup(tableHTML, type);
            }
        });
    };
    this.buildObjectsTable = function(type){
        var typeMap = {
            users: "user",
            roles: "role",
            profiles: "profile"
        },
            data = [];
        this.data["issueObjects"].forEach(function(dataItem){
            if(dataItem["objectType"] == typeMap[type]) {
                data.push(dataItem);
            }
        });
        var params = {
                data: this.parentObj.conversionFields(data, this.conversionFieldRules["issueObjects"]),
                ticks: {"__addition": true},
                option: {
                    namespace: "projects",
                    type: "SODRisksMatrixObjects",
                    sort_default: { column: 1, order: "asc" }
                }
            };
        this.controls.objectsTable = new Table(params);
        return this.controls.objectsTable.getHTML();
    };
    this.filterIssuesByTree = function($treeItem, data){
        var filteredData = [],
            itemType = $treeItem.data("type"),
            itemVal = $treeItem.data("id"),
            $parentItem = $treeItem.parents(".tree-item");
        data.forEach(function(dataItem){
            if(dataItem["targetSystem"][itemType] == itemVal){
                filteredData.push(dataItem);
            }
        });
        if($parentItem.length > 0) {
            return this.filterIssuesByTree($parentItem, filteredData);
        } else {
            return filteredData;
        }
    };
    this.showObjectsPopup = function(content, type){
        var params = {
            class: 'SODRisksMatrixObjects',
            title: this.locale["risksMatrix"]["objectsPopup"][type],
            content: content
        };
        this.controls.objectsPopup = new Popup(params);
        this.controls.objectsPopup.show();
        this.controls.objectsTable.build();
        this.controls.objectsPopup.setSize();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    };
    this.load = function() {
        this.getData();
        this.handlers();
    };
    this.clearEvents = function(){
        $(document.body).off("click", this.sectionSelector + " .system-list .tree .tree-item .text");
        $(document.body).off('click', this.sectionSelector + ' .sod-matrix table td a[data-link="showObjects"]');
    };
    this.clear = function() {
        this.clearEvents();
        this.treeObj.clear();
        $("section.content.tab-" + this.name + "").empty();
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", o.sectionSelector + " .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if ($item.data("type") == null) {
                return false;
            }
            o.section.find(" .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.section.find('.sod-matrix').remove();
            var filteredData = o.filterIssuesByTree($item, o.data["issues"]),
                dataForMatrix = o.convertDataForMatrix(filteredData);
            o.buildMatrix(dataForMatrix);
            return false;
        });
        $(document.body).on('click', o.sectionSelector + ' .sod-matrix .cell a[data-link="showObjects"]', function(e){
            var target = $(e.target),
                cell = target.closest("td"),
                type = target.closest("[data-type]").data("type"),
                issueId = cell.data("issueId");
            o.loadObjects(issueId, type);
        });
    };
};