Jobs.SoDRolesByActions = function(obj, name) {
    this.parentObj = obj;
    this.name = name;
    this.section = $(".tab-sod_roles_by_actions");
    this.controls = {};
    this.locale = null;
    this.path = ['assess', 'projects', 'sod'];
    this.data = {};
    this.templates = {
        loading: "<div class='loading'></div>",
        systemList: '<div class="system-list"></div>',
        link: '<a class="info" href="#">{link}</a>',
        riskMark: '<div class="riskMark" data-risk="{risk}">{text}</div>',
        usedMark: "<div class='usedMark' data-number='{number}'></div>"
    };
    this.queries = {
        "rolesBySODIssues": function(){
            return {
                title: "Roles by SOD issues",
                id: "rolesBySODIssues",
                queryObj: {
                    "from": [
                        {
                            entity: "Issue"
                        },
                        {
                            entity: "Object"
                        }
                    ],
                    "query": "o.objectType = :type1 AND o.issueId = i.id",
                    "fields": [
                        "i.id as id",
                        "i.conflict as conflict",
                        "i.statistic.users as users",
                        "i.targetSystem as targetSystem",
                        "i.riskSeverity as riskSeverity",
                        "o.fields as fields"
                    ],
                    "parameters": [
                        {
                            name: "type1",
                            value: "role"
                        }
                    ]
                }
            }
        }
    };
    this.createSODBusinessRolesMap = function(policy){
        var map = {};
        policy.structure.forEach(function(structureItem){
            structureItem["businessRoles"].forEach(function(businessRole){
                map[businessRole["id"]] = businessRole["name"];
            });
        });
        return map;
    };
    this.getIssuesByRole = function() {
        var o = this;
        ERPScan.API.query("queries", "rolesBySODIssues", {
            params: {
                query: JSON.stringify(this.queries["rolesBySODIssues"].queryObj)
            },
            type: "POST",
            callback: function(response) {
                o.data["rolesByIssues"] = response["data"]["result"];
                var appServerTreeData = o.buildSODAppServersTree(o.data["rolesByIssues"]);
                o.buildView(appServerTreeData);
                var usedTransactionsQueries = o.createUsedTransactionsStatisticQueries(o.data["rolesByIssues"]);
                o.getUsedTransactionsStatistic(usedTransactionsQueries);
            }
        });
    };
    this.getRoles = function(issues){
        var roles = {};
        issues.forEach(function(issue){
            if(typeof roles[issue["fields"]["profile"]] == "undefined") {
                roles[issue["fields"]["profile"]] = [];
            }
            roles[issue["fields"]["profile"]].push([
                {
                    content: issue["fields"]["roleName"]
                },
                {
                    content: issue["users"]
                },
                {
                    content: issue["users"] + 24
                }
            ])
        });
        return roles;
    };
    this.getTransactions = function(issues){
        var o = this,
            transactions = {},
            businessRole;
        transactions["_first"] = [];
        transactions["_first"].push({
            content: this.locale["rolesByActions"]["matrix"]["users"],
            class: "users"
        });
        transactions["_first"].push({
            content: this.locale["rolesByActions"]["matrix"]["conflicts"],
            class: "conflicts"
        });
        issues.forEach(function(issue){
            issue["conflict"].forEach(function(conflict){
                businessRole = o.data["SODBusinessRolesMap"][conflict["businessRoleId"]];
                if(typeof transactions[businessRole] == "undefined") {
                    transactions[businessRole] = [];
                }
                transactions[businessRole].push([{
                    content: conflict["actionName"],
                    dataAttr: {
                        brole: conflict["businessRoleId"]
                    }
                }]);
            });
        });
        return transactions;
    };
    this.showTransaction = function(username, data) {
        var formattedData = [];
        for (var i = 0; i < data.length; i++) {
            formattedData.push({
                "date": {
                    value: data[i]["_source"]["rt"],
                    attr: {
                        name: "ts",
                        value: new Date(data[i]["_source"]["rt"]).getTime()
                    }
                },
                "transaction": data[i]["_source"]["saptcode"],
                "SAPProgram": data[i]["_source"]["sapprogram"]
            });
        }
        var table = new Table({
            title: "",
            ticks: {"__addition": true},
            data: formattedData,
            option: {
                namespace: "projects",
                type: "transactionsByUser",
                sort_default: { column: 1, order: "desc" }
            }
        });
        $(".tansactionsByUser.popup .block-content .content").html(table.getHTML());
        table.build();
        this.controls.transactionsPopup.setSize();
    };
    this.getUsedTransactionsStatistic = function(queries){
        var o = this,
            params = {
                searchType: "count"
            };
        o.data["usedTransactionsQueries"] = queries;
        new Elasticsearch(params).multipleQuery(queries, function(responses) {
            o.data["dataForMatrix"] = {};
            o.data["usedTransactions"] = responses["responses"];
            var convertedData = {};
            convertedData["roles"] = o.getRoles(o.data["rolesByIssues"]);
            convertedData["transactions"] = o.getTransactions(o.data["rolesByIssues"]);
            convertedData["data"] = o.convertDataForMatrix(o.data["rolesByIssues"]);
            o.buildMatrix(convertedData)
        });
    };
    this.createUsedTransactionsStatisticQueries = function(issues){
        var queriesArr = [];
        issues.forEach(function(issue){
            issue["conflict"].forEach(function(conflict){
                queriesArr.push({});
                queriesArr.push({
                    "query": {
                        "bool": {
                            "must": [
                                {
                                    "match": {
                                        "suser": issue["fields"]["user"]
                                    }
                                },
                                {
                                    "match": {
                                        "saptcode": conflict["actionName"]
                                    }
                                }
                            ]
                        }
                    }
                });
            });
        });
        return queriesArr;
    };
    this.createUsedTransactionInfoQueries = function(businessRoleId, role, transaction){
        var queriesArr = [];
        this.data["rolesByIssues"].forEach(function(issue){
            issue["conflict"].forEach(function(conflict){
                if(issue["fields"]["roleName"] == role && conflict["actionName"] == transaction && conflict["businessRoleId"] == businessRoleId) {
                    queriesArr.push({});
                    queriesArr.push({
                        "query": {
                            "bool": {
                                "must": [
                                    {
                                        "match": {
                                            "suser": issue["fields"]["user"]
                                        }
                                    },
                                    {
                                        "match": {
                                            "saptcode": conflict["actionName"]
                                        }
                                    }
                                ]
                            }
                        }
                    });
                }
            });
        });
        return queriesArr;
    };
    this.getUsedTransactionInfo = function(queries, role){
        var o = this;
        this.controls.transactionsPopup = new Popup({
            class: 'transactionsByRole',
            title: this.locale["rolesByActions"]["transactionsPopupTitle"].format({role: role}),
            content: this.templates.loading
        });
        this.controls.transactionsPopup.show();
        new Elasticsearch().multipleQuery(queries, function(responses) {
            o.showTransaction.call(o, responses["responses"], queries);
        });
    };
    this.showTransaction = function(responses, queries) {
        var formattedData = [];
        for(var i = 0; i < responses.length; i++){
            responses[i]["hits"]["hits"].forEach(function(data){
                formattedData.push({
                    "date": {
                        value: data["_source"]["rt"],
                        attr: {
                            name: "ts",
                            value: new Date(data["_source"]["rt"]).getTime()
                        }
                    },
                    "transaction": data["_source"]["saptcode"],
                    "SAPProgram": data["_source"]["sapprogram"],
                    "user": i == 0 ? queries[1]["query"]["bool"]["must"][0]["match"]["suser"] : queries[i * 2 + 1]["query"]["bool"]["must"][0]["match"]["suser"]
                });
            });
        }
        var table = new Table({
            title: "",
            ticks: {"__addition": true},
            data: formattedData,
            option: {
                namespace: "projects",
                type: "transactionsByRole",
                sort_default: { column: 1, order: "desc" }
            }
        });
        $(".transactionsByRole.popup .block-content .content").html(table.getHTML());
        table.build();
        this.controls.transactionsPopup.setSize();
    };
    this.getActionUseCount = function(brole, role, transaction){
        var query,
            useCount = 0,
            users = [];
        this.data["rolesByIssues"].forEach(function(data){
            if(data["fields"]["roleName"] == role && users.indexOf(data["fields"]["user"]) == -1) {
                data["conflict"].forEach(function(conflict){
                    if(transaction == conflict["actionName"] && brole == conflict["businessRoleId"]) {
                        users.push(data["fields"]["user"]);
                    }
                });
            }
        });
        for(var i = 0; i < this.data["usedTransactionsQueries"].length; i++) {
            query = this.data["usedTransactionsQueries"][i];
            if(typeof query["query"] != "undefined" && users.indexOf(query["query"]["bool"]["must"][0]["match"]["suser"]) != -1 && query["query"]["bool"]["must"][1]["match"]["saptcode"] == transaction) {
                useCount += this.data["usedTransactions"][(i - 1) / 2]["hits"]["total"];
            }
        }
        return useCount;
    };
    this.convertDataForMatrix = function(issues) {
        var o = this,
            convertedData = [],
            businessRole,
            actionUseCount,
            obj;
        issues.forEach(function(issue){
            issue["conflict"].forEach(function(conflict){
                businessRole = o.data["SODBusinessRolesMap"][conflict["businessRoleId"]];
                obj = {};
                obj["rowGroup"] = issue["fields"]["profile"];
                obj["rowName"] = issue["fields"]["roleName"];
                obj["colGroup"] = businessRole;
                obj["colName"] = conflict["actionName"];
                actionUseCount = o.getActionUseCount(conflict["businessRoleId"], issue["fields"]["roleName"], conflict["actionName"]);
                if(actionUseCount > 0) {
                    obj["content"] = o.templates.riskMark.format({
                        risk: issue["riskSeverity"].toLowerCase(),
                        text: ""
                    });
                    obj["content"] += o.templates.usedMark.format({
                        number: actionUseCount
                    });
                    obj["dataAttr"] = {
                        transaction: conflict["actionName"],
                        role: issue["fields"]["roleName"],
                        brole: conflict["businessRoleId"]
                    };
                } else {
                    obj["content"] = o.templates.riskMark.format({
                        risk: issue["riskSeverity"].toLowerCase(),
                        text: ""
                    });
                }
                convertedData.push(obj);
            });
        });
        return convertedData;
    };
    this.buildSODAppServersTree = function(result) {
        var tree = [],
            obj,
            hierarchy = ["sid", "client"],
            hierarchyCreate = function(cred, level, nodes) {
                var currentHierarchy = hierarchy[level];
                if(typeof currentHierarchy != 'undefined') {
                    var isNodeRepeat = false;
                    nodes.forEach(function(node){
                        if(node.text == cred[currentHierarchy]) {
                            hierarchyCreate(cred, level+1, node.nodes);
                            isNodeRepeat = true;
                        }
                    });
                    if(!isNodeRepeat) {
                        obj = {
                            text: cred[currentHierarchy],
                            data: {
                                id: cred[currentHierarchy],
                                type: currentHierarchy
                            }
                        };
                        if(currentHierarchy != "client") {
                            obj["nodes"] = [];
                        }
                        nodes.push(obj);
                        hierarchyCreate(cred, level+1, nodes[nodes.length-1].nodes);
                    }
                }
            };
        for (var i = 0; i < result.length; i++) {
            var cred = result[i]["targetSystem"];
            hierarchyCreate(cred, 0, tree);
        }
        return tree;
    };
    this.matrixScaleCallback = function(){
        var $this = $(this),
            $matrixCells = $this.find(".cell"),
            $tableCells = $this.find(".dataTable .cell");
        if($tableCells.outerHeight() < 30) {
            $matrixCells.removeClass("smallCell");
            $matrixCells.addClass("miniCell");
        } else if($tableCells.outerHeight() < 60) {
            $matrixCells.removeClass("mediumCell");
            $matrixCells.addClass("smallCell");
            $matrixCells.removeClass("miniCell");
        } else {
            $matrixCells.removeClass("smallCell");
            $matrixCells.addClass("mediumCell");
        }
    };
    this.buildMatrix = function(data) {
        var params = {
            bottomHeadling: this.locale["rolesByActions"]["matrixBottomHeadling"],
            topHeadling: this.locale["rolesByActions"]["matrixTopHeadling"],
            colLinks: true,
            scaleCallback: this.matrixScaleCallback,
            colNames: data["transactions"],
            rowNames: data["roles"],
            data: data["data"],
            class: 'sodRolesByActions'
        };
        var matrix = new Matrix(params),
            html = matrix.getHTML(),
            html = new Block({
                title: this.locale["risksMatrix"]["matrixTitle"],
                content: html,
                class: "sod-matrix"
            }).getHTML();
        this.section.append(html);
        matrix.setSize();
        matrix.setColGroupBorders();
        Helpers.setBlockHeight(this.section);
    };
    this.filterIssuesByTree = function($treeItem, data){
        var filteredData = [],
            itemType = $treeItem.data("type"),
            itemVal = $treeItem.data("id"),
            $parentItem = $treeItem.parents(".tree-item");
        data.forEach(function(dataItem){
            if(dataItem["targetSystem"][itemType] == itemVal){
                filteredData.push(dataItem);
            }
        });
        if($parentItem.length > 0) {
            return this.filterIssuesByTree($parentItem, filteredData);
        } else {
            return filteredData;
        }
    };
    this.buildView = function(data) {
        var block = new Block({
            title: this.locale["risksTable"].systemsTree,
            content: this.templates.systemList,
            class: "systems-tree"
        }).getHTML();
        this.section.html(block);
        this.treeObj = new Tree(this.section.find(".system-list"), data, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
    };
    this.filterUsersByTransaction = function(transactionName, brole){
        var filteredData = [],
            arr = [];
        this.data["rolesByIssues"].forEach(function(dataItem){
            dataItem["conflict"].forEach(function(conflict){
                if(conflict["actionName"] == transactionName && conflict["businessRoleId"] == brole && arr.indexOf(dataItem["fields"]["user"]) == -1) {
                    arr.push(dataItem["fields"]["user"]);
                    filteredData.push({
                        name: dataItem["fields"]["user"]
                    });
                }
            });
        });
        return filteredData;
    };
    this.filterUsersByBusinessRole = function(roleName){
        var o = this,
            arr = [],
            filteredData = [];
        this.data["rolesByIssues"].forEach(function(dataItem){
            dataItem["conflict"].forEach(function(conflict){
                if(o.data["SODBusinessRolesMap"][conflict["businessRoleId"]] == roleName && arr.indexOf(dataItem["fields"]["user"]) == -1) {
                    arr.push(dataItem["fields"]["user"]);
                    filteredData.push({
                        name: dataItem["fields"]["user"]
                    });
                }
            });
        });
        return filteredData;
    };
    this.showInfoPopup = function(content, entity){
        var params = {
            class: 'sod-conflicts-matrix-info',
            title: this.locale["rolesByActions"]["infoPopup"].format({entity: entity}),
            content: content
        };
        this.controls.infoPopup = new Popup(params);
        this.controls.infoPopup.show();
        this.controls.infoTable.build();
        this.controls.infoPopup.setSize();
    };
    this.buildInfoTable = function(data){
        var params = {
            data: data,
            ticks: {"__addition": true},
            option: {
                namespace: "projects",
                type: "SODRolesByActions",
                sort_default: { column: 1, order: "asc" }
            }
        };
        this.controls.infoTable = new Table(params);
        return this.controls.infoTable.getHTML();
    };
    this.handlers = function(){
        var o = this;
        $(document.body).on("click", this.section.selector + " .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if ($item.data("type") == null) {
                return false;
            }
            o.section.find(" .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.section.find('.sod-matrix').remove();
            var filteredData = o.filterIssuesByTree($item, o.data["rolesByIssues"]),
                convertedData = {};
            convertedData["roles"] = o.getRoles(filteredData);
            convertedData["transactions"] = o.getTransactions(filteredData);
            convertedData["data"] = o.convertDataForMatrix(filteredData);
            o.buildMatrix(convertedData);
            return false;
        });
        $(document.body).on("click", this.section.selector + " .matrix .groupCell .matrixHeadTextBox", function(){
            var filteredData = o.filterUsersByBusinessRole($(this).text()),
                table = o.buildInfoTable(filteredData);
            o.showInfoPopup(table, $(this).text());
        });
        $(document.body).on("click", this.section.selector + " .matrix .colGroup .cell .matrixHeadTextBox", function(){
            var filteredData = o.filterUsersByTransaction($(this).text(), $(this).closest(".cell").data("brole")),
                table = o.buildInfoTable(filteredData);
            o.showInfoPopup(table, $(this).text());
        });
        $(document.body).on("click", this.section.selector + " .matrix .usedMark", function(e) {
            var cell = $(e.target).closest(".cell"),
            queries = o.createUsedTransactionInfoQueries(cell.data("brole"), cell.data("role"), cell.data("transaction"));
            o.getUsedTransactionInfo(queries, cell.data("role"));
        });
        $(document.body).on("mouseenter", this.section.selector + " .matrix .usedMark", function(e) {
            new Tooltip({
                text: "Used "+ $(e.target).data("number") +" times",
                e: e
            }).show();
        });
    };
    this.load = function() {
        this.data["SODBusinessRolesMap"] = this.createSODBusinessRolesMap(this.parentObj.data.scan.policy);
        this.getIssuesByRole();
        this.handlers();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    };
    this.clear = function() {
        $("section.content.tab-" + this.name + "").empty();
    };
};