Jobs.SoDTable = function(obj, name) {
    this.parentObj = obj;
    this.sectionSelector = '.tab-' + name;
    this.section = $(this.sectionSelector);
    this.locale = null;
    this.data = null;
    this.controls = {};
    this.path = ['assess', 'projects', "sod"];
    this.treeObj = null;
    this.table = null;
    this.data = {};
    this.templates = {
        systemList: '<div class="system-list"></div>',
        table: "<div class='block col-2 sod-table'><div class='table-wrapper'><h3>%s</h3>%s</div></div>",
        tr: "<tr>%s</tr>",
        td: "<td class='risk-%s'>%s</td>",
        select: "<select name='risk'>%s</select>",
        option: "<option value='%d'>%s</option>",
        no_entry: '<div class="no_entry">' + Locale.ui.page["no_entry"] + '</div>',
        policy_title: '<h3>%s</h3>',
        actions: '<button class="save">' + Locale.ui.page["save"] + '</button>',
        cell: '<div class="users">Users: <b>%s</b></div><div class="roles">Roles: <b>%s</b></div>',
        link: '<a href="#">{content}</a>',
        riskMark: '<span data-risk="{risk}"></span>'
    };
    this.conversionFieldRules = {
        "issueObjects": {
            "appServer": null,
            "objectType": null,
            "fields": null,
            "id": null,
            "objectName": null,
            "name": function(fields){
                return fields.objectName
            }
        },
        "SODRisksTable": {
            "businessProcess": function(fields){
                var businessProc = "";
                this.parentObj.data.scan.policy.structure.forEach(function(structureItem){
                    structureItem["businessRoles"].forEach(function(businessRole){
                        fields["conflict"].forEach(function(conflict){
                            if(conflict["businessRoleId"] == businessRole["id"]) {
                                businessProc = structureItem["name"];
                                return true;
                            }
                        });
                    });
                });
                return businessProc;
            },
            "ruleID": function(fields){
                var ruleId = "";
                fields["conflict"].forEach(function(conflict){
                    ruleId += conflict["businessRoleId"] + "+";
                });
                return {
                    value: ruleId.substr(0, ruleId.length - 1),
                    attr: {
                        name: "issueid",
                        value: fields["id"]
                    }
                }
            },
            "ruleName": function(fields){
                var o = this,
                    ruleName = "";
                fields["conflict"].forEach(function(conflict){
                    ruleName += o.data["SODBusinessRolesMap"][conflict["businessRoleId"]] + "+";
                });
                return ruleName.substr(0, ruleName.length - 1);
            },
            "ruleDescription": function(fields){
                var ruleDescription = "";
                this.parentObj.data.scan.policy.structure.forEach(function(structureItem){
                    structureItem["businessRoles"].forEach(function(businessRole){
                        fields["conflict"].forEach(function(conflict){
                            if(conflict["businessRoleId"] == businessRole["id"]) {
                                ruleDescription = "Rule description";
                            }
                        });
                    });
                });
                return ruleDescription;
            },
            "risk": function(fields){
                return this.templates.riskMark.format({risk: fields["riskSeverity"].toLowerCase()}) + fields["riskSeverity"]
            },
            "function1Name": function(fields) {
                return fields["conflict"][0]["actionName"];
            },
            "function2Name": function(fields) {
                return fields["conflict"][1]["actionName"];
            },
            "function3Name": function(fields) {
                if(typeof fields["conflict"][2] != "undefined") {
                    return fields["conflict"][2]["actionName"]
                } else {
                    return ""
                }
            },
            "function4Name": function(fields) {
                if(typeof fields["conflict"][3] != "undefined") {
                    return fields["conflict"][3]["actionName"]
                } else {
                    return ""
                }
            },
            "function5Name": function(fields) {
                if(typeof fields["conflict"][4] != "undefined") {
                    return fields["conflict"][4]["actionName"]
                } else {
                    return ""
                }
            },
            "numberOfUsers": function(fields) {
                return {
                    value: this.templates.link.format({content: fields.statistic.users}),
                    attr: {
                        name: "type",
                        value: "users"
                    },
                    cellClass: "abortRowOpening"
                }
            },
            "numberOfRoles": function(fields) {
                return {
                    value: this.templates.link.format({content: fields.statistic.roles}),
                    attr: {
                        name: "type",
                        value: "roles"
                    },
                    cellClass: "abortRowOpening"
                }
            },
            "numberOfProfiles": function(fields) {
                return {
                    value: this.templates.link.format({content: fields.statistic.profiles}),
                    attr: {
                        name: "type",
                        value: "profiles"
                    },
                    cellClass: "abortRowOpening"
                }
            }
        }
    };
    this.policyLevels = [
        {
            name: "businessProcesses",
            copiedFields: ["name", "type"]
        },
        {
            name: "businessRoles",
            copiedFields: ["name"]
        },
        {
            name: "actions",
            copiedFields: ["authObjects", "description", "name", "type"]
        }
    ];
    this.query = function(scanId) {
        return {
            title: "SOD conflicts",
            id: "issueSODConflict",
            queryObj: {
                "from": [
                    {
                        entity: "Issue"
                    }
                ],
                "query": "i.resultId = :type1 AND i.issueType = :type2",
                "fields": [

                ],
                "parameters": [
                    {
                        name: "type1",
                        value: scanId
                    },
                    {
                        name: "type2",
                        value: "issueSODConflict"
                    }
                ]
            }
        }
    };
    this.getData = function() {
        var o = this;
        ERPScan.API.query("resultsSODConflict", "list", {
            params: {
                scanId: o.parentObj.data.scan.scanId
            },
            type: "POST",
            callback: function(response) {
                var policy = o.parentObj.data.scan["policy"]["structure"],
                    policyMap = {};
                o.buildMapPolicyElements(policy["structure"], policyMap, 1);
                var rulesMap = o.buildMapPolicyRules(o.parentObj.data.scan["policy"]["rules"]["rules"]),
                    data = o.convertDataForTable(response.data.result, policyMap, rulesMap);
                o.buildView();
                o.buildTable(data);
            }
        });
    };
    this.filterIssuesByConflictsNumber = function(data){
        var filteredData = [];
        data.forEach(function(dataItem){
            if(dataItem["conflict"].length >= 2) {
                filteredData.push(dataItem);
            }
        });
        return filteredData;
    };
    this.buildSODAppServersTree = function(result) {
        var tree = [],
            obj,
            hierarchy = ["sid", "client"],
            hierarchyCreate = function(cred, level, nodes) {
                var currentHierarchy = hierarchy[level];
                if(typeof currentHierarchy != 'undefined') {
                    var isNodeRepeat = false;
                    nodes.forEach(function(node){
                        if(node.text == cred[currentHierarchy]) {
                            hierarchyCreate(cred, level+1, node.nodes);
                            isNodeRepeat = true;
                        }
                    });
                    if(!isNodeRepeat) {
                        obj = {
                            text: cred[currentHierarchy],
                            data: {
                                id: cred[currentHierarchy],
                                type: currentHierarchy
                            }
                        };
                        if(currentHierarchy != "client") {
                            obj["nodes"] = [];
                        }
                        nodes.push(obj);
                        hierarchyCreate(cred, level+1, nodes[nodes.length-1].nodes);
                    }
                }
            };
        for (var i = 0; i < result.length; i++) {
            var cred = result[i]["targetSystem"];
            hierarchyCreate(cred, 0, tree);
        }
        return tree;
    };
    this.buildView = function(data) {
        var block = new Block({
            title: this.locale.risksTable.systemsTree,
            content: this.templates.systemList,
            class: "systems-tree"
        }).getHTML();
        this.section.html(block);
        /*
        this.treeObj = new Tree(this.section.find(".system-list"), data, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
        */
    };
    this.getFunctionColumnsToDelete = function(issues){
        var maxFunctionsCount = 0,
            columnsToDelete = [];

        issues.forEach(function(issue){
            if (maxFunctionsCount < issue["conflict"].length) {
                maxFunctionsCount = issue["conflict"].length;
            }
        });
        for(var i = 0; i < 5 - maxFunctionsCount; i++) {
            columnsToDelete.push("function" + (5 - i) + "Name");
        }
        return columnsToDelete;
    };
    this.buildTable = function(data) {
        var params = {
            title: "Risks Table",
            data: data,
            ticks: {'__addition': true},
            option: {
                namespace: 'projects',
                type: 'SODRisksTable',
                count_one_page: 50,
                sort_default: {column: 1, order: "desc"}
            }
        };
        var tableObj = new Table(params),
            html = tableObj.getHTML();
        this.section.append(html);
        tableObj.build();
        Helpers.setBlockHeight(this.section);
    };
    this.loadObjects = function(issueId, type){
        var o = this;
        ERPScan.API.query("objects", "list", {
            params: {
                issueId: issueId
            },
            callback: function(response) {
                o.data["issueObjects"] = response["data"]["result"];
                var tableHTML = o.buildObjectsTable(type);
                o.showObjectsPopup(tableHTML, type);
            }
        });
    };
    this.buildObjectsTable = function(type){
        var typeMap = {
                users: "user",
                roles: "role",
                profiles: "profile"
            },
            data = [];
        this.data["issueObjects"].forEach(function(dataItem){
            if(dataItem["objectType"] == typeMap[type]) {
                data.push(dataItem);
            }
        });
        var params = {
            data: this.parentObj.conversionFields(data, this.conversionFieldRules["issueObjects"]),
            ticks: {"__addition": true},
            option: {
                namespace: "projects",
                type: "SODRisksMatrixObjects",
                sort_default: { column: 1, order: "asc" }
            }
        };
        this.controls["objectsTable"] = new Table(params);
        return this.controls["objectsTable"].getHTML();
    };
    this.showObjectsPopup = function(content, type){
        var params = {
            class: 'SODRisksMatrixObjects',
            title: this.locale["risksMatrix"]["objectsPopup"][type],
            content: content
        };
        this.controls.objectsPopup = new Popup(params);
        this.controls.objectsPopup.show();
        this.controls.objectsTable.build();
        this.controls.objectsPopup.setSize();
    };
    this.filterIssuesByTree = function($treeItem, data){
        var filteredData = [],
            itemType = $treeItem.data("type"),
            itemVal = $treeItem.data("id"),
            $parentItem = $treeItem.parents(".tree-item");
        data.forEach(function(dataItem){
            if(dataItem["targetSystem"][itemType] == itemVal){
                filteredData.push(dataItem);
            }
        });
        if($parentItem.length > 0) {
            return this.filterIssuesByTree($parentItem, filteredData);
        } else {
            return filteredData;
        }
    };
    this.convertDataForTable = function(results, policyMap, rules) {
        var resultsFormatted = [];
        for (var i = 0; i < results.length; i++) {
            var businessRoles = [];
            resultsFormatted[i] = {};
            for (var j = 0; j < results[i]["addresses"].length; j++) {
                var businessRoleName = policyMap[results[i]["addresses"][j][0]].child[results[i]["addresses"][j][1]]["params"]["name"]["en_US"];
                businessRoles.push(businessRoleName);
                resultsFormatted[i]["function" + (j + 1) + "Name"] = businessRoleName;
            }
            var criterionArray = businessRoles.sort(),
                criterionString = criterionArray.join("|"),
                rule = rules[criterionString];
            resultsFormatted[i]["ruleID"] = criterionArray.join(" + ");
            resultsFormatted[i]["businessProcess"] = policyMap[results[i]["addresses"][0][0]].params["name"]["en_US"];
            resultsFormatted[i]["ruleDescription"] = rule["description"];
            resultsFormatted[i]["risk"] = rule["risk"];
            resultsFormatted[i]["ruleName"] = rule["name"];
            resultsFormatted[i]["users"] = results[i]["statistic"]["users"];
            resultsFormatted[i]["roles"] = results[i]["statistic"]["roles"];
            resultsFormatted[i]["profiles"] = results[i]["statistic"]["profiles"];
        }
        return resultsFormatted;
    };
    this.buildMapPolicyElements = function(item, map, level) {
        for (var i = 0; i < item.length; i++) {
            map[item[i]["id"]] = {
                params: {}
            };
            var copiedFields = this.policyLevels[level - 1].copiedFields;
            for (var j = 0; j < copiedFields.length; j++) {
                map[item[i]["id"]].params[copiedFields[j]] = item[i][copiedFields[j]];
            }
            if (level < this.policyLevels.length) {
                map[item[i]["id"]].child = {};
                this.buildMapPolicyElements(item[i][this.policyLevels[level].name], map[item[i]["id"]].child, level + 1);
            }
        }
    };
    this.buildMapPolicyRules = function(rules) {
        var hashMap = {};
        for (var i = 0; i < rules.length; i++) {
            rules[i]["criterion"] = rules[i]["criterion"].sort();
            var criterionString = rules[i]["criterion"].join("|");
            hashMap[criterionString] = rules[i];
        }
        return hashMap;
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", this.sectionSelector + " .system-list .tree .tree-item .text", function() {
            var $this = $(this),
                $item = $this.parent(".tree-item-content").parent(".tree-item");
            o.section.find(".system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.section.find(".table").remove();
            var policy = o.parentObj.data.scan["policy"]["structure"],
                policyMap = {};
            o.buildMapPolicyElements(policy, policyMap, 1);
            var filteredData = o.convertDataForTable(o.data["issuesWithMoreThatOneConflict"], policyMap),
                dataForTable = o.parentObj.conversionFields.call(o, filteredData, o.conversionFieldRules["SODRisksTable"]);
            o.buildIssuesTable(dataForTable);
            return false;
        });
        $(document.body).on("click", this.sectionSelector + " td[data-type]", function() {
            var $this = $(this),
                type = $this.data("type"),
                issueId = $this.closest("tr").find("td[data-issueid]").data("issueId");
            o.loadObjects(issueId, type);
        });
    };
    this.clear = function() {
        $(document.body).off("click", this.sectionSelector + " .system-list .tree .tree-item .text");
        $(document.body).off("click", this.sectionSelector + " td[data-type]");
        this.treeObj.clear();
        $("section.content.tab-" + this.name + "").empty();
    };
    this.load = function() {
        this.getData();
        this.handlers();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    }
};