Jobs.Statistic = function(obj) {
    this.parentObj = obj;
    this.locale = null;
    this.path = ['assess', 'projects', 'statistic'];
    this.section = $("section.content.tab-statistic");
    this.data = {};
    this.treeObj = null;
    this.templates = {
        systemList: '<div class="system-list"></div>',
        no_plots: '<div class="wrap no_plots"><h3>{title}</h3>{content}</div>',
        linearPlot: '<div class="linearPlot"><div class="wrapper"></div><div class="legendBox"></div></div>',
        block_tabs: '<div class="block-tabs">{tabs}<select class="plotsList"></select></div>',
        plotsListItem: '<option data-link="{link}">{content}</option>',
        tab: '<a href="#" data-name="{name}" class="block-tab {class}">{text}</a>',
        plotsBlockTitle: '<h3>{title}</h3>'
    };
    this.plots = null;
    this.colorTypes = {
        noToCritical: {
            "critical": "#e24d42",
            "high": "#f9934e",
            "medium": "#eab839",
            "low": "#f3d597",
            "no": "#C7C7C7"
        },
        noToInfo: {
            "critical": "#e24d42",
            "high": "#f9934e",
            "medium": "#eab839",
            "low": "#f3d597",
            "unknown": "#C7C7C7"
        },
        statuses: {
            "negative": "#e24d42",
            "positive": "#7fae70"
        },
        success: {
            "uninstalled": "#e24d42",
            "installed": "#7fae70"
        }
    };
    this.colors = {
        "criticality": this.colorTypes.noToCritical,
        "probability": this.colorTypes.noToCritical,
        "installed": this.colorTypes.success,
        "aggregatedRisk": this.colorTypes.noToInfo,
        "aggregatedStatus": this.colorTypes.statuses,
        "conflicts": this.colorTypes.noToCritical,
        "users_conflicts": this.colorTypes.noToCritical,
        "users_risk": this.colorTypes.noToCritical,
        "roles_conflicts": this.colorTypes.noToCritical,
        "roles_risk": this.colorTypes.noToCritical
    };
    this.labelTransform = function(label, value) {
        return  label.substring(0,1).toUpperCase() + label.substring(1).toLowerCase() + " (" + value + ")";
    };
    this.plotsBuild = function() {
        var o = this,
            section = $("section.content.tab-statistic"),
            aggregatedResults = this.parentObj.data.scan.aggregatedResults,
            circlePlotsList = [],
            linearPlotsList = [],
            linearPlotBlock = null,
            dataset = [], //for linear plots
            colorsForLinear = [],
            data = [],
            dataForLinear = [],
            tabs = '',
            firstPlotsEntity = null,
            optionsForLinear = function(legendContainer, colorsForLinear) {
                return {
                    series: {
                        points: {
                            show: true,
                            radius: 2
                        },
                        shadowSize: 0,
                        lines: { show: true }
                    },
                    yaxis: {
                        tickDecimals: 0
                    },
                    xaxis: {
                        mode: "time",
                        min: (new Date("2016/01/01")).getTime(),
                        max: (new Date("2016/02/01")).getTime(),
                        timeformat: Locale.ui.common.time_format
                    },
                    grid: {
                        hoverable: true,
                        clickable: true,
                        mouseActiveRadius: 10,
                        borderWidth: 0
                    },
                    legend: {
                        marker: {
                            type: "circle",
                            size: 4
                        },
                        container: legendContainer,
                        noColumns: 10,
                        labelBoxBorderColor: 'none'
                    },
                    colors: colorsForLinear
                }
            };

        /* Set plots settings */
        $.each(aggregatedResults.circlePlots, function(entity){
            $.each(this, function(metric, aggrDataForCircle){
                data = [];
                dataset = [];
                $.each(o.parentObj.plotsConfig.criticality[metric], function (num, name) {
                    /* Pie */
                    data.push({
                        color: o.colors[metric][name],
                        data: aggrDataForCircle[name],
                        label: o.labelTransform(name, aggrDataForCircle[name])
                    });

                    /* Linear */
                    dataForLinear = [];
                    $.each(aggregatedResults.linearPlots, function(scanId, aggrData){
                        o.parentObj.data.scanList.forEach(function(scan) {
                            if(scan.id === scanId) {
                                var scanDate = new Date(scan.finishDate);
                                if (dataForLinear.length > 0) {
                                    for (var i = 0; i < dataForLinear.length; i++) {
                                        if (scanDate < dataForLinear[i][0]) {
                                            dataForLinear.splice(i, 0, [scanDate,aggrData[entity][metric][name]]);
                                            return;
                                        } else if ( i == dataForLinear.length - 1) {
                                            dataForLinear.push([scanDate,aggrData[entity][metric][name]]);
                                            return;
                                        }
                                    }
                                } else {
                                    dataForLinear.push([scanDate, aggrData[entity][metric][name]]);
                                }
                            }
                        });
                    });
                    dataset.push({label: Helpers.firstLetterToUpperCase(name), data: dataForLinear, points: {symbol: "circle"}, myData: "data" });
                });
                circlePlotsList.push({
                    type: "Pie",
                    title: o.locale.circlePlots[entity][metric],
                    field: metric,
                    columns: 2,
                    data: data
                });
                linearPlotsList.push(dataset);
            });
        });
        /* Tooltips for linear plots */
        $.fn.UseTooltip = function () {
            var previousPoint = null;
            $(this).bind("plothover", function (event, pos, item) {
                if (item) {
                    if (previousPoint != item.dataIndex) {
                        previousPoint = item.dataIndex;
                        $(".linearPlot_tooltip").remove();
                        var date = new Date(item.datapoint[0]).format('m/d/y h:m');
                        showTooltip(item.pageX, item.pageY, item.series.color,
                            date + "</br>"  + Helpers.firstLetterToUpperCase(item.series.label) + ": " + item.datapoint[1]);
                    }
                }
                else {
                    $(".linearPlot_tooltip").remove();
                    previousPoint = null;
                }
            });
        };
        function showTooltip(x, y, color, contents) {
            $('<div class="linearPlot_tooltip">' + contents + '</div>').css({
                top: y + 5,
                left: x + 10,
                size: 10,
                borderColor: color
            }).appendTo("body").fadeIn(200);
            var tooltip = $('.linearPlot_tooltip');
            if (x + tooltip.outerWidth() + 20 > window.innerWidth) {
                tooltip.css({ left: x - 100, top: y - 50 });
            }
        }

        /* Generate and init plots */
        if (circlePlotsList.length > 0) {
            Chart.generate(circlePlotsList, false, "section.content.tab-statistic");
            Chart.handlers();
        } else {
            section.append(this.templates.no_plots.format({title: '', content: this.locale.no_plots}));
        }
        var chartCells = section.find('.charts-cell');

        $.each(chartCells, function(key){
            linearPlotBlock = $(o.templates.linearPlot).insertAfter($(this));
            var linearPlotWrapper = linearPlotBlock.find('.wrapper');
            colorsForLinear = [];
            $.each(o.colors[circlePlotsList[key].field], function(key, val){
                colorsForLinear.push(val);
            });
            var plot = $.plot(linearPlotWrapper, linearPlotsList[key], optionsForLinear('.legendBox:eq('+ key +')', colorsForLinear));
            linearPlotWrapper.UseTooltip();
            linearPlotWrapper.bind("plotclick", function (event, pos, item) {
                if (item) {
                    plot.highlight(item.series, item.datapoint);
                } else {
                    plot.unhighlight();
                }
            });
        });

        /* Create tabs and set data attr to plots */
        var tempIndex = 0,
            linearChartCells = section.find('.linearPlot');
        $.each(aggregatedResults.circlePlots, function(entity, metrics){
            if (!firstPlotsEntity) {
                firstPlotsEntity = entity;
            }
            tabs += o.templates.tab.format({text: Helpers.firstLetterToUpperCase(entity), name: entity});
            $.each(metrics, function(metric){
                chartCells[tempIndex].setAttribute('data-entity', entity);
                linearChartCells[tempIndex].setAttribute('data-entity', entity);
                $(linearChartCells[tempIndex]).attr('data-metric', metric);
                $(chartCells[tempIndex]).attr('data-metric', metric);
                tempIndex++;
            });
        });
        o.changeVisiblePlotsByEntity(firstPlotsEntity);
        tabs = o.templates.block_tabs.format({tabs: tabs});

        /* Init tabs */
        var plotsWrapper = section.find('.charts');
        plotsWrapper.prepend(tabs);
        plotsWrapper.find('.block-tabs a').first().addClass('active');
        o.changeMetricFilterOptions();
        plotsWrapper.prepend(this.templates.plotsBlockTitle.format({title: this.locale.title}));
        $.each(linearChartCells, function(){
            $(this).prepend(o.templates.plotsBlockTitle.format({title: o.locale.linearPlots[this.getAttribute('data-entity')][this.getAttribute('data-metric')]}));
        });
    };
    this.addBorderToAllPlotsButLast = function (){
        this.section.find('.linearPlot:visible, .charts-cell:visible').addClass('plotBorderBottom');
        this.section.find('.linearPlot:visible:last, .charts-cell:visible:last').removeClass('plotBorderBottom');
    };
    this.changeVisiblePlotsByEntity = function(activeTab){
        var section = $("section.content.tab-statistic"),
            plots = section.find('[data-entity]');
        plots.hide();
        section.find('[data-entity="'+ activeTab +'"]').show();
        this.addBorderToAllPlotsButLast();
    };
    this.changeVisiblePlotsByMetric = function(activeLink){
        var section = $("section.content.tab-statistic"),
            entity = section.find('.charts .block-tabs a.active').attr('data-name'),
            plots = section.find('.linearPlot, .charts-cell'),
            plotsToActivate = activeLink == 'all' ? plots.filter('[data-entity="'+ entity +'"]') :plots.filter('[data-entity="'+ entity +'"][data-metric="'+ activeLink +'"]');
        plots.hide();
        plotsToActivate.show();
        this.addBorderToAllPlotsButLast();
    };
    this.changeMetricFilterOptions = function(){
        var o = this,
            metricFilter = this.section.find('select.plotsList'),
            defaultOptionHtml = this.templates.plotsListItem.format({link: 'all', content: o.locale.selectMetric}),
            visiblePlots = this.section.find('.linearPlot:visible, .charts-cell:visible'),
            metricsToAdd = [],
            metric = null,
            options = defaultOptionHtml;
        $.each(visiblePlots, function(){
            metric = this.getAttribute('data-metric');
            if (metricsToAdd.indexOf(metric) == -1) {
                metricsToAdd.push(metric);
            }
        });
        metricsToAdd.forEach(function(metric){
            options += o.templates.plotsListItem.format({link: metric, content: o.locale.metricsForSelect[metric]});
        });
        metricFilter.html(options);
    };
    this.applyCred = function($item) {
        var section = $("section.content.tab-statistic"),
            creds = String($item.data("info")).split(","),
            tabs = '';
        $("section.content.tab-statistic .system-list .tree .tree-item").removeClass("active");
        $item.addClass("active");

        $("section.content.tab-statistic > .wrap").remove();
        this.plotsBuild();
        Helpers.setBlockHeight(section);
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", "section.content.tab-statistic .system-list .tree .tree-item .text", function() {
            var $this = $(this);
            var $item = $this.parent(".tree-item-content").parent(".tree-item");
            if ($item.data("type") == null) {
                return false;
            }
            $("section.content.tab-statistic .system-list .tree .tree-item").removeClass("active");
            $item.toggleClass("active");
            o.applyCred($item);
            return false;
        });
        $(document.body).on("click", ".charts .block-tab", function(e) {
            var target = $(e.target),
                activeTabName = target.data('name');
            target.siblings('a').removeClass('active');
            target.addClass('active');
            o.changeVisiblePlotsByEntity(activeTabName);
            o.changeMetricFilterOptions();
        });
        $(document.body).on("change", ".charts .plotsList", function(e) {
            var target = $(e.target),
                activeLink = target.find('option:checked').attr('data-link');
            o.changeVisiblePlotsByMetric(activeLink);
        });
    };
    this.clear = function() {
        $("section.content.tab-statistic").empty();
        $(document.body).off("click", "section.content.tab-statistic .system-list .tree .tree-item .text");
        $(document.body).off("click", ".charts .block-tab");
        $(document.body).off("change", ".charts .plotsList");
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
        this.handlers();
    };
    this.load = function(){
        var section = $("section.content.tab-statistic"),
            block = new Block({
                title: this.locale.systemsTree,
                content: this.templates.systemList,
                class: "systems-tree"
            }).getHTML();
        section.empty();
        section.html(block);
        this.treeObj = new Tree($("section.content.tab-statistic .system-list"), this.parentObj.data.scan.appServersTree, { expandWhenClick: false });
        this.treeObj.init();
        this.treeObj.expandAll();
        this.afterInit();
    };
    this.afterInit = function() {
        this.applyCred($("section.content.tab-statistic .system-list .tree > .tree-item").eq(0));
    }
};