Jobs.Test = function(obj) {
    this.parentObj = obj;
    this.locale = null;
    this.path = ['assess', 'projects', 'test'];
    this.results = null;
    this.aggregatedResults = null;
    this.templates = {
        no_entry: "<div class='no_results'>%s</div>"
    };
    this.conversionFieldRules = {
        "results": {
            "credential": null,
            "id": null,
            "checkId": function(fields, name) {
                if(typeof this.info.checks[fields["check"]] != "undefined") {
                    return {
                        value: this.info.checks[fields["check"]]["common"]["name"],
                        attr: { name: 'check_id', value: fields["check"] }
                    };
                } else {
                    return "SSCA_555340"
                }
            },
            "checkName": function(fields, name) {
                if(typeof this.info.checks[fields["check"]] != "undefined") {
                    return this.info.checks[fields["check"]]["view"]["title"];
                }
            },
            "aggregatedStatus": function (fields, name) {
                if(typeof fields[name] != "undefined") {
                    return fields[name]["negative"] > 0 ? "negative" : "positive";
                }
            },
            "aggregatedRisk": function(fields, name) {
                if(typeof fields[name] != "undefined") {
                    var max_risk = 0,
                        max_risk_type = null;
                    for (var risk in fields[name]) {
                        if (fields[name][risk] > max_risk) {
                            max_risk = max_risk;
                            max_risk_type = risk;
                        }
                    }
                    return max_risk_type;
                }
            },
            "issues": function (fields, name) {
                if(typeof fields["aggregatedStatus"] != "undefined") {
                    return {
                        value: fields["aggregatedStatus"]["positive"],
                        attr: {
                            name: "result_id",
                            value: fields["id"]
                        }
                    };
                }
            },
            "sid": function (fields, name) {
                if(typeof fields["credential"] != "undefined") {
                    return fields["credential"]["sid"];
                } else if (typeof fields["targetSystem"] != "undefined") {
                    return fields["targetSystem"]["sid"];
                }
            },
            "host": function (fields, name) {
                if(typeof fields["credential"] != "undefined") {
                    return fields["credential"]["host"];
                } else if (typeof fields["targetSystem"] != "undefined") {
                    return fields["targetSystem"]["host"];
                }
            },
            "appServer": function (fields, name) {
                if(typeof fields["credential"] != "undefined") {
                    return {
                        value: this.info.appservers[fields["credential"]["appServer"]]["name"],
                        attr: {
                            name: "id",
                            value: fields["credential"]["appServer"]
                        }
                    };
                }
            },
            "entity": function (fields, name) {
                if(typeof fields["credential"] != "undefined") {
                    var entity = null;
                    if (entity == null) {
                        entity = "service";
                    }
                    var entity_name = Locale.fields['projects']["projects_result"][entity],
                        value;

                    if (entity == "service") {
                        value = entity_name + ": " + this.info.appservers[fields["credential"]["appServer"]]["services"][fields["credential"]["service"]]["name"];
                        entity += "|" + fields["credential"]["service"];
                    } else {
                        value = entity_name + ": " + fields["credential"]["customParameters"][entity];
                        entity += "|" + fields["credential"]["customParameters"][entity];
                    }
                    return {
                        value: value,
                        attr: {
                            name: "type",
                            value: entity
                        }
                    }
                }
            },
            "exploitable": function (fields, name) {
                return fields[name];
            }
        }
    };
    this.tableLoad = function() {
        var params = {
            title: "Results",
            data: this.parentObj.conversionFields(this.parentObj.data.scan.results, this.conversionFieldRules.results),
            ticks: {"__addition": true},
            option: {
                namespace: "projects",
                type: "result",
                sort_default: { column: 1, order: "asc" }
            }
        };
        var table_obj = new Table(params);
        var html = table_obj.getHTML(),
            section = $('section.content.tab-test');
        section.html(html);
        table_obj.build();
        section.find(".chart-actions").after().remove();
    };
    this.clear = function() {
        $("section.content.tab-test").empty();
    };
    this.handlers = function() {

    };
    this.init = function() {
        $("section.content.tab-test").empty();
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
        this.tableLoad();
    }
};