Page.Render = {
    init: function() {
        var fields = ["queries", "search"],
            locale = "en";
        Locale.set(locale, { fields: fields });
        Locale.loadPage([Page.path.module, Page.path.page, Page.path.tab]);
        Header.load([Page.path.module, Page.path.page, Page.path.tab], { tabs: true });
        new SimpleSearch().init();
    }
};

function SimpleSearch() {
    this.chart_types = ["pie", "bar"];
    this.chart_number = 1;
    this.queries = null;
    this.query = null;
    this.fields = null;
    this.comparators = {
        '=':':"%s"',
        '!=':':!"%s"',
        '>':':>"%s"',
        '<':':<"%s"',
        '>=':':>="%s"',
        '<=':':<="%s"',
        'LIKE':':%s'
    };
    this.templates = {
        wrap: "<div class='search'><div class='search-field'><input type='text' /><a href='#' class='search-icon'></a></div><div class='preset-queries'></div></div>",
        presetQuery: "<a href='#' class='preset-query' data-id='{queryId}'>{queryName}</a>",
        dashboard: "",
        loading: "<div class='loading'></div>",
        no_entry: "<div class='no_entry'>" + Locale.ui.page["no_entry"] + "</div>",
        charts_add: '<div class="charts-cell col-2"><div class="chart"><h3>' + Locale.ui.page["adding"]["block_title"] + '</h3><div class="param-item chart-type"><div class="chart-type">' + Locale.ui.page["adding"]["chart_type"] + ':</div><div class="param-value"><select>%s</select></div></div><div class="add-chart-button"><button>' + Locale.ui.page["adding"]["button"] + '</button></div></div></div>',
        charts_add_param_item: "<option value='%s'>%s</option>",
        chart_add_fields: {
            pie: '<div class="param-item item-specific-chart chart-metric"><div class="chart-type">' + Locale.ui.page["adding"]["chart_metric"] + ':</div><div class="param-value"><select>%s</select></div></div>',
            bar: '<div class="param-item item-specific-chart chart-metric"><div class="chart-type">' + Locale.ui.page["adding"]["chart_metric"] + ':</div><div class="param-value"><select>%s</select></div></div>' +
            '<div class="param-item item-specific-chart chart-metric-x"><div class="chart-type">' + Locale.ui.page["adding"]["chart_metric_x"] + ':</div><div class="param-value"><select>%s</select></div></div>',
            line: '<div class="param-item item-specific-chart chart-metric"><div class="chart-type">' + Locale.ui.page["adding"]["chart_metric"] + ':</div><div class="param-value"><select>%s</select></div></div>'
        },
        found: "<h2>" + Locale.ui.page["found"] + "<h2>",
        filters: {
            operator: '<a href="#" class="operator%s" data-type="%s">%s</a>',
            time_filter: '<input type="text" class="time_filter_input%s" />',
            value: '<input type="text" class="value" />',
            remove_filter: '<a href="#" class="remove"></a>',
            select: "<select name='%s'>%s</select>",
            option: "<option value='%s'>%s</option>",
            add_item: "<button class='add_item'>Add item</button>"
        }
    };
    this.transformDataForTable = function(data) {
        for (var i = 0; i < data.length; i++) {
            //data[i]["__mark"] = '<input type="checkbox" />';
            data[i]["__addition"] = '<a href="#"></a>';
            data[i]["time"] = {
                value: data[i]["rt"],
                attr: { name: 'ts', value:  new Date(data[i]["rt"]).getTime() }
            };
        }
        return data;
    };
    this.setTransposition = function() {
        var disposition = $.cookie("events-disposition") || "left";
        var $wrap = $(".wrap");
        var $charts = $wrap.find(".charts");
        var $events = $wrap.find(".events");
        var charts = $charts.get(0);
        $charts.detach();
        switch(disposition) {
            case "top":
                $events.after(charts);
                break;
            case "left":
                $events.after(charts);
                break;
            case "right":
                $events.before(charts);
                break;
            case "bottom":
                $events.before(charts);
                break;
        }
        $events.removeAttr("class").addClass("events").addClass(disposition);
        $charts.removeAttr("class").addClass("charts").addClass(disposition);
        Chart.reDrawing();
    };
    this.getTableTicks = function(data) {
        var ticks = {
            //'__mark': 0,
            '__addition': 0
        };
        var order = 1;
        var fields_count = 5;
        for (var field in data[0]) {
            if (field == "message") {
                continue;
            }
            ticks[field] = order > fields_count ? -1 : order++;
        }
        return ticks;
    };
    this.getBarTicks = function(data) {
        var ticks = [];
        for (var i = 0; i < data.length; i++) {
            if (data[i].key == '""') {
                continue;
            }
            ticks[i] = [i, data[i].key];
        }
        return ticks;
    };
    this.buildResult = function(data, id) {
        for (var i = 0; i < data.length; i++) {
            data[i]["__addition"] = '<a href=""></a>';
        }
        $("section.content .loading").remove();
        Chart.generate([ ], {
            type: "Table",
            ticks: {"__addition": true},
            title: Locale.ui.page["all_events"],
            data: data,
            option: {
                namespace: "queries",
                type: "customQuery" + id,
                sort_default: { column: 1, order: "asc"}
            }
        });
        Chart.handlers();
        var chart_types = this.templates.charts_add_param_item.sprintf(0, Locale.ui.page.adding.chart_types_select);
        for (var i = 0; i < this.chart_types.length; i++) {
            chart_types += this.templates.charts_add_param_item.sprintf(this.chart_types[i], Locale.ui.page.adding.chart_types[this.chart_types[i]]);
        }
        $("section.content .charts").html(this.templates.charts_add.sprintf(chart_types));
    };
    this.run = function(queryObj, id) {
        $("section.content .loading").remove();
        $("section.content .wrap").remove();
        $("section.content .no_entry").remove();
        this.query = {
            id: id,
            obj: queryObj
        };
        $("section.content").append(this.templates.loading);
        var o = this;
        ERPScan.API.query("queries", "execute", {
            callback: function(response) {
                $("header .title h2").remove();
                $("header .title h1").after(o.templates.found.sprintf(response["data"]["total"]));
                if (response["data"]["total"] == 0) {
                    $("section.content .loading").remove();
                    $("section.content").append(o.templates.no_entry);
                    return;
                }
                o.buildResult.call(o, response["data"]["result"], id);
            },
            type: "POST",
            params: {
                query: JSON.stringify(queryObj)
            }
        });
    };
    this.transformForChart = function(data, type) {
        var output = [];
        var ticks = [];
        switch (type) {
            case "Pie":
                for (var stat in data) {
                    output.push({
                        "label": data[stat] + " (" + (stat == "" ? "none": stat) + ") ",
                        "data": data[stat]
                    });
                }
                break;
            case "Bar":
                for (var field1 in data) {
                    var index = output.push({
                        "label": field1,
                        "data": [],
                        "bars": {
                            "show": true,
                            "barWidth": 0.15,
                            "fill": true,
                            "lineWidth": 0,
                            "order": 1
                        }
                    }) - 1,
                        counter = 1;
                    for (var field2 in data[field1]) {
                        output[index].data.push([counter++, data[field1][field2]]);
                    }
                }
                for (var field1 in data) {
                    for (var field2 in data[field1]) {
                        ticks.push(field2);
                    }
                    break;
                }
                break;
        }
        return {
            data: output,
            ticks: ticks
        };
    };
    this.pasteFieldsAdding = function(o) {
        var $this = $(this);
        var chart_type = $this.find("option:selected").val();
        $("section.content .charts .chart .item-specific-chart").remove();
        if (chart_type == 0) {
            return false;
        }
        var metrics = "";
        metrics += o.templates.charts_add_param_item.sprintf(0, Locale.ui.page.adding.chart_metric_select);
        for (var field in Chart.table.fields.ticks) {
            if (field.substr(0,2) == "__") {
                continue;
            }
            metrics += o.templates.charts_add_param_item.sprintf(field, field);
        }
        $this.parents(".param-item").after(o.templates.chart_add_fields[chart_type].sprintf(metrics, metrics));
    };
    this.addChart = function($this) {
        var $chart_type = $this.parents(".chart").find(".param-item.chart-type select option:selected"),
            $metric = $this.parents(".chart").find(".param-item.chart-metric select option:selected"),
            o = this;
        if ($chart_type.val() == 0 || $metric.val() == 0) {
            return false;
        }
        var chart_type = $chart_type.text();
        var metric = $metric.text();
        var data = {
            'query': this.query,
            'ts_gte': this.ts_gte,
            'ts_lte': this.ts_lte,
            'metric': "",
            'submetric': ""
        }, type;
        switch (chart_type) {
            case "Pie":
                data.metric = metric;
                type = "Statistic1D";
                break;
            case "Bar":
                var $submetric = $this.parents(".chart").find(".param-item.chart-metric-x select option:selected");
                metric = metric + "_" + $submetric.text();
                data.metric = metric;
                type = "Statistic2D";
                break;
        }
        ERPScan.API.query("queries", "statisticQuery" + this.query.id + type, {
            callback: function(response) {
                var chart_data = o.transformForChart(type == "Statistic2D" ? response["data"][metric]["data"]  : response["data"][metric], chart_type);
                $("section.content .charts .chart .item-specific-chart").remove();
                $("section.content .charts .chart .param-item.chart-type .param-value select option")[0].selected = true;
                var html = Chart.generateChart({
                    title: metric,
                    field: metric,
                    type: chart_type,
                    columns: 2,
                    removable: true
                }, o.chart_number);
                $("section.content .charts .charts-cell").eq(0).after(html);
                switch (chart_type) {
                    case "Pie":
                        Chart[chart_type].build(chart_data.data, {
                            number: o.chart_number++,
                            field: metric
                        });
                        break;
                    case "Bar":
                        Chart[chart_type].build(chart_data.data, {
                            number: o.chart_number++,
                            ticks: chart_data.ticks,
                            field: metric
                        });
                        break;
                }
            },
            type: "POST",
            params: {
                query: JSON.stringify(this.query.obj)
            }
        });
        return false;
    };
    this.getFieldsHTML = function() {
        return "";
    };
    this.getComparatorsHTML = function() {
        var comparators = "";
        for (var cmp in this.comparators) {
            comparators += this.templates.filters.option.sprintf(cmp, cmp);
        }
        return this.templates.filters.select.sprintf("types", comparators);
    };
    this.generateSearchString = function() {
        var search_string = "";
        var o = this;
        $("section.content .table.time_filters tbody tr").each(function() {
            var $this = $(this);
            var from = $this.find(".time_filter_input.from").data("ts");
            var to = $this.find(".time_filter_input.to").data("ts");
            var operator = $this.find(".operator.active").data("type");
            var value = "@timestamp:[" + from + " TO " + to + "]";
            if (search_string) {
                search_string += " " + operator + " " + value;
            } else {
                search_string += " " + value;
            }
        });
        $("section.content .table.parameters_filters tbody tr").each(function() {
            var $this = $(this);
            var field = $this.find("select[name=\"names\"] option:selected").html();
            var comparator = $this.find("select[name=\"types\"] option:selected").text();
            var value = $this.find("input.value").val();
            var operator = $this.find(".operator.active").data("type");
            if (search_string) {
                search_string += " " + operator + " " + field + o.comparators[comparator].sprintf(value || "");
            } else {
                search_string += " " + field + o.comparators[comparator].sprintf(value || "");
            }
        });
        $(".search-field input[type=\"text\"]").val(search_string);
    };
    this.filtersPaste = function() {
        var data = [
            {
                operator: {
                    value: "",
                    attr: {
                        name: "order",
                        value: "0"
                    }
                },
                start_time: this.templates.filters.time_filter.sprintf(" from"),
                end_time: this.templates.filters.time_filter.sprintf(" to"),
                actions: ""
            }
        ];
        var params = {
            title: "Time filter",
            data: data,
            option: {
                namespace: "queries",
                type: "timeFiltering",
                filtering: false,
                expandable: false,
                sort_default: { column: 0, order: "asc" }
            }
        };
        var table_obj = new Table(params),
            html = table_obj.getHTML();
        $("section.content .blocks").eq(0).html($(html).addClass("block time_filters filters col-2"));
        table_obj.build();

        data = [ ];
        params = {
            title: "Parameters filter",
            data: data,
            option: {
                namespace: "queries",
                type: "parameterFiltering",
                expandable: false,
                filtering: false,
                sort_default: { column: 0, order: "desc" }
            }
        };
        table_obj = new Table(params);
        html = table_obj.getHTML();
        $("section.content .blocks").eq(1).html($(html).addClass("block parameters_filters filters col-2"));
        table_obj.build();

        $("section.content .table h3").before(this.templates.filters.add_item);
        $("section.content .table.filters .time_filter_input.from").datetimepicker({
            onSelect: function(t) {
                $(this).data("ts", new Date(t).getTime());
            }
        }).datepicker('setDate', '-5y').data("ts", new Date($("section.content .table.filters .time_filter_input.from").datepicker('getDate')).getTime());
        $("section.content .table.filters .time_filter_input.to").datetimepicker({
            onSelect: function(t) {
                $(this).data("ts", new Date(t).getTime());
            }
        }).datepicker('setDate', 'now').data("ts", new Date().getTime());
    };
    this.pastePresetQueries = function() {
        $(".preset-queries").empty();
        for (var i = 1; i <= this.presetQueries.length; i++) {
            $(".preset-queries").append(this.templates.presetQuery.format({
                queryId: i,
                queryName: this.presetQueries[i - 1].title
            }))
        }
    };
    this.loadPresetQueries = function() {
        var $presetQueries = $(".preset-queries");
        $presetQueries.html("Loading...").slideToggle("normal").css("display", "inline-block");
        var o = this;
        $.ajax({
            url: "/data/presetQueries.json",
            success: function(response) {
                o.presetQueries = response;
                o.pastePresetQueries.call(o);
                $presetQueries.animate({
                    width: $(".search-field").width() - 62
                }, function() {
                    var defaultHeight = $presetQueries.height();
                    $presetQueries.css({height: "auto"});
                    var height = $presetQueries.height();
                    $presetQueries.css({height: defaultHeight}).animate({
                        height: height
                    }, function() {
                        $presetQueries.find(".preset-query").fadeTo(500, 1.0);
                    });
                });
            }
        })
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", ".preset-queries .preset-query", function() {
            var $this = $(this);
            o.run.call(o, o.presetQueries[$this.data("id") * 1 - 1]["queryObj"], $this.data("id"));
            return false;
        });
        $(document.body).on("click", ".disposition .left", function() {
            $.cookie('events-disposition', 'left');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            o.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .top", function() {
            $.cookie('events-disposition', 'top');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            o.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .right", function() {
            $.cookie('events-disposition', 'right');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            o.setTransposition();
            return false;
        });
        $(document.body).on("click", ".disposition .bottom", function() {
            $.cookie('events-disposition', 'bottom');
            $(".disposition a").removeClass("active");
            $(this).addClass("active");
            o.setTransposition();
            return false;
        });
        $("section.content .search .search-field input[type=\"text\"]").on("keyup", function(event) {
            if (event.keyCode == 13) {
                o.run();
            }
        });
        $("section.content .search .search-field .search-icon").on("click", function() {
            o.run();
            return false;
        });
        $(document.body).on("change", "section.content .charts .chart .param-item.chart-type .param-value select", function() {
            o.pasteFieldsAdding.call(this, o);
        });
        $(document.body).on("click", "section.content .charts .chart .chart-actions .remove", function() {
            $(this).parents(".charts-cell").remove();
            return false;
        });
        $(document.body).on("click", "section.content .charts .chart .add-chart-button button", function() {
            var $this = $(this);
            o.addChart.call(o, $this);
        });
        $(document.body).on("click", "section.content .search .date-filter a.from", function(){
            $("section.content .search .date-filter input.from").datetimepicker('show');
        });
        $(document.body).on("click", "section.content .search .date-filter a.to", function(){
            $("section.content .search .date-filter input.to").datetimepicker('show');
        });
        $(document.body).on("click", "section.content .block .block-tabs .block-tab", function() {
            var $this = $(this);
            var name = $this.data("name");
            $("section.content .block .block-content-tab").removeClass("active");
            $("section.content .block .block-content-tab." + name).addClass("active");
            $("section.content .block .block-tabs .block-tab").removeClass("active");
            $this.addClass("active");
            return false;
        });
        $(document.body).on("click", "section.content .table.filters .operator", function() {
            var $this = $(this);
            $this.parents("tr").find(".operator").removeClass("active");
            $this.addClass("active");
            o.generateSearchString.call(o);
            return false;
        });
        $(document.body).on("click", "section.content .table.parameters_filters .add_item", function() {
            var $this = $(this);
            var names = o.getFieldsHTML.call(o);
            var types = o.getComparatorsHTML.call(o);
            var row = '<tr><td>' + o.templates.filters.operator.sprintf(" active", "OR", "OR") + o.templates.filters.operator.sprintf("", "AND", "AND") + '</td><td>' + names + '</td><td>' + types + '</td><td>' + o.templates.filters.value + '</td><td>' + o.templates.filters.remove_filter + '</td></tr>',
                $row = $(row),
                resort = true;
            $this.parents(".table").find("table")
                .find('tbody').append($row)
                .trigger('addRows', [$row, resort]);
            o.generateSearchString.call(o);
            return false;
        });
        $(document.body).on("click", "section.content .table.time_filters .add_item", function() {
            var $this = $(this);
            var row = '<tr><td data-order="1">' + o.templates.filters.operator.sprintf(" active", "OR", "OR") + o.templates.filters.operator.sprintf("", "AND", "AND") + '</td><td>' + o.templates.filters.time_filter.sprintf(" from") + '</td><td>' + o.templates.filters.time_filter.sprintf(" to") + '</td><td>' + o.templates.filters.remove_filter + '</td></tr>',
                $row = $(row),
                resort = true;
            $this.parents(".table").find("table")
                .find('tbody').append($row)
                .trigger('addRows', [$row, resort]);
            $row.find(".time_filter_input.from").val("Select time").datetimepicker({
                onSelect: function(t) {
                    $(this).data("ts", new Date(t).getTime());
                }
            }).data("ts", new Date($row.find(".time_filter_input.from").datepicker('getDate')).getTime());
            $row.find(".time_filter_input.to").val("Select time").datetimepicker({
                onSelect: function(t) {
                    $(this).data("ts", new Date(t).getTime());
                }
            }).data("ts", new Date().getTime());
            o.generateSearchString.call(o);
            return false;
        });
        $(document.body).on("click", "section.content .table.parameters_filters .remove", function() {
            var $this = $(this);
            $this.parents("tr").remove();
            $("table").trigger('update');
            o.generateSearchString.call(o);
        });
        $(document.body).on("click", "section.content .table.time_filters .remove", function() {
            var $this = $(this);
            $this.parents("tr").remove();
            $("table").trigger('update');
            o.generateSearchString.call(o);
        });
        $(document.body).on("change", "section.content .table.time_filters .time_filter_input.from, section.content .table.time_filters .time_filter_input.to", function(){
            o.generateSearchString.call(o);
        });
        $(document.body).on("change", "section.content .table.parameters_filters select", function() {
            o.generateSearchString.call(o);
        });
        $(document.body).on("keyup", "section.content .table.parameters_filters input", function() {
            o.generateSearchString.call(o);
        });
    };
    this.fieldsLoad = function() {
        var o = this;
        o.loadPresetQueries();
        //o.filtersPaste();
        //o.generateSearchString.call(o);
        o.handlers();
    };
    this.init = function() {
        $("section.content").append(this.templates.wrap);
        this.fieldsLoad();
    };
}

var original_gotoToday = $.datepicker._gotoToday;

$.datepicker._gotoToday = function(id) {
    var target = $(id),
        inst = this._getInst(target[0]);
    original_gotoToday.call(this, id);
    $(id).data("now", true);
    this._selectDate(id, this._formatDate(inst, inst.selectedDay, inst.drawMonth, inst.drawYear));
};