/**
 * threat-map.js
 * http://erpscan.com
 *
 * Threat Map for ERPScan
 *
 * @version 1.0.0
 * @date    2015-10-14
 *
 * @license
 * Copyright (C) 2015 Petukhov V.A, http://erpscan.com
 */

var ThreatMap = {
    templates: {
        wrap: '<div class="setting"> <h3>Settings</h3> <div class="filters"></div> </div> <div id="threat-map"></div>'
    },
    node: {
        selected: false,
        count: 0,
        defaults: function(params) {
            for (var value in ThreatMap.config.nodes.default)
                params[value] = params[value] || ThreatMap.config.nodes.default[value];
            return params;
        },
        allocation: {
            circles: 1,
            number: 0,
            residue: [ ],
            grouping: function(nodes) {
                var grouped_nodes = {
                    scanner: [],
                    some: [],
                    network: []
                }
                for (var i = 0; i < nodes.length; i++)
                    grouped_nodes[nodes[i].system].push(nodes[i]);
                return grouped_nodes;
            },
            algorithm: function(node, residue, type) {
                if (ThreatMap.node.allocation.residue[0] != ThreatMap.node.allocation.circles) {
                    ThreatMap.node.allocation.residue[0] = ThreatMap.node.allocation.circles;
                    ThreatMap.node.allocation.residue[1] = residue;
                }
                residue = ThreatMap.node.allocation.residue[1];
                var exact_count = Math.round(ThreatMap.config.nodes.allocation[type].count_coef * (ThreatMap.node.allocation.circles + 1)),
                    next_count = Math.round(ThreatMap.config.nodes.allocation[type].count_coef * (ThreatMap.node.allocation.circles + 2));
                var is_fit = residue < (exact_count + next_count);
                if (!is_fit && exact_count <= ThreatMap.node.allocation.number) {
                    ThreatMap.node.allocation.circles++;
                    ThreatMap.node.allocation.number = 0;
                    return ThreatMap.node.allocation.algorithm(node, residue - exact_count, type);
                }
                var count = is_fit ? residue : exact_count,
                    tick = ThreatMap.config.nodes.allocation[type].tick_coef * Math.PI * 2 / count,
                    radius = 170 * ThreatMap.node.allocation.circles,
                    deg = tick * ThreatMap.node.allocation.number++;
                if (ThreatMap.config.nodes.allocation[type].transofrms) {
                    for (var bound in ThreatMap.config.nodes.allocation[type].transofrms) {
                        var obj = ThreatMap.config.nodes.allocation[type].transofrms[bound];
                        if (deg > obj.range[0] && deg < obj.range[1]) {
                            deg += obj.addition;
                        }
                    }
                }
                if (ThreatMap.config.nodes.allocation[type].transform_deg)
                    deg = ThreatMap.config.nodes.allocation[type].transform_deg(deg);
                return {
                    x: radius * Math.cos(deg),
                    y: radius * Math.sin(deg)
                }
            },
            run: function(nodes) {
                var all_nodes = [];
                for (var type in nodes) {
                    for (var i = 0; i < nodes[type].length; i++) {
                        var coordinates = ThreatMap.node.allocation.algorithm(nodes[type][i], nodes[type].length - i, type);
                        nodes[type][i].x = coordinates.x;
                        nodes[type][i].y = coordinates.y;
                        all_nodes.push(nodes[type][i]);
                    }
                    ThreatMap.node.allocation.circles++;
                    ThreatMap.node.allocation.number = 0;
                }
                return all_nodes;
            }
        },
        calcTagsLines: function(tags) {
            var container = "<div class='__tags'>%s</div>",
                tag = "<div class='item'>#%s</div>",
                tags_html = '',
                line_height = 10;
            for (var i = 0; i < tags.length; i++) {
                tags_html += tag.sprintf(tags[i]);
            }
            container = container.sprintf(tags_html);
            var o = $(container).appendTo($('body')),
                h = o.height();
            o.remove();
            return Math.round(h / line_height);
        },
        add: function(params) {
            params = ThreatMap.node.defaults(params);
            params.tags_lines = params.tags ? ThreatMap.node.calcTagsLines(params.tags) : 0;
            var icon = "http://%s:%s%s?".sprintf(ThreatMap.config.general.server, ThreatMap.config.general.port, ThreatMap.config.nodes.renders.path) + $.param(params);
            return {
                id: ++ThreatMap.node.count,
                shape: 'image',
                image: icon,
                x: params.x,
                y: params.y,
                shapeProperties: { useImageSize: true }
            }
        },
        load: function(systems) {
            systems = ThreatMap.node.allocation.grouping(systems);
            systems = ThreatMap.node.allocation.run(systems);
            var nodes = [];
            for (var i = 0; i < systems.length; i++)
                nodes.push(ThreatMap.node.add(systems[i]));
            console.log(nodes);
            return new vis.DataSet(nodes);
        },
        select: function(node) {
            ThreatMap.node.selected = node;
            ThreatMap.graph.dfs.init(node);
            var node = ThreatMap.data.allNodes[node];
            node.image = node.image.setParamURI({opacity: 1, expanded: 1});
        },
        unselect: function() {
            var node = ThreatMap.node.selected;
            if (!node)
                return;
            ThreatMap.node.selected = false;
            var node = ThreatMap.data.allNodes[node];
            node.image = node.image.setParamURI({expanded: 0});
        },
        highlight: function(node) {
            var node = ThreatMap.data.allNodes[node];
            node.image = node.image.setParamURI({opacity: 1});
        },
        highlightAll: function() {
            for (var node in ThreatMap.data.allNodes)
                ThreatMap.node.highlight(node);
        },
        unhighlight: function(node) {
            var node = ThreatMap.data.allNodes[node];
            node.image = node.image.setParamURI({opacity: .2});
        },
        unhighlightAll: function() {
            for (var node in ThreatMap.data.allNodes)
                ThreatMap.node.unhighlight(node);
        },
        update: function() {
            var updateArray = [];
            for (var nodeId in ThreatMap.data.allNodes) {
                if (ThreatMap.data.allNodes.hasOwnProperty(nodeId)) {
                    updateArray.push(ThreatMap.data.allNodes[nodeId]);
                }
            }
            ThreatMap.data.nodes.update(updateArray);
        },
        getCountVisibleEdges: function(node) {
            var edges = ThreatMap.object.getConnectedEdges(node);
            var count = 0;
            for (var i = 0; i < edges.length; i++) {
                if (!ThreatMap.data.allEdges[edges[i]].hidden)
                    count++;
            }
            return count;
        },
        hide: function(node) {
            ThreatMap.data.allNodes[node].hidden = true;
            if (ThreatMap.node.selected == node) {
                ThreatMap.edge.highlightAll();
                ThreatMap.node.highlightAll();
                ThreatMap.node.unselect();
            }
        },
        hideWithAdjacent: function(node) {
            ThreatMap.node.hide(node);
            var nodes = ThreatMap.object.getConnectedNodes(node);
            var edges = ThreatMap.object.getConnectedEdges(node);
            for (var i = 0; i < edges.length; i++)
                ThreatMap.edge.hide(edges[i]);
            for (var i = 0; i < nodes.length; i++) {
                var uri = ThreatMap.data.allNodes[nodes[i]].image.parseURI();
                if (uri.params.system != "scanner" && ThreatMap.node.getCountVisibleEdges(nodes[i]) == 0)
                    ThreatMap.node.hide(nodes[i]);
            }
        },
        show: function(node) {
            ThreatMap.data.allNodes[node].hidden = false;
        },
        showAll: function() {
            for (var node in ThreatMap.data.allNodes)
                ThreatMap.node.show(node);
        }
    },
    edge: {
        connections: [ ],
        popup: {
            setSize: function() {
                var height = 500;
                $(".popup").css({
                    height: height,
                    marginTop: -Math.round((height) / 2)
                })
                $(".popup .content").css({ height: height - ThreatMap.config.edges.popup.paddingContent });
            },
            close: function() {
                $(window).off("resize");
                $(window).on("resize", ThreatMap.setSize);
                ThreatMap.object.unselectAll();
            },
            new: function(edge) {
                var fields = [ 'bname', 'uflag', 'trdat', 'descrypted_password', 'instance', 'client', 'access_in_system1', 'access_in_system2' ];
                var events = ThreatMap.config.edges.popup.table;
                events.data = [ ];
                for (var i = 0; i < edge.connections.length; i++) {
                    var row = { '__addition': '<a href="#"></a>' };
                    for (var j = 0; j < fields.length; j++) {
                        row[fields[j]] = edge.connections[i][fields[j]];
                    }
                    events.data.push(row);
                }
                var table_obj = new Table(events);
                var content = table_obj.getHTML(1);
                Popup.new({
                    class: "edge-full-info",
                    title: "Connections",
                    content: content,
                    open: ThreatMap.edge.popup.setSize,
                    close: ThreatMap.edge.popup.close
                });
                $(window).off("resize");
                $(window).on("resize", ThreatMap.edge.popup.setSize);
                ThreatMap.edge.popup.setSize();
                table_obj.build();
            }
        },
        hover: function(params) {
            var edge = ThreatMap.data.allEdges[params.edge];
            var items = '';
            var total_connections = 0;
            for (var type in edge.types) {
                items += ThreatMap.config.edges.tooltip.item.sprintf(type + " connections", edge.types[type]);
                total_connections += edge.types[type];
            }
            items += ThreatMap.config.edges.tooltip.item.sprintf("Total", total_connections);
            $(ThreatMap.config.general.selector).append(ThreatMap.config.edges.tooltip.wrap.sprintf(window.event.clientX + 10, window.event.clientY + 10, items));
            document.onmousemove = function() {
                $(ThreatMap.config.general.selector).find(ThreatMap.config.edges.tooltip.selector).css({left: window.event.clientX + 10, top: window.event.clientY + 10});
            }
        },
        blur: function() {
            document.onmousemove = null;
            $(ThreatMap.config.general.selector).find(ThreatMap.config.edges.tooltip.selector).remove();
        },
        select: function(params) {
            ThreatMap.edge.blur();
            var edge = ThreatMap.data.allEdges[params.edges[0]];
            ThreatMap.edge.popup.new(edge);
        },
        getConnection: function(from, to) {
            if (Array.isArray(ThreatMap.edge.connections[from]) && typeof ThreatMap.edge.connections[from][to] !== "undefined")
                return { type: "direct", id: ThreatMap.edge.connections[from][to] };
            if (Array.isArray(ThreatMap.edge.connections[to]) && typeof ThreatMap.edge.connections[to][from] !== "undefined")
                return { type: "reverse", id: ThreatMap.edge.connections[to][from] };
            return false;
        },
        setConnection: function(from, to, i) {
            if (!Array.isArray(ThreatMap.edge.connections[from]))
                ThreatMap.edge.connections[from] = [ ];
            ThreatMap.edge.connections[from][to] = i;

            if (!Array.isArray(ThreatMap.edge.connections[to]))
                ThreatMap.edge.connections[to] = [ ];
            ThreatMap.edge.connections[to][from] = i;
        },
        defaults: function(params) {
            for (var value in ThreatMap.config.edges.default)
                params[value] = params[value] === false ? false : (params[value] == 0 ? params[value] : (params[value] || ThreatMap.config.edges.default[value]));
            return params;
        },
        add: function(params) {
            params = ThreatMap.edge.defaults(params);
            ThreatMap.edge.connections[params.from] = params.to;
            ThreatMap.edge.connections[params.to] = params.from;
            var option = {
                from: params.from,
                to: params.to,
                color: ThreatMap.edge.getColor(1, params.weight),
                connections: [ ],
                weight: params.weight,
                weights: [ params.weight ],
                types: { },
                dashes: params.potential == 1 ? true : false,
                count: 1,
                arrows: params.has_arrow ? { to: { enable: true, scaleFactor: 0.5 } } : null,
                length: 1,
                hoverWidth: function (width) {return width + 1;},
                smooth: { type: 'continuous' },
                width: params.width,
                selectionWidth: 1
            }
            option.connections[0] = { };
            for (var item in params)
                option.connections[0][item] = params[item];
            return option;
        },
        getColor: function(count, weight) {
            for (var state in ThreatMap.config.edges.scales.weight) {
                var obj = ThreatMap.config.edges.scales.weight[state];
                if (obj.min <= weight && obj.max >= weight) {
                    return obj.color;
                }
            }
        },
        load: function(connections) {
            var edges = [];
            var count = 0;
            for (var i = 0; i < connections.length; i++) {
                var connection = ThreatMap.edge.getConnection(connections[i].from, connections[i].to);
                if (connection !== false) {
                    if (connection.type == "reverse") {
                        ThreatMap.edge.setConnection(connections[i].from, connections[i].to, connection.id);
                        edges[connection.id].arrows = edges[connection.id].arrows || { };
                        edges[connection.id].arrows.from = { enable: true, scaleFactor: 0.5 };
                    }
                    edges[connection.id].weights = edges[connection.id].weights || [ ];
                    edges[connection.id].weights.push(connections[i].weight);
                    edges[connection.id].types[connections[i].type] =
                        edges[connection.id].types[connections[i].type] ?
                        edges[connection.id].types[connections[i].type] + 1 : 1,
                        edges[connection.id].count++;
                    var total_weight = 0;
                    for (var j = 0; j < edges[connection.id].weights.length; j++) {
                        total_weight += edges[connection.id].weights[j];
                    }
                    edges[connection.id].weight = total_weight / (edges[connection.id].count + 1);
                    edges[connection.id].color = ThreatMap.edge.getColor(edges[connection.id].count, edges[connection.id].weight);
                    var count_connections = edges[connection.id].connections.push({ });
                    for (var item in connections[i])
                        edges[connection.id].connections[count_connections - 1][item] = connections[i][item];
                } else {
                    var edge = ThreatMap.edge.add(connections[i]);
                    edge.types[connections[i].type] = 1;
                    edges.push(edge);
                    ThreatMap.edge.setConnection(connections[i].from, connections[i].to, count++);
                }
            }
            return new vis.DataSet(edges);
        },
        highlight: function(edge) {
            var edge = ThreatMap.data.allEdges[edge];
            if (edge.highlight === false) {
                edge.color = Helpers.getColorRestore(edge.color, .2, "#ffffff");
            }
            edge.highlight = true;
        },
        highlightAll: function() {
            for (var edge in ThreatMap.data.allEdges)
                ThreatMap.edge.highlight(edge);
        },
        unhighlight: function(edge) {
            var edge = ThreatMap.data.allEdges[edge];
            if (edge.highlight !== false) {
                edge.color = Helpers.getColorTransparency(edge.color, .2, "#ffffff");
                edge.highlight = false;
            }
        },
        unhighlightAll: function() {
            for (var edge in ThreatMap.data.allEdges)
                ThreatMap.edge.unhighlight(edge);
        },
        update: function() {
            var updateArray = [];
            for (var edgeId in ThreatMap.data.allEdges) {
                if (ThreatMap.data.allEdges.hasOwnProperty(edgeId))
                    updateArray.push(ThreatMap.data.allEdges[edgeId]);
            }
            ThreatMap.data.edges.update(updateArray);
        },
        hide: function(edge) {
            ThreatMap.data.allEdges[edge].hidden = true;
        },
        show: function(edge) {
            ThreatMap.data.allEdges[edge].hidden = false;
        },
        showAll: function() {
            for (var edge in ThreatMap.data.allEdges)
                ThreatMap.edge.show(edge);
        }
    },
    graph: {
        dfs: {
            vertexes: { },
            init: function(root) {
                for (var node in ThreatMap.data.allNodes)
                    ThreatMap.graph.dfs.vertexes[node] = false;
                ThreatMap.graph.dfs.main(root);
            },
            main: function(root) {
                var connectedEdges = ThreatMap.object.getConnectedEdges(root);
                for (var i = 0; i < connectedEdges.length; i++) {
                    var edge = ThreatMap.data.allEdges[connectedEdges[i]];
                    var need_direction = edge.from == root ? "to" : "from";
                    if (!edge.arrows[need_direction] || !edge.arrows[need_direction].enable || edge.hidden)
                        continue;
                    var nodes = ThreatMap.object.getConnectedNodes(connectedEdges[i]);
                    if (nodes.length == 1)
                        return;
                    var node = nodes[0] == root ? nodes[1] : nodes[0];
                    ThreatMap.edge.highlight(connectedEdges[i]);
                    if (ThreatMap.graph.dfs.vertexes[node])
                        continue;
                    ThreatMap.graph.dfs.vertexes[root] = true;
                    ThreatMap.graph.dfs.main(node);
                    ThreatMap.node.highlight(node);
                }
            }
        },
        click: function(params) {
            var has_selected = ThreatMap.node.selected;
            if (params.nodes.length > 0 && !has_selected && ThreatMap.data.allNodes[params.nodes[0]].hidden !== true) {
                ThreatMap.node.unhighlightAll();
                ThreatMap.edge.unhighlightAll();
                ThreatMap.node.select(params.nodes[0]);
            } else if (has_selected) {
                ThreatMap.edge.highlightAll();
                ThreatMap.node.highlightAll();
                ThreatMap.node.unselect();
            }
            if (params.nodes.length > 0 || has_selected) {
                ThreatMap.node.update();
                ThreatMap.edge.update();
            }
        },
        load: function() {
            ThreatMap.data = {
                nodes: ThreatMap.node.load(Helpers.loadJSON("../data/plugs/generator/systems.json")),
                edges: ThreatMap.edge.load(Helpers.loadJSON("../data/plugs/generator/systems.json"))
            };
            ThreatMap.object = new vis.Network($(ThreatMap.config.general.selector)[0], ThreatMap.data, ThreatMap.config.graph);
            ThreatMap.navigation.add("fitToScreen");
            ThreatMap.data.allNodes = ThreatMap.data.nodes.get({ returnType: "Object" });
            ThreatMap.data.allEdges = ThreatMap.data.edges.get({ returnType: "Object" });
        }
    },
    setSize: function() {
        var height;
        if (Helpers.fullscreen.isset()) {
            height = window.screen.height;
            $("section.content").addClass("fullscreen");
        } else {
            height = Helpers.workspace.height() - $("header nav.main").outerHeight() - $("header nav.sub").outerHeight();
            $("section.content").removeClass("fullscreen");
        }
        $("#threat-map").height(height);
    },
    filters: {
        params: {
            nodes: [],
            edges: []
        },
        callbacks: {
            weightByBoundary: function(params, min, max) {
                if (params.weights.length < 2)
                    return false;
                var max_weight = 0;
                for (var i = 0; i < params.weights.length; i++) {
                    if (params.weights[i] > max_weight)
                        max_weight = params.weights[i];
                }
                if (params.weight < max_weight && max_weight <= max && max_weight >= min)
                    return true;
                return false;
            }
        },
        apply: {
            all: function(params) {
                ThreatMap.node.showAll();
                ThreatMap.edge.showAll();
                ThreatMap.filters.apply.edges(params.type);
                ThreatMap.filters.apply.nodes(params.type);
                ThreatMap.edge.update();
                ThreatMap.node.update();
            },
            check: function(object, params, type) {
                var is_visible = true;
                for (var i = 0; i < object.length; i++) {
                    var filter = object[i];
                    if (!filter.values)
                        continue;
                    if (type == "interval") {
                        var is_numeric_ticks = typeof filter.ticks == "number",
                            min = is_numeric_ticks ? filter.values.min : filter.ticks[filter.values.min].min,
                            max = is_numeric_ticks ? filter.values.max : filter.ticks[filter.values.max].max;
                        is_visible = is_visible && ((params[filter.field] >= min && params[filter.field] <= max) || (filter.callback && ThreatMap.filters.callbacks[filter.callback](params, min, max)));
                    } else {
                        is_visible = params[filter.field] == filter.values.value;
                    }
                }
                return is_visible;
            },
            nodes: function(filter_type) {
                for (var node in ThreatMap.data.allNodes) {
                    var params = ThreatMap.data.allNodes[node].image.parseURI().params;
                    if (params.system != "scanner")
                        continue;
                    if (!ThreatMap.filters.apply.check(ThreatMap.filters.params.nodes, params, filter_type == "checkbox" ? "exact" : "interval"))
                        ThreatMap.node.hideWithAdjacent(node);
                }
            },
            edges: function(filter_type) {
                for (var edge in ThreatMap.data.allEdges) {
                    var params = ThreatMap.data.allEdges[edge];
                    if (!ThreatMap.filters.apply.check(ThreatMap.filters.params.edges, params, filter_type == "checkbox" ? "exact" : "interval"))
                        ThreatMap.edge.hide(edge);
                }
            }
        },
        checkbox: {
            count: 0,
            set: function() {
                var $this = $(this);
                var $filter = $this.parents(".filter-checkbox");
                var id = $filter.data("id").split("-")[1],
                    scope = $filter.data("scope");
                var potential = $filter.find("input[data-value=\"0\"]")[0].checked;
                var direct = $filter.find("input[data-value=\"1\"]")[0].checked;
                if (potential && !direct) {
                    ThreatMap.filters.params[scope][id - 1].values = { value: false }
                } else if (!potential && direct) {
                    ThreatMap.filters.params[scope][id - 1].values = { value: true }
                } else {
                    ThreatMap.filters.params[scope][id - 1].values = null;
                }
                ThreatMap.filters.apply.all({ type: "checkbox" });
            },
            build: function(params) {
                var ticks = "";
                for (var i = 0; i < params.ticks.length; i++)
                    ticks += ThreatMap.config.filters.checkbox.template.tick.sprintf(params.ticks[i].value, params.ticks[i].title);
                ThreatMap.filters.params[params.scope].push(params);
                var id = params.scope + "-" + ThreatMap.filters.params[params.scope].length;
                var html = ThreatMap.config.filters.checkbox.template.wrap.sprintf(params.name, id, params.scope, ticks);
                $(ThreatMap.config.filters.selector).append(html);
                $(".filter-checkbox[data-id=\"%d\"]".sprintf(id)).find("input[type=\"checkbox\"]").on("change", ThreatMap.filters.checkbox.set);
            }
        },
        slider: {
            count: 0,
            set: function(event, ui) {
                var id = $(event.target).data("id").split("-")[1],
                    scope = $(event.target).data("scope");
                ThreatMap.filters.params[scope][id - 1].values = { min: ui.values[0], max: ui.values[1] }
                ThreatMap.filters.apply.all({ type: "slider" });
            },
            build: function(params) {
                var is_numeric_ticks = typeof params.ticks == "number",
                    ticks = '',
                    min = params.min || 0,
                    max = params.max || params.ticks.length - 1,
                    step = is_numeric_ticks ? (params.max - params.min) / (params.ticks - 1) : 1;
                for (var i = min; i <= max; i += step)
                    ticks += ThreatMap.config.filters.slider.template.tick.sprintf(is_numeric_ticks ? i.toFixed(params.precision || 0) : params.ticks[i].title);
                ThreatMap.filters.params[params.scope].push(params);
                var id = params.scope + "-" + ThreatMap.filters.params[params.scope].length;
                var html = ThreatMap.config.filters.slider.template.wrap.sprintf(params.name, id, params.scope, ticks);
                $(ThreatMap.config.filters.selector).append(html);
                $(".filter-slider[data-id=%d]".sprintf(id)).slider({
                    range: true,
                    min: min,
                    max: max,
                    step: params.step || 1,
                    values: [ min, max ],
                    slide: ThreatMap.filters.slider.set
                })
            }
        },
        load: function() {
            var filters = Helpers.loadJSON(ThreatMap.config.filters.source);
            for (var i = 0; i < filters.length; i++)
                ThreatMap.filters[filters[i].type].build(filters[i]);
        }
    },
    navigation: {
        add: function(type) {
            $(ThreatMap.config.general.selector + " .vis-network .vis-navigation").append(ThreatMap.config.navigation.template[type]);
        },
        fullscreen: {
            toggle: function() {
                $(this).toggleClass("active");
                $("header").toggle();
                Helpers.fullscreen.toggle();
            },
            off: function() {
                if (!Helpers.fullscreen.isset()) {
                    $("header").show();
                    $(ThreatMap.selector + " .vis-network .vis-navigation .vis-fitToScreen").removeClass("active");
                }
            }
        }
    },
    handlers: function() {
        $(window).on("resize", ThreatMap.setSize);
        ThreatMap.object.on("click", ThreatMap.graph.click);
        ThreatMap.object.on("hoverEdge", ThreatMap.edge.hover);
        ThreatMap.object.on("blurEdge", ThreatMap.edge.blur);
        ThreatMap.object.on("selectEdge", ThreatMap.edge.select);
        ThreatMap.object.on("hoverNode", ThreatMap.edge.blur);
        $(document.body).on("click", ThreatMap.config.general.selector + " .vis-network .vis-navigation .vis-fitToScreen", ThreatMap.navigation.fullscreen.toggle)
        Helpers.fullscreen.change(ThreatMap.navigation.fullscreen.off);
    },
    init: function() {
        $("section.content").html(ThreatMap.templates.wrap);
        ThreatMap.setSize();
        ThreatMap.config = Helpers.loadJSON("/media/config/monitor/threatmap.json");
        ThreatMap.filters.load();
        ThreatMap.graph.load();
        ThreatMap.handlers();
    }
};

Page.Render = {
    init: function() {
        var page = ["monitor", "threat_map"];
        Locale.set("en", { fields: ["connections"] });
        Locale.loadPage(page);
        Header.load(page, { tabs: false, type: "threat-map" });
        ThreatMap.init();
    }
};