Jobs.CodeIssuesByProgram = function(obj, option) {
    this.sectionName = '.tab-codeIssuesByProgram';
    this.section = $(this.sectionName);
    this.parentObj = obj;
    this.tabs = {};
    this.locale = null;
    this.path = ['assess', 'projects', 'code_vulnerability'];
    this.results = {};
    this.tables = {};
    this.codeIssues = {};
    this.templates = {
        no_entry: "<p>" + Locale.ui.page.no_entry + "</p>",
        description_content: '<div class="block-tabs">{tabs}</div><div class="block-content">{content}</div>',
        description_empty: '<div class="block-content">{content}</div>',
        block_content: {
            description: '<p><b>{id_name}</b>: {id_value}</p><p><b>{description_name}:</b></p><p>{description_value}</p><p><b>{threat_name}:</b></p><p>{threat_value}</p>',
            solution: '<p><b>{solution_name}:</b></p><p>{solution_value}</p><p><b>{links_name}:</b></p><p>{links_value}</p>',
            issues: '{issues}',
            stats: '<p><b>{stats_name}:</b></p><p>{stats_value} ms</p>'
        },
        table_loading: "<div class='loading-image'><img src='/media/images/loading.gif' /></div>",
        system_tree: "",
        loading: "<div class='loading'></div>",
        object: "<div class='objects-list'><div class='object-type'>{object_type}:</div><div class='object-name'><a href='#'>{object_name}</a></div></div>"
    };
    this.query = {
        title: "Code issues by program",
        id: "codeIssueByProgram",
        queryObj: {
            "from": [
                {
                    entity: "Result"
                },
                {
                    entity: "Issue"
                }
            ],
            "query": "r1.id = i1.resultId AND i1.issueType = :type1",
            "fields": [
                "r.id",
                "r.credential.host",
                "r.credential.sid",
                "r.credential.appServer",
                "r.credential.customParameters",
                "r.risk",
                "r.status",
                "i.id",
                "i.resultId",
                "i.fields.$AbapProgram.programName"
            ],
            "parameters": [
                {
                    name: "type1",
                    value: "abapProgramIssue"
                }
            ]
        }
    };
    this.runQuery = function(){
        var o = this;
        ERPScan.API.query("queries", "codeIssueByProgram", {
            params: {
                query: JSON.stringify(this.query.queryObj)
            },
            type: "POST",
            callback: function(response) {
                o.codeIssues.results = o.parentObj.conversionFields.call(o.parentObj, response["data"]["result"], o.parentObj.conversionFieldRules.codeIssues);
                o.codeIssues.categorizedCodeIssues = o.categorizedCodeIssues(o.codeIssues.results);
                o.tableLoad();
                // var data = o.conversionFields(result["data"]["result"], o.conversionFieldRules[o.queries[i].id]);
                // o.buildTable(data, i);
            }
        });
    };
    this.categorizedCodeIssues = function(data){
        var programs = [],
            result =[],
            j = 1;

        $.each(data, function(){
            if (programs.indexOf(this.program_name) === -1) {
                programs.push(this.program_name);
            }
        });
        for (var i = 0; i < programs.length; i++) {
            j = 1;
            result.push({
                _id: "type_" + (i+1),
                // check_name: type_obj["title"],
                'program_name': programs[i],
                'check_id': '',
                'issues': "",
                'exploitable': "",
                'sid': "",
                'host': "",
                'appServer': "",
                'entity': "",
                '_is_category': true
            });
            $.each(data, function(){
                if (programs[i] === this.program_name) {
                    this._id = "check_" + (i+1) + "_" + j;
                    this._parent = "type_" + (i+1);
                    this._is_category = false;
                    this.program_name = null;
                    result.push(this);
                }
            });
        }
        return result;
    };
    this.tableLoad = function() {
        var params = {
            ticks: {'__addition': true},
            title: "Issues by program",
            data: this.codeIssues.categorizedCodeIssues,
            option: {
                expandable: false,
                namespace: "projects",
                type: "code_issues",
                tree: true,
                treeColumn: 1
            }
        };
        html = new Block({
            title: this.locale.description_block.title,
            content: this.locale.description_block.empty,
            class: "checkDescription"
        }).getHTML();
        this.section.html(html);

        this.tables.main = new Table(params);
        html = this.tables.main.getHTML();
        this.section.append(html);
        this.tables.main.build(params.data, params.option);
        this.handlers(this.tables.main.number);

        Helpers.setBlockHeight(this.section);
    };
    this.showObjects = function($row, issueInfo) {
        var objects_html = this.templates.object.format({object_type: issueInfo.object.objectType, object_name: issueInfo.object.objectName}),
            fields = [],
            tempObj = {};
        $.each(issueInfo.fields, function(key, val){
            tempObj = {};
            tempObj['name'] = key;
            tempObj['value'] = val;
            fields.push(tempObj);
        });
        this.tables.issues.showAllFields($row);
        $row.find('.addition-field .arrow').after(objects_html);
        $row.removeClass("loading");

    };
    this.showIssues = function(resultInfo) {
        var params = {
            ticks: {"__addition": true},
            title: "Issue list",
            data: this.parentObj.conversionFields(this.results.issues, this.parentObj.conversionFieldRules.issues),
            option: {
                expandable: false,
                namespace: "projects",
                type: "abapProgramIssue",
                count_one_page: 50
            }
        };
        this.tables.issues = new Table(params);
        var $table = $(this.tables.issues.getHTML()),
            table_id = $table.attr("id"),
            issues = $table.find("table")[0].outerHTML;
        var checkInfo = this.parentObj.info.checks[resultInfo.check_id.attr.value];
        var description_content = this.templates.block_content.description.format({
            id_name: this.locale.description_block.content.description.id,
            id_value: checkInfo["common"]["name"],
            description_name: this.locale.description_block.content.description.description,
            description_value: checkInfo["view"]["description"][Locale.name],
            threat_name: this.locale.description_block.content.description.threat,
            threat_value: checkInfo["view"]["threat"][Locale.name]
        });
        var solution_content = this.templates.block_content.solution.format({
            solution_name: this.locale.description_block.content.solution.solution,
            solution_value: checkInfo["view"]["solution"][Locale.name],
            links_name: this.locale.description_block.content.solution.links,
            links_value: checkInfo["view"]["links"][Locale.name]
        });
        var issues_content = this.templates.block_content.issues.format({
            issues: "<div class='' id='" + table_id + "'>" + issues + "</div>"
        });
        var stats_content = this.templates.block_content.stats.format({
            stats_name: this.locale.description_block.content.stats.runtime,
            stats_value: ""
        });
        this.tabs.descriptionBlock = new Tabs({
            container: this.section.find(" .checkDescription .block-content"),
            locale: this.locale.description_block.tabs,
            tabs: {
                issues: issues_content,
                description: description_content,
                solution: solution_content,
                stats: stats_content
            }
        });
        var content = this.tabs.descriptionBlock.getHTML();
        this.section.find(" .checkDescription .block-content").html(content);
        this.section.find(" .checkDescription h3").html(checkInfo["title_" + Locale.name]);
        this.tables.issues.build();
    };
    this.loadIssues = function(resultInfo) {
        var o = this;
        this.section.find(".checkDescription .block-content").html(this.templates.loading);
        ERPScan.API.query("issues", "list", {
            params: {
                resultId: resultInfo.resultId
            },
            callback: function(result) {
                o.results.issues = result.data.result;
                o.showIssues(resultInfo);
            }
        });
    };
    this.clear = function() {
        $(document.body).off("click", this.sectionName + " table.tablesorter tr.leaf");
        $(document.body).off("mousedown", this.sectionName + " table.tablesorter tr.branch");
        $(document.body).off("click", this.sectionName + " .checkDescription .block-tabs .block-tab");
        $(document.body).off("click", this.sectionName + " .system-list .tree .tree-item .text");
        $(document.body).off("click", this.sectionName + " .block.checkDescription [data-name='issues']  table.tablesorter tbody tr");
        $(document.body).off("click", this.sectionName +  " .block.checkDescription [data-name='issues']  table.tablesorter tr.active .toggle-info a");
        $(this.sectionName + ' iframe.blockResize')[0].contentWindow.onresize = null;
        this.section.empty();
    };
    this.handlers = function(tableNum) {
        var o = this,
            table_selector = ' #table-' + tableNum;
        $(document.body).on("click", o.sectionName + table_selector + " table.tablesorter tbody tr.leaf", function() {
            var $this = $(this),
                table = $this.closest('.table'),
                rows = table.find('tr'),
                resultInfo = o.codeIssues.categorizedCodeIssues[$this.attr('data-number')];

            rows.removeClass("active");
            $this.toggleClass("active");
            o.loadIssues(resultInfo);
        });
        $(document.body).on("mousedown", o.sectionName + table_selector + " table.tablesorter tbody tr.branch", function(event) {
            if ($(event.target).parent(".indenter").length > 0) {
                return false;
            }
            var $this = $(this);
            var expanded = $this.hasClass("expanded");
            $(o.sectionName + " table.tablesorter.treetable").treetable(expanded ? "collapseNode" : "expandNode", $this.data("ttId"));
        });
        $(document.body).on("click", o.sectionName + " .block.checkDescription [data-name='issues']  table.tablesorter tbody tr", function(e) {
            var $this = $(this),
                $target = $(e.target);

            if ($target.hasClass(".addition-field") || $target.parents(".addition-field").length > 0) {
                return false;
            }
            if ($this.hasClass("active")) {
                o.tables.issues.hideAllFields($this);
            } else {
                o.showObjects.call(o, $this, o.results.issues[$this.data("number")]);
            }
            return false;
        });
        $(document.body).on("click", o.sectionName + " .block.checkDescription [data-name='issues']  table.tablesorter tr.active .toggle-info a", function(e) {
            var $tr =  $(this).parents("tr.active"),
                $target = $(e.target);

            if ($target.is(".object-name a")) {
                return false;
            }
            o.tables.issues.hideAllFields($tr);
            return false;
        });
        $(o.sectionName + ' .checkDescription iframe.blockResize')[0].contentWindow.onresize = function() {
            var activeRow = o.section.find('.checkDescription [data-name="issues"] tbody tr.active'),
                additionField = activeRow.find('.addition-field');
            additionField.outerWidth(activeRow.width());
        };
    };
    this.load = function() {
        this.runQuery();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
    }
};