Jobs.CodeVulnerability = function(obj, option) {
    this.option = option;
    this.parentObj = obj;
    this.locale = null;
    this.class = { id: 4 };
    this.path = ['assess', 'jobs', 'code_vulnerability'];
    this.results = null;
    this.table = null;
    this.templates = {
        wrap: '<div class="blocks">%s<div class="block col-3 check-description"><h3>%s</h3>%s</div>',
        no_entry: "<p>" + Locale.ui.page.no_entry + "</p>",
        description_content: '<div class="block-tabs">%s</div><div class="block-content">%s</div>',
        description_empty: '<div class="block-content">%s</div>',
        block_tab: '<a href="#" data-name="%s" class="block-tab %s">%s</a>',
        block_content: {
            wrap: '<div class="block-content-tab %s">%s</div>',
            description: '<p><b>%s</b>: %s</p><p><b>%s:</b></p><p>%s</p><p><b>%s:</b></p><p>%s</p>',
            solution: '<p><b>%s:</b></p><p>%s</p><p><b>%s:</b></p><p>%s</p>',
            additional: '<p><b>%s:</b></p><p>%s</p>',
            stats: '<p><b>%s:</b></p><p>%s ms</p>'
        },
        table_loading: "<div class='loading-image'><img src='/media/images/loading.gif' /></div>"
    };
    this.categorizedChecks = function(data) {
        var checks_by_cats = {};
        for (var i = 0; i < data.length; i++) {
            var check = data[i];
            if (!checks_by_cats[check.type]) {
                checks_by_cats[check.type] = { };
            }
            if (!checks_by_cats[check.type][check.subtype]) {
                checks_by_cats[check.type][check.subtype] = [ ];
            }
            checks_by_cats[check.type][check.subtype].push(check);
        }
        var checks_result = [];
        for (var type in checks_by_cats) {
            var type_obj = this.parentObj.checkCats[this.class.id].child[this.class.id + "." + type];
            if (!type_obj) {
                continue;
            }
            checks_result.push({
                name: type_obj.name,
                _id: "type_" + type,
                id: "",
                value: "",
                exploitable: "",
                sid: ""
            });
            for (var subtype in checks_by_cats[type]) {
                var subtype_obj = type_obj.child[this.class.id + "." + type + "." + subtype];
                if (!subtype_obj) {
                    continue;
                }
                checks_result.push({
                    name: subtype_obj.name,
                    _id: "type_" + type + "_" + subtype,
                    _parent: "type_" + type,
                    id: "",
                    value: "",
                    exploitable: "",
                    sid: ""
                });
                var checks = checks_by_cats[type][subtype];
                for (var i = 0; i < checks.length; i++) {
                    checks[i]._id = "check_" + type + "_" + subtype + "_" + i;
                    checks[i]._parent = "type_" + type + "_" + subtype;
                    checks_result.push(checks[i]);
                }
            }
        }
        return checks_result;
    };
    this.pasteCheckInfo = function(check_id, exectime, value, info) {
        var descr_tabs = this.templates.block_tab.sprintf("description", "active", this.locale.description_block.tabs.description) +
            this.templates.block_tab.sprintf("solution", "", this.locale.description_block.tabs.solution) +
            this.templates.block_tab.sprintf("additional", "", this.locale.description_block.tabs.additional) +
            this.templates.block_tab.sprintf("stats", "", this.locale.description_block.tabs.stats);
        var description_content = this.templates.block_content.description.sprintf(this.locale.description_block.content.description.id, check_id, this.locale.description_block.content.description.description, info.data["description_" + Locale.name], this.locale.description_block.content.description.threat, info.data["threat_" + Locale.name]);
        var solution_content = this.templates.block_content.solution.sprintf(this.locale.description_block.content.solution.solution, info.data["solution_" + Locale.name], this.locale.description_block.content.solution.links, info.data["links_" + Locale.name]);
        var additional_content = this.templates.block_content.additional.sprintf(this.locale.description_block.content.additional.check_value, value);
        var stats_content = this.templates.block_content.stats.sprintf(this.locale.description_block.content.stats.runtime, exectime);
        var content = this.templates.block_content.wrap.sprintf("description active", description_content) +
            this.templates.block_content.wrap.sprintf("solution", solution_content) +
            this.templates.block_content.wrap.sprintf("additional", additional_content) +
            this.templates.block_content.wrap.sprintf("stats", stats_content);
        content = this.templates.description_content.sprintf(descr_tabs, content);
        $(".check-description .block-content").html(content);
        $(".check-description h3").html(info.data["title_" + Locale.name]);
    };
    this.conversionCheckList = function() {
        var results = this.parentObj.scanInfo.results;
        var i;
        this.data = [ ];
        for (i = 0; i < results.length; i++) {
            var checkData = this.parentObj.checks[results[i].name].data;
            if (checkData.class != this.class.id) {
                continue;
            }
            var elementObj = {
                name: checkData["title_" + Locale.name],
                id: {
                    value: results[i].name,
                    attr: {
                        name: "exectime",
                        value: results[i].exectime
                    }
                },
                value: results[i].result,
                exploitable: "",
                sid: this.parentObj.SIDByCreds[results[i].cred_id],
                type: checkData.type,
                subtype: checkData.subtype
            };
            this.data.push(elementObj);
        }
    };
    this.buildView = function() {
        var html = this.templates.wrap.sprintf("<div class='table'>" + this.templates.table_loading + "</div>", this.locale.description_block.title, this.templates.description_empty.sprintf(this.locale.description_block.empty));
        $("section.content.tab-" + this.option.type + "").html(html);
    };
    this.tableLoad = function() {
        var data = this.categorizedChecks(this.data);
        var params = {
            title: "Check List",
            data: data,
            option: {
                expandable: false,
                fields: "jobs_check_list",
                ticks: {
                    'name': 0,
                    'id': 1,
                    'value': 2,
                    'exploitable': 3,
                    'sid': ""
                },
                sorting: {
                    0: false,
                    1: false,
                    2: false,
                    3: false,
                    4: false,
                    5: false,
                    6: false,
                    7: false
                },
                count_one_page: 50,
                tree: true
            }
        };
        this.table = new Table(params);
        var html = this.table.getHTML(),
            section = $("section.content.tab-" + this.option.type);
        section.find('.blocks .table').remove();
        section.find('.blocks .block').eq(0).after(html);
        if (this.parentObj.scanInfo.results.length == 0) {
            section.find(".table table.tablesorter").after(this.templates.no_entry);
            section.find(" .table table.tablesorter, section.content.tab-vulnerability_by_" + this.option.type + " .table .pager").remove();
            return;
        }
        this.table.build();
        Helpers.setBlockHeight(section);
    };
    this.clear = function() {
        $("section.content.tab-" + this.option.type).empty();
        $(document.body).off("click", "section.content.tab-" + this.option.type + " table.tablesorter tbody tr.leaf");
        $(document.body).off("mousedown", "section.content.tab-" + this.option.type + " table.tablesorter tbody tr.branch");
        $(document.body).off("click", "section.content.tab-" + this.option.type + " .check-description .block-tabs .block-tab");
    };
    this.handlers = function() {
        var o = this;
        $(document.body).on("click", "section.content.tab-" + o.option.type + " table.tablesorter tbody tr.leaf", function() {
            $("section.content.tab-" + o.option.type + " table.tablesorter tr").removeClass("active");
            var $this = $(this);
            var $id = $this.find("td").eq(o.table.getColumnNumber("id"));
            var check_id = $id.text();
            var value = $this.find("td").eq(o.table.getColumnNumber("value")).text();
            var exectime = $id.data("exectime");
            if (o.parentObj.checks[check_id]) {
                $this.toggleClass("active");
                o.pasteCheckInfo.call(o, check_id, exectime, value, o.parentObj.checks[check_id]);
            }
        });
        $(document.body).on("mousedown", "section.content.tab-" + o.option.type + " table.tablesorter tbody tr.branch", function(event) {
            if ($(event.target).parent(".indenter").length > 0) {
                return false;
            }
            var $this = $(this);
            var expanded = $this.hasClass("expanded");
            $("section.content.tab-" + o.option.type + " table.tablesorter").treetable(expanded ? "collapseNode" : "expandNode", $this.data("ttId"));
        });
        $(document.body).on("click", "section.content.tab-" + o.option.type + " .check-description .block-tabs .block-tab", function() {
            var $this = $(this);
            var name = $this.data("name");
            $("section.content.tab-" + o.option.type + " .check-description .block-content-tab").removeClass("active");
            $("section.content.tab-" + o.option.type + " .check-description .block-content-tab." + name).addClass("active");
            $("section.content.tab-" + o.option.type + " .check-description .block-tabs .block-tab").removeClass("active");
            $this.addClass("active");
            return false;
        });
    };
    this.load = function() {
        this.conversionCheckList();
        this.tableLoad();
        this.handlers();
    };
    this.init = function() {
        if (!this.locale) {
            this.locale = Locale.loadPage(this.path, true);
        }
        this.buildView();
    }
};